/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/







#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>
#include <errno.h>


#ifdef WIN32
#include <winsock2.h>
#include <sys/Timeb.h>
#else
#include <unistd.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <sys/time.h>
#include <termios.h>
#include <netinet/in.h>
#include <err.h>
#include <netdb.h>
#include <arpa/inet.h>
#endif



#include "libkeyinput.h"
#include "lki_define.h"


int	giCmdFD;						
int	giEvFD;							
int	giRecErrCntPW;					

static void LKI_InitLib(void) __attribute__((constructor));


static void LKI_InitLib(void)
{
	giCmdFD = -1;
	giEvFD  = -1;
	giRecErrCntPW = 0;
}


int LKI_OpenSocket(const char* pAddr)
{
	struct	sockaddr_in		stDestaddr;
	struct	protoent		*pTp = getprotobyname("tcp");
	unsigned long int		ulAddr  = 0;
	int iRv;

#ifdef ___DEBUG___
	fprintf(stderr, "----- LKI_OpenSocket start -----\n");
#endif
	
	if((giCmdFD != -1) && (giEvFD != -1)){
		return LKI_SUCCESS;
	}

	
	LKI_InitLib();

	
	
	if(pAddr == NULL){


#ifdef WIN32
		ulAddr = (unsigned int)inet_addr(LOCALHOST_ADDR);
#else
		ulAddr = (unsigned int)inet_network(LOCALHOST_ADDR);
#endif


	}else{


#ifdef WIN32
		ulAddr = (unsigned int)inet_addr(pAddr);
#else
		ulAddr = (unsigned int)inet_network(pAddr);
#endif


	}

	
	giCmdFD = socket(PF_INET, SOCK_STREAM, pTp->p_proto);
	if(giCmdFD < 0){
#ifdef ___DEBUG___
		fprintf(stderr, "socket open...error\n");
#endif
		LKI_CloseSocket();
		return LKI_ERR_FATAL;
	}
#ifdef ___DEBUG___
	fprintf(stderr, "socket open...ok\n");
#endif
 
	
	stDestaddr.sin_family		= AF_INET;
	stDestaddr.sin_port			= htons(IM_TCP_PORT_NO);


#ifdef WIN32
	stDestaddr.sin_addr.s_addr	= ulAddr;
#else
	stDestaddr.sin_addr.s_addr	= htonl(ulAddr);
#endif


	iRv = connect(giCmdFD, (struct sockaddr*)&stDestaddr, sizeof(stDestaddr));
	if(iRv < 0){
#ifdef ___DEBUG___
		fprintf(stderr, "command socket connect...error\n");
#endif
		LKI_CloseSocket();
		return LKI_ERR_FATAL;
	}
#ifdef ___DEBUG___
	fprintf(stderr, "command socket connect...ok\n");
#endif

	
	giEvFD = socket(PF_INET, SOCK_STREAM, pTp->p_proto);
	if(giCmdFD < 0){
#ifdef ___DEBUG___
		fprintf(stderr, "event socket open...error\n");
#endif
		LKI_CloseSocket();
		return LKI_ERR_FATAL;
	}
#ifdef ___DEBUG___
	fprintf(stderr, "event socket open...ok\n");
#endif
 
	
	memset(&stDestaddr, 0, sizeof(stDestaddr));
	stDestaddr.sin_family		= AF_INET;
	stDestaddr.sin_port			= htons(IM_TCP_PORT_NO);


#ifdef WIN32
	stDestaddr.sin_addr.s_addr	= ulAddr;
#else
	stDestaddr.sin_addr.s_addr	= htonl(ulAddr);
#endif


	iRv = connect(giEvFD, (struct sockaddr*)&stDestaddr, sizeof(stDestaddr));
	if(iRv < 0){
#ifdef ___DEBUG___
		fprintf(stderr, "event socket connect...error\n");
#endif
		LKI_CloseSocket();
		return LKI_ERR_FATAL;
	}
#ifdef ___DEBUG___
	fprintf(stderr, "event socket connect...ok\n");
#endif


#ifdef ___DEBUG___
	fprintf(stderr, "----- LKI_OpenSocket end -----\n");
#endif

	return LKI_SUCCESS;

}


int LKI_CloseSocket(void)
{
	int iRv;

	if(giCmdFD >= 0){


#ifdef WIN32
		iRv = closesocket(giCmdFD);
#else
		iRv = close(giCmdFD);
#endif


		giCmdFD = -1;
		if(iRv < -1){
			return LKI_ERR_FATAL;
		}
	}

	if(giEvFD >= 0){


#ifdef WIN32
		iRv = closesocket(giEvFD);
#else
		iRv = close(giEvFD);
#endif


		giEvFD = -1;
		if(iRv < -1){
			return LKI_ERR_FATAL;
		}
	}

	return LKI_SUCCESS;

}

