/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/






#include "Windows.h"
#include "ReadBuf.h"





CReadBuf::CReadBuf()
{
	InitializeCriticalSection(&m_crtSec);
}

CReadBuf::~CReadBuf()
{
	DeleteCriticalSection(&m_crtSec);
}

int CReadBuf::ReadBufferCommon(char * pBuff,char * pData,unsigned int size)
{
	int ret = 0;
	if(pBuff){
		EnterCriticalSection(&m_crtSec);

		BufferHeader *pHeader =  (BufferHeader *)pBuff;
		unsigned long ixRead  = pHeader->ixRead ;
		unsigned long ixWrite = pHeader->ixWrite;
		unsigned long buffersize = pHeader->BufferSize;
		unsigned long overFlow = pHeader->OverFlow;

		char *buffer = pBuff + sizeof(BufferHeader);

		if(pHeader->OverFlow == BUFFER_OVERFLOW_OFF) 
		{
			if( ixRead > ixWrite ) {
				ixWrite += buffersize;
			}

			if ( ixRead+size <= ixWrite ){
				if(ixRead+size < buffersize){
					memcpy(pData,buffer+ixRead,size);
					ixRead += size;
				} else {
					memcpy(pData,buffer+ixRead,buffersize-ixRead);
					if(buffersize < ixRead+size){ 
	
	
						memcpy(pData+(buffersize-ixRead),buffer,size - (buffersize-ixRead) );
					}
					ixRead = ixRead+size-buffersize;
				}
				ret = size;
	
	
			}
		}
		else 
		{
			pHeader->ixRead = ixWrite;

			ret = 1;
		}
		LeaveCriticalSection(&m_crtSec);

	} else {
		return -1;
	}
	return ret;
}

BOOL CReadBuf::MoveReadPointer(char * pBuff,unsigned int size)
{
	if(pBuff){
		EnterCriticalSection(&m_crtSec);

		BufferHeader *pHeader =  (BufferHeader *)pBuff;
		unsigned long ixRead  = pHeader->ixRead ;
		unsigned long buffersize = pHeader->BufferSize;
		if(ixRead + size < buffersize)
			pHeader->ixRead = ixRead + size;
		else
			pHeader->ixRead = ixRead + size - buffersize;

	    LeaveCriticalSection(&m_crtSec);
		return TRUE;
	} else
		return FALSE;
}

BOOL CReadBuf::ReSetOverFlow(char * pBuff)
{
	if(pBuff)
	{
		EnterCriticalSection(&m_crtSec);

		BufferHeader *pHeader =  (BufferHeader *)pBuff;

		pHeader->OverFlow = BUFFER_OVERFLOW_OFF;

		LeaveCriticalSection(&m_crtSec);

		return TRUE;
	} 
	else 
	{
		return FALSE;
	}
}

BOOL CReadBuf::SetDCHandle(char * pBuff,unsigned long dcHandle)
{
	if(pBuff)
	{
		EnterCriticalSection(&m_crtSec);

		BufferHeader *pHeader =  (BufferHeader *)pBuff;

		pHeader->dcHandle = dcHandle;

		LeaveCriticalSection(&m_crtSec);

		return TRUE;
	} 
	else 
	{
		return FALSE;
	}
}