/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
#include "StdAfx.h"
#include "Windows.h"
#include "MirrorBuffer.h"
#include "..\Common\IoCtrlCode.h" 
#include "..\Common\HeadBuff.h" 









#define MAPPED_FILE_NAME "C:\\RDISPLAY.MAP"






#define MUTEX_TIMEOUT 30000	

#include "LogFile.h"
extern CLogFile buffLog; 




MirrorBuffer::MirrorBuffer(){


	m_hDevice=NULL;
	m_ptr    =NULL;

	InitializeCriticalSection(&m_bufferLock); 

}

MirrorBuffer::~MirrorBuffer(){

	DeleteCriticalSection(&m_bufferLock); 

	if(m_hDevice!=NULL){
		Finalize();
	}
}

void MirrorBuffer::Finalize(){

	if(m_hDevice==NULL || m_hDevice==INVALID_HANDLE_VALUE){
		return;
	}

	buffLog.TimePrintHex("Finalize m_hDevice",(unsigned long)m_hDevice);
	buffLog.TimePrintHex("Finalize m_ptr",(unsigned long)m_ptr);

	DWORD dw;
	BOOL bIoCtrl = DeviceIoControl(
		m_hDevice,

		IOCTL_MIRROR_UNMAPPING_MDL,

		&m_ptr,sizeof(ULONG),NULL,0,&dw,NULL
	);
	if(!bIoCtrl){
		return;
	}
	::CloseHandle(m_hDevice);


	m_hDevice=NULL;
	m_ptr    =NULL;
	return;
}

BOOL MirrorBuffer::Initialize(HANDLE hProccess){

	if(m_hDevice!=NULL && m_hDevice!=INVALID_HANDLE_VALUE){
		return FALSE;
	}

	ULONG physicaladdress=0;
	{
		HANDLE hFile,hMap;
		hFile = CreateFile(
			(LPCTSTR)MAPPED_FILE_NAME,
			GENERIC_READ|GENERIC_WRITE,
			FILE_SHARE_READ|FILE_SHARE_WRITE,
			NULL,
			OPEN_ALWAYS,
			FILE_ATTRIBUTE_NORMAL,
			NULL
		);

		if(hFile == INVALID_HANDLE_VALUE)
		{
			buffLog.TimePrint("Initialize CreateFile Error");
			return FALSE;
		}
		hMap=CreateFileMapping(
			hFile,
			NULL,
			PAGE_READWRITE | SEC_COMMIT,


			0, sizeof(ULONG), 
			NULL
		);
		if(hMap == NULL)
		{
			buffLog.TimePrint("Initialize CreateFileMapping Error");
			return FALSE;
		}


		char *ptr = (char*)::MapViewOfFile(hMap,FILE_MAP_ALL_ACCESS,0,0,sizeof(ULONG));
		if(ptr == NULL)
		{
			buffLog.TimePrint("Initialize MapViewOfFile Error");
			return FALSE;
		}
		memcpy(&physicaladdress,ptr,sizeof(ULONG));
		::UnmapViewOfFile(ptr);
		::CloseHandle(hMap);
		::CloseHandle(hFile);	
	}


    m_hDevice = CreateFile(
                "\\\\.\\GpdDev",
                GENERIC_READ,
                FILE_SHARE_READ,
                NULL,
                OPEN_EXISTING,
                0,
                NULL
                );

	if(m_hDevice && m_hDevice != INVALID_HANDLE_VALUE){
		ULONG address=0;
		DWORD dw=0;
		BOOL bIoCtrl;
		bIoCtrl = DeviceIoControl(
			m_hDevice,

			IOCTL_MIRROR_MAPPING_MDL,

			&physicaladdress,sizeof(ULONG),&address,sizeof(ULONG),&dw,NULL
		);
		if(!bIoCtrl){
			char  c[1024];
			FormatMessage(
					FORMAT_MESSAGE_FROM_SYSTEM|
					FORMAT_MESSAGE_IGNORE_INSERTS,
					NULL,
					GetLastError(),
					MAKELANGID(LANG_NEUTRAL,SUBLANG_DEFAULT),
					(LPTSTR)c,
					1023,NULL
					);
			return FALSE;
		}
		m_ptr = (char*)address;

		BufferHeader *pHeader =  (BufferHeader *)m_ptr;


		EnterCriticalSection(&m_bufferLock); 

		pHeader->ixRead  =0;	
		pHeader->ixWrite =0;	
		pHeader->OverFlow =0;	
	
		LeaveCriticalSection(&m_bufferLock); 

		
		buffLog.TimePrintHex("Initialize m_hDevice",(unsigned long)m_hDevice);
		buffLog.TimePrintHex("Initialize m_ptr",(unsigned long)m_ptr);









		return TRUE;
	}
	return FALSE;
}


int MirrorBuffer::ReadBuffer(char * pData,unsigned int size){

	int getSize = m_buffRead.ReadBufferCommon(m_ptr,pData,size);

	CONTAINER *buff = (CONTAINER *)pData;

	BufferHeader *pHeader =  (BufferHeader *)m_ptr;






	if(getSize == 1)
	{
		buffLog.TimePrint("ReadBufferCommon OverFlow");
		return getSize;
	}
	else if(getSize == (int)size)
	{
		if(m_buffRead.MoveReadPointer(m_ptr,size) == TRUE)
		return getSize;
	}
	return 0;






}

int MirrorBuffer::PeekBuffer(char * pData,unsigned int size){

	return m_buffRead.ReadBufferCommon(m_ptr,pData,size);












}

BOOL MirrorBuffer::MoveReadPointer(unsigned int size)
{
	return m_buffRead.MoveReadPointer(m_ptr,size);
}


BOOL MirrorBuffer::ReSetOverFlow()
{
	return m_buffRead.ReSetOverFlow(m_ptr);
}


BOOL MirrorBuffer::SetDCHandle(unsigned long dcHandle)
{
	return m_buffRead.SetDCHandle(m_ptr,dcHandle);
}