/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/



#include "stdafx.h"
#include "Registry.h"
#include <stdio.h>
#include <malloc.h>
#include <stdlib.h>
#include <commctrl.h>

#define ACL_BUFFER_SIZE     1024


CRegistry::CRegistry()
{
	m_hKey = NULL;
}

CRegistry::~CRegistry()
{
	if (m_hKey)
		close();
}

BOOL
CRegistry::create(
	HKEY	hKey,
	LPCTSTR	pSubKey,
	LPTSTR	pClass,
	REGSAM	samDesired)
{
	LONG	rv;
	DWORD	dwDisposition;

	LVITEM *itemStr = new LVITEM;

	SECURITY_ATTRIBUTES sa;
	SECURITY_DESCRIPTOR sdPermissions;

    PSID  psidEveryone = NULL; 
    PACL  paclKey      = NULL; 
  
	SID_IDENTIFIER_AUTHORITY  WorldSidAuthority= SECURITY_WORLD_SID_AUTHORITY; 
 
   
   
 
   sa.nLength              = sizeof(SECURITY_ATTRIBUTES); 
   sa.bInheritHandle       = FALSE; 
   sa.lpSecurityDescriptor = &sdPermissions; 

   
   
 
	if (!AllocateAndInitializeSid(&WorldSidAuthority, 1, SECURITY_WORLD_RID, 
									0, 0, 0, 0, 0, 0, 0, &psidEveryone ) ) {
		delete itemStr;
		return FALSE;
	}
	if (!InitializeSecurityDescriptor(&sdPermissions, SECURITY_DESCRIPTOR_REVISION1)){
 		delete itemStr;
		return FALSE;
	}

   
   
 
   
   
   
 
	if (!(paclKey = (PACL)_alloca(ACL_BUFFER_SIZE))) {
 		delete itemStr;
		return FALSE;
	}
	if (!InitializeAcl(paclKey, ACL_BUFFER_SIZE, ACL_REVISION2)) {
 		delete itemStr;
		return FALSE;
	}
 
   
   
   
 
   if (!AddAccessAllowedAce(paclKey, ACL_REVISION2, KEY_ALL_ACCESS, psidEveryone)) {
 		delete itemStr;
		return FALSE;
	}
   
   
 
	if (!SetSecurityDescriptorDacl(&sdPermissions, TRUE, paclKey, FALSE)) {
 		delete itemStr;
		return FALSE;
	}
	
	rv = RegCreateKeyEx(hKey, pSubKey, 0, pClass, REG_OPTION_NON_VOLATILE
						,  samDesired, &sa, &m_hKey, &dwDisposition);
	delete itemStr;
	if (rv == ERROR_SUCCESS) {
		return TRUE;
	} else {
		m_hKey = NULL;
		return FALSE;
	}
}

BOOL
CRegistry::open(
	HKEY	hKey,
	LPCTSTR	pSubKey,
	REGSAM	samDesired)
{
	LONG	rv;
	rv = RegOpenKeyEx(hKey, pSubKey, 0, samDesired, &m_hKey);
	if (rv == ERROR_SUCCESS) {
		return TRUE;
	} else {
		m_hKey = NULL;
		return FALSE;
	}
}

BOOL
CRegistry::close(
	void)
{
	if (m_hKey) {
		LONG	rv;
		rv = RegCloseKey(m_hKey);
		m_hKey = NULL;
		return (rv == ERROR_SUCCESS);
	} else
		return FALSE;
}

BOOL
CRegistry::setBOOL(
	LPCTSTR	szValue,
	BOOL	bData)
{
	if (m_hKey) {
		LONG	rv;
		rv = RegSetValueEx(m_hKey, szValue, 0, REG_DWORD
							, (const BYTE *)&bData, sizeof(bData));
		return (rv == ERROR_SUCCESS);
	} else
		return FALSE;
}

BOOL
CRegistry::setINT(
	LPCTSTR	szValue,
	INT		dwData)
{
	if (m_hKey) {
		LONG	rv;
		rv = RegSetValueEx(m_hKey, szValue, 0, REG_DWORD
							, (const BYTE *)&dwData, sizeof(dwData));
		return (rv == ERROR_SUCCESS);
	} else
		return FALSE;
}

BOOL
CRegistry::setString(
	LPCTSTR	szValue,
	LPCTSTR	szData)
{
	if (m_hKey) {
		LONG	rv;
		rv = RegSetValueEx(m_hKey, szValue, 0, REG_SZ
							, (const BYTE *)szData, strlen(szData)+1);
		return (rv == ERROR_SUCCESS);
	} else
		return FALSE;
}

BOOL
CRegistry::getBOOL(
	LPCTSTR	szValue,
	BOOL	* pbData)
{
	if (m_hKey) {
		DWORD	cbData = sizeof(BOOL);
		LONG	rv = RegQueryValueEx(m_hKey, szValue, NULL, NULL
						, (BYTE *)pbData, &cbData);
		return (rv == ERROR_SUCCESS);
	} else
		return FALSE;
}

BOOL
CRegistry::getINT(
	LPCTSTR	szValue,
	INT		* pdwData)
{
	if (m_hKey) {
		DWORD	cbData = sizeof(DWORD);
		LONG	rv = RegQueryValueEx(m_hKey, szValue, NULL, NULL
						, (BYTE *)pdwData, &cbData);
		return (rv == ERROR_SUCCESS);
	} else
		return FALSE;
}

BOOL
CRegistry::getString(
	LPCTSTR	szValue,
	LPCTSTR	* pData)
{
	if (m_hKey) {
		DWORD	cbData = 0;
		LONG	rv = RegQueryValueEx(m_hKey, szValue, NULL, NULL
						, NULL, &cbData);
		char	* buf = new char[cbData];
		rv = RegQueryValueEx(m_hKey, szValue, NULL, NULL
						, (BYTE *)buf, &cbData);
		if (rv == ERROR_SUCCESS)
			*pData = buf;
		delete [] buf;
		return (rv == ERROR_SUCCESS);
	} else
		return FALSE;
}



