/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/




#include "stdafx.h"
#include "AdaptersInfo.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif





CAdaptersInfo::CAdaptersInfo()
{
	m_nAdapterCount = 0;
	m_pAdapterInfoBuf = NULL;
	m_AdaptersPList.RemoveAll();
}

CAdaptersInfo::~CAdaptersInfo()
{
	Clear(); 









}

	
int
CAdaptersInfo::Init()
{
	if( m_pAdapterInfoBuf )
		return ADPINFO_ERR_ALREADYINIT;

	
	ULONG bufsize = sizeof(IP_ADAPTER_INFO);
	m_pAdapterInfoBuf = new BYTE[bufsize];
	if( !m_pAdapterInfoBuf )
		return ADPINFO_ERR_FAILALLOCMEM;

	
	DWORD ret = GetAdaptersInfo((IP_ADAPTER_INFO*)m_pAdapterInfoBuf, &bufsize);
	if( ret == ERROR_BUFFER_OVERFLOW )
	{
		
		delete m_pAdapterInfoBuf;
		m_pAdapterInfoBuf = new BYTE[bufsize];
		if( !m_pAdapterInfoBuf )
			return ADPINFO_ERR_FAILALLOCMEM;

		ret = GetAdaptersInfo((IP_ADAPTER_INFO*)m_pAdapterInfoBuf, &bufsize);
	}

	
	if( ret != ERROR_SUCCESS )
	{
		delete m_pAdapterInfoBuf;
		m_pAdapterInfoBuf = NULL;

		if( ret == ERROR_NO_DATA )
			return ADPINFO_ERR_NODATA;

		else if( ret == ERROR_NOT_SUPPORTED )
			return ADPINFO_ERR_NOTSUPPORTED;

		else
			return ADPINFO_ERR_UNKNOWN;
	}

	
	void *pNext = (void*)m_pAdapterInfoBuf;
	while(pNext){
		m_AdaptersPList.AddTail(pNext);
		IP_ADAPTER_INFO *pAdpInf = (IP_ADAPTER_INFO*)pNext;
		pNext = pAdpInf->Next;
	}

	
	m_nAdapterCount = m_AdaptersPList.GetCount();

	return ADPINFO_NOERR;
}


	
void
CAdaptersInfo::Clear()
{
	m_AdaptersPList.RemoveAll();

	if( m_pAdapterInfoBuf ){
		delete m_pAdapterInfoBuf;
		m_pAdapterInfoBuf = NULL;
	}
}


	
UINT
CAdaptersInfo::GetCount()
{
	return m_nAdapterCount;
}


	
char*
CAdaptersInfo::GetAdapterName(int index)
{
	IP_ADAPTER_INFO *pAdpInf = GetAdapterPoi(index);
	if( !pAdpInf ) return NULL;

	return pAdpInf->AdapterName;
}


	
int
CAdaptersInfo::GetAdapterType(int index)
{
	IP_ADAPTER_INFO *pAdpInf = GetAdapterPoi(index);
	if( !pAdpInf ) return ADPINFO_ERR_GETADPINFO;

	return pAdpInf->Type;
}


	


int
CAdaptersInfo::GetIPAddress(ADP_IPADDR *pAddr, int index, int pos)
{
	IP_ADAPTER_INFO *pAdpInf = GetAdapterPoi(index);
	if( !pAdpInf ) return ADPINFO_ERR_GETADPINFO;

	IP_ADDR_STRING *IPpos = &(pAdpInf->IpAddressList);
	
	if( pAddr ){
		for(int i=1; i<=pos; i++){
			if( IPpos == NULL ) return ADPINFO_ERR_GETIPADDR;
			IPpos = IPpos->Next;
		}
		if( IPpos == NULL ) return ADPINFO_ERR_GETIPADDR;
	
		pAddr->uIPAddr = (ULONG)inet_addr(IPpos->IpAddress.String);
		pAddr->uSubnet = (ULONG)inet_addr(IPpos->IpMask.String);
		return ADPINFO_NOERR;
	}else
	{
		int cnt = 0;
		while(IPpos){
			IPpos = IPpos->Next;
			cnt++;
		}
		return cnt;
	}
}

#if 0 
	
ULONG
CAdaptersInfo::GetSubnetMask(int index)
{
	IP_ADAPTER_INFO *pAdpInf = GetAdapterPoi(index);
	if( !pAdpInf ) return NULL;

	IP_ADDR_STRING *IPpos = &(pAdpInf->IpAddressList);
	for(int i=1; i<=pos; i++){
		if( IPpos == NULL ) return NULL;
		IPpos = IPpos->Next;
	}
	if( IPpos == NULL ) return NULL;
	
	return (ULONG)inet_addr(IPpos->IpMask.String);
}
#endif


	
ULONG
CAdaptersInfo::GetGatewayAddr(int index, int pos)
{
	IP_ADAPTER_INFO	 *pAdpInf = GetAdapterPoi(index);
	if( !pAdpInf ) return NULL;
	
	IP_ADDR_STRING *GWpos = &(pAdpInf->GatewayList);
	for(int i=1; i<=pos; i++){
		if( GWpos == NULL ) return NULL;
		GWpos = GWpos->Next;
	}
	if( GWpos == NULL ) return NULL;

	return (ULONG)inet_addr(GWpos->IpAddress.String);
}



	
IP_ADAPTER_INFO*
CAdaptersInfo::GetAdapterPoi(int index)
{
	if( m_AdaptersPList.IsEmpty() || index < 0)
		return NULL;

	POSITION pos = m_AdaptersPList.FindIndex(index);
	if( !pos ) return NULL;

	IP_ADAPTER_INFO *pAdpInf = (IP_ADAPTER_INFO *)m_AdaptersPList.GetAt(pos);

	return pAdpInf;
}



	
BYTE*
CAdaptersInfo::GetMacAddr(int index)
{
	IP_ADAPTER_INFO *pAdpInf = GetAdapterPoi(index);
	if( !pAdpInf ) return NULL;

	return (BYTE*)pAdpInf->Address;
}


	
int
CAdaptersInfo::GetAdapterIndex(BYTE *pMacAddr)
{
	for(int i=0; i<m_AdaptersPList.GetCount(); i++)
	{
		if( memcmp(GetMacAddr(i), pMacAddr, ADPINFO_MACADDRESS_MAXLEN)==0)
			return i;
	}
	return 0;
}



	
char*
CAdaptersInfo::GetDescription(int index)
{
	IP_ADAPTER_INFO *pAdpInf = GetAdapterPoi(index);
	if( !pAdpInf ) return NULL;

	return (char*)pAdpInf->Description;
}