/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/




#include "stdafx.h"
#include "CreateFile.h"

#include <commctrl.h>
#include <malloc.h>
#include <stdio.h>
#include <tchar.h>

#define ACL_BUFFER_SIZE     1024















CCreateFile::CCreateFile()
{
	m_hFile = NULL;
}

CCreateFile::~CCreateFile()
{
	if(m_hFile)
		Close();
}


int CCreateFile::GetOSVersion()
{
	OSVERSIONINFO info;
	memset(&info,0,sizeof(OSVERSIONINFO));
	info.dwOSVersionInfoSize=sizeof(OSVERSIONINFO);
	if (!GetVersionEx(&info)){
		return -1;
	}
	if(info.dwPlatformId==VER_PLATFORM_WIN32_NT){ 
		return 1;


	}else if(info.dwPlatformId==VER_PLATFORM_WIN32_WINDOWS){ 
		return 0;


	} else {
		return -1;
	}
}


BOOL CCreateFile::Create(LPCTSTR path)
{
	LVITEM *itemStr = new LVITEM;

	SECURITY_ATTRIBUTES sa;
	SECURITY_DESCRIPTOR sdPermissions;

	
	ZeroMemory(&sa,sizeof(SECURITY_ATTRIBUTES));

    PSID  psidEveryone = NULL; 
    PACL  paclKey      = NULL; 
  
	SID_IDENTIFIER_AUTHORITY  WorldSidAuthority= SECURITY_WORLD_SID_AUTHORITY; 
 
   
   
 
   sa.nLength              = sizeof(SECURITY_ATTRIBUTES); 
   sa.bInheritHandle       = FALSE; 
   sa.lpSecurityDescriptor = &sdPermissions; 

   
   
 
   if(GetOSVersion() != 1)	
		return TRUE;

	if (!AllocateAndInitializeSid(&WorldSidAuthority, 1, SECURITY_WORLD_RID, 
									0, 0, 0, 0, 0, 0, 0, &psidEveryone ) ) {
		delete itemStr;
		return FALSE;
	}
	if (!InitializeSecurityDescriptor(&sdPermissions, SECURITY_DESCRIPTOR_REVISION1)){
 		delete itemStr;
		return FALSE;
	}

   
   

   
   
   

	if (!(paclKey = (PACL)_alloca(ACL_BUFFER_SIZE))) {
 		delete itemStr;
		return FALSE;
	}

	if (!InitializeAcl(paclKey, ACL_BUFFER_SIZE, ACL_REVISION)) {
 		delete itemStr;
		return FALSE;
	}

   
   
   


   if (!AddAccessAllowedAce(paclKey, ACL_REVISION, KEY_ALL_ACCESS, psidEveryone)) {
 		delete itemStr;
		return FALSE;
	}

   

	if (!SetSecurityDescriptorDacl(&sdPermissions, TRUE, NULL, FALSE)) {

 		delete itemStr;
		return FALSE;
	}

	m_hFile = ::CreateFile(path
						,GENERIC_WRITE | GENERIC_READ
						,FILE_SHARE_DELETE | FILE_SHARE_READ | FILE_SHARE_WRITE
						,&sa
						,CREATE_ALWAYS
						,FILE_ATTRIBUTE_NORMAL
						,NULL);
	delete itemStr;
	if(m_hFile == INVALID_HANDLE_VALUE)
		return FALSE;
	return TRUE;
}

BOOL CCreateFile::Close()
{
	BOOL ret = ::CloseHandle(m_hFile);
	m_hFile=NULL;

	return ret;
}

BOOL CCreateFile::Remove(LPCTSTR path)
{








	return _tremove(path);
}

BOOL CCreateFile::Write(LPCVOID lpBuffer,DWORD bytes)
{
	DWORD result;


	return ::WriteFile(m_hFile
					  ,lpBuffer
					  ,bytes
					  ,&result
					  ,NULL);
}

void  
CCreateFile::GetErrorMessage(void)
{
#ifdef _DEBUG
	LPVOID lpMsgBuf;
	::FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER | 
					 FORMAT_MESSAGE_FROM_SYSTEM | 
					 FORMAT_MESSAGE_IGNORE_INSERTS,
					 NULL, 
					 GetLastError(), 
					 MAKELANGID(LANG_NEUTRAL,SUBLANG_DEFAULT), 
					 (LPTSTR) &lpMsgBuf,	
					 0, 
					 NULL);
	
	::MessageBox(NULL,(LPCTSTR)lpMsgBuf,"CCreateFile Class",MB_OK);
#endif
}
