/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/



#include "stdafx.h"
#include "ECButton.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif




CECButton::CECButton(CEMPLocalize* pLocal)
{
	m_plczRes = pLocal;
	m_bDefault   = TRUE;

	
	m_bRepeat = FALSE;
	m_nRpTimer= NULL;
	
}

CECButton::~CECButton()
{
	
	if( m_bRepeat ){
		if( m_nRpTimer ){
			KillTimer(m_nRpTimer);
			m_nRpTimer = NULL;
		}
	}
	
}


BEGIN_MESSAGE_MAP(CECButton, CButton)
	
	ON_WM_CREATE()
	ON_WM_GETDLGCODE()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_TIMER()
	ON_WM_KILLFOCUS()
	
	ON_MESSAGE(BM_SETSTYLE,OnBsSetStyle)
END_MESSAGE_MAP()



UINT CECButton::OnGetDlgCode() 
{
	DWORD dwStyle;
	dwStyle	=	GetStyle();
	if( dwStyle & WS_TABSTOP)
		return DLGC_BUTTON | DLGC_DEFPUSHBUTTON;
	else
	
		return DLGC_STATIC;
}


LONG CECButton::OnBsSetStyle( UINT wParam, LONG lParam)
{
	wParam;
	lParam;
    return 0; 
}

int CECButton::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CButton::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	
	if( !m_plczRes )
		return -1;

	
	int nID = GetDlgCtrlID();
	PLCCONTROL pCtl = m_plczRes->GetCtlInf(nID);
	if( !pCtl )
		return -1;

	

	
	lpCreateStruct->x = pCtl->rec.left;
	lpCreateStruct->y = pCtl->rec.top;
	lpCreateStruct->cx = pCtl->rec.right - pCtl->rec.left;
	lpCreateStruct->cy = pCtl->rec.bottom - pCtl->rec.top;

	
	
	if( pCtl->textID ){
		
		LOGFONT fnt;
		memset(&fnt, 0, sizeof(LOGFONT));
		::GetObject(GetStockObject(DEFAULT_GUI_FONT), sizeof(LOGFONT), &fnt);

		
		fnt.lfHeight = -(pCtl->fontsize);
		fnt.lfWidth = 0;
		lstrcpy(fnt.lfFaceName, pCtl->font);

		
		if( !m_Font.CreateFontIndirect(&fnt) )
			return FALSE;

		this->SetFont(&m_Font);
	}
	m_bRepeat = pCtl->repeat;

	return 0;
}

void CECButton::DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct) 
{
	int		bDownFlg, bDisable;
	COLORREF cTxtColor;	

	
	if( lpDrawItemStruct->CtlType != ODT_BUTTON || !m_plczRes)
		return;

	CDC *dc = CDC::FromHandle(lpDrawItemStruct->hDC);
	LCCONTROL *pCtl = m_plczRes->GetCtlInf(GetDlgCtrlID());
	int nBmpWid = pCtl->rec.right - pCtl->rec.left;
	int nBmpHei = pCtl->rec.bottom - pCtl->rec.top;



	bDownFlg = 0;
	bDisable = 0;
	cTxtColor = pCtl->txtColor;
	switch(lpDrawItemStruct->itemAction){
	case ODA_DRAWENTIRE:
		if( lpDrawItemStruct->itemState & ODS_DISABLED ){
			m_plczRes->PaintDIBmp(*dc, 0, 0, pCtl->bmpID, nBmpWid, nBmpHei ,0, nBmpHei*3);
			if( pCtl->txtDisableOff == 0 )
				bDisable = 1;
		}else if( lpDrawItemStruct->itemState & ODS_FOCUS){
			m_plczRes->PaintDIBmp(*dc, 0, 0, pCtl->bmpID, nBmpWid, nBmpHei ,0, nBmpHei*2);
		}
		else{
			m_plczRes->PaintDIBmp(*dc, 0, 0, pCtl->bmpID, nBmpWid, nBmpHei ,0, 0);
		}
		SetRgn(pCtl->bmpID, nBmpWid, nBmpHei, 0);
		break;
	case ODA_FOCUS:
		if( lpDrawItemStruct->itemState & ODS_FOCUS){
			m_plczRes->PaintDIBmp(*dc, 0, 0, pCtl->bmpID, nBmpWid, nBmpHei ,0, nBmpHei*2);
		}
		break;
	case ODA_SELECT:
		if( lpDrawItemStruct->itemState & ODS_SELECTED ){
			m_plczRes->PaintDIBmp(*dc, 0, 0, pCtl->bmpID, nBmpWid, nBmpHei ,0, nBmpHei);
			cTxtColor = pCtl->txtColorDown;
			bDownFlg = 1;
		}else
		{
			if( lpDrawItemStruct->itemState & ODS_FOCUS){
				m_plczRes->PaintDIBmp(*dc, 0, 0, pCtl->bmpID, nBmpWid, nBmpHei ,0, nBmpHei*2);
			}else
			{
				m_plczRes->PaintDIBmp(*dc, 0, 0, pCtl->bmpID, nBmpWid, nBmpHei ,0, 0);
			}
		}
		break;
	default:
		break;
	}

	
	RECT recTxt;
	if( pCtl->textID ){
		
		recTxt.left = pCtl->textX + bDownFlg;
		recTxt.top = pCtl->textY + bDownFlg + 2;
		recTxt.right = recTxt.left + pCtl->rec.right - pCtl->rec.left + bDownFlg;
		recTxt.bottom = recTxt.top + pCtl->rec.bottom - pCtl->rec.top + bDownFlg + 2;

		
		dc->SetBkMode(TRANSPARENT);

		
		if( bDisable ){
			if( pCtl->txtDisableColor1 != 0 )
				dc->SetTextColor(pCtl->txtDisableColor1);
			else
				dc->SetTextColor(RGB(255,255,255));
		}else
			dc->SetTextColor(cTxtColor);

		
		char *pStr = m_plczRes->GetString(pCtl->textID);
		if( pStr ){
			
			dc->DrawText(pStr, lstrlen(pStr), &recTxt,  DT_NOPREFIX | DT_CENTER | DT_VCENTER );
			if( bDisable ){	
				recTxt.left -= 1;
				recTxt.top -= 1;
				recTxt.right -= 1;
				recTxt.bottom -= 1;
				if( pCtl->txtDisableColor2 != 0 )
					dc->SetTextColor(pCtl->txtDisableColor2);
				else
					dc->SetTextColor(RGB(200,200,200));
				dc->DrawText(pStr, lstrlen(pStr), &recTxt,  DT_NOPREFIX | DT_CENTER | DT_VCENTER );
			}
		}
	}	
}



void CECButton::OnLButtonDown(UINT nFlags, CPoint point) 
{
	CButton::OnLButtonDown(nFlags, point);

	if( m_bRepeat ){
		if( m_nRpTimer ){
			KillTimer(m_nRpTimer);
			m_nRpTimer = NULL;
		}
		
		m_nRpTimer = SetTimer(IDC_REPEAT_TIMER, ECON_REPEAT_TIME, NULL);
		::SendMessage(((CWnd*)GetParent())->m_hWnd, WM_ECBUTTON_REPEAT,
							REPEAT_WPARAM_DOWN, GetDlgCtrlID());

		PostMessage(WM_TIMER, IDC_REPEAT_TIMER, NULL);
	}
}



void CECButton::OnLButtonUp(UINT nFlags, CPoint point) 
{
	CButton::OnLButtonUp(nFlags, point);
	if( m_bRepeat ){
		if( m_nRpTimer ){
			KillTimer(m_nRpTimer);
			m_nRpTimer = NULL;
		}
		::SendMessage(((CWnd*)GetParent())->m_hWnd, WM_ECBUTTON_REPEAT,
						REPEAT_WPARAM_UP, GetDlgCtrlID());
	}
}



void CECButton::OnTimer(UINT nIDEvent) 
{
	if( m_bRepeat ){
		::SendMessage(((CWnd*)GetParent())->m_hWnd, WM_ECBUTTON_REPEAT,
						REPEAT_WPARAM_HOLD, GetDlgCtrlID());
	}
	CButton::OnTimer(nIDEvent);
}


void
CECButton::OnKillFocus(CWnd* pNewWnd) 
{
	CButton::OnKillFocus(pNewWnd);
	if( m_bRepeat ){
		if( m_nRpTimer ){
			KillTimer(m_nRpTimer);
			m_nRpTimer = NULL;
		}
		::SendMessage(((CWnd*)GetParent())->m_hWnd, WM_ECBUTTON_REPEAT,
						REPEAT_WPARAM_UP, GetDlgCtrlID());
	}
	
}


BOOL
CECButton::SetRgn(int nBmpID, int nWid, int nHei, int nTop)
{
	HRGN hRgn = NULL;
	BYTE *pBits = NULL;

	LCBITMAPDATA *pBmp = m_plczRes->GetBmpData(nBmpID);
	if( !pBmp )
		return FALSE;

	
	if( !pBmp->lcBmp.useMask )
		return TRUE;

	
	LCBITMAPDATA *pMask = m_plczRes->GetBmpData(pBmp->lcBmp.maskID);
	if( !pMask )
		return FALSE;

	int nLineSize = WIDTHBYTES( nWid * pMask->lcBmp.binf.biBitCount);
	pBits = (BYTE*)pMask->pBits + nLineSize * nTop;

	
	{
        int xx;            

        
        
        
        
        DWORD maxRects = 200;

        
        HANDLE hData = GlobalAlloc(GMEM_MOVEABLE, sizeof(RGNDATAHEADER) + (sizeof(RECT) * maxRects));
        if (!hData)
            return FALSE;

        
        RGNDATA *pRgnData = (RGNDATA *)GlobalLock(hData);
        if (!pRgnData)
        {
            
            GlobalFree(hData);
            return FALSE;
        }

        
        pRgnData->rdh.dwSize = sizeof(RGNDATAHEADER);
        pRgnData->rdh.iType = RDH_RECTANGLES;
        pRgnData->rdh.nCount = pRgnData->rdh.nRgnSize = 0;
        SetRect(&pRgnData->rdh.rcBound, MAXLONG, MAXLONG, 0, 0);

        
        pBits += (nHei - 1) * nWid * 3;

        
        for (int y = 0; y < nHei; y++)  
        {
            for (int x = 0; x < nWid; x++)    
            {
                if (*pBits != 0xFF && *(pBits+1) != 0xFF && *(pBits+2) != 0xFF )    
                {                                  
                    xx = x;    

                    
                    for (x = x + 1; x < nWid; x++)
                    {
                        
                        pBits+=3;

                        
                        if (*pBits == 0xFF && *(pBits+1) == 0xFF && *(pBits+2) == 0xFF)
                            break;
                    }

                    
                    RECT *pr = (RECT *)pRgnData->Buffer;
                    SetRect(&pr[pRgnData->rdh.nCount], xx, y, x, y+1);

                    
                    if (xx < pRgnData->rdh.rcBound.left)
                        pRgnData->rdh.rcBound.left = xx;
                    if (y < pRgnData->rdh.rcBound.top)
                        pRgnData->rdh.rcBound.top = y;
                    if (x > pRgnData->rdh.rcBound.right)
                        pRgnData->rdh.rcBound.right = x;
                    if (y+1 > pRgnData->rdh.rcBound.bottom)
                        pRgnData->rdh.rcBound.bottom = y+1;

                    
                    pRgnData->rdh.nCount++;

                    
                    
                    if (maxRects <= pRgnData->rdh.nCount)
                    {
                        HRGN hRgnTmp = ExtCreateRegion(NULL, sizeof(RGNDATAHEADER) + (sizeof(RECT) * maxRects), pRgnData);
                        if (!hRgnTmp)
                        {
                            
                            if (hRgn)
                                DeleteObject(hRgn);
                            
                            GlobalUnlock(hData);
                            GlobalFree(hData);

                            return FALSE;
                        }

                        
                        
                        
                        if (hRgn)
                        {
                            CombineRgn(hRgn, hRgn, hRgnTmp, RGN_OR);
                            DeleteObject(hRgnTmp);
                        }
                        else
                            hRgn = hRgnTmp;

                        pRgnData->rdh.nCount = 0;
                        SetRect(&pRgnData->rdh.rcBound, MAXLONG, MAXLONG, 0, 0);
                    }
                }
                
                pBits += 3;
            }
            
            pBits -= 2 * nWid *3;
        }

        
        HRGN hRgnTmp = ExtCreateRegion(NULL, sizeof(RGNDATAHEADER) + (sizeof(RECT) * maxRects), pRgnData);
        if (!hRgnTmp)
        {
            
            if (hRgn)
                DeleteObject(hRgn);
        }
        else
        {
            
            
            
            if (hRgn)
            {
                CombineRgn(hRgn, hRgn, hRgnTmp, RGN_OR);
                DeleteObject(hRgnTmp);
            }
            else
                hRgn = hRgnTmp;
        }

        
        GlobalUnlock(hData);
		GlobalFree(hData);

    }  
	SetWindowRgn(NULL, FALSE);
	return SetWindowRgn(hRgn, TRUE);
	
}
