/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/



#include "stdafx.h"
#include "ECDialog.h"
#include "ECStatic.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif





CECDialog::CECDialog(UINT nResID, CEMPLocalize *pLocal, CWnd* pParent )
	: CDialog(nResID, pParent)
{
	m_pLocal = pLocal;
	m_nResID = nResID;
	m_lstButton.RemoveAll();
	m_lstStatic.RemoveAll();

	m_nTabCount = 0;
	for(int i=0; i<ECDLG_TABORDER_MAXSIZE; i++)
		m_nTabOrder[i] = 0;

	
		
	
}


void CECDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	
		
	
}


BEGIN_MESSAGE_MAP(CECDialog, CDialog)
	
	ON_WM_CREATE()
	ON_WM_NCHITTEST()
	ON_WM_SYSCOMMAND()
	ON_MESSAGE(WM_CTLCOLORSTATIC ,OnCtlColorStatic)
	ON_WM_PAINT()
	
END_MESSAGE_MAP()




int CECDialog::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CDialog::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	
	if( !m_pLocal )
		return -1;

	

	PLCDIALOG pDlg = m_pLocal->GetDlgInf(m_nResID);
	if( !pDlg )
		return -1;

	
	if( pDlg->font ){
		
		LOGFONT fnt;
		memset(&fnt, 0, sizeof(LOGFONT));
		::GetObject(GetStockObject(DEFAULT_GUI_FONT), sizeof(LOGFONT), &fnt);

		
		fnt.lfHeight = -(pDlg->fontsize);
		fnt.lfWidth = 0;
		lstrcpy(fnt.lfFaceName, pDlg->font);

		
		if( !m_Font.CreateFontIndirect(&fnt) )
			return -1;

		this->SetFont(&m_Font);
	}

	
	ModifyStyle(WS_CAPTION, 0);
	SetMenu(0);
	DrawMenuBar();

	
	RECT recDlg;
	GetWindowRect(&recDlg);
	LCBITMAPDATA *plcBmp = m_pLocal->GetBmpData(pDlg->backgndID);
	if( plcBmp ){
		recDlg.right = recDlg.left + plcBmp->lcBmp.binf.biWidth;
		recDlg.bottom = recDlg.top + plcBmp->lcBmp.binf.biHeight;
	}else
	{
		recDlg.right = recDlg.left + pDlg->wid;
		recDlg.bottom = recDlg.top + pDlg->hei;
	}
	MoveWindow(&recDlg, TRUE);

	
	HRGN hRgn = CreateRoundRectRgn( 
							0,      
							0,      
							recDlg.right - recDlg.left + 1,     
							recDlg.bottom - recDlg.top + 1,    
							7,		
							7		
							);
	SetWindowRgn(hRgn, TRUE);

	
	for(int i=0; i < m_pLocal->GetHeader()->nCtlCnt; i++)
	{
		LCCONTROL *pCtl = m_pLocal->GetCtlInf(ECON_CTRLRESOURCE_ID + i);
		if( !pCtl )
			return -1;

		if( pCtl->dlgID == (int)m_nResID && pCtl->tapStop > 0 && pCtl->tapStop < 512 ){
			m_nTabOrder[pCtl->tapStop] = pCtl->rsID;
			m_nTabCount++;
		}

		if( pCtl->type == CTLTYPE_BUTTON && pCtl->dlgID == (int)m_nResID)
		{
			DWORD style = WS_VISIBLE | BS_OWNERDRAW | WS_CHILD;
			CECButton *pBtn = new CECButton(m_pLocal);
			if( pBtn )
			{
				if( pCtl->tapStop)
					style |= WS_TABSTOP | BS_DEFPUSHBUTTON | WS_GROUP;

				pBtn->Create("", style , pCtl->rec, this, ECON_CTRLRESOURCE_ID + i);
				try{
					m_lstButton.AddTail(pBtn);
				}catch(CMemoryException* e){
					e->Delete();
					return -1;
				}
			}
		
		}else if( pCtl->type == CTLTYPE_STATIC && pCtl->dlgID == (int)m_nResID)
		{
			DWORD style = WS_VISIBLE | SS_NOPREFIX;
			
			if( pCtl->textAlign == CTLALIGN_LEFT )
				style |= SS_LEFT;
			else if( pCtl->textAlign == CTLALIGN_RIGHT )
				style |= SS_RIGHT;
			else if( pCtl->textAlign == CTLALIGN_CENTER )
				style |= SS_CENTER;

			CECStatic *pSTxt = new CECStatic(m_pLocal);
			if( pSTxt )
			{
				pSTxt->Create("", style, pCtl->rec, this, ECON_CTRLRESOURCE_ID + i);
				try{
					m_lstStatic.AddTail(pSTxt);
				}catch(CMemoryException* e){
					e->Delete();
					return -1;
				}
			}
		}
	}

	return 0;
}


UINT
CECDialog::OnNcHitTest(CPoint point) 
{
	UINT nHitTest = CDialog::OnNcHitTest(point);
	CPoint clpos = point;
	ScreenToClient(&clpos);

	if( m_nResID == 5006 ) {
		return nHitTest;
	}else
	{

		if( clpos.y > 3 && clpos.y < 25 )
			return (nHitTest ==  HTCLIENT) ? HTCAPTION : nHitTest;
		else


			return nHitTest;


	}

}


HBRUSH
CECDialog::OnCtlColorStatic(WPARAM wParam, LPARAM lParam)
{
	lParam;
	SetBkMode((HDC) wParam, TRANSPARENT);
	int nCtlID = ::GetDlgCtrlID((HWND)lParam);
	if( nCtlID ){
		LCCONTROL *pCtl = m_pLocal->GetCtlInf(nCtlID);
		if( pCtl ){
			SetTextColor((HDC)wParam, pCtl->txtColor);
		}
	}

	return (HBRUSH)GetStockObject(NULL_BRUSH);
}

BOOL CECDialog::PreTranslateMessage(MSG* pMsg) 
{
	
	if( pMsg->message == WM_KEYDOWN){
		if( pMsg->wParam == 0x1B ){
			return TRUE;
		}
	}
	return CDialog::PreTranslateMessage(pMsg);
}

void CECDialog::OnSysCommand(UINT nID, LPARAM lParam) 
{
	
	if( nID == SC_KEYMENU || nID == SC_CLOSE) return;

	CDialog::OnSysCommand(nID, lParam);
}

BOOL CECDialog::DestroyWindow() 
{
	
	
	if( !m_lstButton.IsEmpty()){
		POSITION pos = m_lstButton.GetHeadPosition();
		while(pos){
			CECButton *ecb = (CECButton*)m_lstButton.GetNext(pos);
			if( ecb ) delete ecb;
		}
	}
	m_lstButton.RemoveAll();

	
	if( !m_lstStatic.IsEmpty()){
		POSITION pos = m_lstStatic.GetHeadPosition();
		while(pos){
			CECStatic *ecb = (CECStatic*)m_lstStatic.GetNext(pos);
			if( ecb ) delete ecb;
		}
	}
	m_lstStatic.RemoveAll();

	return CDialog::DestroyWindow();
}

void CECDialog::OnPaint() 
{
	CPaintDC dc(this); 

	PLCDIALOG pDlg = m_pLocal->GetDlgInf(m_nResID);
	if( pDlg )
		m_pLocal->PaintDIBmp(dc, 0, 0, pDlg->backgndID);
}


BOOL
CECDialog::SetCtrlState(int nDlgItemID, int nResID)
{
	if( !m_pLocal) return FALSE;

	CWnd *wnd = (CWnd*)this->GetDlgItem(nDlgItemID);
	LCCONTROL *lcCtl = m_pLocal->GetCtlInf(nResID);
	if( !wnd || !lcCtl) return FALSE;

	
	wnd->SetFont(&m_Font, TRUE);

	
	if( lcCtl->type == CTLTYPE_CHECKBOX){
		wnd->SetWindowText(m_pLocal->GetString(lcCtl->textID));
	}


	
	wnd->SetWindowPos(0,lcCtl->rec.left,
						lcCtl->rec.top,
						lcCtl->rec.right - lcCtl->rec.left,
						lcCtl->rec.bottom - lcCtl->rec.top,
						SWP_NOZORDER
						);








	return TRUE;
}



void
CECDialog::SetTabOrder()
{
	int i;
	for(i=2; i<m_nTabCount; i++){
		CWnd *preWnd = this->GetDlgItem(m_nTabOrder[i-1]);
		CWnd *nowWnd = this->GetDlgItem(m_nTabOrder[i]);
		if( preWnd && nowWnd ){
			nowWnd->SetWindowPos(preWnd,0,0,0,0,SWP_NOMOVE | SWP_NOSIZE );
		}
	}
}
