/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/




#include "stdafx.h"
#include "Log.h"
#include "ECSearchClient.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif
extern Log searchlog;


UINT SearchReceiveProc( LPVOID pParam );

UINT SearchReceiveProc( LPVOID pParam )
{
	
	
	
#ifndef _AFXDLL
#define _AFX_SOCK_THREAD_STATE AFX_MODULE_THREAD_STATE
#define _afxSockThreadState AfxGetModuleThreadState()
	_AFX_SOCK_THREAD_STATE* pState = _afxSockThreadState;
	if (pState->m_pmapSocketHandle == NULL)
		pState->m_pmapSocketHandle = new CMapPtrToPtr;
	if (pState->m_pmapDeadSockets == NULL)
		pState->m_pmapDeadSockets = new CMapPtrToPtr;
	if (pState->m_plistSocketNotifications == NULL)
		pState->m_plistSocketNotifications = new CPtrList;
#endif

	BYTE rcvBuf[ECUDP_UDP_BUFFSIZE];

	CECSearchClient *pSrch = (CECSearchClient*)pParam;
	if( !pSrch ) return 0;

	
	CAsyncSocket* sock = new CAsyncSocket();
	if( !sock )
		return 1;

	in_addr addr;
	addr.S_un.S_addr = pSrch->m_ulSvrAddr;
	if( !sock->Create(pSrch->m_nPort, SOCK_DGRAM, FD_READ | FD_WRITE, inet_ntoa(addr)) ){
		::PostMessage(pSrch->m_hParentWnd, ECS_MESSAGE, 0, e_sockerror);
		delete sock;
		return 1;
	}

	BOOL bVal = TRUE;
	if( !sock->SetSockOpt(SO_BROADCAST, &bVal, sizeof(BOOL)) ){
		::PostMessage(pSrch->m_hParentWnd, ECS_MESSAGE, 0, e_sockerror);
		delete sock;
		return 1;
	}

	pSrch->SetRunThread(TRUE);


	UINT	rcvPort;
	CString rcvIP;
	for(int i=0; i<ECS_RECEIVING_TIME; i++){
		if( pSrch->GetKill() ) break;
		Sleep(ECS_SLEEP_TIME);
		if( sock->ReceiveFrom(rcvBuf, ECUDP_UDP_BUFFSIZE, rcvIP, rcvPort) != SOCKET_ERROR){
			pSrch->OnReceive(rcvBuf, (LPCTSTR)rcvIP);
		}else
		{
			int err = sock->GetLastError();
			if( err != WSAEWOULDBLOCK ){
				::PostMessage(pSrch->m_hParentWnd, ECS_MESSAGE, 0, e_sockerror);
				delete sock;
				return 1;
			}
		}
	}

	delete sock;

	pSrch->SetRunThread(FALSE);
	return 0;
}




CECSearchClient::CECSearchClient(HWND hPntWnd, ULONG ulSvrAddr, UINT nPort)
{
	m_ClientList.RemoveAll();
	m_hParentWnd = hPntWnd;
	m_pSock = NULL;
	m_nPort = nPort;
	m_ulSvrAddr = ulSvrAddr;
	m_bKill = FALSE;
	m_bRunThread = FALSE;
	m_bListChangeFlg = FALSE; 

	InitializeCriticalSection(&m_csLock);
}

CECSearchClient::~CECSearchClient()
{
	ClearClientList();
	EndReceiveUDP();
	DeleteCriticalSection(&m_csLock);
	searchlog.Print(LL_STATE, "Exit destructor of CECClientThread class.\n");
}

	
BOOL
CECSearchClient::SearchClient(ULONG ulHostAddr, BOOL bNoCheck)
{
	searchlog.Print(LL_STATE, "Enter SearchClient Function.\n");
	m_ulHostAddr = ulHostAddr;

	
	if( !bNoCheck )
		CheckLostCount();


	















	

	return SendSearchCommand(ulHostAddr);
}


	
void
CECSearchClient::CheckLostCount()
{
	searchlog.Print(LL_STATE, "Enter CheckLostCount Function.\n");

	POSITION pos1, pos2;

	
	EnterCriticalSection(&m_csLock);

	if( !m_ClientList.IsEmpty() ){
		for(pos1=m_ClientList.GetHeadPosition(); (pos2=pos1) != NULL;){
			CLIENTINFO *inf = (CLIENTINFO*)m_ClientList.GetNext(pos1);
			if( inf ){
				
				if( inf->m_nLostCounter >= ECSC_PACKETLOST_LIMIT ){
					delete inf;
					m_ClientList.RemoveAt(pos2);
					m_bListChangeFlg = TRUE; 
				}else
				{
					inf->m_nLostCounter++;
				}
			}
		}
	}
	LeaveCriticalSection(&m_csLock);
}

	
BOOL
CECSearchClient::SendSearchCommand(ULONG ulHostAddr)
{
	searchlog.Print(LL_STATE, "Enter SendSearchCommand Function.\n");

	
	CEconCommand econ(m_ulSvrAddr, m_nPort);
	if( ulHostAddr == 0 )
		econ.MakeHeader(e_cmd_easysearch);
	else
		econ.MakeHeader(e_cmd_ipsearch);

	
	if( !SendUDP(&econ, ulHostAddr) ){
		return FALSE;
	}

	return TRUE;
}

	
void
CECSearchClient::StartReceiveUDP()
{
	if( !GetRunThread() ){
		
		SetKill(FALSE);

		
		AfxBeginThread(SearchReceiveProc, this);

		
		int timeout = ECS_THREADWAIT_TIME;
		while(! GetRunThread() && --timeout > 0){
			Sleep(ECS_SLEEP_TIME);
		}
	}
}


	
void
CECSearchClient::EndReceiveUDP()
{
	if( GetRunThread() ){
		
		SetKill(TRUE);

		
		int timeout = ECS_THREADWAIT_TIME;
		while(GetRunThread() && --timeout > 0){
			Sleep(ECS_SLEEP_TIME);
		}

	}
}



void
CECSearchClient::OnReceive(BYTE *pBuf, LPCTSTR rcvIP)
{
	searchlog.Print(LL_STATE, "Enter OnReceive Function.from %s\n", rcvIP);

	unsigned long ulRecvIP = inet_addr(rcvIP);

	if( !pBuf ){
		::PostMessage(m_hParentWnd, ECS_MESSAGE, 0, e_sockerror);
		return;
	}

	
	int cmdID=0;
	CEconCommand econ;
	cmdID = econ.CheckHeader(pBuf);
	if( cmdID == ECMD_ERR_MAGICCODE || cmdID == ECMD_ERR_PROTOVERSION){
		::PostMessage(m_hParentWnd, ECS_MESSAGE, 0, e_commanderror);
		return;
	}

	if( cmdID == e_cmd_clientinfo ){

		econ.Recv(pBuf, pBuf+sizeof(ECONHEADER));
		ECONHEADER *pHead = (ECONHEADER*)pBuf;
		ECONCOMMAND *pEcon = econ.GetEconCommand();
		ECONRECORD *pRecord = econ.GetEconRecord(0);

		if( !pEcon || !pRecord ){
			::PostMessage(m_hParentWnd, ECS_MESSAGE, 0, e_commanderror);
			return;
		}

		
		
		if( memcmp(&m_ulSvrAddr, pHead->IPaddress, ECON_IPADDRESS_SIZE) !=0 ){
			searchlog.Print(LL_STATE, "Server IPAddress missmatch!\n");
			return;
		}
		

		
		CLIENTINFO inf;
		if( GetClientInfo(pRecord->projUniqInfo, &inf) )
		{

			
			if( inf.m_projState != pEcon->command.clientinfo.projState ){
				inf.m_projState = pEcon->command.clientinfo.projState;
				m_bListChangeFlg = TRUE;
			}

			if( inf.m_projUseKeyword != pEcon->command.clientinfo.useKeyword ){
				inf.m_projUseKeyword = pEcon->command.clientinfo.useKeyword;
				m_bListChangeFlg = TRUE;
			}

			if( inf.m_nDisplayType != pEcon->command.clientinfo.displayType ){
				inf.m_nDisplayType = pEcon->command.clientinfo.displayType;
				m_bListChangeFlg = TRUE;
			}
			
			
			if(	lstrcmp(inf.m_projName, pEcon->command.clientinfo.projName)!= 0 ){
				lstrcpy(inf.m_projName, pEcon->command.clientinfo.projName);
				m_bListChangeFlg = TRUE;
			}
			




			inf.m_nLostCounter = 0;
			
			SetClientInfo(&inf);
		}else
		{
			
			CLIENTINFO *newinf = new CLIENTINFO;
			if( !newinf ){
				::PostMessage(m_hParentWnd, ECS_MESSAGE, 0, e_memerror);
				return;
			}
			memcpy(newinf->m_projUniqInfo, pRecord->projUniqInfo, ECON_UNIQINFO_LENGTH );


			memcpy(&newinf->m_projIPaddress, (void*)&ulRecvIP, ECON_IPADDRESS_SIZE);


			lstrcpy(newinf->m_projName, pEcon->command.clientinfo.projName);

			newinf->m_projState = pEcon->command.clientinfo.projState;
			newinf->m_projUseKeyword = pEcon->command.clientinfo.useKeyword;
			newinf->m_nLostCounter = 0;

			newinf->m_nDisplayType = pEcon->command.clientinfo.displayType;
			searchlog.Print(LL_STATE, "add new client info. displaytype=%d\n",newinf->m_nDisplayType);

	
			try{
				m_ClientList.AddTail(newinf);
				m_bListChangeFlg = TRUE; 
			}catch(CMemoryException *e)
			{
				::PostMessage(m_hParentWnd, ECS_MESSAGE, 0, e_memerror);
				e->Delete();
				delete newinf;
				return;
			}
		}
	}
}


	
int
CECSearchClient::GetClientCount()
{
	EnterCriticalSection(&m_csLock);
	int cnt = m_ClientList.GetCount();
	LeaveCriticalSection(&m_csLock);
	return cnt;
}


	
BOOL
CECSearchClient::GetClientInfo(int index, CLIENTINFO *pCltInf)
{
	if( !pCltInf ) return FALSE;

	EnterCriticalSection(&m_csLock);
	if( !m_ClientList.IsEmpty() ){
		POSITION pos = m_ClientList.FindIndex(index);
		if( pos ){
			CLIENTINFO *inf = (CLIENTINFO*)m_ClientList.GetAt(pos);
			if( inf ){
				memcpy(pCltInf, inf, sizeof(CLIENTINFO));
				LeaveCriticalSection(&m_csLock);
				return TRUE;
			}
		}
	}
	LeaveCriticalSection(&m_csLock);
	return FALSE;
}


	
BOOL
CECSearchClient::GetClientInfo(unsigned char* pUniqInfo, CLIENTINFO *pCltInf)
{
	if( !pUniqInfo || !pCltInf) return FALSE;

	EnterCriticalSection(&m_csLock);
	if( !m_ClientList.IsEmpty() ){
		POSITION pos = m_ClientList.GetHeadPosition();
		while(pos){
			CLIENTINFO *inf = (CLIENTINFO*)m_ClientList.GetNext(pos);
			if( inf ){
				if( memcmp(inf->m_projUniqInfo, pUniqInfo, ECON_UNIQINFO_LENGTH) == 0 )
				{
					memcpy(pCltInf, inf, sizeof(CLIENTINFO));
					LeaveCriticalSection(&m_csLock);
					return TRUE;
				}
			}
		}
	}
	LeaveCriticalSection(&m_csLock);
	return FALSE;
}


	
BOOL
CECSearchClient::GetClientInfoByIP(ULONG ipaddr, CLIENTINFO *pCltInf)
{
	EnterCriticalSection(&m_csLock);
	if( !m_ClientList.IsEmpty() ){
		POSITION pos = m_ClientList.GetHeadPosition();
		while(pos){
			CLIENTINFO *inf = (CLIENTINFO*)m_ClientList.GetNext(pos);
			if( inf ){
				if( inf->m_projIPaddress == ipaddr )
				{
					memcpy(pCltInf, inf, sizeof(CLIENTINFO));
					LeaveCriticalSection(&m_csLock);
					return TRUE;
				}
			}
		}
	}
	LeaveCriticalSection(&m_csLock);
	return FALSE;
}



	
BOOL
CECSearchClient::SetClientInfo(CLIENTINFO *pCltInf)
{
	if( !pCltInf ) return FALSE;

	EnterCriticalSection(&m_csLock);

	if( !m_ClientList.IsEmpty() ){
		POSITION pos = m_ClientList.GetHeadPosition();
		while(pos){
			CLIENTINFO *inf = (CLIENTINFO*)m_ClientList.GetNext(pos);
			if( inf ){
				if( memcmp(inf->m_projUniqInfo, pCltInf->m_projUniqInfo, ECON_UNIQINFO_LENGTH) == 0 )
				{
					memcpy(inf, pCltInf, sizeof(CLIENTINFO));
					LeaveCriticalSection(&m_csLock);
					m_bListChangeFlg = TRUE;
					return TRUE;
				}
			}
		}
	}
	LeaveCriticalSection(&m_csLock);
	return FALSE;
}	


	
void
CECSearchClient::ClearClientList()
{
	
	EnterCriticalSection(&m_csLock);
	if( !m_ClientList.IsEmpty() )
	{
		POSITION pos = m_ClientList.GetHeadPosition();
		while(pos)
		{
			CLIENTINFO *inf = (CLIENTINFO*)m_ClientList.GetNext(pos);
			delete inf;
		}
		m_ClientList.RemoveAll();
	}
	LeaveCriticalSection(&m_csLock);
}




BOOL
CECSearchClient::GetRunThread()
{
	return m_bRunThread;
}


void
CECSearchClient::SetRunThread(BOOL run)
{
	EnterCriticalSection(&m_csLock);
	m_bRunThread = run;
	LeaveCriticalSection(&m_csLock);
}



BOOL
CECSearchClient::GetKill()
{
	return m_bKill;
}


void
CECSearchClient::SetKill(BOOL bKill)
{
	EnterCriticalSection(&m_csLock);
	m_bKill = bKill;
	LeaveCriticalSection(&m_csLock);
}


	
BOOL
CECSearchClient::SendUDP(CEconCommand *econ, ULONG ulHostAddr)
{
	if( !econ ) return FALSE;

	CAsyncSocket sock;

	in_addr addr;
	addr.S_un.S_addr = m_ulSvrAddr;
	if( !sock.Create(0, SOCK_DGRAM, FD_READ | FD_WRITE, inet_ntoa(addr)) )
		return FALSE;

	BOOL bVal = TRUE;
	if( !sock.SetSockOpt(SO_BROADCAST, &bVal, sizeof(BOOL)) )
		return FALSE;

	if( !econ->Send(&sock, TRUE, ulHostAddr))
		return FALSE;

	return TRUE;
}