/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/




#include "stdafx.h"

#include "EMPLocalize.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif





CEMPLocalize::CEMPLocalize()
{
	memset(&m_lcHeader, 0, sizeof(LCHEADER));
	m_lstDialog.RemoveAll();
	m_lstControl.RemoveAll();
	m_lstString.RemoveAll();
	m_lstBitmap.RemoveAll();
}

CEMPLocalize::~CEMPLocalize()
{
	ClearAllList();
}


	
int
CEMPLocalize::Load(char *sFilePath)
{
	int i=0;
	try{
		
		CFile in(sFilePath, CFile::modeRead );
	
		
		in.Read(&m_lcHeader, sizeof(LCHEADER));

		
		if( memcmp(m_lcHeader.magic, magic_code, MAGICCODE_LENGTH) != 0 ){
			in.Close();
			return LCZ_ERR_NOEMPLOCALFILE;
		}

		
		for(i=0; i<m_lcHeader.nDlgCnt; i++){
			LCDIALOG *lcDlg = new LCDIALOG;
			if( !lcDlg ){
				in.Close();
				goto LcErrMemoryAlloc;
			}
			in.Read(lcDlg, sizeof(LCDIALOG));
			try{
				m_lstDialog.AddTail(lcDlg);
			}catch(CMemoryException* e){
				e->Delete();
				in.Close();
				goto LcErrMemoryAlloc;
			}
		}

		
		for(i=0; i<m_lcHeader.nCtlCnt; i++){
			LCCONTROL *lcCtl = new LCCONTROL;
			if( !lcCtl ){
				in.Close();
				goto LcErrMemoryAlloc;
			}
			in.Read(lcCtl, sizeof(LCCONTROL));
			try{
				m_lstControl.AddTail(lcCtl);
			}catch(CMemoryException* e){
				e->Delete();
				in.Close();
				goto LcErrMemoryAlloc;
			}
		}

		
		for(i=0; i<m_lcHeader.nStrCnt; i++){
			LCSTRING *lcStr = new LCSTRING;
			if( !lcStr ){
				in.Close();
				goto LcErrMemoryAlloc;
			}
			in.Read(lcStr, sizeof(LCSTRING));
			try{
				m_lstString.AddTail(lcStr);
			}catch(CMemoryException* e){
				e->Delete();
				in.Close();
				goto LcErrMemoryAlloc;
			}
		}

		
		for(i=0; i<m_lcHeader.nBmpCnt+m_lcHeader.nMskCnt; i++){
			
			LCBITMAPDATA *lcBmp = new LCBITMAPDATA;
			if( !lcBmp ){
				in.Close();
				goto LcErrMemoryAlloc;
			}
			in.Read(&lcBmp->lcBmp, sizeof(LCBITMAP));

			
			BYTE *pBits = new BYTE[lcBmp->lcBmp.bitSize];
			if( !pBits ){
				delete lcBmp;
				in.Close();
				goto LcErrMemoryAlloc;
			}
			in.Read(pBits, lcBmp->lcBmp.bitSize);
			lcBmp->pBits = pBits;
			

			
			try{
				m_lstBitmap.AddTail(lcBmp);
			}catch(CMemoryException* e){
				e->Delete();
				in.Close();
				goto LcErrMemoryAlloc;
			}
		}
		in.Close();

	}catch(CFileException* e)
	{
		e->Delete();
		return LCZ_ERR_FILEACCESS;
	}

	return LCZ_NOERR;

LcErrMemoryAlloc:
	ClearAllList();
	return LCZ_ERR_MEMALLOC;
}



LCDIALOG*
CEMPLocalize::GetDlgInf(int nResID)
{
	if( !m_lstDialog.IsEmpty() )
	{
		POSITION pos = m_lstDialog.GetHeadPosition();
		while(pos)
		{
			LCDIALOG *lcDlg = (LCDIALOG*)m_lstDialog.GetNext(pos);
			if( lcDlg )
				if( lcDlg->rsID == nResID )
					return lcDlg;
		}
	}
	return (LCDIALOG*)NULL;
}



LCCONTROL*
CEMPLocalize::GetCtlInf(int nResID)
{
	if( !m_lstControl.IsEmpty() )
	{
		POSITION pos = m_lstControl.GetHeadPosition();
		while(pos)
		{
			LCCONTROL *lcCtl = (LCCONTROL*)m_lstControl.GetNext(pos);
			if( lcCtl )
				if( lcCtl->rsID == nResID )
					return lcCtl;
		}
	}
	return (LCCONTROL*)NULL;
}



char*
CEMPLocalize::GetString(int nResID)
{
	if( !m_lstString.IsEmpty() )
	{
		POSITION pos = m_lstString.GetHeadPosition();
		while(pos)
		{
			LCSTRING *lcStr = (LCSTRING*)m_lstString.GetNext(pos);
			if(lcStr)
				if( lcStr->rsID == nResID )
					return lcStr->rsString;
		}
	}
	return (char*)NULL;
}



LCBITMAPDATA*
CEMPLocalize::GetBmpData(int nResID)
{
	if( !m_lstBitmap.IsEmpty() )
	{
		POSITION pos = m_lstBitmap.GetHeadPosition();
		while(pos)
		{
			LCBITMAPDATA *lcBmp = (LCBITMAPDATA*)m_lstBitmap.GetNext(pos);
			if(lcBmp)
				if( lcBmp->lcBmp.rsID== nResID )
					return lcBmp;
		}
	}
	return (LCBITMAPDATA*)NULL;
}



BOOL
CEMPLocalize::PaintDIBmp(CDC &dc, int nXDst, int nYDst, int nResID, 
						int nWid, int nHei, int nXSrc, int nYSrc)
{
	DWORD rop = SRCCOPY;
	
	LCBITMAPDATA *pMaskdat = NULL;
	LCBITMAPDATA *pBmpdat = GetBmpData(nResID);
	if( !pBmpdat ) return FALSE;

	if( pBmpdat->lcBmp.useMask ){
		pMaskdat = GetBmpData(pBmpdat->lcBmp.maskID);
	}
		
	if(nWid < 0) nWid = pBmpdat->lcBmp.binf.biWidth;
	if(nHei < 0) nHei = pBmpdat->lcBmp.binf.biHeight;

	
	BITMAPINFO inf;
	inf.bmiHeader = pBmpdat->lcBmp.binf;
	HBITMAP hMask = NULL;
	HBITMAP hBmp = CreateDIBitmap(	dc.m_hDC, 
									&(pBmpdat->lcBmp.binf),
									CBM_INIT, pBmpdat->pBits,
									&inf, DIB_RGB_COLORS);
	if( !hBmp ) return FALSE;


	if( pBmpdat->lcBmp.useMask ){
		hMask = CreateDIBitmap(	dc.m_hDC, 
								&(pMaskdat->lcBmp.binf),
								CBM_INIT, pMaskdat->pBits,
								&inf, DIB_RGB_COLORS);
		if( !hMask ) return FALSE;
	}

	CBitmap *pOldBitmap=NULL;
	CDC MemDC,MemDC2;

	if( pBmpdat->lcBmp.useMask ){
		MemDC2.CreateCompatibleDC(&dc);
		CBitmap *pcBmp = CBitmap::FromHandle(hMask);
		pOldBitmap = MemDC2.SelectObject(pcBmp);

		if( !dc.BitBlt(nXDst, nYDst, nWid, nHei, &MemDC2, nXSrc, nYSrc, SRCAND))
			return FALSE;

		MemDC2.SelectObject(pOldBitmap);
	
		DeleteObject(hMask);
	}


	
	MemDC.CreateCompatibleDC(&dc);
	CBitmap *pcBmp = CBitmap::FromHandle(hBmp);
	pOldBitmap = MemDC.SelectObject(pcBmp);

	if( pBmpdat->lcBmp.useMask ) rop = SRCINVERT;

	if( !dc.BitBlt(nXDst, nYDst, nWid, nHei, &MemDC, nXSrc, nYSrc, rop))
		return FALSE;

	MemDC.SelectObject(pOldBitmap);

	DeleteObject(hBmp);



	return TRUE;
}


void
CEMPLocalize::ClearAllList()
{
	
	if( !m_lstDialog.IsEmpty() )
	{
		POSITION pos = m_lstDialog.GetHeadPosition();
		while(pos)
		{
			LCDIALOG *lcDlg = (LCDIALOG*)m_lstDialog.GetNext(pos);
			delete lcDlg;
		}
		m_lstDialog.RemoveAll();
	}

	
	if( !m_lstControl.IsEmpty() )
	{
		POSITION pos = m_lstControl.GetHeadPosition();
		while(pos)
		{
			LCCONTROL *lcCtl = (LCCONTROL*)m_lstControl.GetNext(pos);
			delete lcCtl;
		}
		m_lstControl.RemoveAll();
	}

	
	if( !m_lstString.IsEmpty() )
	{
		POSITION pos = m_lstString.GetHeadPosition();
		while(pos)
		{
			LCSTRING *lcStr = (LCSTRING*)m_lstString.GetNext(pos);
			delete lcStr;
		}
		m_lstString.RemoveAll();
	}

	
	if( !m_lstBitmap.IsEmpty() )
	{
		POSITION pos = m_lstBitmap.GetHeadPosition();
		while(pos)
		{
			LCBITMAPDATA *lcBmp = (LCBITMAPDATA*)m_lstBitmap.GetNext(pos);
			if( lcBmp->pBits )
				delete[] lcBmp->pBits;
			delete lcBmp;
		}
		m_lstBitmap.RemoveAll();
	}
}