/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/




#include "stdafx.h"
#include "EMPCommon.h"
#include "Log.h"
#include "EconCommand.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

extern Log netlog;




CEconCommand::CEconCommand(ULONG ulSvrAddr, int port)
{
	m_nPort = port;
	m_bHeaderOnly = TRUE;
	m_header.datasize = 0;
	memcpy(m_header.IPaddress, &ulSvrAddr, ECON_IPADDRESS_SIZE);
	memcpy(m_header.magicnum, ECON_MAGIC_NUMBER, ECON_MAGICNUM_SIZE);

	memcpy(m_header.version, ECON_PROTO_VERSION, ECON_PROTOVER_MAXLEN);

}

CEconCommand::CEconCommand()
{
}

CEconCommand::~CEconCommand()
{
	Clear();
}


	
void
CEconCommand::SetParam(ULONG ulSvrAddr, int port)
{
	m_nPort = port;
	m_bHeaderOnly = TRUE;
	m_header.datasize = 0;
	memcpy(m_header.IPaddress, &ulSvrAddr, ECON_IPADDRESS_SIZE);
	memcpy(m_header.magicnum, ECON_MAGIC_NUMBER, ECON_MAGICNUM_SIZE);

	memcpy(m_header.version, ECON_PROTO_VERSION, ECON_PROTOVER_MAXLEN);

}


	
void
CEconCommand::MakeHeader(int cmdID)
{
	m_header.commandID = cmdID;
}


	
void
CEconCommand::MakeReqConnect(	BOOL useEnc,
								unsigned char *encKey,
								ULONG subnet,
								ULONG gateway )
{
	m_bHeaderOnly = FALSE;
	MakeHeader(e_cmd_reqconnect);
	m_command.recordCount = (unsigned char)m_lstRecord.GetCount();
	m_command.command.reqconnect.useEncryption = (unsigned char)useEnc;
	memcpy(m_command.command.reqconnect.EncPassword, encKey, ECON_ENCRYPTION_MAXLEN);
	memcpy(m_command.command.reqconnect.subnetMask, &subnet, ECON_IPADDRESS_SIZE);
	memcpy(m_command.command.reqconnect.gateAddress, &gateway, ECON_IPADDRESS_SIZE);
}


	
void
CEconCommand::MakeDispWait()
{
	m_bHeaderOnly = FALSE;
	MakeHeader(e_cmd_displaywait);
	m_command.recordCount = (unsigned char)m_lstRecord.GetCount();
}























BOOL
CEconCommand::AddRecord(	unsigned char* pUniqInf,
							char* pKeyword,
							ULONG ulPingAddr )
{
	m_bHeaderOnly = FALSE;

	ECONRECORD *erec = new ECONRECORD;
	if( !erec ) return FALSE;

	memcpy(erec->projUniqInfo, pUniqInf, ECON_UNIQINFO_LENGTH);
	memset(erec->projKeyword, 0, ECON_KEYWORD_MAXLEN);
	strcpy((char*)erec->projKeyword, pKeyword);
	memcpy(erec->IPaddress, &ulPingAddr, ECON_IPADDRESS_SIZE);
	try{
		m_lstRecord.AddTail(erec);
	}catch(CMemoryException* e){
		e->Delete();
		delete erec;
		return FALSE;
	}

	return TRUE;
}



BOOL
CEconCommand::DeleteRecord(	unsigned char* pUniqInf )
{
	
	POSITION pos = m_lstRecord.GetHeadPosition(),posbak;
	while(pos){
		posbak = pos;
		ECONRECORD *pErec = (ECONRECORD*)m_lstRecord.GetNext(pos);
		if( !pErec ) return FALSE;
		if( memcmp(pErec->projUniqInfo, pUniqInf, ECON_UNIQINFO_LENGTH)==0){
			delete pErec;
			m_lstRecord.RemoveAt(posbak);
			return TRUE;
		}
	}
	return FALSE;
}



void
CEconCommand::Clear()
{
	memset(&m_header, 0, sizeof(ECONHEADER));
	memset(&m_command, 0, sizeof(ECONCOMMAND));
	if( !m_lstRecord.IsEmpty() ){
		POSITION pos = m_lstRecord.GetHeadPosition();
		while(pos){
			ECONRECORD *erec = (ECONRECORD *)m_lstRecord.GetNext(pos);
			if( erec ) delete erec;
		}
	}
	m_lstRecord.RemoveAll();
	m_bHeaderOnly = TRUE;
}



BOOL
CEconCommand::Send(	CAsyncSocket *sock,
					BOOL	bByUDP,
					ULONG	hostaddr )
{
	int nSendBufSize = sizeof(ECONHEADER);
	int nDataBufSize = 0;
	BYTE *pSendBuf = NULL;
	BYTE *pDataBuf = NULL;

	if( !sock ) return FALSE;

	if( !m_bHeaderOnly ){

		
		int nRecCnt = m_lstRecord.GetCount();

		
		nDataBufSize = sizeof(ECONCOMMAND) + sizeof(ECONRECORD)*nRecCnt;
		pDataBuf = new BYTE[nDataBufSize];
		if( !pDataBuf) return FALSE;


		
		m_command.recordCount = (unsigned char)nRecCnt;
		memcpy(pDataBuf, &m_command, sizeof(ECONCOMMAND));
		for(int i=0; i<nRecCnt; i++){
			POSITION pos = m_lstRecord.FindIndex(i);
			memcpy(pDataBuf + sizeof(ECONCOMMAND) + sizeof(ECONRECORD)*i, 
				m_lstRecord.GetAt(pos), sizeof(ECONRECORD));
		}

		nSendBufSize += nDataBufSize;
	}

	
	pSendBuf = new BYTE[nSendBufSize];
	if( !pSendBuf ){
		delete[] pDataBuf;
		return FALSE;
	}

	
	m_header.datasize = nDataBufSize;
	memcpy(pSendBuf, &m_header, sizeof(ECONHEADER));
	if( !m_bHeaderOnly )
		memcpy(pSendBuf+sizeof(ECONHEADER), pDataBuf, nDataBufSize);

	delete[] pDataBuf;

	
	char *sAddress;
	if( hostaddr == 0 ){
		sAddress = NULL;
	}else{
		in_addr addr;
		addr.S_un.S_addr = hostaddr;
		sAddress = inet_ntoa(addr);
	}
	if( bByUDP ){
		if( sock->SendTo(pSendBuf, nSendBufSize, m_nPort, sAddress) == SOCKET_ERROR ){
			delete[] pSendBuf;
			return FALSE;
		}
	}else{
		if( sock->Send(pSendBuf, nSendBufSize) == SOCKET_ERROR ){
			delete[] pSendBuf;
			return FALSE;
		}
		
	}
	delete[] pSendBuf;

	netlog.Print(LL_STATE, "<<<<<<< Sent EconCommand Data >>>>>>>\n");
	OutputLog();

	return TRUE;
}



int
CEconCommand::CheckHeader(BYTE *pHeader)
{
	ECONHEADER *ph = (ECONHEADER*)pHeader;
	if( memcmp(ph->magicnum, ECON_MAGIC_NUMBER, ECON_MAGICNUM_SIZE)!=0)
		return ECMD_ERR_MAGICCODE;

	if( memcmp(ph->version, ECON_PROTOCOL_VERSION, 2) != 0 )
		return ECMD_ERR_PROTOVERSION;

	return ph->commandID;
}


		
void
CEconCommand::Recv(BYTE *pHeader, BYTE *pDataBuf)
{
	netlog.Print(LL_STATE, "<<<<<<< Receive EconCommand Data >>>>>>>\n");


	Clear();
	memcpy(&m_header, pHeader, sizeof(ECONHEADER));
	if( m_header.datasize > 0 && pDataBuf){
		memcpy(&m_command, pDataBuf, sizeof(ECONCOMMAND));


		ECONRECORD *erec;
		for(int i=0; i<m_command.recordCount; i++){
			erec = (ECONRECORD*)(pDataBuf + sizeof(ECONCOMMAND) + sizeof(ECONRECORD)*i);
			this->AddRecord(erec->projUniqInfo, (char*)erec->projKeyword, 0);
		}
	}

	OutputLog();
}


	
int
CEconCommand::GetRecordNum()
{
	return m_lstRecord.GetCount();
}

	
ECONCOMMAND*
CEconCommand::GetEconCommand()
{
	return &m_command;
}


	
ECONRECORD*
CEconCommand::GetEconRecord(int index)
{
	POSITION pos = m_lstRecord.FindIndex(index);
	return (ECONRECORD*)m_lstRecord.GetAt(pos);
}



void
CEconCommand::OutputLog()
{
	char tmpstr[MAX_PATH];
	
	switch(m_header.commandID){
	case e_cmd_easysearch:
		strcpy(tmpstr, "e_cmd_easysearch");
		break;
	case e_cmd_ipsearch:
		strcpy(tmpstr, "e_cmd_ipsearch");
		break;
	case e_cmd_clientinfo:
		strcpy(tmpstr, "e_cmd_clientinfo");
		break;
	case e_cmd_reqconnect:
		strcpy(tmpstr, "e_cmd_reqconnect");
		break;
	case e_cmd_connected:
		strcpy(tmpstr, "e_cmd_connected");
		break;
	case e_cmd_reqrestart:
		strcpy(tmpstr, "e_cmd_reqrestart");
		break;
	case e_cmd_finishrestart:
		strcpy(tmpstr, "e_cmd_finishrestart");
		break;
	case e_cmd_disconclient:
		strcpy(tmpstr, "e_cmd_disconclient");
		break;
	case e_cmd_interrupt:
		strcpy(tmpstr, "e_cmd_interrupt");
		break;
	case e_cmd_keepalive:
		strcpy(tmpstr, "e_cmd_keepalive");
		break;







	case e_cmd_clienterror:
		strcpy(tmpstr, "e_cmd_clienterror");
		break;
	case e_cmd_resendfullscrid:
		strcpy(tmpstr, "e_cmd_resendfullscrid");
		break;

	case e_cmd_displaywait:
		strcpy(tmpstr, "e_cmd_displaywait");
		break;

	default:
		strcpy(tmpstr, "?????");
		break;
	};

	netlog.Print(LL_STATE, "[ ECONHEADER ]\n");
	netlog.Print(LL_STATE, "\tcommandID = %s\n", tmpstr);
	netlog.Print(LL_STATE, "\tIPAddress = %s\n", inet_ntoa(*(in_addr*)&m_header.IPaddress));
	netlog.Print(LL_STATE, "\tdatasize = %ld\n", m_header.datasize);

	if( m_header.datasize == 0 ) return;

	
	netlog.Print(LL_STATE, "[ ECONCOMMAND ]\n");
	switch(m_header.commandID){
	case e_cmd_clientinfo:
		netlog.Print(LL_STATE, "\tRecord Count = %d\n", m_command.recordCount);
		netlog.Print(LL_STATE, "\tProjector Name = %s\n", m_command.command.clientinfo.projName);
		netlog.Print(LL_STATE, "\tstatus = %d\n", m_command.command.clientinfo.projState);
		netlog.Print(LL_STATE, "\tuseKeyword = %d\n", m_command.command.clientinfo.useKeyword);
		break;
	case e_cmd_reqconnect:
		netlog.Print(LL_STATE, "\tRecord Count = %d\n", m_command.recordCount);
		netlog.Print(LL_STATE, "\tuseEncryption = %d\n", m_command.command.reqconnect.useEncryption);
		memcpy(tmpstr, m_command.command.reqconnect.EncPassword, ECON_ENCRYPTION_MAXLEN);
		tmpstr[ECON_ENCRYPTION_MAXLEN] = '\0';
		netlog.Print(LL_STATE, "\tuseEncryption = %s\n", tmpstr);
		netlog.Print(LL_STATE, "\tsubnetMask = %s\n", 
			inet_ntoa(*((in_addr*)&m_command.command.reqconnect.subnetMask)));
		netlog.Print(LL_STATE, "\tgateAddress = %s\n", 
			inet_ntoa(*((in_addr*)&m_command.command.reqconnect.gateAddress)));
		netlog.Print(LL_STATE, "\tScreen Format\n");
		netlog.Print(LL_STATE, "\tframebufferWidth = %d\n", m_command.command.reqconnect.vnesInitMsg.framebufferWidth);
		netlog.Print(LL_STATE, "\tframebufferHeight = %d\n", m_command.command.reqconnect.vnesInitMsg.framebufferHeight);
		netlog.Print(LL_STATE, "\tnameLength = %d\n", m_command.command.reqconnect.vnesInitMsg.nameLength);
		netlog.Print(LL_STATE, "\tbitsPerPixel = %d\n", m_command.command.reqconnect.vnesInitMsg.format.bitsPerPixel);
		netlog.Print(LL_STATE, "\tdepth = %d\n", m_command.command.reqconnect.vnesInitMsg.format.depth);
		netlog.Print(LL_STATE, "\tbigEndian = %d\n", m_command.command.reqconnect.vnesInitMsg.format.bigEndian);
		netlog.Print(LL_STATE, "\ttrueColour = %d\n", m_command.command.reqconnect.vnesInitMsg.format.trueColour);
		netlog.Print(LL_STATE, "\tredMax = %d\n", m_command.command.reqconnect.vnesInitMsg.format.redMax);
		netlog.Print(LL_STATE, "\tgreenMax = %d\n", m_command.command.reqconnect.vnesInitMsg.format.greenMax);
		netlog.Print(LL_STATE, "\tblueMax = %d\n", m_command.command.reqconnect.vnesInitMsg.format.blueMax);
		netlog.Print(LL_STATE, "\tredShift = %d\n", m_command.command.reqconnect.vnesInitMsg.format.redShift);
		netlog.Print(LL_STATE, "\tgreenShift = %d\n", m_command.command.reqconnect.vnesInitMsg.format.greenShift);
		netlog.Print(LL_STATE, "\tblueShift = %d\n", m_command.command.reqconnect.vnesInitMsg.format.blueShift);
		break;
	case e_cmd_connected:
		netlog.Print(LL_STATE, "\tRecord Count = %d\n", m_command.recordCount);
		netlog.Print(LL_STATE, "\tProjector Name = %s\n", m_command.command.connected.projName);
		netlog.Print(LL_STATE, "\tstatus = %d\n", m_command.command.connected.projState);
		break;
	case e_cmd_keepalive:
		netlog.Print(LL_STATE, "\tRecord Count = %d\n", m_command.recordCount);

		break;

	case e_cmd_resendfullscrid:
		netlog.Print(LL_STATE, "\tResendID = %d\n", m_command.command.resendid.resendID);
		break;

	case e_cmd_displaywait:
		netlog.Print(LL_STATE, "\tRecord Count = %d\n", m_command.recordCount);
		break;

	default:
		netlog.Print(LL_STATE, "\tnone....\n");
		break;
	};

	if( m_header.datasize > 0 ){
		for( int i=0; i<m_command.recordCount; i++){
			netlog.Print(LL_STATE, "[ ECONRECORD%d ].\n",i);
			ECONRECORD *erec = GetEconRecord(i);
			if( erec ){
				
				tmpstr[ECON_UNIQINFO_LENGTH] = '\0';
				netlog.Print(LL_STATE, "\tUniq Info = %02x-%02x-%02x-%02x-%02x-%02x\n", erec->projUniqInfo[0],
																						erec->projUniqInfo[1],
																						erec->projUniqInfo[2],
																						erec->projUniqInfo[3],
																						erec->projUniqInfo[4],
																						erec->projUniqInfo[5]
																						);
				netlog.Print(LL_STATE, "\tKeyword = %s\n",erec->projKeyword);
				netlog.Print(LL_STATE, "\tno-use IPAddress = %s\n", inet_ntoa(*((in_addr*)erec->IPaddress)));
			}
		}
	}
}







		

		

	





