/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/



#include "stdafx.h"
#include "IPAddress.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif




CIPAddress::CIPAddress()
{
	m_nEdtWidth = 0;
	m_nEdtHeight= 0;
}

CIPAddress::~CIPAddress()
{

}


BEGIN_MESSAGE_MAP(CIPAddress, CWnd)
	
	ON_WM_PAINT()
	ON_WM_SETFOCUS()
	ON_WM_DESTROY()
	
END_MESSAGE_MAP()



BOOL
CIPAddress::Create(LPCTSTR lpszClassName, LPCTSTR lpszWindowName, DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID, CCreateContext* pContext) 
{
	BOOL nRet = CWnd::Create(lpszClassName, lpszWindowName, dwStyle, rect, pParentWnd, nID, pContext);
	if( nRet ){
		m_recIPRect = rect;
		m_nEdtWidth = (rect.right-rect.left - 19)/4;
		m_nEdtHeight= rect.bottom- rect.top - 7;
	
		
		RECT iniRec;
		for(int i=0; i<EDIT_COUNT; i++){
			
			 
			iniRec.top = EDIT_TOP;
			iniRec.left = EDIT_LEFT+(m_nEdtWidth+4)*i;
			iniRec.right = iniRec.left + m_nEdtWidth;
			iniRec.bottom = iniRec.top + m_nEdtHeight;

			
			m_pEdit[i] = new CIPEdit();
			if( m_pEdit[i] == NULL )
				return FALSE;

			if( !m_pEdit[i]->Create(WS_VISIBLE|ES_CENTER, iniRec, this, IPADR_EDIT_CTRLID+i))
			return FALSE;

			
			m_pEdit[i]->SetLimitText(EDIT_MAXLEN);
			m_pEdit[i]->LineScroll(1);
			if( i > 0 ){
				m_pEdit[i-1]->m_pNextEdit = m_pEdit[i];
				m_pEdit[i]->m_pPrevEdit = m_pEdit[i-1];
			}
		}
		m_pEdit[0]->m_pPrevEdit = NULL;
		m_pEdit[EDIT_COUNT-1]->m_pNextEdit = NULL;
		return TRUE;
	}
	
	return FALSE;
}



void
CIPAddress::OnPaint() 
{
	CPaintDC dc(this); 
	
	
	RECT filRec;
	filRec.top = 0;
	filRec.left = 0;
	filRec.right = m_recIPRect.right - m_recIPRect.left;
	filRec.bottom = m_recIPRect.bottom - m_recIPRect.top;
	dc.FillSolidRect(&filRec, IPADR_COLOR_BACKGOURND);
	
	
	CPen pnDot;
	if( pnDot.CreatePen( PS_SOLID, 2, IPADR_COLOR_BLACK) ){
		CPen* pOldPen = dc.SelectObject( &pnDot );
		for(int i=1; i<EDIT_COUNT; i++){
			dc.MoveTo(EDIT_LEFT+(m_nEdtWidth+4)*i-3, m_nEdtHeight);
			dc.LineTo(EDIT_LEFT+(m_nEdtWidth+4)*i-3, m_nEdtHeight);
		}
		dc.SelectObject( pOldPen );
	}

	
	CPen pnliGray, pndrGray;
	if( pnliGray.CreatePen( PS_SOLID, 1, IPADR_COLOR_LIGHTPEN) &&
		pndrGray.CreatePen( PS_SOLID, 1, IPADR_COLOR_DARKPEN)){
		
		CPen* pOldPen = dc.SelectObject( &pndrGray );
		dc.MoveTo(filRec.left, filRec.bottom-1);
		dc.LineTo(filRec.left, filRec.top);
		dc.LineTo(filRec.right-1, filRec.top);
		dc.SelectObject( &pnliGray );
		dc.MoveTo(filRec.right-1, filRec.top);
		dc.LineTo(filRec.right-1, filRec.bottom-1);
		dc.LineTo(filRec.left, filRec.bottom-1);

		dc.SelectObject( pOldPen );
	}

	
	CPen pnBlack, pnmiGray;
	if( pnBlack.CreatePen( PS_SOLID, 1, IPADR_COLOR_BLACK) &&
		pnmiGray.CreatePen( PS_SOLID, 1, IPADR_COLOR_MIDGRAY)){
		
		CPen* pOldPen = dc.SelectObject( &pnBlack );
		dc.MoveTo(filRec.left+1, filRec.bottom-2);
		dc.LineTo(filRec.left+1, filRec.top+1);
		dc.LineTo(filRec.right-2, filRec.top+1);
		dc.SelectObject( &pnmiGray );
		dc.MoveTo(filRec.right-2, filRec.top+1);
		dc.LineTo(filRec.right-2, filRec.bottom-2);
		dc.LineTo(filRec.left+1, filRec.bottom-2);

		dc.SelectObject( pOldPen );
	}
}



void
CIPAddress::SetFont(CFont* pFont, BOOL bRedraw)
{
	for(int i=0; i<EDIT_COUNT; i++)
		if( m_pEdit[i] )
			m_pEdit[i]->SetFont(pFont, bRedraw);

	CWnd::SetFont(pFont, bRedraw);
}



void
CIPAddress::SetAddress(ULONG ulAddr)
{
	unsigned char *cAddr = (unsigned char*)&ulAddr;
	for(int k=EDIT_COUNT-1; k>=0; k--){
		if( m_pEdit[k] )
			m_pEdit[k]->SetValue(cAddr[k]);
	}
}



BOOL
CIPAddress::GetAddress(ULONG& ulAddr)
{
	CString sIPAddr;
	for(int k=0; k<EDIT_COUNT; k++){
		if( m_pEdit[k] ){
			CString str;
			m_pEdit[k]->GetWindowText(str);
			if( str.IsEmpty() )
				return FALSE;

			sIPAddr += str;
			if( k != EDIT_COUNT-1) 
				sIPAddr += ".";
		}
	}
	ulAddr = inet_addr((LPCTSTR)sIPAddr);
	if( ulAddr == 0xffffffff || ulAddr == 0)
		return FALSE;

	return TRUE;
}



void
CIPAddress::OnSetFocus(CWnd* pOldWnd) 
{
	CWnd::OnSetFocus(pOldWnd);
	
	
	if( m_pEdit[0] ){
		m_pEdit[0]->SetFocus();
		m_pEdit[0]->SetSel(0xff00);
	}
}



void CIPAddress::OnDestroy() 
{
	CWnd::OnDestroy();
	
	
	for(int i=0; i<EDIT_COUNT; i++){
		delete (CIPEdit*)m_pEdit[i];
	}		
}


void CIPAddress::Clear()
{
	for(int i=0; i<EDIT_COUNT; i++){
		m_pEdit[i]->SetWindowText("");
	}
}

