/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/




#include "stdafx.h"
#include "Log.h"
#include "IniFileUtil.h"
#include "EP_INIFILE.h"
#include "string.h"


#include "CreateFile.h"


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

extern Log inilog;




CIniFileUtil::CIniFileUtil(CPtrList *pList,CString sPath)
{
	m_sFilePath = sPath;
	m_pPList = pList;

	m_pbSectionNames = NULL;		
	m_dwSecNamesBuffSize = 0; 
	m_dwSecNamesReadSize = 0;

	
	
	

}

CIniFileUtil::~CIniFileUtil()
{
	if(m_pbSectionNames != NULL)
	{
		delete [] m_pbSectionNames;
		m_pbSectionNames = NULL;
		m_dwSecNamesBuffSize = 0; 
		m_dwSecNamesReadSize = 0;
	}

	
}


BOOL 
CIniFileUtil::SaveListToFile()
{
	inilog.Print(0,"INI FILE: Saving List to file...");

	if(m_pPList == NULL || m_sFilePath.IsEmpty())
		return false;

	int nVersion = GetOSVersion();
	if(nVersion == 1)
	{
		
		
		
		CCreateFile file;
		if(!file.Create(m_sFilePath))
		{
			inilog.Print(0,"Failed to create new file. SaveListToFile().");
			return false;
		}
		file.Close();
		
	}
	else 
	{
		
		try
		{
			
			
			CFile file(m_sFilePath,CFile::modeCreate);
			file.Close();
		}
		catch(CFileException *pFileExt)
		{
			inilog.Print(0,"ERROR: Failed to open file");
			return false;
			pFileExt->Delete();
		}
		
	}
	

	POSITION pos;

	for(pos = m_pPList->GetHeadPosition(); pos != NULL;)
	{
		EPNSD_IPGROUP *pGroup = (EPNSD_IPGROUP*)m_pPList->GetNext(pos);
		
		if(pGroup == NULL)
			continue;

		
		
		if(!WriteGroupToFile(pGroup))
			return false;
	}


	return true;
}


BOOL 
CIniFileUtil::WriteSectionName(CString sSecName)
{
	BOOL bRet = false;

	char temp[2];
	ZeroMemory(temp,sizeof(char)*2);

	bRet = WritePrivateProfileSection(sSecName,temp,m_sFilePath);

	return bRet;
}


BOOL 
CIniFileUtil::WriteSectionIPs(EPNSD_IPGROUP *pGroup)
{
	CString sIPAddr;
	CString sKeyName;

	for(int i = 1; i <= ECON_PROJLIST_IPLISTMAXCOUNT; i++)
	{
		sKeyName.Empty();
		sKeyName.Format("%s%d",ECON_INIFILE_KEYNAME,i);
		
		
		if(pGroup->dwIPAdd[i-1] == 0)
			continue;

		
		sIPAddr.Empty();
		in_addr address;
		address.S_un.S_addr = pGroup->dwIPAdd[i-1];
		sIPAddr = inet_ntoa(address);

		if(!WritePrivateProfileString(pGroup->sGroupName			
						,sKeyName									
						,sIPAddr									
						,m_sFilePath))								
		{
			inilog.Print(0,"ERROR: WritePrivateProfileString() failed index %d",i);
			return false;
		}
	}

	return true;
}


BOOL 
CIniFileUtil::ReadFromFile()
{
	inilog.Print(0,"INI FILE: Reading... (case-sensitive)");

	if(m_pPList == NULL || m_sFilePath.IsEmpty())
		return false;

	DWORD dwBuffSize = 0;

	
	dwBuffSize = ECON_GROUPNAME_MAXLEN * ECON_NORMAL_GROUPCOUNT;
	if(!CheckSectionNameBuffSize(dwBuffSize))
		return false;

	
	DWORD dwLoop = 2;
	while(!SetSectionNames())
	{
		
		
		dwBuffSize = ECON_GROUPNAME_MAXLEN * ECON_NORMAL_GROUPCOUNT * dwLoop;
		if(!CheckSectionNameBuffSize(dwBuffSize))
			return false;

		dwLoop++;
	}
	inilog.Print(0,"INI FILE: Section names");

	
	if(!SetGroups())
		return false;

	inilog.Print(0,"INI FILE: Groups");
	return true;
}


BOOL 
CIniFileUtil::ReadFromFileNoCase()
{
	
	
	inilog.Print(0,"INI FILE: Reading... (case-sensitive)");

	if(m_pPList == NULL || m_sFilePath.IsEmpty())
		return false;

	DWORD dwBuffSize = 0;

	
	dwBuffSize = ECON_GROUPNAME_MAXLEN * ECON_NORMAL_GROUPCOUNT;
	if(!CheckSectionNameBuffSize(dwBuffSize))
		return false;

	
	DWORD dwLoop = 2;
	while(!SetSectionNames())
	{
		
		
		dwBuffSize = ECON_GROUPNAME_MAXLEN * ECON_NORMAL_GROUPCOUNT * dwLoop;
		if(!CheckSectionNameBuffSize(dwBuffSize))
			return false;

		dwLoop++;
	}
	inilog.Print(0,"INI FILE: Section names");

	
	if(!SetGroupsNoCase())
		return false;

	inilog.Print(0,"INI FILE: Groups");
	return true;
}

BOOL 
CIniFileUtil::CheckSectionNameBuffSize(DWORD dwSize)
{
	if(m_dwSecNamesBuffSize >= dwSize)
		return true;

	if(m_pbSectionNames != NULL)
	{
		delete [] m_pbSectionNames;
		m_pbSectionNames = NULL;
		m_dwSecNamesBuffSize = 0;
		m_dwSecNamesReadSize = 0;
	}

	m_pbSectionNames = new BYTE[dwSize];
	
	if(m_pbSectionNames == NULL)
		return false;

	m_dwSecNamesBuffSize = dwSize;

	return true;
}





BOOL 
CIniFileUtil::SetSectionNames()
{
	
	DWORD dwRetSize = 0;

	dwRetSize = GetPrivateProfileSectionNames((char *)m_pbSectionNames
					,m_dwSecNamesBuffSize
					,m_sFilePath);

	if(dwRetSize == (m_dwSecNamesBuffSize - 2))
	{
		
		m_dwSecNamesReadSize = 0;
		return false;
	}

	
	
	m_dwSecNamesReadSize = dwRetSize;

	return true;
}


BOOL 
CIniFileUtil::SetGroups()
{
	DWORD dwBuffPtr = 0;
	CString sSectionName;

	
	if(m_dwSecNamesReadSize == 0)
		return true;

	for(dwBuffPtr = 0; dwBuffPtr < m_dwSecNamesReadSize;)
	{
		
		sSectionName.Empty();
		
		
		sSectionName = (char *)(m_pbSectionNames + dwBuffPtr);

		
		dwBuffPtr += sSectionName.GetLength() + 1;

		if(!InsertSectionToList(sSectionName))
			return false;

	}
	return true;
}


BOOL 
CIniFileUtil::SetGroupsNoCase()
{
		DWORD dwBuffPtr = 0;
	CString sSectionName;

	
	if(m_dwSecNamesReadSize == 0)
		return true;

	for(dwBuffPtr = 0; dwBuffPtr < m_dwSecNamesReadSize;)
	{
		
		sSectionName.Empty();
		
		
		sSectionName = (char *)(m_pbSectionNames + dwBuffPtr);

		
		dwBuffPtr += sSectionName.GetLength() + 1;

		if(!InsertSectionToListNoCase(sSectionName))
			return false;
	}
	return true;
}


BOOL
CIniFileUtil::InsertSectionToList(CString &sSecName)
{
	
	if(sSecName.GetLength() > ECON_GROUPNAME_MAXLEN ||
	   sSecName.GetLength() <= 0)
	{
		
		inilog.Print(0,"Invalid Group Name : Ignoring group: %s",sSecName);
		return true;
	}

	try
	{
		
		CFile file(m_sFilePath,CFile::modeRead
			| CFile::shareDenyWrite
			| CFile::modeNoTruncate);
		
		
		DWORD dwFileSize = file.GetLength();

		
		if(dwFileSize <= 0)
		{
			file.Close();
			return true;
		}
		
		
		CString sBuff;

		
		UINT nBytesRead = file.Read(sBuff.GetBuffer(dwFileSize),dwFileSize);

		
		sBuff.ReleaseBuffer();

		file.Close();

		
		if(nBytesRead == 0)
		{
			inilog.Print(0,"ERROR: file.Read() failed. %lu bytes read.",nBytesRead);
			return true;
		}
		
		
		int nSecStart = 0;
		int nNextSecStart = 0;
		DWORD dwBuffPtr = 0;

		
		CString sTempSecName;
		sTempSecName.Format("[%s]",sSecName);

		nSecStart = sBuff.Find(sTempSecName,dwBuffPtr);
		if(nSecStart == -1)
		{
			
			return true;
		}

		
		dwBuffPtr = nSecStart + sTempSecName.GetLength();

		
		nNextSecStart = sBuff.Find('[',dwBuffPtr);
		if(nNextSecStart == -1)
			nNextSecStart = sBuff.GetLength();
	
		
		EPNSD_IPGROUP *pGroup = new EPNSD_IPGROUP;

		if(pGroup == NULL)
			return false;

		
		ZeroMemory(pGroup->dwIPAdd,sizeof(ULONG) * ECON_PROJLIST_IPLISTMAXCOUNT);
		pGroup->sGroupName = sSecName;

	
		CString sSectionIPs;
		sSectionIPs = sBuff.Mid(dwBuffPtr+1,nNextSecStart-(dwBuffPtr + 1));
		
		if(!SetIPsFromString(pGroup,sSectionIPs))
		{
			delete pGroup;
			return false;
		}

		if(!IsIPRegistered(pGroup))
		{
			
			delete pGroup;
			
			return true;
		}
		
		m_pPList->AddTail(pGroup);

	}
	catch(CFileException *pFileExt)
	{
		inilog.Print(0,"ERROR: Failed to open file: InsertSectionToList");
		return false;
		pFileExt->Delete();
	}

	return true;
}


BOOL 
CIniFileUtil::InsertSectionToListNoCase(CString &sSecName)
{
	
	if(sSecName.GetLength() > ECON_GROUPNAME_MAXLEN ||
	   sSecName.GetLength() <= 0)
	{
		
		inilog.Print(0,"Invalid Group Name : Ignoring group: %s",sSecName);
		return true;
	}

	
	EPNSD_IPGROUP *pGroup = new EPNSD_IPGROUP;

	if(pGroup == NULL)
		return false;

	
	
	
	
	
	ZeroMemory(pGroup->dwIPAdd,sizeof(ULONG) * ECON_PROJLIST_IPLISTMAXCOUNT);
	pGroup->sGroupName = sSecName;
	

	CString sIPAddr;
	CString sKeyName;
	DWORD dwRet = 0;
	for(int i = 1; i <= ECON_PROJLIST_IPLISTMAXCOUNT; i++)
	{
		sKeyName.Empty();
		sKeyName.Format("%s%d",ECON_INIFILE_KEYNAME,i);
		
		dwRet = GetPrivateProfileString(sSecName					
						,sKeyName									
						,""											
						,sIPAddr.GetBuffer(ECON_IPADDRSTR_MAXLEN)	
						,ECON_IPADDRSTR_MAXLEN						
						,m_sFilePath);								
		
		
		
		sIPAddr.ReleaseBuffer();

		
		ULONG ulIPAddr = inet_addr(sIPAddr);

		
		if(ulIPAddr == INADDR_NONE)
			ulIPAddr = 0;

		
		pGroup->dwIPAdd[i-1] = ulIPAddr;
	}
	
	if(!IsIPRegistered(pGroup))
	{
		
		delete pGroup;
		
		return true;
	}

	
	m_pPList->AddTail(pGroup);
	
	inilog.Print(0,"INI FILE: Group: %s Added to Tail",sSecName);
	
	return true;
}


BOOL 
CIniFileUtil::CopyToNewFile(CString sNewFilePath)
{
	return CopyFile(m_sFilePath,sNewFilePath,false);
}


BOOL 
CIniFileUtil::WriteGroupToFileNoCase(EPNSD_IPGROUP *pGroup)
{
	
	if(!WriteSectionName(pGroup->sGroupName))
		return false;

	
	if(!WriteSectionIPs(pGroup))
		return false;

	return true;
}


BOOL 
CIniFileUtil::WriteGroupToFile(EPNSD_IPGROUP *pGroup)
{
	try{
		CFile file(m_sFilePath,CFile::modeWrite 
				| CFile::shareDenyWrite 
				| CFile::modeNoTruncate);

		CString sWrite;
		sWrite.Empty();

		CString sIPAddr;
		sIPAddr.Empty();

		
		file.SeekToEnd();

		
		sWrite.Format("[%s]\r\n",pGroup->sGroupName);
		
		file.Write(sWrite,sWrite.GetLength());

		
		for(int i = 1 ; i <= ECON_PROJLIST_IPLISTMAXCOUNT; i++)
		{
			
			if(pGroup->dwIPAdd[i-1] == 0)
				continue;

			
			sIPAddr.Empty();
			in_addr address;
			address.S_un.S_addr = pGroup->dwIPAdd[i-1];
			sIPAddr = inet_ntoa(address);

			sWrite.Empty();
			sWrite.Format("%s%d=%s\r\n",ECON_INIFILE_KEYNAME,i,sIPAddr);

			
			file.Write(sWrite,sWrite.GetLength());
		}

		file.Close();
	}
	catch(CFileException *pFileExt)
	{
		inilog.Print(0,"ERROR: Failed to open file: WriteGroupToFile");
		
		switch(pFileExt->m_cause)
		{
		case CFileException::fileNotFound:
			inilog.Print(0,"fileNotFound: Failed to open file: WriteGroupToFile");
			break;
		case CFileException::badPath:
			inilog.Print(0,"badPath: Failed to open file: WriteGroupToFile");
			break;
		case CFileException::accessDenied:
			inilog.Print(0,"accessDenied: Failed to open file: WriteGroupToFile");
			break;
		case CFileException::generic:
			inilog.Print(0,"generic: Failed to open file: WriteGroupToFile");
			break;
		case CFileException::sharingViolation:
			inilog.Print(0,"sharingViolation: Failed to open file: WriteGroupToFile");
			break;
		case CFileException::none:
			inilog.Print(0,"none: Failed to open file: WriteGroupToFile");
			break;
		}
	
		pFileExt->Delete();
		return false;
	}
	return true;
}


BOOL
CIniFileUtil::ReadFile()
{
	inilog.Print(0,"INI FILE: Reading...(Case-sensitive)");

	if(m_pPList == NULL || m_sFilePath.IsEmpty())
		return false;

	try
	{
		
		CFile file(m_sFilePath,CFile::modeRead
			| CFile::shareDenyWrite
			| CFile::modeNoTruncate);
		
		
		DWORD dwFileSize = file.GetLength();

		
		if(dwFileSize <= 0)
		{
			file.Close();
			return true;
		}
		
		
		CString sBuff;

		
		UINT nBytesRead = file.Read(sBuff.GetBuffer(dwFileSize),dwFileSize);

		
		sBuff.ReleaseBuffer();

		file.Close();

		
		if(nBytesRead == 0)
		{
			inilog.Print(0,"ERROR: file.Read() failed. %lu bytes read.",nBytesRead);
			return true;
		}
		
		
		if(!SetSectionsFromString(sBuff,nBytesRead))
			return false;
	}
	catch(CFileException *pFileExt)
	{
		inilog.Print(0,"ERROR: Failed to open file ReadFile");
		return false;
		pFileExt->Delete();
	}

	return true;
}


BOOL		
CIniFileUtil::SetSectionsFromString(CString &sBuff, DWORD dwBuffSize)
{
	
	int nSecStart = 0;
	int nSecEnd = 0;
	DWORD dwBuffPtr = 0;
		
	while(dwBuffPtr < dwBuffSize)
	{
		
		
		
		nSecStart = sBuff.Find('[',dwBuffPtr);
		if(nSecStart == -1)
		{
			
			return true;
		}

		
		nSecEnd = sBuff.Find(']',nSecStart);
		if(nSecEnd == -1)
		{
			
			return true;
		}

		
		CString sSectionName = sBuff.Mid(nSecStart+1,nSecEnd-nSecStart-1);
		
		DWORD dwSecNameLength = sSectionName.GetLength();
		if(dwSecNameLength <= 0 || dwSecNameLength > ECON_GROUPNAME_MAXLEN)
		{
			
			dwBuffPtr = nSecEnd;
			continue;
		}

		
		EPNSD_IPGROUP *pGroup = new EPNSD_IPGROUP;
		if(pGroup == NULL)
		{
			inilog.Print(0,"ERROR: (INI) failed to allocate mem for EPNSD_IPGROUP");
			return false;
		}

		
		ZeroMemory(pGroup->dwIPAdd,sizeof(ULONG) * ECON_PROJLIST_IPLISTMAXCOUNT);		
		
		pGroup->sGroupName = sSectionName;

		
		int nNextSection = sBuff.Find("[",nSecEnd);
		if(nNextSection == -1)
			nNextSection = sBuff.GetLength();
			
		
		dwBuffPtr = nNextSection - 1;

		CString sSectionIPs;
		sSectionIPs = sBuff.Mid(nSecEnd+1,nNextSection-(nSecEnd+1));

		if(!SetIPsFromString(pGroup,sSectionIPs))
		{
			delete pGroup;
			return false;
		}

		if(!IsIPRegistered(pGroup))
		{
			
			delete pGroup;
			
			return true;
		}

		
		m_pPList->AddTail(pGroup);
	}
	return true;
}


BOOL		
CIniFileUtil::SetIPsFromString(EPNSD_IPGROUP *pGroup, CString &sBuff)
{
	if(sBuff.GetLength() <= 0)
		return true;

	int nKeyStart = 0;
	int nKeyEnd = 0;

	for(int i = 1 ; i <= ECON_PROJLIST_IPLISTMAXCOUNT; i++)
	{
		
		CString sKeyName;
		sKeyName.Format("%s%d=",ECON_INIFILE_KEYNAME,i);

		int nKeyStart = sBuff.Find(sKeyName,0);
		if(nKeyStart == -1)
		{
			
			continue;
		}

		
		int nKeyEnd = sBuff.Find('=',nKeyStart);
		
		CString sIPAddr;
		sIPAddr = sBuff.Mid(nKeyEnd+1,ECON_IPADDRSTR_MAXLEN);
				
		
		ULONG ulIPAddr = inet_addr(sIPAddr);

		
		if(ulIPAddr == INADDR_NONE)
			ulIPAddr = 0;

		
		pGroup->dwIPAdd[i-1] = ulIPAddr;
	}

	return true;
}


BOOL		
CIniFileUtil::GetGroup(CString sGroupName,EPNSD_IPGROUP &nsdGroup)
{
	
	
	nsdGroup.sGroupName = sGroupName;
	ZeroMemory(nsdGroup.dwIPAdd,sizeof(ULONG) * ECON_PROJLIST_IPLISTMAXCOUNT);

	try
	{
		
		CFile file(m_sFilePath,CFile::modeRead
			| CFile::shareDenyWrite
			| CFile::modeNoTruncate);
		
		
		DWORD dwFileSize = file.GetLength();

		
		if(dwFileSize <= 0)
		{
			file.Close();
			return false;
		}
		
		
		CString sBuff;

		
		UINT nBytesRead = file.Read(sBuff.GetBuffer(dwFileSize),dwFileSize);

		
		sBuff.ReleaseBuffer();

		file.Close();

		
		if(nBytesRead == 0)
		{
			inilog.Print(0,"ERROR: file.Read() failed. %lu bytes read.",nBytesRead);
			return false;
		}
		
		
		int nSecStart = 0;
		int nNextSecStart = 0;
		DWORD dwBuffPtr = 0;

		
		CString sTempSecName;
		sTempSecName.Format("[%s]",sGroupName);

		nSecStart = sBuff.Find(sTempSecName,dwBuffPtr);
		if(nSecStart == -1)
		{
			
			return false;
		}

		
		dwBuffPtr = nSecStart + sTempSecName.GetLength();

		
		nNextSecStart = sBuff.Find('[',dwBuffPtr);
		if(nNextSecStart == -1)
			nNextSecStart = sBuff.GetLength();
	
		CString sSectionIPs;
		sSectionIPs = sBuff.Mid(dwBuffPtr+1,nNextSecStart-(dwBuffPtr + 1));
		
		if(!SetIPsFromString(&nsdGroup,sSectionIPs))
			return false;

		if(!IsIPRegistered(&nsdGroup))
		{
			nsdGroup.sGroupName.Empty();
			return false;
		}
	}
	catch(CFileException *pFileExt)
	{
		inilog.Print(0,"ERROR: Failed to open file");
		return false;
		pFileExt->Delete();
	}

	return true;
}


BOOL		
CIniFileUtil::IsIPRegistered(EPNSD_IPGROUP *pGroup)
{
	for(int i = 0; i < ECON_PROJLIST_IPLISTMAXCOUNT; i++)
	{
		if(pGroup->dwIPAdd[i] != 0x00000000 && pGroup->dwIPAdd[i] != 0xFFFFFFFF)
			return true;
	}
	return false;
}

int CIniFileUtil::GetOSVersion()
{
	OSVERSIONINFO info;
	memset(&info,0,sizeof(OSVERSIONINFO));
	info.dwOSVersionInfoSize=sizeof(OSVERSIONINFO);
	if (!GetVersionEx(&info)){
		return -1;
	}
	if(info.dwPlatformId==VER_PLATFORM_WIN32_NT){ 
		return 1;


	}else if(info.dwPlatformId==VER_PLATFORM_WIN32_WINDOWS){ 
		return 0;


	} else {
		return -1;
	}
}



