/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/



#include "stdafx.h"
#include "Log.h"





const int Log::ToDebug   =  1;
const int Log::ToFile    =  2;
const int Log::ToConsole =  4;

const static int LINE_BUFFER_SIZE = 1024;


#ifdef	_DEBUG
	Log inilog(Log::ToFile|Log::ToDebug,LL_ALL, "NSINILOG.txt",false);
	Log eclog(Log::ToFile|Log::ToDebug,LL_ALL, "ECLOG.txt", false);
	Log ecctlog(Log::ToFile|Log::ToDebug,LL_ALL, "ECTHREADLOG.txt", false);
	Log netlog(Log::ToFile|Log::ToDebug,LL_ALL, "ECNETLOG.txt", false);
	Log listenlog(Log::ToFile|Log::ToDebug,LL_ALL, "LISTENLOG.txt", false);
	Log searchlog(Log::ToFile|Log::ToDebug,LL_ALL, "SEARCHLOG.txt", false);
	Log escvplog(Log::ToFile|Log::ToDebug,LL_ALL, "ESCVPLOG.txt", false);
	Log pinglog(Log::ToFile|Log::ToDebug,LL_ALL, "PINGLOG.txt", false);
#else
	Log inilog(NULL,LL_ALL, "NSINILOG.txt", false);
	Log eclog(NULL,LL_ALL, "ECLOG.txt", false);
	Log ecctlog(NULL,LL_ALL, "ECTHREADLOG.txt", false);
	Log netlog(NULL,LL_ALL, "ECNETLOG.txt", false);
	Log listenlog(NULL,LL_ALL, "LISTENLOG.txt", false);
	Log searchlog(NULL,LL_ALL, "SEARCHLOG.txt", false);
	Log escvplog(NULL,LL_ALL, "ESCVPLOG.txt", false);
	Log pinglog(NULL,LL_ALL, "PINGLOG.txt", false);
#endif

Log::Log(int mode, int level, LPSTR filename, bool append)
{
	m_level = level;
	m_lastLogTime = 0;
	m_filename = NULL;
	m_append = false;
    hlogfile = NULL;
    m_todebug = false;
    m_toconsole = false;
    m_tofile = false;
    SetFile(filename, append);
    SetMode(mode);
}

void Log::SetMode(int mode) {
    
    if (mode & ToDebug)
        m_todebug = true;
    else
        m_todebug = false;

    if (mode & ToFile)  {
		if (!m_tofile)
			OpenFile();
	} else {
		CloseFile();
        m_tofile = false;
    }
    
    if (mode & ToConsole) {
        if (!m_toconsole)
            AllocConsole();
        m_toconsole = true;
    } else {
        m_toconsole = false;
    }
}


void Log::SetLevel(int level) {
    m_level = level;
}

void Log::SetFile(LPSTR filename, bool append) 
{
	CloseFile();
	if (m_filename != NULL)
		free(m_filename);
	m_filename = strdup(filename);
	m_append = append;
	if (m_tofile)
		OpenFile();
}

void Log::OpenFile()
{
	
	if (m_filename == NULL)
	{
        m_todebug = true;
        m_tofile = false;
        Print(0, "Error opening log file\n");
		return;
	}

    m_tofile  = true;
    
    
    hlogfile = CreateFile(
        m_filename,  GENERIC_WRITE, FILE_SHARE_READ, NULL,
        OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL  );
    
    if (hlogfile == INVALID_HANDLE_VALUE) {
        
        m_todebug = true;
        m_tofile = false;
        Print(0, "Error opening log file %s\n", m_filename);
    }
    if (m_append) {
        SetFilePointer( hlogfile, 0, NULL, FILE_END );
    } else {
        SetEndOfFile( hlogfile );
    }
}


void Log::CloseFile() {
    if (hlogfile != NULL) {
        CloseHandle(hlogfile);
        hlogfile = NULL;
    }
}

inline void Log::ReallyPrintLine(LPSTR line) 
{
    if (m_todebug) OutputDebugString(line);
    if (m_toconsole) {
        DWORD byteswritten;
        WriteConsole(GetStdHandle(STD_OUTPUT_HANDLE), line, strlen(line), &byteswritten, NULL); 
    };
    if (m_tofile && (hlogfile != NULL)) {
        DWORD byteswritten;
        WriteFile(hlogfile, line, strlen(line), &byteswritten, NULL); 
		WriteFile(hlogfile, "\r\n", 2, &byteswritten,NULL); 
    }
}

void Log::ReallyPrint(LPSTR format, va_list ap) 
{
	time_t current = time(0);
	if (current != m_lastLogTime) {
		
		
		ReallyPrintLine(ctime(&current));
		m_lastLogTime = current;
	}

	
	TCHAR line[LINE_BUFFER_SIZE];
    vsprintf(line, format, ap);
	ReallyPrintLine(line);
}

Log::~Log()
{
	if (m_filename != NULL)
		free(m_filename);
    CloseFile();
}