/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


#include "stdafx.h"
#include "ProjListBox.h"
#include "Log.h"
#include "AdaptersInfo.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern Log eclog;



CProjListBox::CProjListBox()
{	
	bkBrush.CreateSolidBrush(PLIST_COLOR_BACKGOURND);
	m_CheckedItemCnt = 0;
	m_bListEnable = TRUE;
	m_pFont = NULL;
	m_bFocus = FALSE;	

	m_maxProjectors=0;				

	m_nSrchTimer = 0;
	m_pSrchClient = NULL;
	m_bExistNowSrch = FALSE;
	m_nConIPNum = 0;

	memset(m_bMACAddr, 0 , ECON_MACADDRESS_MAXLEN);

	m_nAnimeFlag = 0;
	m_nAnimTimer = 0;
}

CProjListBox::~CProjListBox()
{
}


BEGIN_MESSAGE_MAP(CProjListBox, CListBox)
	
	ON_CONTROL_REFLECT(LBN_DBLCLK, OnDblclk)
	ON_WM_LBUTTONUP()
	ON_WM_CTLCOLOR_REFLECT()
	ON_WM_SETFOCUS()
	ON_WM_KILLFOCUS()
	ON_WM_TIMER()
	ON_WM_DESTROY()
	
END_MESSAGE_MAP()


void
CProjListBox::OnDestroy() 
{
	CWnd::OnDestroy();

	StopSearchClient();
	DeleteSearchClient();
}



void
CProjListBox::DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct) 
{
	CString		str;
	CBrush		*pbrBack;
	COLORREF	TextColor,BackColor,oldTextColor,oldBackColor;
	int			nCheckBoxWid=0;

	
	CDC*		pDC = CDC::FromHandle(lpDrawItemStruct->hDC);

	
	
	if( lpDrawItemStruct->CtlType != ODT_LISTBOX ||
		lpDrawItemStruct->itemID < 0 )
		return;


	
	
	if( lpDrawItemStruct->itemState & ODS_SELECTED && m_bFocus ){
	
		TextColor = PLIST_COLOR_SELTEXT;
 		BackColor = PLIST_COLOR_SELBACKGND;
	}else
	{
		TextColor = PLIST_COLOR_TEXT;
		BackColor = PLIST_COLOR_BACKGOURND;
	}















	oldTextColor = pDC->SetTextColor(TextColor);		
 	oldBackColor = pDC->SetBkColor(BackColor);			
	
	
	pbrBack = new CBrush(BackColor);		
	pDC->FillRect(&lpDrawItemStruct->rcItem, pbrBack);
	delete pbrBack;
	
	
	CPen pnliGray, pndrGray;
	if( pnliGray.CreatePen( PS_SOLID, 1, PLIST_COLOR_LIGHTPEN) &&
		pndrGray.CreatePen( PS_SOLID, 1, PLIST_COLOR_DARKPEN)){
		
		CPen* pOldPen = pDC->SelectObject( &pnliGray );
		pDC->MoveTo(lpDrawItemStruct->rcItem.left, lpDrawItemStruct->rcItem.top);
		pDC->LineTo(lpDrawItemStruct->rcItem.right, lpDrawItemStruct->rcItem.top);
		pDC->SelectObject( &pndrGray );
		pDC->MoveTo(lpDrawItemStruct->rcItem.left, lpDrawItemStruct->rcItem.bottom-1);
		pDC->LineTo(lpDrawItemStruct->rcItem.right, lpDrawItemStruct->rcItem.bottom-1);

		pDC->SelectObject( pOldPen );
	}

	
	ECITEM *ecItem = (ECITEM*)GetItemDataPtr(lpDrawItemStruct->itemID);
	if( (int)ecItem == -1 ) return;


	if( m_dwStyle & PLIST_STYLE_CHECKBOX )
	{

		


		int nCkHei = 0;
		if( ecItem->projState == PLIST_STATUS_SEARCHING || ecItem->projState == PLIST_STATUS_NOREPLY )
			nCkHei = 28;
		else if( ecItem->bSelected || IsCheckedItem(ecItem->projUniqInfo) )
			nCkHei = 14;

		m_pLocal->PaintDIBmp(	*pDC, 
							lpDrawItemStruct->rcItem.left + 8, 
							lpDrawItemStruct->rcItem.top + 4,
							BMP_PLIST_CHECKBOX,
							14, 14, 0, nCkHei );

		nCheckBoxWid = 20;
	}


		

	
	int nIconLeft=0;
	if( pDC->GetDeviceCaps(BITSPIXEL) <= 8 )
		nIconLeft = 18;

	int nIconState = (ecItem->projState)*PLIST_IMAGE_HEIGHT;
	if( ecItem->projState == PLIST_STATUS_SEARCHING )
		nIconState += m_nAnimeFlag*PLIST_IMAGE_HEIGHT*2;

	m_pLocal->PaintDIBmp(	*pDC, 
							lpDrawItemStruct->rcItem.left + 8 + nCheckBoxWid, 
							lpDrawItemStruct->rcItem.top + 3,
							BMP_PLIST_PROJICON,
							PLIST_IMAGE_WIDTH,
							PLIST_IMAGE_HEIGHT,
							nIconLeft, nIconState);

	
	CFont *pOldFont=NULL, *pNSFont;
	if( !m_pFont )
		pNSFont = this->GetParent()->GetFont();
	else
		pNSFont = m_pFont;

	

	
	LOGFONT logFont;
	CFont	fnProjName;

	memset(&logFont, 0, sizeof(logFont));
	::GetObject(GetStockObject(DEFAULT_GUI_FONT), sizeof(LOGFONT), &logFont);

	LCCONTROL *pCtl = m_pLocal->GetCtlInf(ECON_LBXITEM_PROJNAME);
	ASSERT(pCtl);
	logFont.lfHeight = -(pCtl->fontsize);
	logFont.lfWidth = 0;
	if( lstrlen(pCtl->font) != 0 ){
		lstrcpy(logFont.lfFaceName, pCtl->font);
	}
	fnProjName.CreateFontIndirect(&logFont);

	pOldFont = pDC->SelectObject(&fnProjName);


	RECT txtRec = { lpDrawItemStruct->rcItem.left + pCtl->rec.left + nCheckBoxWid,
					lpDrawItemStruct->rcItem.top + pCtl->rec.top,
					lpDrawItemStruct->rcItem.left+pCtl->rec.right + nCheckBoxWid,
					lpDrawItemStruct->rcItem.bottom -2};

	
	
	pDC->DrawText(ecItem->projName, -1, &txtRec, DT_LEFT | DT_VCENTER | DT_SINGLELINE | DT_NOPREFIX);
	
	pDC->SelectObject(pOldFont);

	
	if( m_dwStyle & PLIST_STYLE_IPADDRESS ){
		TCHAR sIPStr[16];
		LOGFONT logFont;
		CFont	fnIPAddr;

		memset(&logFont, 0, sizeof(logFont));
		::GetObject(GetStockObject(DEFAULT_GUI_FONT), sizeof(LOGFONT), &logFont);

		LCCONTROL *pCtl = m_pLocal->GetCtlInf(ECON_LBXITEM_IPADDRESS);
		ASSERT(pCtl);
		logFont.lfHeight = -(pCtl->fontsize);
		logFont.lfWidth = 0;
		if( lstrlen(pCtl->font) != 0 ){
			lstrcpy(logFont.lfFaceName, pCtl->font);
		}
		fnIPAddr.CreateFontIndirect(&logFont);

		pOldFont = pDC->SelectObject(&fnIPAddr);


		RECT txtRec = { lpDrawItemStruct->rcItem.left + pCtl->rec.left + nCheckBoxWid,
						lpDrawItemStruct->rcItem.top + pCtl->rec.top,
						lpDrawItemStruct->rcItem.left+pCtl->rec.right + nCheckBoxWid,
						lpDrawItemStruct->rcItem.bottom -2};

		
		in_addr addr;
		addr.S_un.S_addr = ecItem->ulIPAddr;
		sprintf(sIPStr, "%3d.%3d.%3d.%3d", addr.S_un.S_un_b.s_b1,
										   addr.S_un.S_un_b.s_b2,
										   addr.S_un.S_un_b.s_b3,
										   addr.S_un.S_un_b.s_b4);
		
		
		pDC->DrawText(sIPStr, -1, &txtRec, DT_LEFT | DT_VCENTER | DT_SINGLELINE | DT_NOPREFIX);
		pDC->SelectObject(pOldFont);
	}


	

	if( ecItem->projState == PLIST_STATUS_USING || 
		ecItem->projState == PLIST_STATUS_OTHERPC || 
		ecItem->projState == PLIST_STATUS_APPUSING)


	{
		CString usingTxt;







		usingTxt = m_pLocal->GetString(IDS_RES_STRING0005);

		CSize tsize = pDC->GetTextExtent(usingTxt);


		RECT useRec = { lpDrawItemStruct->rcItem.right - tsize.cx-15,

					lpDrawItemStruct->rcItem.top + 2,
					lpDrawItemStruct->rcItem.right,
					lpDrawItemStruct->rcItem.bottom -2
		};


		pOldFont = pDC->SelectObject(&fnProjName);
		
		
		pDC->DrawText(usingTxt, &useRec, DT_LEFT | DT_VCENTER | DT_SINGLELINE | DT_NOPREFIX);
		pDC->SelectObject(pOldFont);
	}

	pDC->SetTextColor(oldTextColor);		
 	pDC->SetBkColor(oldBackColor);			

}


BOOL


CProjListBox::AddECItem(ECITEM *pEC, int nIndexIns)

{
	if( !pEC )
		return FALSE;

	
	
	return AddECItem(pEC->projName, pEC->projUniqInfo, pEC->projState, pEC->ulIPAddr,nIndexIns);
	
}

BOOL
CProjListBox::UpdateECItem(int index, ECITEM* pEC)
{
	if( !pEC )
		return FALSE;

	return UpdateECItem(index, pEC->projName, pEC->projUniqInfo, pEC->projState, pEC->ulIPAddr);
}


BOOL




CProjListBox::AddECItem(char* projName, unsigned char* UniqInfo, unsigned char state, ULONG ulIPAddr, int nIndexIns )


{
	
	if( lstrlen(projName) >= PROJNAME_MAXLEN ||
		UniqInfo == NULL ||
		(state < PLIST_STATUS_NOUSE || state > PLIST_STATUS_NOREPLY) )
		return FALSE;

	if( state == PLIST_STATUS_SEARCHING)
		m_bExistNowSrch = TRUE;


	
	ECITEM *ecItem = new ECITEM;
	if( !ecItem ) return FALSE;

	
	lstrcpy(ecItem->projName, projName);
	memcpy(ecItem->projUniqInfo, UniqInfo, UNIQINFO_LENGTH);
	ecItem->projState = state;
	ecItem->bSelected = FALSE;
	
	ecItem->ulIPAddr = ulIPAddr;
	

	
	
	
	
	int index = 0;
	if(nIndexIns < 0)
		index = AddString((LPCSTR)ecItem->projName);
	else
		index = InsertString(nIndexIns,(LPCSTR)ecItem->projName);
	

	if( index == LB_ERR || index == LB_ERRSPACE )
	{
		delete ecItem;
		return FALSE;
	}

	
	
	if( SetItemDataPtr(index, ecItem)== LB_ERR ){
		delete ecItem;
		return FALSE;
	}

	if( m_pSrchClient )
		m_pSrchClient->SearchClient(ecItem->ulIPAddr, TRUE);

	return TRUE;
}


BOOL


CProjListBox::UpdateECItem(int index, char* projName, unsigned char* UniqInfo, unsigned char state, ULONG ulIPAddr)

{
	ECITEM *ecItem = (ECITEM*)GetItemDataPtr(index);
	if( (int)ecItem == -1 )
		return FALSE;

	lstrcpy(ecItem->projName, projName);
	memcpy(ecItem->projUniqInfo, UniqInfo, UNIQINFO_LENGTH);
	ecItem->projState = state;
	
	ecItem->ulIPAddr = ulIPAddr;
	

	return TRUE;
}


	
void
CProjListBox::DeleteItem(LPDELETEITEMSTRUCT lpDeleteItemStruct) 
{
	ECITEM *ecItem = (ECITEM*)GetItemDataPtr(lpDeleteItemStruct->itemID);
	if( ecItem )
		delete ecItem;
}


	
void
CProjListBox::OnDblclk() 
{


}

void
CProjListBox::OnLButtonUp(UINT nFlags, CPoint point) 
{
	if( m_bListEnable ){
		int	sel;
		RECT itemRec;

		sel = GetCurSel();
		GetItemRect(sel, &itemRec);

		if( point.x > itemRec.left && point.x < itemRec.right ){
			if( point.y > itemRec.top && point.y < itemRec.bottom){





				ChangeCheckBoxValue(sel);












			}
		}
	}

	CListBox::OnLButtonUp(nFlags, point);
}


	
int
CProjListBox::CompareItem(LPCOMPAREITEMSTRUCT lpCompareItemStruct) 
{
	ECITEM *ecItem1 = (ECITEM *)lpCompareItemStruct->itemData1;
	ECITEM *ecItem2 = (ECITEM *)lpCompareItemStruct->itemData2;
	return strcmp(ecItem1->projName, ecItem2->projName);
}


	
HBRUSH
CProjListBox::CtlColor(CDC* pDC, UINT nCtlColor) 
{
	pDC;
	nCtlColor;
	return bkBrush;
}


	
unsigned char* 
CProjListBox::GetUniqInfo(int index)
{
	ECITEM *pECItem = (ECITEM*)GetItemDataPtr(index);
	if( (int)pECItem == -1 ) return NULL;

	return pECItem->projUniqInfo;
}




void
CProjListBox::DeleteECItem(int index)
{
	DeleteString(index);
}


	
BOOL
CProjListBox::GetSelected(int index)
{
	ECITEM *pECItem = (ECITEM*)GetItemDataPtr(index);
	if( (int)pECItem == -1 ) return FALSE;
	
	return pECItem->bSelected;
}


			
BOOL
CProjListBox::CheckSelectedProjctors()
{
	int selCnt=0;
	for(int i=0; i<GetCount(); i++){
		if( GetSelected(i) ){
			selCnt++;
		}
	}


	if( selCnt >= (int)m_maxProjectors ){



		MessageBox(m_pLocal->GetString(IDS_WRN_STRING0009), ECON_PRODUCTNAME, MB_OK|MB_ICONWARNING);



		return FALSE;
	}
	return TRUE;
}


	
void
CProjListBox::SaveCheckedItem()
{
	m_CheckedItemCnt = 0;
	for(int i=0; i<GetCount(); i++){
		if( GetSelected(i) ){
			ECITEM *pECItem = (ECITEM*)GetItemDataPtr(i);
			ASSERT(pECItem);
			memcpy(m_CheckedItem+m_CheckedItemCnt*UNIQINFO_LENGTH, pECItem->projUniqInfo, UNIQINFO_LENGTH);
			m_CheckedItemCnt++;
		}
	}
}


	
void
CProjListBox::ClearCheckedItem()
{
	m_CheckedItemCnt = 0;
	for(int i=0; i<GetCount(); i++){
		ECITEM *pECItem = (ECITEM*)GetItemDataPtr(i);
		if(pECItem){
			pECItem->bSelected = FALSE;
		}
	}
}


	
BOOL
CProjListBox::IsCheckedItem(unsigned char *pUniqInfo)
{
	for(int i=0; i<m_CheckedItemCnt; i++){
		if( memcmp(m_CheckedItem+i*UNIQINFO_LENGTH, pUniqInfo, UNIQINFO_LENGTH)==0 )
			return TRUE;
	}
	return FALSE;
}







void
CProjListBox::SetListEnable(BOOL bEnable)
{
	m_bListEnable = bEnable;



}



void
CProjListBox::OnSetFocus(CWnd* pOldWnd) 
{
	CListBox::OnSetFocus(pOldWnd);

	
	if( GetCurSel() == LB_ERR )
		SetCurSel(0);

	
	m_bFocus = TRUE;
	RedrawWindow();
}


void
CProjListBox::OnKillFocus(CWnd* pNewWnd) 
{
	CListBox::OnKillFocus(pNewWnd);
	
	
	m_bFocus = FALSE;
	RedrawWindow();
}


int
CProjListBox::VKeyToItem(UINT nKey, UINT nIndex) 
{
	
	
	
	
	


	if( nKey == VK_SPACE && m_bListEnable )

	{


		int sel = GetCurSel();
	
		if( sel == LB_ERR )
			sel = 0;
		nIndex;
	
		
		
		if( SetCurSel(sel) != LB_ERR )
			ChangeCheckBoxValue(sel);
		

	

	}
	return -1;
}


void
CProjListBox::ChangeCheckBoxValue(int sel)
{
	if( sel == LB_ERR ) return;


	if( !(m_dwStyle & PLIST_STYLE_CHECKBOX) )
		return;

	
	ECITEM *evItem = (ECITEM*)GetItemDataPtr(sel);

	if( (int)evItem == -1 ) return;


	
	if( evItem->projState == PLIST_STATUS_SEARCHING ||
		evItem->projState == PLIST_STATUS_NOREPLY){
		return;
	}


	if( evItem->bSelected )
		evItem->bSelected = FALSE;
	else{
		if( CheckSelectedProjctors() )
			evItem->bSelected = TRUE;
		else
			evItem->bSelected = FALSE;
	}

	RedrawWindow();
}




BOOL

CProjListBox::StartSearchClient(ULONG ulSvrIPAddr, UINT nPort, BYTE *bMACAddr, int nMACSize)
{
	if( m_pSrchClient )
		delete m_pSrchClient;


	memcpy(m_bMACAddr, bMACAddr, nMACSize);

	
	m_pSrchClient = new CECSearchClient((HWND)GetParent(), ulSvrIPAddr, nPort);
	if( !m_pSrchClient )
		return FALSE;

	
	SendSearchPacket();


	
	for(int i=0; i<GetCount(); i++){
		ECITEM *pECItem = (ECITEM*)GetItemDataPtr(i);
		if( pECItem )
			m_bExistNowSrch = TRUE;
	}

	
	if( m_nSrchTimer == NULL )
		m_nSrchTimer = SetTimer(PLIST_SRCHTIMER_EVENT, PLIST_SRCHTIMER_TIME, NULL);

	
	if( m_nAnimTimer == NULL )
		m_nAnimTimer = SetTimer(PLIST_ANIMATION_EVENT, PLIST_ANIMATION_TIME, NULL);

	return TRUE;
}


void
CProjListBox::StopSearchClient()
{
	if( m_nSrchTimer ){
		KillTimer(m_nSrchTimer);
		m_nSrchTimer = 0;
	}
	if( m_nAnimTimer ){
		KillTimer(m_nAnimTimer);
		m_nAnimTimer = 0;
	}
}


void
CProjListBox::DeleteSearchClient()
{
	if( m_pSrchClient ){
		m_pSrchClient->EndReceiveUDP();
		delete m_pSrchClient;
		m_pSrchClient = NULL;
	}
}


BOOL
CProjListBox::IsSearching()
{
	return (m_nSrchTimer != 0);
}


BOOL
CProjListBox::GetClientInfo(int index, CLIENTINFO *pCltInf)
{
	if( !m_pSrchClient )
		return FALSE;

	ECITEM *pECItem = (ECITEM*)GetItemDataPtr(index);
	if( !pECItem ) return FALSE;
	
	return m_pSrchClient->GetClientInfoByIP(pECItem->ulIPAddr, pCltInf);
}


CECSearchClient *
CProjListBox::GetSrchClient()
{
	return m_pSrchClient;
}


void
CProjListBox::OnTimer(UINT nIDEvent) 
{
	if( !m_pSrchClient )
		return;

	if( nIDEvent == PLIST_SRCHTIMER_EVENT ){

		m_pSrchClient->EndReceiveUDP();
		
		if( m_pSrchClient->GetListChangeFlg() || m_bExistNowSrch){
			UpdateIPList();
			m_bExistNowSrch = FALSE;
		}
		SendSearchPacket();

	}else if( nIDEvent == PLIST_ANIMATION_EVENT){
		if( m_nAnimeFlag == 0 )
			m_nAnimeFlag = 1;
		else
			m_nAnimeFlag = 0;
		if( m_bExistNowSrch ){
			RedrawWindow(NULL);
		}
	}


	CListBox::OnTimer(nIDEvent);
}





void
CProjListBox::SendSearchPacket()
{
	int i;
	if( !m_pSrchClient )
		return;

	
	CAdaptersInfo adpinf;
	BOOL bFindIP = FALSE;
	if( adpinf.Init() != ADPINFO_NOERR ){


		::PostMessage(::GetParent(this->m_hWnd), PLIST_NETWORKERR, 0, 0);

		StopSearchClient();
		DeleteSearchClient();
		return;
	}

	for(i=0; i<(int)adpinf.GetCount(); i++){

		
		if( memcmp(m_bMACAddr, adpinf.GetMacAddr(i), MAC_ADDR_LENGTH )==0 ){
			
			
			for(int j=0; j<adpinf.GetIPAddress(NULL,i); j++){
				ADP_IPADDR addr;
				adpinf.GetIPAddress(&addr,i,j);
				if( addr.uIPAddr == m_pSrchClient->m_ulSvrAddr ){
					bFindIP = TRUE;
					break;
				}
			}
		}
	}
	if( !bFindIP ){
		::PostMessage(::GetParent(this->m_hWnd), PLIST_NETWORKERR, 0, 0);
		StopSearchClient();
		DeleteSearchClient();
		return;
	}

	m_pSrchClient->ResetListChangeFlg();
	m_pSrchClient->StartReceiveUDP();

	
	m_pSrchClient->CheckLostCount();

	
	for(i=0; i<GetCount(); i++){
		ECITEM *pECItem = (ECITEM*)GetItemDataPtr(i);
		if(pECItem){
			if( pECItem->ulIPAddr )
				if( !m_pSrchClient->SendSearchCommand(pECItem->ulIPAddr) ){
					eclog.Print(LL_ERROR, "!!! Failed to Send IP Search Command.\n");
				}
		}
		Sleep(1);
	}
}




void CProjListBox::UpdateIPList(void)
{
	int i,j;
	eclog.Print(LL_STATE, "Enter UpdateIPList Function.");

	for(i=0; i<GetCount(); i++){
		ECITEM *pECItem = (ECITEM*)GetItemDataPtr(i);
		if(pECItem){
			CLIENTINFO cinf;
			
			if( m_pSrchClient->GetClientInfoByIP(pECItem->ulIPAddr, &cinf) ){
				
				switch(cinf.m_projState){
				case e_pstat_nouse:
					pECItem->projState = PLIST_STATUS_NOUSE;
					break;
				case e_pstat_using:
					pECItem->projState = PLIST_STATUS_OTHERPC;
					for(j=0; j<m_nConIPNum; j++){
						if( m_ConIPList[j] == pECItem->ulIPAddr ){
							pECItem->projState = PLIST_STATUS_USING;
							break;
						}
					}
					break;
				case e_pstat_appusing:
					pECItem->projState = PLIST_STATUS_APPUSING;
					break;
				default:
					break;
				}
				
				lstrcpy(pECItem->projName, cinf.m_projName);
					
				memcpy(pECItem->projUniqInfo, cinf.m_projUniqInfo, UNIQINFO_LENGTH);
			}else
			{
				
				pECItem->projState = PLIST_STATUS_NOREPLY;
				lstrcpy(pECItem->projName, "");
				memset(pECItem->projUniqInfo, 0, UNIQINFO_LENGTH);
			}
		}
	}
		
	this->RedrawWindow();
}