/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


#if !defined(__ECONPROTOCOL__)
#define __ECONPROTOCOL__


#define ECON_PORTNUMBER				3620	
#define ECON_MAGICNUM_SIZE			4		
#define ECON_IPADDRESS_SIZE			4		
#define ECON_UNIQINFO_LENGTH		6		
#define	ECON_PROJNAME_MAXLEN		32		
#define ECON_KEYWORD_MAXLEN			16		
#define ECON_ENCRYPTION_MAXLEN		8		


#ifdef LITTLE_ENDIAN_HOST

#define Swap16IfLE(s) \
    ((CARD16) ((((s) & 0xff) << 8) | (((s) >> 8) & 0xff)))
#define Swap32IfLE(l) \
    ((CARD32) ((((l) & 0xff000000) >> 24) | \
     (((l) & 0x00ff0000) >> 8)  | \
	 (((l) & 0x0000ff00) << 8)  | \
	 (((l) & 0x000000ff) << 24)))

#else

#define Swap16IfLE(s) (s)
#define Swap32IfLE(l) (l)

#endif



#define ECON_MAGIC_NUMBER		"EEMP"		
#define ECON_PROTO_VERSION		"0100"		
#define ECON_PROTOVER_MAXLEN	4			


enum _e_ProjStat {
	e_pstat_nouse			=	1,			
	e_pstat_using			=	2,			
	e_pstat_appusing		=	3			




};



enum _e_EmpCommand {
	e_cmd_easysearch		=	1,			
	e_cmd_ipsearch			=   2,			
	e_cmd_clientinfo		=	3,			
	e_cmd_reqconnect		=	4,			
	e_cmd_connected			=	5,			
	e_cmd_reqrestart		=	6,			
	e_cmd_finishrestart		=	7,			
	e_cmd_disconclient		=	8,			
	e_cmd_interrupt			=	9,			
	e_cmd_keepalive			=	10,			


	e_cmd_sendrequests		=	12,			
	e_cmd_clienterror		=	13,			
	e_cmd_resendfullscrid   =   14,			
	e_cmd_displaywait		=	15,			
    e_cmd_sendkey           =   16          
};



enum _e_ProjType {
	e_ptype_IM_X			=	1,			
	e_ptype_IM_XP1			=	2,			
	e_pType_IM_XP4			=	3			
};

enum _e_SendReq {
	e_sendreq_OK			=	1,			
	e_sendreq_NG			=	2			
};


enum _e_Keyword {
	e_keyword_nouse			=	0,			
	e_keyword_use			=	1			
};



enum _e_Encryption {
	e_encrypt_nouse			=	0,			
	e_encrypt_use			=	1			
};



enum _e_Error {
	e_error_keyword			=	0			
};





typedef struct {

    unsigned char bitsPerPixel;		

    unsigned char depth;			

    unsigned char bigEndian;		

    unsigned char trueColour;		

    

    unsigned short redMax;			

    unsigned short greenMax;		

    unsigned short blueMax;			

    unsigned char redShift;			

    unsigned char greenShift;		

    unsigned char blueShift;		

    unsigned char pad1;
    unsigned short pad2;

} rfbPixelFormat;



typedef struct {

    unsigned short framebufferWidth;

    unsigned short framebufferHeight;
    
	rfbPixelFormat format;
    
	unsigned long nameLength;

} rfbServerInitMsg;



typedef struct {

	char			projName[ECON_PROJNAME_MAXLEN];		

	unsigned char	projState;							

	unsigned char	useKeyword;							
														

	unsigned char	displayType;						
														
														
														
} e_command_clientinfo;


typedef struct {

	unsigned char	useEncryption;						
														

	unsigned char	EncPassword[ECON_ENCRYPTION_MAXLEN];

	unsigned char	subnetMask[ECON_IPADDRESS_SIZE];	

	unsigned char	gateAddress[ECON_IPADDRESS_SIZE];	

	rfbServerInitMsg	vnesInitMsg;					

} e_command_reqconnect;


typedef struct {

	char			projName[ECON_PROJNAME_MAXLEN];		

	unsigned char	projState;							

} e_command_connected;



typedef struct {

	unsigned char	errorNo;								

} e_command_clienterror;



typedef struct {

	unsigned int	resendID;								

} e_command_resendfullscrid;


typedef struct {

    unsigned int    keyID;                              

} e_command_sendkey;



struct econ_header{

	char			magicnum[ECON_MAGICNUM_SIZE];		

	char			version[ECON_PROTOVER_MAXLEN];		

	unsigned char	IPaddress[ECON_IPADDRESS_SIZE];		

	unsigned long	commandID;							

	unsigned long	datasize;							

};

typedef econ_header ECONHEADER, *PECONHEADER;



struct econ_command{

	unsigned char recordCount;					

	union e_commands {
		
		e_command_clientinfo     clientinfo;	

		e_command_reqconnect     reqconnect;	
		
		e_command_connected      connected;		

		e_command_clienterror	 clienterror;	

		e_command_resendfullscrid	resendid;	

        e_command_sendkey       sendkey;        

	} command;
};

typedef econ_command ECONCOMMAND, *PECONCOMMAND;



struct econ_record {

	unsigned char	projUniqInfo[ECON_UNIQINFO_LENGTH];	

	unsigned char	projKeyword[ECON_KEYWORD_MAXLEN];	

	unsigned char	IPaddress[ECON_IPADDRESS_SIZE];		

};

typedef econ_record ECONRECORD, *PECONRECORD;


#endif
