/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/




#include "stdafx.h"
#include "EP_NSS.h"
#include "DialogsMng.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

extern Log eclog;




CDialogsMng::CDialogsMng()
{
	memset(&m_dlgdata, 0, sizeof(EPNSD_DLGDATA));
	m_hNSSOPLib = NULL;

	m_pfnEPNSD_Init = NULL;
	m_pfnEPNSD_GetDlgData = NULL;
	m_pfnEPNSD_DoOptionDlg = NULL;
	m_pfnEPNSD_DoLANSelDlg = NULL;
	m_pfnEPNSD_DoGroupDlg = NULL;
	m_pfnEPNSD_DoVersionDlg = NULL;
}

CDialogsMng::~CDialogsMng()
{
	if( m_hNSSOPLib ){
		::FreeLibrary(m_hNSSOPLib);
		m_hNSSOPLib = NULL;
	}
}


BOOL
CDialogsMng::Load(const char* sFileName)
{
	if( !sFileName){
		eclog.Print(LL_ERROR, "CDialogsMng::Load/Parameter Error.\n");
		return FALSE;
	}

	m_hNSSOPLib = (HMODULE)::LoadLibrary(sFileName);
	if ( ! m_hNSSOPLib ) {	
		eclog.Print(LL_ERROR, "CDialogsMng::Load/Failed to LoadLibrary.\n");
		return FALSE;		
	}

	
	m_pfnEPNSD_Init = (PROC_EPNSD_Init)::GetProcAddress(m_hNSSOPLib,"EPNSD_Init");
	m_pfnEPNSD_GetDlgData = (PROC_EPNSD_GetDlgData)::GetProcAddress(m_hNSSOPLib,"EPNSD_GetDlgData");
	m_pfnEPNSD_DoOptionDlg = (PROC_EPNSD_DoOptionDlg)::GetProcAddress(m_hNSSOPLib,"EPNSD_DoOptionDlg");
	m_pfnEPNSD_DoLANSelDlg = (PROC_EPNSD_DoLANSelDlg)::GetProcAddress(m_hNSSOPLib,"EPNSD_DoLANSelectDlg");
	m_pfnEPNSD_DoGroupDlg = (PROC_EPNSD_DoGroupDlg)::GetProcAddress(m_hNSSOPLib,"EPNSD_DoGroupDlg");
	m_pfnEPNSD_DoVersionDlg = (PROC_EPNSD_DoVersionDlg)::GetProcAddress(m_hNSSOPLib,"EPNSD_DoVersionDlg");
	if( !m_pfnEPNSD_Init || !m_pfnEPNSD_GetDlgData || !m_pfnEPNSD_DoOptionDlg 
		|| !m_pfnEPNSD_DoLANSelDlg || !m_pfnEPNSD_DoGroupDlg || !m_pfnEPNSD_DoVersionDlg){
		eclog.Print(LL_ERROR, "CDialogsMng::Load/Failed to GetProcAddress.\n");
		return FALSE;
	}
	return TRUE;
}



int
CDialogsMng::Show_OptionDlg(CWnd* pParent)
{
	if( !m_pfnEPNSD_Init(&m_dlgdata) )
		return -1;

	int rc = m_pfnEPNSD_DoOptionDlg(pParent);
	m_pfnEPNSD_GetDlgData(&m_dlgdata, sizeof(EPNSD_DLGDATA));
	return rc;
}


int
CDialogsMng::Show_LANSelDlg(CWnd* pParent)
{
	if( !m_pfnEPNSD_Init(&m_dlgdata) )
		return -1;

	int rc = m_pfnEPNSD_DoLANSelDlg(pParent);
	m_pfnEPNSD_GetDlgData(&m_dlgdata, sizeof(EPNSD_DLGDATA));
	return rc;
}



int
CDialogsMng::Show_GroupDlg(CWnd* pParent)
{
	if( !m_pfnEPNSD_Init(&m_dlgdata) )
		return -1;

	int rc = m_pfnEPNSD_DoGroupDlg(pParent);
	m_pfnEPNSD_GetDlgData(&m_dlgdata, sizeof(EPNSD_DLGDATA));
	return rc;
}


int
CDialogsMng::Show_VersionDlg(CWnd* pParent)
{
	if( !m_pfnEPNSD_Init(&m_dlgdata) )
		return -1;

	return m_pfnEPNSD_DoVersionDlg(pParent);
}