/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "stdafx.h"
#include "ECClientThread.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_DYNAMIC(CECClientThread, CWinThread)


CECClientThread::CECClientThread(HWND hMainWnd, ULONG ulSvrAddr, int nPort)
{
	m_hMainWnd		= hMainWnd;
	m_ulSvrAddr		= ulSvrAddr;
	m_nPort			= nPort;
	m_hSocket		= NULL;
	m_nState		= 0;
	m_bsent_restart = FALSE;
	m_ulIPAddr		= 0;
	m_dwVNeSLostTime= 0;
	memset(m_cUniqInfo, 0, ECON_UNIQINFO_LENGTH);
	memset(m_projName, 0, ECON_PROJNAME_MAXLEN);



#ifdef __USE_KEEPALIVE__ 
	m_dwKeepAliveSentTime = 0;


#endif 


	
	m_hEventKill = CreateEvent(NULL, TRUE, FALSE, NULL);
	m_hEventDead = CreateEvent(NULL, TRUE, FALSE, NULL);

	InitializeCriticalSection(&m_csLock);
}

CECClientThread::~CECClientThread()
{
	DeleteCriticalSection(&m_csLock);

	CloseHandle(m_hEventKill);
	CloseHandle(m_hEventDead);
	ecctlog.Print(LL_STATE, "Exit destructor of CECClientThread class.\n");
}

BOOL
CECClientThread::InitInstance()
{
	
	
	
#ifndef _AFXDLL
#define _AFX_SOCK_THREAD_STATE AFX_MODULE_THREAD_STATE
#define _afxSockThreadState AfxGetModuleThreadState()
	_AFX_SOCK_THREAD_STATE* pState = _afxSockThreadState;
	if (pState->m_pmapSocketHandle == NULL)
		pState->m_pmapSocketHandle = new CMapPtrToPtr;
	if (pState->m_pmapDeadSockets == NULL)
		pState->m_pmapDeadSockets = new CMapPtrToPtr;
	if (pState->m_plistSocketNotifications == NULL)
		pState->m_plistSocketNotifications = new CPtrList;
#endif

	
	m_pSock = new CClientSock();
	if( !m_pSock ){
		SetStatus(GetStatus() | e_disablesock);
		ecctlog.Print(LL_ERROR, "failed to alloc memory for create new socket.\n");
	}else
	{
		
		m_pSock->m_pClientThread = this;
		m_pSock->Attach(m_hSocket);
		ecctlog.Print(LL_STATE, "complete to attach the new thread socket.\n");
	}
	return TRUE;
}

void
CECClientThread::Delete()
{
	if( m_pSock ){
		delete m_pSock;
		m_pSock = NULL;
	}
	VERIFY(SetEvent(m_hEventDead));
}

int
CECClientThread::ExitInstance()
{
	CWinThread::ExitInstance();
	return 0;
}

BEGIN_MESSAGE_MAP(CECClientThread, CWinThread)
	
	
	ON_THREAD_MESSAGE(WM_TERM_CTTHREAD, OnTermThread)
END_MESSAGE_MAP()


	
void
CECClientThread::OnReceive(CClientSock *sock, int nErrorCode)
{
	int cmdID = 0;
	CEconCommand econ;
	ECONHEADER eh;
	ECONCOMMAND *pEcon = NULL;
	ECONRECORD *pRec = NULL;
	CString peerIP;
	UINT peerPort;
	
	if(GetStatus() & e_disablesock ) return;

	
	if( !sock || nErrorCode == WSAENETDOWN ){
		SetStatus(GetStatus() | e_disablesock);
		if( GetStatus() & e_connected ){
			ecctlog.Print(LL_STATE, "Notice to mail window 'e_networkerr'.\n");
			NoticeToMain(e_networkerr);
		}
		ecctlog.Print(LL_ERROR, "!!!OnReceive Parameter 'nErrorCode' is WSAENETDOWN or sock is NULL.\n");
		return;
	}

	
	

	
	int nRcv = sock->Receive(&eh, sizeof(ECONHEADER));
	if( nRcv == SOCKET_ERROR ){
		if( GetLastError() != WSAEWOULDBLOCK ){
			SetStatus(GetStatus() | e_disablesock);
			if( GetStatus() & e_connected ){
				ecctlog.Print(LL_STATE, "Notice to mail window 'e_networkerr'.\n");
				NoticeToMain(e_networkerr);
			}
			ecctlog.Print(LL_ERROR, "!!!TCP Socket Error on Receive header.\n");
			return;
		}
	}else if( nRcv > 0){

		BYTE pDataBuf[ECM_RECVBUF_MAXLEN];
		
		
		cmdID = econ.CheckHeader((BYTE*)&eh);
		if( cmdID == ECMD_ERR_MAGICCODE || cmdID == ECMD_ERR_PROTOVERSION){
			
			
			SetStatus(GetStatus() | e_disablesock);
			if( GetStatus() & e_connected ){
				ecctlog.Print(LL_STATE, "Notice to mail window 'e_networkerr'.\n");
				NoticeToMain(e_networkerr);
			}
			ecctlog.Print(LL_ERROR, "!!!Magic code missmatch.\n");
			
			return;
		}

		
		if( eh.datasize > 0 ){
			int nDatasize = eh.datasize;
			if( nDatasize > ECM_RECVBUF_MAXLEN )
				nDatasize = ECM_RECVBUF_MAXLEN;

			int nRcv = sock->Receive(pDataBuf, nDatasize);
			if( nRcv == SOCKET_ERROR ){
				if( GetLastError() != WSAEWOULDBLOCK ){
					SetStatus(GetStatus() | e_disablesock);
					if( GetStatus() & e_connected ){
						ecctlog.Print(LL_STATE, "Notice to mail window 'e_networkerr'.\n");
						NoticeToMain(e_networkerr);
					}
					ecctlog.Print(LL_ERROR, "!!!TCP Socket Error on Receive data.\n");
					return;
				}		
			}
		}

		econ.Recv((BYTE*)&eh, pDataBuf);
		
		pEcon = econ.GetEconCommand();

	}else
	{
		
		SetStatus(GetStatus() | e_disablesock);
		if( GetStatus() & e_connected ){
			ecctlog.Print(LL_STATE, "Notice to mail window 'e_networkerr'.\n");
			NoticeToMain(e_networkerr);
		}
		ecctlog.Print(LL_ERROR, "!!!Unknown Error on Receive data.\n");
		return;
	}

	
	
	switch(cmdID){

	
	case e_cmd_connected:
		ecctlog.Print(LL_STATE, "Receive 'e_cmd_connected' command.\n");

		
		if( econ.GetRecordNum() <= 0 || !pEcon) break;
		pRec = econ.GetEconRecord(0);
		if( !pRec ) break;

		sock->GetPeerName(peerIP, peerPort);
		m_ulIPAddr = inet_addr((LPCTSTR)peerIP);
		memcpy(m_cUniqInfo, pRec->projUniqInfo, ECON_UNIQINFO_LENGTH);
		memcpy(m_projName, pEcon->command.connected.projName, ECON_PROJNAME_MAXLEN);

		SetStatus(GetStatus() | e_recvconnect);
		break;

	
	case e_cmd_finishrestart:
		ecctlog.Print(LL_STATE, "Receive 'e_cmd_finishrestart' command.\n");
		SetStatus(GetStatus() | e_restarted | e_disablesock);
		break;

	
	case e_cmd_disconclient:
		ecctlog.Print(LL_STATE, "Receive 'e_cmd_disconclient' command.\n");
		SetStatus(GetStatus() | e_disablesock);
		if( GetStatus() & e_connected ){
			ecctlog.Print(LL_STATE, "Notice to mail window 'e_disconclient'.\n");
			NoticeToMain(e_disconclient);
		}
		break;

	
	case e_cmd_interrupt:
		ecctlog.Print(LL_STATE, "Receive 'e_cmd_interrupt' command.\n");
		SetStatus(GetStatus() | e_disablesock);
		if( GetStatus() & e_connected ){
			ecctlog.Print(LL_STATE, "Notice to mail window 'e_interrupt'.\n");
			NoticeToMain(e_interrupt);
		}	
		break;




















	
BOOL 
CECClientThread::OnIdle( LONG lCount )
{
	lCount;
	
	if( GetStatus() & e_restart && !m_bsent_restart)
	{
		CEconCommand econ(m_ulSvrAddr, m_nPort);
		econ.MakeHeader(e_cmd_reqrestart);

		if( !econ.Send(m_pSock, FALSE) ){
			ecctlog.Print(LL_ERROR, "!! Failed to send e_cmd_reqrestart command.\n");
			SetStatus(GetStatus() | e_disablesock);
			if( GetStatus() & e_connected ){
				ecctlog.Print(LL_STATE, "Notice to mail window 'e_networkerr'.\n");
				NoticeToMain(e_networkerr);
			}
		}
		m_bsent_restart = TRUE;
	}

	
	
	if( !(GetStatus() & e_disablesock) && (GetStatus() & e_vneslost) )
	{
		ecctlog.Print(LL_STATE, "Check the time after VNeS Lost.\n");

		
		if( m_dwVNeSLostTime == 0 ){
			m_dwVNeSLostTime = ::GetTickCount();
			ecctlog.Print(LL_STATE, "Set m_dwVNeSLostTime = %d.\n", m_dwVNeSLostTime);
		}

		
		if( (::GetTickCount() - m_dwVNeSLostTime) > VNESLOST_TIMEOUT){
			ecctlog.Print(LL_ERROR, "!! VNeS Lost Time is Over.\n");
			SetStatus(GetStatus() | e_disablesock);
			if( GetStatus() & e_connected ){
				ecctlog.Print(LL_STATE, "Notice to mail window 'e_networkerr'.\n");
				NoticeToMain(e_networkerr);
			}
		}
	}

#ifdef __USE_KEEPALIVE__ 
	else if( !(GetStatus() & e_restart) && (GetStatus() & e_recvconnect) && 
			(::GetTickCount() - m_dwKeepAliveSentTime) > KEEPALIVE_TIME && !(GetStatus() & e_disablesock)){
		CEconCommand econ(m_ulSvrAddr, m_nPort);
		econ.MakeHeader(e_cmd_keepalive);

		if( !econ.Send(m_pSock, FALSE) ){
			ecctlog.Print(LL_ERROR, "!! Failed to send keepalive command.\n");
			SetStatus(GetStatus() | e_disablesock);
			if( GetStatus() & e_connected ){
				ecctlog.Print(LL_STATE, "Notice to mail window 'e_networkerr'.\n");
				NoticeToMain(e_networkerr);
			}
		}else
		{
			ecctlog.Print(LL_STATE, "Send keepalive command ot client.\n");
			
			m_dwKeepAliveSentTime = ::GetTickCount();
		

			






		}
	}
#endif 

	Sleep(THREAD_SLEEP_TIME);
	return TRUE;
}

	
void
CECClientThread::KillThread()
{
	::PostThreadMessage(m_nThreadID, WM_TERM_CTTHREAD, 0, 0);
	SetThreadPriority(THREAD_PRIORITY_ABOVE_NORMAL);
	WaitForSingleObject(m_hEventDead, INFINITE);
	WaitForSingleObject(m_hThread, INFINITE);
	delete this;
}


	
void CECClientThread::OnTermThread(UINT, LONG)
{
	ecctlog.Print(LL_STATE, "exit thread loop.\n");
	
	::PostQuitMessage(0);
}


	
unsigned char* 
CECClientThread::GetUniqInfo()
{
	return m_cUniqInfo;
}

	
char*
CECClientThread::GetProjName()
{
	return m_projName;
}


	


	
ULONG CECClientThread::GetIPAddress()
{
	return m_ulIPAddr;
}


	




DWORD
CECClientThread::GetStatus()
{
	EnterCriticalSection(&m_csLock);
	DWORD ret = m_nState;
	LeaveCriticalSection(&m_csLock);
	return ret;
}


void
CECClientThread::SetStatus(DWORD state)
{
	EnterCriticalSection(&m_csLock);
	m_nState = state;
	ecctlog.Print(LL_STATE, "changed client status. status=%x\n",m_nState);
	LeaveCriticalSection(&m_csLock);
}



void
CECClientThread::NoticeToMain(int cmd)
{
	
	::PostMessage(m_hMainWnd, ECM_CLIENTCOMMAND, cmd, (LPARAM)this);
}



void
CECClientThread::OnSockClose(void)
{
	ecctlog.Print(LL_STATE, "Enter OnSockClose Function.\n");

	if( (GetStatus() & e_connected) && !(GetStatus() & e_disablesock) ){
		ecctlog.Print(LL_STATE, "Notice to mail window 'e_networkerr'.\n");
		NoticeToMain(e_networkerr);
	}
	SetStatus(GetStatus() | e_disablesock);
}
