/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/




#include "stdafx.h"
#include "EP_NSS.h"
#include "ECPing.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif



UINT ECPingProc( LPVOID pParam );

UINT ECPingProc( LPVOID pParam )
{
	pinglog.Print(LL_STATE, "Enter ECPingProc.\n");

	CECPing *ping = (CECPing*)pParam;
	if( !ping ) return 0;

	for(int i=0; i<ECPING_IPCOUNTMAX; i++){
		Sleep(ECP_SLEEP_TIME);
		if( !ping->SearchIPAddr() || ping->GetKill())
			break;
	}
	ping->SetRunThread(FALSE);
	pinglog.Print(LL_STATE, "Exit ECPingProc.\n");
	return 0;
}





CECPing::CECPing()
{
	pinglog.Print(LL_STATE, "Enter constructor of CECPing class.\n");

	m_hICMPMofule			= NULL;
	m_pfnICMPCreateFile		= NULL;
	m_pfnICMPSendEcho		= NULL;
	m_pfnICMPCloseHandle	= NULL;
	m_nCurPos				= 1;
	m_bCompleteSearchIP		= FALSE;
	m_bRunThread			= FALSE;
	m_bKill					= FALSE;

	for(int i=0; i<ECPING_IPCOUNTMAX; i++)
		m_ulIPList[i] = 0;

	InitializeCriticalSection(&m_csLock);
	pinglog.Print(LL_STATE, "Exit constructor of CECPing class.\n");
}

CECPing::~CECPing()
{
	pinglog.Print(LL_STATE, "Enter destructor of CECPing class.\n");

	
	EnterCriticalSection(&m_csLock);
	m_bKill = TRUE;
	LeaveCriticalSection(&m_csLock);

	
	int timeout = ECP_THREADWAIT_TIME;
	while(GetRunThread() && --timeout > 0){
		Sleep(1);
	}

	m_pfnICMPCreateFile		= NULL;
	m_pfnICMPSendEcho		= NULL;
	m_pfnICMPCloseHandle	= NULL;

	if(m_hICMPMofule){
		FreeLibrary(m_hICMPMofule);
		m_hICMPMofule =NULL;
	}

	DeleteCriticalSection(&m_csLock);

	pinglog.Print(LL_STATE, "Exit destructor of CECPing class.\n");
}


BOOL
CECPing::Init(ULONG ulAddr, ULONG ulMask, ULONG ulGateway)
{
	pinglog.Print(LL_STATE, "Enter Init function of CECPing class.\n");

	m_ulPCAddr = ntohl(ulAddr);
	m_ulPCMask = ntohl(ulMask);
	m_uIPCGateway = ntohl(ulGateway);
	m_nPingCount = 0;

	m_hICMPMofule = LoadLibrary(ECPING_DLLNAME);
	if( !m_hICMPMofule ){
		pinglog.Print(LL_ERROR, "!!! Failed to load DLL.\n");
		return FALSE;
	}

	m_pfnICMPCreateFile  = (PROC_IcmpCreateFile) GetProcAddress(m_hICMPMofule,_T("IcmpCreateFile"));
	m_pfnICMPSendEcho  = (PROC_IcmpSendEcho) GetProcAddress(m_hICMPMofule,_T("IcmpSendEcho"));
	m_pfnICMPCloseHandle  = (PROC_IcmpCloseHandle) GetProcAddress(m_hICMPMofule,_T("IcmpCloseHandle"));

	if( m_pfnICMPCreateFile == NULL || 
		m_pfnICMPSendEcho == NULL ||
		m_pfnICMPCloseHandle == NULL ){
		pinglog.Print(LL_ERROR, "!!! Failed to get function pointer of DLL.\n");
		return FALSE;
	}

	SetRunThread(TRUE);
	AfxBeginThread(ECPingProc, this);

	pinglog.Print(LL_STATE, "Exit Init function of CECPing class.\n");
	return TRUE;
}



BOOL
CECPing::SearchIPAddr()
{
	pinglog.Print(LL_STATE, "Enter SearchIPAddr function of CECPing class.\n");

	ULONG findaddr;

	if( m_nCurPos >= (int)~m_ulPCMask ){
		pinglog.Print(LL_STATE, "Complete all ip seatch.\n");
		return FALSE;
	}
	
	findaddr = (m_ulPCAddr & m_ulPCMask ) | (~m_ulPCMask & m_nCurPos);
	if( findaddr == m_ulPCAddr || findaddr == m_uIPCGateway){
		m_nCurPos++;
		pinglog.Print(LL_STATE, "skip address server IP or server gateway IP.\n");
		return TRUE;
	}

	EnterCriticalSection(&m_csLock);
	int nRet = ICMPEcho(findaddr, ECPING_TIMEOUT);
	if( nRet == ECPING_NOREPLY )
	{
		m_ulIPList[m_nPingCount] = findaddr;
		m_nPingCount++;
		if( m_nPingCount >= ECPING_IPCOUNTMAX ){
			LeaveCriticalSection(&m_csLock);
			pinglog.Print(LL_STATE, "Complete to find IPAddress.\n");
			return FALSE;
		}
	}
	LeaveCriticalSection(&m_csLock);

	m_nCurPos++;
	pinglog.Print(LL_STATE, "Exit SearchIPAddr function of CECPing class. m_nCurPos = %d\n",m_nCurPos);
	return TRUE;
}



int
CECPing::EchoCheck(ULONG ulAddr, DWORD timeout)
{
	pinglog.Print(LL_STATE, "Enter EchoCheck function of CECPing class.\n");

	int ret;
	EnterCriticalSection(&m_csLock);

	ret = ICMPEcho(ulAddr,timeout);

	LeaveCriticalSection(&m_csLock);

	pinglog.Print(LL_STATE, "Exit EchoCheck function of CECPing class.\n");
	return ret;
}




int
CECPing::ICMPEcho(ULONG ulAddr, DWORD timeout)
{
	DWORD	cnt;
	unsigned char request_data[32];
	unsigned char reBuff[sizeof(ICMP_ECHO_REPLY)+200];
	ICMP_ECHO_REPLY *p;
	IP_OPTION_INFORMATION ipopt,*pipopt;
	HANDLE hIcmpFH;
	
	hIcmpFH = m_pfnICMPCreateFile();
	if( hIcmpFH < 0 )
		return ECPING_ERROR;

	pipopt=NULL;
	memset(&ipopt,0,sizeof(ipopt));
	ipopt.Ttl = 0xff;
	ipopt.OptionsData = request_data;
	pipopt = &ipopt;
	memset(reBuff,0,sizeof(ICMP_ECHO_REPLY)+200);
	memset(request_data,0,sizeof(request_data));

	
	cnt = m_pfnICMPSendEcho((void*)hIcmpFH, (ULONG)htonl(ulAddr), (void*)request_data,
		(unsigned short)sizeof(request_data), pipopt, (void*)reBuff, (ULONG)sizeof(reBuff), (ULONG)timeout);

	m_pfnICMPCloseHandle(hIcmpFH);

	p=(ICMP_ECHO_REPLY *)reBuff;
	if( cnt <= 0 ){
		if( p->Status == IP_REQ_TIMED_OUT )
			return ECPING_NOREPLY;
	}else
	{
		if( p->Status == IP_SUCCESS )
			return ECPING_SUCCESS;
	}
	return ECPING_ERROR;
}



ULONG
CECPing::GetIPAddr(int index)
{
	ULONG ulIPAddr;

	if( index >= 0 && index < m_nPingCount )
		ulIPAddr =  ntohl(m_ulIPList[index]);
	else
		ulIPAddr = 0;

	return ulIPAddr;
}



BOOL
CECPing::GetRunThread()
{
	return m_bRunThread;
}


void
CECPing::SetRunThread(BOOL run)
{
	EnterCriticalSection(&m_csLock);
	m_bRunThread = run;
	LeaveCriticalSection(&m_csLock);
}



BOOL
CECPing::GetKill()
{
	return m_bKill;
}