/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


#include "stdafx.h"
#include "EP_NSS.h"
#include "ECServer.h"


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


CECServer::CECServer(HWND hPntWnd, ULONG ulSvrIP, UINT nPort)
{
	m_hParentWnd = hPntWnd;
	m_ilSvrAddr = ulSvrIP;
	m_nPort = nPort;


	m_TCPThreadList.RemoveAll();
	m_EconCmd.SetParam(ulSvrIP, nPort); 
	m_nKwdErrCount = 0;
	m_pListener = NULL;

	InitializeCriticalSection(&m_csLock);
}

CECServer::~CECServer()
{
	eclog.Print(LL_STATE, "Enter CECServer's Destructor.\n");
	CloseListen();
	eclog.Print(LL_STATE, "Closed Listener Thread.\n");

	
	ClearThreadList();
	eclog.Print(LL_STATE, "Clear All Client Threads.\n");

	DeleteCriticalSection(&m_csLock);
}



int





CECServer::StartEasyConnect(CECSearchClient *pECsc, CLIENTINFO* pConCltInf,
							UINT nConCltNum, BOOL bEasyMode, int *keyError)


{
	

	eclog.Print(LL_STATE, "StartEasyConnect 1.\n");


	long nTimeCounter = 0;
	int i = 0, nThdCnt = 0,nRecordNum=0;
	
	nRecordNum = (UINT)m_EconCmd.GetRecordNum();
	ASSERT( nRecordNum > 0 );

	if( !pECsc )
		return ECS_ERR_PARAM;

	
	m_pListener->ClearClient();
	eclog.Print(LL_STATE, "Clear Listener List. Listener Thread Count=%d\n",
				m_pListener->GetClientCount());
	
	




	if( bEasyMode )
	{
		if( !pECsc->SendUDP(&m_EconCmd, 0))
			return ECS_ERR_SOCKETERROR;
	}else{	
		for(int i=0; i<(int)nConCltNum; i++)
		{
			if( !pECsc->SendUDP(&m_EconCmd, pConCltInf[i].m_projIPaddress))
				return ECS_ERR_SOCKETERROR;
		}
	}


	while(++nTimeCounter < ECON_PROJLIST_CONNECTTIMEOUT){
		if( m_pListener->GetClientCount() + GetThreadCount() == nRecordNum ) break;
		Sleep(ECS_SLEEPTIME);
	}

	
	for(i=0; i<m_pListener->GetClientCount(); i++)
		OnAcceptClient(m_pListener->GetClient(i));

	m_pListener->ClearClient();



	




	nThdCnt = GetThreadCount();


	eclog.Print(LL_STATE, "Thread Count is %d.\n",nThdCnt);

	
	int nRcvCnt = 0;
	nTimeCounter = 0;
	while(++nTimeCounter < ECS_TIMEOVER_CONNECT)
	{
		nRcvCnt = 0;
		m_nKwdErrCount = 0;
		for(i=0; i<nThdCnt; i++){
			CECClientThread *ect = GetClientThread(i);
			if( ect ){
				if( ect->GetStatus() & e_recvconnect )
					nRcvCnt++;
				else if( ect->GetStatus() & e_keyworderror ){		
					
					memcpy(	&m_KwdErrList[m_nKwdErrCount],
							ect->GetUniqInfo(),
							ECON_UNIQINFO_LENGTH
							);
					m_nKwdErrCount++;
				}
			}
		}
		if( (nRcvCnt + m_nKwdErrCount) >= nThdCnt )
			break;
		Sleep(ECS_SLEEPTIME);
	}
	eclog.Print(LL_STATE, "RecvConnected Count = %d\n",nRcvCnt);
	eclog.Print(LL_STATE, "Keyword Error Count = %d\n",m_nKwdErrCount );






	int keyUse = nConCltNum;
	for(i=0; i<(int)nConCltNum; i++)

	{




		if(pConCltInf[i].m_projUseKeyword)

		{
			BOOL getKey= TRUE;
			for(int j=0; j<GetThreadCount(); j++)
			{
				CECClientThread *dummy = GetClientThread(j);


				if(memcmp(pConCltInf[i].m_projUniqInfo, dummy->m_cUniqInfo, ECON_UNIQINFO_LENGTH)==0)

				{
					keyUse--;
					getKey=FALSE;
					break;
				}
			}
			if(getKey)
			{
				memcpy(	&m_KwdErrList[m_nKwdErrCount],


						pConCltInf[i].m_projUniqInfo,

						ECON_UNIQINFO_LENGTH
						);
				m_nKwdErrCount++;
			}
		}
		else
		{
			keyUse--;
		}
	}


	
	int nPos = 0;
	for(int j=0; j<nThdCnt; j++){
		CECClientThread *ect = GetClientThread(nPos);
		if( ect ){
			if( !(ect->GetStatus() & e_recvconnect) || (ect->GetStatus() & e_disablesock)){
				KillClientThread(ect->GetUniqInfo());
				continue;
			}
		}
		nPos++;
	}

	eclog.Print(LL_STATE, "return thread count = %d\n",GetThreadCount());


	*keyError = keyUse;

	return GetThreadCount();
}



BOOL
CECServer::StartListen()
{
	if( m_pListener ){
		eclog.Print(LL_STATE, "Listener Already Start.\n");
		return FALSE;
	}

	eclog.Print(LL_STATE, "Create Listener Thread.\n");
	m_pListener = new CListenThread(m_hParentWnd, this);
	if( !m_pListener )
		return FALSE;
	
	eclog.Print(LL_STATE, "Start Listen Thread.\n");
	if( !m_pListener->CreateThread() ){
		eclog.Print(LL_ERROR, "!!!Failed to create Listener Thread.\n");
		delete m_pListener;
		m_pListener = NULL;
		return FALSE;
	}
	
	return TRUE;
}



void
CECServer::CloseListen()
{
	if( m_pListener ){
		m_pListener->KillThread();
		m_pListener = NULL;
	}
}


int
CECServer::RestartClients()
{
	int nRestartedCnt=0;
	long nTimeCounter = 0;
	int nClientCnt = GetThreadCount();

	
	SetStatusAllClients(e_restarted, FALSE);
	SetStatusAllClients(e_restart, TRUE);

	
	while(++nTimeCounter < ECS_TIMEOVER_DISCONNECT){
		nRestartedCnt = 0;
		for(int i=0; i<nClientCnt; i++){
			if( CECClientThread *ect = GetClientThread(i) ){
				if( ect->GetStatus() & e_restarted )
					nRestartedCnt++;
			}
		}
		if( nRestartedCnt == nClientCnt ) break;
		Sleep(ECS_SLEEPTIME);
	}
	if( nRestartedCnt == nClientCnt )
		return ECS_NOERR;
	else if( nRestartedCnt < nClientCnt )
		return ECS_ERR_NOTALLRECEIVE;
	else
		return ECS_ERR_UNKNOWN;
}


	
void
CECServer::ClearThreadList()
{
	while (!m_TCPThreadList.IsEmpty())
	{
		((CECClientThread*)m_TCPThreadList.RemoveHead())->KillThread();
	}
}


	
int
CECServer::GetThreadCount()
{
	return m_TCPThreadList.GetCount();
}



CECClientThread*
CECServer::GetClientThread(int index)
{
	if( !m_TCPThreadList.IsEmpty() ){
		POSITION pos = m_TCPThreadList.FindIndex(index);
		if( pos ){
			return (CECClientThread*)m_TCPThreadList.GetAt(pos);
		}
	}
	return (CECClientThread*)NULL;
}



CECClientThread*
CECServer::GetClientThread(unsigned char* pUniqInfo)
{
	if( !pUniqInfo ){
		return NULL;
	}

	if( m_TCPThreadList.IsEmpty() )
		return NULL;

	
	POSITION pos = m_TCPThreadList.GetHeadPosition();
	while(pos){
		CECClientThread *ect = (CECClientThread*)m_TCPThreadList.GetNext(pos);
		if( !ect ){
			return NULL;
		}
		if( memcmp(ect->GetUniqInfo(), pUniqInfo, ECON_UNIQINFO_LENGTH)==0){
			return ect;
		}
	}
	return NULL;
}






	




BOOL
CECServer::KillClientThread(unsigned char* pUniqInfo)
{
	if( !pUniqInfo ){
		return NULL;
	}
	if( m_TCPThreadList.IsEmpty() )
		return FALSE;

	
	POSITION pos = m_TCPThreadList.GetHeadPosition(),posbak;
	while(pos){
		posbak = pos;
		CECClientThread *ect = (CECClientThread*)m_TCPThreadList.GetNext(pos);
		if( !ect ){
			return FALSE;
		}
		if( memcmp(ect->GetUniqInfo(), pUniqInfo, ECON_UNIQINFO_LENGTH)==0){
			ect->KillThread();
			m_TCPThreadList.RemoveAt(posbak);
			return TRUE;
		}
	}
	return FALSE;
}


	
void
CECServer::SetStatusAllClients(DWORD status, BOOL flg)
{
	if( !m_TCPThreadList.IsEmpty() ){
		POSITION pos = m_TCPThreadList.GetHeadPosition();
		while(pos){
			CECClientThread *ect = (CECClientThread*)m_TCPThreadList.GetNext(pos);
			if( ect ){
				DWORD stat = ect->GetStatus();
				if( flg )
					stat |= status;
				else
					stat &= ~status;

				ect->SetStatus(stat);
			}		
		}
	}
}


	
void
CECServer::OnAcceptClient(CECClientThread* pThread)
{
	eclog.Print(LL_STATE, "Add new client thread.\n");
	try{
		m_TCPThreadList.AddTail(pThread);
	}catch(CMemoryException *e){
		e->Delete();
		pThread->KillThread();
	}
}