/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/



#include "stdafx.h"
#include "EP_NSS.h"
#include "EP_NSSDlg.h"
#include "Registry.h"
#include <malloc.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#include "..\\common\\econproto.h"





BEGIN_MESSAGE_MAP(CEP_NSSApp, CWinApp)
	
	
	
END_MESSAGE_MAP()





CEP_NSSApp::CEP_NSSApp()
{
	m_nPort = ECON_PORTNUMBER;
	m_bOutputLogFile = 0;
	m_bUseEncryption = FALSE;
	m_bUseIPConnect = FALSE;
	m_sLocalizeFName[0] = '\0';
	m_sAppPath[0] = '\0';
	m_nAdpIPAddress = 0;
	m_usESCVPPort = 0;

	m_maxProjectors=0;				
	m_resolutionMaxWidth=0;		
	m_resolutionMaxHeight=0;		



	m_bUseGroupList = FALSE;		

	memset(m_sCurrentMACAddr, 0, ECON_MACADDRESS_MAXLEN+1);
	memset(m_sIPConAddr, 0, ECON_IPADDRSTR_MAXLEN);

	m_nIPListMaxCount = ECON_PROJLIST_IPLISTMAXCOUNT;


	m_sNSGroupPath[0] = NULL;

}




CEP_NSSApp theApp;




BOOL CEP_NSSApp::InitInstance()
{
	char sFilePath[MAX_PATH];

	
	if( !LoadRegistry() ){
		eclog.Print(LL_ERROR, "Failed to load registry.\n");





		MessageBox(NULL, "Failed to Load Registry Key.", ECON_PRODUCTNAME, MB_OK|MB_ICONSTOP);


		return FALSE;
	}
	eclog.Print(LL_STATE, "Loaded Registry Info.\n");

	
	if( m_bOutputLogFile ){
		eclog.SetMode(Log::ToFile|Log::ToDebug);
		netlog.SetMode(Log::ToFile|Log::ToDebug);
		ecctlog.SetMode(Log::ToFile|Log::ToDebug);
		listenlog.SetMode(Log::ToFile|Log::ToDebug);
		searchlog.SetMode(Log::ToFile|Log::ToDebug);
		escvplog.SetMode(Log::ToFile|Log::ToDebug);
		pinglog.SetMode(Log::ToFile|Log::ToDebug);
	}

	
	sprintf(sFilePath, "%s\\%s", m_sAppPath, m_sLocalizeFName);
	CEMPLocalize lczRes;
	int nErr = lczRes.Load(sFilePath);
	if( nErr != LCZ_NOERR ){
		CString mes;
		if( nErr == LCZ_ERR_NOEMPLOCALFILE )
			mes.Format("The file '%s' is incorrect format",sFilePath);
		else if( nErr == LCZ_ERR_FILEACCESS )
			mes.Format("Failed to access a localization file '%s'.",sFilePath);
		else if( nErr == LCZ_ERR_MEMALLOC )
			mes.Format("Failed to alloc memory.");
		else
			mes.Format("Failed to load localization file.");





		MessageBox(NULL, mes, ECON_PRODUCTNAME, MB_OK|MB_ICONSTOP);


		eclog.Print(LL_ERROR, "%s\n",mes);
		return FALSE;
	}
	eclog.Print(LL_STATE, "Loaded EP_NSS localization file.\n");


	
	if( !CheckMutex() ){
		HWND	hWndCap;
		CString	strCap;
		eclog.Print(LL_ERROR, "Mutex error.\n");




		MessageBox(NULL, lczRes.GetString(IDS_ERR_STRING0011), ECON_PRODUCTNAME, MB_OK|MB_ICONSTOP);


		
		strCap = ECON_PRODUCTNAME;
		if( (hWndCap = FindWindow(NULL, strCap)) != NULL )
			SetForegroundWindow(hWndCap);	
		return FALSE;
	}

	
	if (!AfxSocketInit())
	{




		MessageBox(NULL, lczRes.GetString(IDS_ERR_STRING0013), ECON_PRODUCTNAME, MB_OK|MB_ICONSTOP);


		return FALSE;
	}


	
	if( !CheckFileExists(&lczRes, ECON_EPNSSOP_FNAME) )
		return FALSE;


	
	
	if( !CheckFileExists(&lczRes, ECON_VNESFILE_EPNSSE) )
		return FALSE;

	
	if( !CheckFileExists(&lczRes, ECON_VNESFILE_ZLIB) )
		return FALSE;

	
	if( GetOSVersion() == 0 ){
		
		if( !CheckFileExists(&lczRes, ECON_VNESFILE_EPNSB2) )
			return FALSE;
		if( !CheckFileExists(&lczRes, ECON_VNESFILE_EPNSB4) )
			return FALSE;
	}else if( GetOSVersion() == 1){
		
		if( !CheckFileExists(&lczRes, ECON_VNESFILE_EPNSBN) )
			return FALSE;
	}

	
	free((void*)m_pszHelpFilePath);
	sprintf(sFilePath, "%s\\%s", m_sAppPath, _T(ECON_HELPFILE_PATH));
	m_pszHelpFilePath=_tcsdup(sFilePath);

#ifdef _AFXDLL
	Enable3dControls();			
#else
	Enable3dControlsStatic();	
#endif


	CEP_NSSDlg dlg(&lczRes);
	m_pMainWnd = &dlg;
	dlg.m_bUseIPConnect = m_bUseIPConnect;
	dlg.m_bUseEnc = m_bUseEncryption;
	lstrcpy(dlg.m_sIPConAddr, m_sIPConAddr);
	dlg.m_usESCVPPort = m_usESCVPPort;

	dlg.m_bUseGroupList = m_bUseGroupList;


	dlg.m_ProjListBox.m_maxProjectors=m_maxProjectors;	
	dlg.m_IPProjListBox.m_maxProjectors=m_maxProjectors;

	dlg.m_resolutionMaxWidth=m_resolutionMaxWidth;		
	dlg.m_resolutionMaxHeight=m_resolutionMaxHeight;		


	
	eclog.Print(LL_STATE, "Show the E.C.DialogBox.\n");
	dlg.DoModal();

	m_bUseIPConnect = dlg.m_bUseIPConnect;
	m_bUseEncryption = dlg.m_bUseEnc;
	lstrcpy(m_sIPConAddr, dlg.m_sIPConAddr);

	
	if( !SaveRegistry() ){
		eclog.Print(LL_ERROR, "Failed to save registry.\n");
	}

	eclog.Print(LL_STATE, "Exit Application.\n");
	return FALSE;
}




BOOL
CEP_NSSApp::CheckMutex()
{
	HANDLE hPrevMutex = NULL;
	hPrevMutex = ::CreateMutex(NULL, TRUE, ECON_INSTANCE_NAME);

	if( ::GetLastError() == ERROR_ALREADY_EXISTS || !hPrevMutex)
		return FALSE;

	return TRUE;
}



int
CEP_NSSApp::GetOSVersion()
{
	OSVERSIONINFO info;
	memset(&info,0,sizeof(OSVERSIONINFO));
	info.dwOSVersionInfoSize=sizeof(OSVERSIONINFO);
	if (!GetVersionEx(&info)){
		return -1;
	}
	if(info.dwPlatformId==VER_PLATFORM_WIN32_NT){ 
		return 1;
	}else if(info.dwPlatformId==VER_PLATFORM_WIN32_WINDOWS){ 
		return 0;
	} else {
		return -1;
	}
}



BOOL
CEP_NSSApp::SaveRegistry()
{
	
	SECURITY_ATTRIBUTES  security, *pseq=NULL;
	
	
	if( GetOSVersion() == 1){
		::ZeroMemory(&security, sizeof(security));
	
		void* pBuf = _alloca(ACL_BUFFER_SIZE);

		if(!pBuf) {
			return FALSE;
		}

		if(!CRegistry::CreateAllUserSecurity(&security, (PACL) pBuf )){
			return FALSE;
		}
		pseq = &security;
	}

	CRegistry reg;
	if( !reg.create(	ECON_REGISTRY_ROOTKEY,
						ECON_REGISTRY_SUBKEY,
						NULL,
						KEY_ALL_ACCESS,
						pseq
					)){
		return FALSE;
	}



	if( !reg.setBOOL(ECON_REG_USEENC, m_bUseEncryption))

		return FALSE;



	if( !reg.setBOOL(ECON_REG_USEIPCON, m_bUseIPConnect))

		return FALSE;



	if( !reg.setString(ECON_REG_IPCONADDR, m_sIPConAddr))

		return FALSE;





	char sMACStr[MAX_PATH];
	sprintf(sMACStr, "%02x_%02x_%02x_%02x_%02x_%02x",m_sCurrentMACAddr[0],m_sCurrentMACAddr[1],
											   m_sCurrentMACAddr[2],m_sCurrentMACAddr[3],
											   m_sCurrentMACAddr[4],m_sCurrentMACAddr[5]);



	if( !reg.setString(ECON_REG_CURRENTLAN, sMACStr))

		return FALSE;

	in_addr addr;
	addr.S_un.S_addr = m_nAdpIPAddress;


	if( !reg.setString(ECON_REG_ADAPTERIPADDR, inet_ntoa(addr)))

		return FALSE;






	


	if( !reg.setBOOL(ECON_REG_USEGROUPLIST, m_bUseGroupList) )

		return FALSE;

	







	if( !reg.close() )
		return FALSE;

	return TRUE;
}



BOOL
CEP_NSSApp::LoadRegistry()
{
	CRegistry reg;
	CString	  tmp;
	if( !reg.open(ECON_REGISTRY_ROOTKEY, ECON_REGISTRY_SUBKEY, KEY_ALL_ACCESS))
		return FALSE;

	


	if( !reg.getBOOL(ECON_REG_OUTPUTLOG, &m_bOutputLogFile) )
		m_bOutputLogFile = 0;


	


	if( !reg.getBOOL(ECON_REG_USEENC, &m_bUseEncryption) )

		m_bUseEncryption = 0;

	m_bUseEncryption = (BOOL)(m_bUseEncryption != 0 );


	


	if( !reg.getBOOL(ECON_REG_USEIPCON, &m_bUseIPConnect) )

		m_bUseIPConnect = 0;

	m_bUseIPConnect = (BOOL)(m_bUseIPConnect != 0 );


	






	
	int portnum;


	if( !reg.getINT(ECON_REG_ESCVPPORT, &portnum) )

		m_usESCVPPort = ESCVP_PORTNUMBER;
	else
		m_usESCVPPort = (unsigned short)portnum;
	

	


	if( reg.getString(ECON_REG_LOCALFNAME, &tmp) ){

		lstrcpy(m_sLocalizeFName, (LPCTSTR )tmp );
		if( strlen(m_sLocalizeFName)==0 )
			lstrcpy(m_sLocalizeFName, ECON_LOCALIZE_DLLNAME);

	}else{
		lstrcpy(m_sLocalizeFName, ECON_LOCALIZE_DLLNAME);
	}

	


	if( reg.getString(ECON_REG_IPCONADDR, &tmp) ){

		lstrcpy(m_sIPConAddr, (LPCTSTR)tmp );
	}else{
		lstrcpy(m_sIPConAddr, "");
	}

	


	if( reg.getString(ECON_REG_CURRENTLAN, &tmp) ){

		char sMACStr[MAX_PATH];
		lstrcpy((char*)sMACStr, (LPCTSTR)tmp );
		sscanf(sMACStr, "%02x_%02x_%02x_%02x_%02x_%02x", &m_sCurrentMACAddr[0],&m_sCurrentMACAddr[1],
														 &m_sCurrentMACAddr[2],&m_sCurrentMACAddr[3],
														 &m_sCurrentMACAddr[4],&m_sCurrentMACAddr[5]);
	}else{
		::ZeroMemory(m_sCurrentMACAddr, ECON_MACADDRESS_MAXLEN);
	}


	if( reg.getString(ECON_REG_ADAPTERIPADDR, &tmp) ){

		m_nAdpIPAddress = inet_addr((LPCTSTR)tmp);
	}else{
		m_nAdpIPAddress = 0;
	}



	if( reg.getString(ECON_REG_APPPATH, &tmp) ){


		lstrcpy(m_sAppPath, (LPCTSTR)tmp );
		
		
		if( strlen(m_sAppPath) == 0 ){
			if( GetAppDir(m_sAppPath) == 0 )
				return FALSE;
		}
	}else{
		
		if( GetAppDir(m_sAppPath) == 0 )
			return FALSE;
	}
	eclog.Print(LL_STATE, "AppPath = %s\n", m_sAppPath);



	int tempInt;


	if( !reg.getINT(ECON_REG_MAXPROJECTOR, &tempInt) )

		m_maxProjectors = ECON_SELECTPROJ_MAXNUM;
	else
		m_maxProjectors = (unsigned short)tempInt;
	if(m_maxProjectors > ECON_SELECTPROJ_MAXNUM_LIMIT)
		m_maxProjectors = ECON_SELECTPROJ_MAXNUM_LIMIT;
	



	if( !reg.getINT(ECON_REG_RESOLUTIONWID, &tempInt) )

		m_resolutionMaxWidth = ECON_UXGA_WIDTH;
	else
		m_resolutionMaxWidth = (unsigned short)tempInt;
	if(m_resolutionMaxWidth > ECON_XGA_WIDTH_LIMIT)
		m_resolutionMaxWidth = ECON_XGA_WIDTH_LIMIT;




	if( !reg.getINT(ECON_REG_RESOLUTIONHEI, &tempInt) )

		m_resolutionMaxHeight = ECON_UXGA_HEIGHT;
	else
		m_resolutionMaxHeight = (unsigned short)tempInt;
	if(m_resolutionMaxHeight > ECON_XGA_HEIGHT_LIMIT)
		m_resolutionMaxHeight = ECON_XGA_HEIGHT_LIMIT;


	


	if( !reg.getBOOL(ECON_REG_USEGROUPLIST, &m_bUseGroupList) )

		m_bUseGroupList = FALSE;


	if( reg.getString(ECON_REG_NSGROUPPATH, &tmp) ){
		lstrcpy(m_sNSGroupPath, (LPCTSTR)tmp );
	}else{
		return FALSE;
	}


	









	reg.close();

	return TRUE;
}				



int
CEP_NSSApp::GetAppDir(char  *pDst)
{
	char szPath[MAX_PATH];
	if (GetModuleFileName(NULL, szPath, MAX_PATH))
	{
		TCHAR szDrive	[MAX_PATH];
		TCHAR szDir		[MAX_PATH];
		TCHAR szFname	[MAX_PATH];
		TCHAR szExt		[MAX_PATH];
		_tsplitpath(szPath, szDrive, szDir, szFname, szExt);
		if( _tcslen(szDir)!=0 && _tcslen(szDrive)!=0 )
		{
			szDir[strlen(szDir)-1] = '\0';
			return sprintf(pDst,"%s%s",szDrive, szDir);
		}
	}
	return 0;
}



BOOL
CEP_NSSApp::CheckFileExists(CEMPLocalize *plocaldata, char* pFName)
{
	char sFilePath[MAX_PATH];
	char sMessage[LOCALSTR_MAXLEN+MAX_PATH];
	sprintf(sFilePath, "%s\\%s", m_sAppPath, pFName);

	CFileStatus status;
	if( !CFile::GetStatus( sFilePath, status ) ){
		sprintf(sMessage,"%s\n%s", plocaldata->GetString(IDS_ERR_STRING0028), sFilePath);




		MessageBox(NULL, sMessage,ECON_PRODUCTNAME, MB_OK|MB_ICONSTOP);


		return FALSE;
	}else
		return TRUE;
}