/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/




#include "stdafx.h"
#include "EP_NSS.h"
#include "ESCVPBase.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif






CESCVPBase::CESCVPBase()
{
	escvplog.Print(LL_STATE, "CESCVPBase constructor called.\n");
	m_ulConnectIP = 0;
}

CESCVPBase::~CESCVPBase()
{
	escvplog.Print(LL_STATE, "CESCVPBase destructor called.\n");
	if( m_ulConnectIP ){
		Close();
		m_ulConnectIP = 0;
	}
}


	
BOOL
CESCVPBase::Open(ULONG ulAddr, ULONG ulSockAddr, unsigned short nPort)
{
	escvplog.Print(LL_STATE, "Enter Open function.addr=%x/sockaddr=%x\n",ulAddr,ulSockAddr);

	
	if( m_ulConnectIP ){
		if( m_ulConnectIP != ulAddr ){
			if( !Close() ){
				return FALSE;
			}
		}else
		{
			return TRUE;
		}
	}

	
	char hostaddr[16], bindaddr[16];
	struct in_addr inaddr;
	inaddr.S_un.S_addr = ulAddr;
	strcpy(hostaddr, inet_ntoa(inaddr));

	inaddr.S_un.S_addr = ulSockAddr;
	strcpy(bindaddr, inet_ntoa(inaddr));

	escvplog.Print(LL_STATE, "host addr=%s / bind addr=%s / portnum=%d\n"
					,hostaddr, bindaddr, nPort);

	int nErr = LKI_OpenSocket(hostaddr, bindaddr, nPort);
	if( nErr==LKI_ERR_FATAL){
		escvplog.Print(LL_ERROR, "!!! Failed to open socket.\n");
		return FALSE;
	}

	
	nErr = LKI_CreateDNetSession(DNETSESSION_TIMEOUTSEC, DNETSESSION_TIMEOUTMSEC);
	if( nErr != LKI_SUCCESS ){
		return FALSE;
	}
	
	m_ulConnectIP = ulAddr;
	escvplog.Print(LL_STATE, "Open ESCVP connection. Address = %s\n", hostaddr);
	return TRUE;
}


	
BOOL
CESCVPBase::Close()
{
	if( LKI_CloseSocket() != LKI_SUCCESS ){
		escvplog.Print(LL_ERROR, "!!! Failed to close socket.\n");
		return FALSE;
	}
	escvplog.Print(LL_STATE, "Close ESCVP connection.\n");
	m_ulConnectIP = 0;	
	return TRUE;
}

	
ULONG
CESCVPBase::GetConnectIP()
{
	return m_ulConnectIP;
}



BOOL
CESCVPBase::SetKeyCommand(char nKeyParam)
{
	char key[ESCVP21_MAXLEN];
	BOOL nRetVal = FALSE;
	char *pRecv = NULL;
	int nErr = 0;
	
	sprintf(key, "%02X", nKeyParam);
	nErr = LKI_ComSequence2(cmdKEY, key, &pRecv);

	if( nErr > 0 && pRecv )
	{
		if( lstrcmp(pRecv, "ERR")==0 ){
			escvplog.Print(LL_STATE, "!!! Failed to send Key command.KeyParam=%s\n",key);


			nRetVal = TRUE;

		}else if( lstrcmp(pRecv, ":")==0 ){
			escvplog.Print(LL_STATE, "Success to send Key command. KeyParam=%s\n",key);
			nRetVal = TRUE;
		}
	}else{
		escvplog.Print(LL_STATE, "!!! LKI_ComSequence2 function failed.KeyParam=%s\n",key);
		nRetVal = FALSE;
	}

	if( pRecv )	free(pRecv);
	return nRetVal;
}



BOOL
CESCVPBase::SetIncDecValue(const COMID eID, int nParam)
{
	char param[ESCVP21_MAXLEN];
	BOOL nRetVal = FALSE;
	char *pRecv = NULL;
	int nErr = 0;

	if( nParam == EVM_PRM_INC)
		lstrcpy(param, "INC");
	else if( nParam == EVM_PRM_DEC )
		lstrcpy(param, "DEC");
	else
		return FALSE;

	nErr = LKI_ComSequence2(eID, (const char*)param, &pRecv);

	if( nErr > 0 && pRecv )
	{
		if( lstrcmp(pRecv, "ERR")==0 ){
			escvplog.Print(LL_STATE, "!!! Failed to send Inc/Dec command.param=%s\n",param);


			nRetVal = TRUE;

		}else if( lstrcmp(pRecv, ":")==0 ){
			escvplog.Print(LL_STATE, "Success to send Inc/Dec command. param=%s\n",param);
			nRetVal = TRUE;
		}
	}else{
		escvplog.Print(LL_STATE, "!!! LKI_ComSequence2 function failed.param=%s\n",param);
		nRetVal = FALSE;
	}

	if( pRecv )	free(pRecv);
	return nRetVal;
}
