/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


#include "stdafx.h"
#include "ListenThread.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


IMPLEMENT_DYNAMIC(CListenThread, CWinThread)

CListenThread::CListenThread(HWND hWnd, CECServer* pServer)
{
	m_hWnd   = hWnd;
	m_nPort  = pServer->m_nPort;
	m_pServer = pServer;
	m_bKill = FALSE;
	
	m_hEventKill = CreateEvent(NULL, TRUE, FALSE, EVENTKILL_NAME);
	listenlog.Print(LL_STATE, "create handle of m_hEventKill.\n");

	m_hEventDead = CreateEvent(NULL, TRUE, FALSE, EVENTDEAD_NAME);
	listenlog.Print(LL_STATE, "create handle of m_hEventDead.\n");

	
	InitializeCriticalSection(&m_csLock);
	listenlog.Print(LL_STATE, "Initialized CriticalSection object.\n");
}

CListenThread::~CListenThread()
{
	listenlog.Print(LL_STATE, "Enter CListenThread destructor.\n");
	
	DeleteCriticalSection(&m_csLock);
	listenlog.Print(LL_STATE, "deleted CriticalSection object.\n");

	CloseHandle(m_hEventKill);
	listenlog.Print(LL_STATE, "closed handle of m_hEventKill.\n");

	CloseHandle(m_hEventDead);
	listenlog.Print(LL_STATE, "closed handle of m_hEventDead.\n");

	listenlog.Print(LL_STATE, "Exit CListenThread destructor.\n");
}

BOOL
CListenThread::InitInstance()
{
	
	
	
#ifndef _AFXDLL
#define _AFX_SOCK_THREAD_STATE AFX_MODULE_THREAD_STATE
#define _afxSockThreadState AfxGetModuleThreadState()
	_AFX_SOCK_THREAD_STATE* pState = _afxSockThreadState;
	if (pState->m_pmapSocketHandle == NULL)
		pState->m_pmapSocketHandle = new CMapPtrToPtr;
	if (pState->m_pmapDeadSockets == NULL)
		pState->m_pmapDeadSockets = new CMapPtrToPtr;
	if (pState->m_plistSocketNotifications == NULL)
		pState->m_plistSocketNotifications = new CPtrList;
#endif

	
	m_pSock = new CListenSock();
	listenlog.Print(LL_STATE, "make new listener socket.\n");

	m_pSock->Create(m_nPort);
	listenlog.Print(LL_STATE, "created listener socket.\n");

	m_pSock->m_pThread = this;
	listenlog.Print(LL_STATE, "set thread pointer.\n");

	m_pSock->m_pServer = m_pServer;
	listenlog.Print(LL_STATE, "set ECServer pointer.\n");

	m_pSock->Listen();
	listenlog.Print(LL_STATE, "start listener.\n");

	return TRUE;
}


void
CListenThread::Delete()
{
	delete m_pSock;
	listenlog.Print(LL_STATE, "deleted listener socket.\n");
	VERIFY(SetEvent(m_hEventDead));
	listenlog.Print(LL_STATE, "set event m_hEventDead.\n");
}


int
CListenThread::ExitInstance()
{
	listenlog.Print(LL_STATE, "ExitInstance.\n");
	CWinThread::ExitInstance();
	return 0;
}

BEGIN_MESSAGE_MAP(CListenThread, CWinThread)
	
	
	ON_THREAD_MESSAGE(WM_TERM_THREAD, OnTermThread)
END_MESSAGE_MAP()




	
void
CListenThread::KillThread()
{
	listenlog.Print(LL_STATE, "Enter KillThread Function.\n");
	
	::PostThreadMessage(this->m_nThreadID, WM_TERM_THREAD, 0, 0);
	listenlog.Print(LL_STATE, "Posted WM_TERM_THREAD Message.\n");

	SetThreadPriority(THREAD_PRIORITY_ABOVE_NORMAL);
	listenlog.Print(LL_STATE, "Change Thread Priority Above Normal.\n");

	WaitForSingleObject(m_hEventDead, INFINITE);
	listenlog.Print(LL_STATE, "Finished to wait m_hEventDead.\n");

	WaitForSingleObject(m_hThread, INFINITE);
	listenlog.Print(LL_STATE, "Finished to wait m_hThread.\n");

	delete this;
}



void CListenThread::OnTermThread(UINT, LONG)
{
	listenlog.Print(LL_STATE, "Receive WM_TERM_THREAD Message.\n");
	AfxEndThread(0,TRUE);
	
	listenlog.Print(LL_STATE, "Call PostQuitMessage.\n");
}



void
CListenThread::AddClient(CECClientThread* pThread, int nErrorCode)
{
	if( nErrorCode != 0 ){
		::PostMessage(m_hWnd, ECM_LISTENERR, 0, 0);
		return;
	}
	EnterCriticalSection(&m_csLock);
	m_ClientList.AddTail(pThread);
	LeaveCriticalSection(&m_csLock);
	listenlog.Print(LL_STATE, "Add Thread pointer.\n");
}



int
CListenThread::GetClientCount()
{
	int nRet;
	EnterCriticalSection(&m_csLock);
	nRet = m_ClientList.GetCount();
	LeaveCriticalSection(&m_csLock);
	return nRet;
}



void
CListenThread::ClearClient()
{
	EnterCriticalSection(&m_csLock);
	m_ClientList.RemoveAll();
	LeaveCriticalSection(&m_csLock);
}



CECClientThread*
CListenThread::GetClient(int index)
{
	CECClientThread *pThread = NULL;
	EnterCriticalSection(&m_csLock);
	if( !m_ClientList.IsEmpty() ){
		POSITION pos = m_ClientList.FindIndex(index);
		if( pos ){
			pThread = (CECClientThread*)m_ClientList.GetAt(pos);
			LeaveCriticalSection(&m_csLock);
			return pThread;
		}
	}
	LeaveCriticalSection(&m_csLock);
	return NULL;
}



	







