/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/



#include "stdafx.h"
#include "Registry.h"
#include <malloc.h>
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif



CRegistry::CRegistry()
{
	m_hKey = NULL;
}

CRegistry::~CRegistry()
{
	if (m_hKey)
		close();
}

BOOL
CRegistry::create(
	HKEY					hKey,
	LPCTSTR					pSubKey,
	LPTSTR					pClass,
	REGSAM					samDesired,
	LPSECURITY_ATTRIBUTES	pSecurity
	)
{
	LONG	rv;
	DWORD	dwDisposition;
	rv = RegCreateKeyEx(hKey, pSubKey, 0, pClass, REG_OPTION_NON_VOLATILE
						,  samDesired, pSecurity, &m_hKey, &dwDisposition);
	if (rv == ERROR_SUCCESS) {
		return TRUE;
	} else {
		m_hKey = NULL;
		return FALSE;
	}
}

BOOL
CRegistry::open(
	HKEY	hKey,
	LPCTSTR	pSubKey,
	REGSAM	samDesired)
{
	LONG	rv;
	rv = RegOpenKeyEx(hKey, pSubKey, 0, samDesired, &m_hKey);
	if (rv == ERROR_SUCCESS) {
		return TRUE;
	} else {
		m_hKey = NULL;
		return FALSE;
	}
}

BOOL
CRegistry::close(
	void)
{
	if (m_hKey) {
		LONG	rv;
		rv = RegCloseKey(m_hKey);
		m_hKey = NULL;
		return (rv == ERROR_SUCCESS);
	} else
		return FALSE;
}

BOOL
CRegistry::setBOOL(
	LPCTSTR	szValue,
	BOOL	bData)
{
	if (m_hKey) {
		LONG	rv;
		rv = RegSetValueEx(m_hKey, szValue, 0, REG_DWORD
							, (const BYTE *)&bData, sizeof(bData));
		return (rv == ERROR_SUCCESS);
	} else
		return FALSE;
}

BOOL
CRegistry::setINT(
	LPCTSTR	szValue,
	INT		dwData)
{
	if (m_hKey) {
		LONG	rv;
		rv = RegSetValueEx(m_hKey, szValue, 0, REG_DWORD
							, (const BYTE *)&dwData, sizeof(dwData));
		return (rv == ERROR_SUCCESS);
	} else
		return FALSE;
}

BOOL
CRegistry::setString(
	LPCTSTR	szValue,
	LPCTSTR	szData)
{
	if (m_hKey) {
		LONG	rv;
		rv = RegSetValueEx(m_hKey, szValue, 0, REG_SZ
							, (const BYTE *)szData, strlen(szData)+1);
		return (rv == ERROR_SUCCESS);
	} else
		return FALSE;
}

BOOL
CRegistry::getBOOL(
	LPCTSTR	szValue,
	BOOL	* pbData)
{
	if (m_hKey) {
		DWORD	cbData = sizeof(BOOL);
		LONG	rv = RegQueryValueEx(m_hKey, szValue, NULL, NULL
						, (BYTE *)pbData, &cbData);
		return (rv == ERROR_SUCCESS);
	} else
		return FALSE;
}

BOOL
CRegistry::getINT(
	LPCTSTR	szValue,
	INT		* pdwData)
{
	if (m_hKey) {
		DWORD	cbData = sizeof(DWORD);
		LONG	rv = RegQueryValueEx(m_hKey, szValue, NULL, NULL
						, (BYTE *)pdwData, &cbData);
		return (rv == ERROR_SUCCESS);
	} else
		return FALSE;
}

BOOL
CRegistry::getString(
	LPCTSTR	szValue,
	CString	* pData)
{
	if (m_hKey) {
		DWORD	cbData = 0;
		LONG	rv = RegQueryValueEx(m_hKey, szValue, NULL, NULL
						, NULL, &cbData);
		char	* buf = new char[cbData];
		rv = RegQueryValueEx(m_hKey, szValue, NULL, NULL
						, (BYTE *)buf, &cbData);
		if (rv == ERROR_SUCCESS)
			*pData = buf;
		delete [] buf;
		return (rv == ERROR_SUCCESS);
	} else
		return FALSE;
}


BOOL CRegistry::deleteKeyValue(LPCTSTR szValue)
{
	if (m_hKey) {
		LONG rv = RegDeleteValue(m_hKey, szValue);
		return (rv == ERROR_SUCCESS);
	} else 
		return FALSE;
}


BOOL CRegistry::CreateAllUserSecurity(LPSECURITY_ATTRIBUTES pSecurity, PACL pPacl )
{
	static SECURITY_DESCRIPTOR	 sdPermissions;
	SECURITY_ATTRIBUTES&	sa = *pSecurity;

    PSID  psidEveryone = NULL; 
    PACL  paclKey      = NULL; 
  
	SID_IDENTIFIER_AUTHORITY  WorldSidAuthority= SECURITY_WORLD_SID_AUTHORITY; 
 
	

	if(!pPacl){
		return FALSE;
	}

	
	
	sa.nLength              = sizeof(SECURITY_ATTRIBUTES); 
	sa.bInheritHandle       = FALSE; 
	sa.lpSecurityDescriptor = &sdPermissions; 

	
	
 	if (!AllocateAndInitializeSid(&WorldSidAuthority, 1, SECURITY_WORLD_RID, 
								  0, 0, 0, 0, 0, 0, 0, &psidEveryone ) ) {
		int err = GetLastError();
		err;
 		return FALSE;
	}
 
	if (!InitializeSecurityDescriptor(&sdPermissions, SECURITY_DESCRIPTOR_REVISION1)) {
 		return FALSE;
	}
 
	
	
 
	 
	
	
 



	paclKey = pPacl;

	if (!InitializeAcl(paclKey, ACL_BUFFER_SIZE, ACL_REVISION2)) {
 		return FALSE;
	}
 
    
    
    
 
    if (!AddAccessAllowedAce(paclKey, ACL_REVISION2, KEY_ALL_ACCESS, psidEveryone)) {
 		return FALSE;
	}
   
	
 
	if (!SetSecurityDescriptorDacl(&sdPermissions, TRUE, paclKey, FALSE)) {
 		return FALSE;
	}
	
	return TRUE;
}







