/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "VNeSLib.h"
#include <stdio.h>





CVNeSLib::CVNeSLib()
{
	m_hLibModule		= NULL;

	m_pfnStartVNeS		= NULL;
	m_pfnStopVNeS		= NULL;
	m_pfnConnect		= NULL;
	m_pfnDisconnect		= NULL;
	m_pfnIsSending		= NULL;
	m_pfnSetSendImage	= NULL;
	m_pfnReSendRect		= NULL;
	m_pfnGetScrFormat	= NULL;
	m_bConnecting		= FALSE;
}


CVNeSLib::~CVNeSLib()
{
	if(m_hLibModule){
		FreeLib();
	}
}


BOOL
CVNeSLib::FreeLib()
{
	
	VNeSStop();

	BOOL ret = FALSE;

	
	m_pfnStartVNeS		= NULL;
	m_pfnStopVNeS		= NULL;
	m_pfnConnect		= NULL;
	m_pfnDisconnect		= NULL;
	m_pfnIsSending		= NULL;
	m_pfnSetSendImage	= NULL;
	m_pfnReSendRect		= NULL;
	m_pfnGetScrFormat	= NULL;

	if(m_hLibModule){
		ret = FreeLibrary(m_hLibModule);
		m_hLibModule = NULL;
	}

	return ret;

}


BOOL
CVNeSLib::LoadLib(char *psAppPath)
{
	char cDLL	[_MAX_PATH];
	memset(cDLL,0,_MAX_PATH);
	
	if(_snprintf(cDLL,_MAX_PATH,"%s\\EP_NSSE.dll",psAppPath) < 0)
	{
		
		return FALSE;
	}

	m_hLibModule = (HMODULE)LoadLibrary(cDLL);

	if(m_hLibModule == NULL)
		return FALSE;

	m_pfnStartVNeS		= (PROC_VNeSStart) GetProcAddress(m_hLibModule,TEXT("StartVNeS"));
	m_pfnStopVNeS		= (PROC_VNeSStop) GetProcAddress(m_hLibModule,TEXT("StopVNeS"));
	m_pfnConnect		= (PROC_VNeSConnect) GetProcAddress(m_hLibModule,TEXT("Connect"));
	m_pfnDisconnect		= (PROC_VNeSDisconnect) GetProcAddress(m_hLibModule,TEXT("Disconnect"));		
	m_pfnIsSending		= (PROC_VNeSIsSending) GetProcAddress(m_hLibModule,TEXT("IsSending"));	 	
	m_pfnSetSendImage	= (PROC_VNeSSetSendImage) GetProcAddress(m_hLibModule,TEXT("SetSendImage"));
	m_pfnReSendRect		= (PROC_VNeSReSendRect) GetProcAddress(m_hLibModule,TEXT("ReSendFullScrRect"));	
	m_pfnGetScrFormat = (PROC_VNeSGetScrFormat) GetProcAddress(m_hLibModule,TEXT("GetScreenFormat"));	

	
	if(m_pfnStartVNeS		== NULL 	
	   || m_pfnStopVNeS		== NULL
	   || m_pfnConnect		== NULL
	   || m_pfnDisconnect	== NULL
	   || m_pfnIsSending	== NULL
	   || m_pfnSetSendImage == NULL
	   || m_pfnReSendRect	== NULL
	   || m_pfnGetScrFormat == NULL
	   )
	{
		return FALSE;
	}
	else
	{
		return TRUE;
	}

}


BOOL  

CVNeSLib::VNeSStart(HWND hWnd)
{
#ifdef _VNES_DUMMY_
	return TRUE;
#else
	if(!m_hLibModule || !m_pfnStartVNeS)
	{
		return FALSE;
	}
	
	return m_pfnStartVNeS(hWnd);
#endif
}


BOOL	
CVNeSLib::VNeSStop()
{
#ifdef _VNES_DUMMY_
	return TRUE;
#else
	if(!m_hLibModule || !m_pfnStopVNeS)
	{
		return FALSE;
	}

	Disconnect();
	
	return m_pfnStopVNeS();
#endif
}


BOOL	
CVNeSLib::Connect(CEVStream &ConnInfo)
{
#ifdef _VNES_DUMMY_
	return TRUE;
#else
	if(m_hLibModule && m_pfnConnect)
	{
		if( m_pfnConnect(ConnInfo) ){
			m_bConnecting = TRUE;
			return TRUE;
		}
	}
	return FALSE;
#endif
}


BOOL	
CVNeSLib::Disconnect()
{
#ifdef _VNES_DUMMY_
	return TRUE;
#else
	if(!m_hLibModule || !m_pfnDisconnect)
	{
		return FALSE;
	}
	m_bConnecting = FALSE;

	return m_pfnDisconnect();
#endif
}


BOOL
CVNeSLib::IsSending(DWORD &dwBytesSent)
{
#ifdef _VNES_DUMMY_
	dwBytesSent = 100;
	return TRUE;
#else
	if(!m_hLibModule || !m_pfnIsSending)
	{
		return FALSE;
	}

	return m_pfnIsSending(dwBytesSent);
#endif
}


BOOL	
CVNeSLib::SetSendImage(BOOL bPause)
{
#ifdef _VNES_DUMMY_
	return TRUE;
#else
	if(!m_hLibModule || !m_pfnSetSendImage)
	{
		return FALSE;
	}

	return m_pfnSetSendImage(bPause);
#endif
}


BOOL	
CVNeSLib::ReSendFullScrRect(DWORD dwID)
{
#ifdef _VNES_DUMMY_
	return TRUE;
#else
	if(!m_hLibModule || !m_pfnReSendRect)
	{
		return FALSE;
	}

	return m_pfnReSendRect(dwID);
#endif
}


BOOL	
CVNeSLib::GetScreenFormat(rfbServerInitMsg &format)
{
#ifdef _VNES_DUMMY_
	return TRUE;
#else
	if(!m_hLibModule || !m_pfnGetScrFormat)
	{
		return FALSE;
	}

	return m_pfnGetScrFormat(format);
#endif
}
