/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
#include "stdhdrs.h"
#include "vncDesktop.h"
#include "VNeSThread.h"
#include "vncBuffer.h"
#include "VNeS.h"
#include "VNeSConst.h"





DWORD WINAPI 
ClientThreadProc(LPVOID lParam)
{
	log.Print(0,"THREAD: ClientThreadProc() called");

	
	vncDesktop *pDesktop = (vncDesktop *)lParam;

	
	if(pDesktop == NULL)
		return -1;

	CClientThread  *pClientTrd   = pDesktop->m_pClientTrd;
	CDDIHookThread *pDDIHookTrd = pDesktop->m_pDDIHookTrd; 

	if(pClientTrd == NULL ||
	   pDDIHookTrd == NULL)
	{
		log.Print(0,"ERROR: pClientTrd or pDDIHookTrd NULL.");
		
		pDesktop->QuitAppMessage(VERROR_Init);
		return -1;
	}
	

	
	vncBuffer *pBuffer = new vncBuffer(pDesktop);
	if(pBuffer == NULL)
	{
		log.Print(0,"ERROR: Memory pBuffer.");
		
		pDesktop->QuitAppMessage(VERROR_Memory);
		return -1;
	}

	
	
	pBuffer->SetQualityLevel(-1);
	pBuffer->SetCompressLevel(5);
    
	
	if(!pClientTrd->Init(pBuffer))
	{
		log.Print(0,"ERROR: Client thread Init() failed.");
		
		if(pBuffer != NULL)
		{
			delete pBuffer;
			pBuffer = NULL;
		}

		
		pDesktop->QuitAppMessage(VERROR_Init);

		return -1;
	}

	
	
	pDDIHookTrd->m_fullscreen = pDesktop->m_pClientTrd->m_fullscreen;

	
	
	pDesktop->m_vrUpdateReg.Clear();
	
	pDesktop->m_vrUpdateReg.AddRect(pDesktop->m_pClientTrd->m_fullscreen);
	
	
	if(!pClientTrd->SetClientFormat(pDesktop->GetServerFormat()))
	{
		log.Print(0,"ERROR: SetClientFormat() failed.");

		if(pBuffer != NULL)
		{
			delete pBuffer;
			pBuffer = NULL;
		}
		
		
		pDesktop->QuitAppMessage(VERROR_Init);
		return -1;
	}

	
	pDesktop->m_bClientReady = TRUE;
	log.Print(0,"CLIENT: m_bClientReady = TRUE");

	
	BOOL bError = FALSE;
	
	
	
	
	DWORD dwWaitDelayFast	= pClientTrd->m_dwWaitDelay;
	DWORD dwWaitDelay		= dwWaitDelayFast;
	DWORD dwWaitDelaySlow	= pClientTrd->m_dwClientWaitDelaySlow;
	
	DWORD dwClientWaitDelaySlowLimitRects = pClientTrd->m_dwClientWaitDelaySlowLimitRects;
	
	
	
	

	while (WaitForSingleObject(pClientTrd->m_hEventKill,dwWaitDelay) 
		   == WAIT_TIMEOUT)
	{	
		
		
		
		WaitForSingleObject(pDesktop->GetSendUpdateEventHandle(), 100);
		

		
		WaitForSingleObject(pDesktop->GetSendBuffsReadyHandle(), INFINITE);
		
		timelog.Print(0,"CAPTURE: Loop Start = %lu ms"
					,GetTickCount()-gdwAppStartTime);

		
		
		{
			CMyMutexLock lock(pDesktop->m_csRegionLock);

			
			{
				rectlist rects;
				
				
				rects.clear();
				

				
				if(!pDesktop->m_vrUpdateReg.IsEmpty())
				{
				
				
					
					pDesktop->m_vrUpdateReg.Rectangles(rects);
				
					
					if(rects.size() > dwClientWaitDelaySlowLimitRects){	
						
						if(dwWaitDelay == dwWaitDelayFast){
							log.Print(0,"Rect$B?t$,5,DjCM$r1[$($?$?$a!"(BWait$B$r(BSlow$B$KJQ99$7$^$7$?(B");
						}	
						dwWaitDelay = dwWaitDelaySlow;
					} else {
						dwWaitDelay = dwWaitDelayFast;
					}
				
				}
				
			}
			

			pClientTrd->CopyUpdateRegion(&pDesktop->m_vrUpdateReg);
		}
		
		
		pDesktop->SignalSendUpdate(FALSE);

		
		
		
		pDesktop->SetMouseRect();
		
		RECT mouseRect = pDesktop->MouseRect();

		
		if(!pClientTrd->SendUpdate(mouseRect))
		{
			
			log.Print(0,"CLIENT THREAD: ERROR SendUpdate() failed.");
			
			pClientTrd->KillLoop();
			bError = TRUE;
			break;
		}

		
		if(pClientTrd->IsNewUpdate())
		{
			if(!pDesktop->CopyToDataBuffer(pClientTrd->GetSendBuff(),pClientTrd->GetSendSize()))
			{
				log.Print(0,"ERROR: CopyToDataBuffer() failed.");
				pClientTrd->KillLoop();
				bError = TRUE;
				break;
			}

			
			pClientTrd->ResetSendBuffPtr();
			pClientTrd->SetNewUpdate(false);

			if(!pDesktop->CopyFromDataBuffer())
			{
				log.Print(0,"ERROR: CopyFromDataBuffer() failed.");
				pClientTrd->KillLoop();
				bError = TRUE;
				break;
			}

			
			
			pDesktop->SignalSendBuffsReady(FALSE);

			
			
			
			
			
			
			

			timelog.Print(0,"CAPTURE: Loop Stop = %lu ms"
					,GetTickCount()-gdwAppStartTime);
			continue;
		}
		

		
		
		mouseRect = pDesktop->MouseRect();
		

		
		if(!pClientTrd->SendFullScr(mouseRect))
		{
			
			log.Print(0,"CLIENT THREAD: ERROR SendFullScr() failed.");
			
			pClientTrd->KillLoop();
			bError = TRUE;
			break;
		}

		
		if(pClientTrd->IsNewUpdate())
		{
			if(!pDesktop->CopyToDataBuffer(pClientTrd->GetSendBuff(),pClientTrd->GetSendSize()))
			{
				log.Print(0,"ERROR: CopyToDataBuffer() failed.");
				pClientTrd->KillLoop();
				bError = TRUE;
				break;
			}
			
			pClientTrd->ResetSendBuffPtr();
			pClientTrd->SetNewUpdate(false);

			if(!pDesktop->CopyFromDataBuffer())
			{
				log.Print(0,"ERROR: CopyFromDataBuffer() failed.");
				pClientTrd->KillLoop();
				bError = TRUE;
				break;
			}
		}
		

		
		
		pDesktop->SignalSendBuffsReady(FALSE);

		timelog.Print(0,"CAPTURE: Loop Stop = %lu ms"
					,GetTickCount()-gdwAppStartTime);
	}

	if(bError)
	{
		
		log.Print(0,"Ending DDIHookThread from Clientthread");
		pDDIHookTrd->KillLoop();
		pDesktop->m_bClientReady = FALSE;
		
		pDesktop->PostCaptureErrorMsg();
	}
	
	if(pBuffer != NULL)
	{
		delete pBuffer;
		pBuffer = NULL;
	}

	
	

	log.Print(0,"THREAD: ClientThreadProc() end");
	return 0;
}
