/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/




#include"stdhdrs.h"

#include <tchar.h>
#include <winbase.h>
#include <stdio.h>
#include "HeadBuff.h"

#include "DDIHook.h"


static DWORD rdtsc(){
	DWORD tcH,tcL;
	__asm{
		push eax    ;
		push edx    ;
		rdtsc       ;
		mov tcH,edx ;
		mov tcL,eax ;
		pop edx     ;
		pop eax     ;
	}
	return  ((tcH & 0x000fffff) <<12) | ((tcL >>20 )& 0x00000fff);
}


DDIHookClass::DDIHookClass(){
	
	m_pfnMovePointer	= NULL;
	m_pfnReSetOverFlow	= NULL;
	m_pfnSetDCHandle	= NULL;
	

	m_hLibModule     = NULL;
	m_pfnInitialize  = NULL;
	m_pfnFinalize    = NULL;
	m_pfnReadBuffer  = NULL;
	m_pfnPeekBuffer  = NULL;
	m_pfnWriteBuffer = NULL;
	m_bIsWin9x       = FALSE;
	
};

DDIHookClass::~DDIHookClass(){
	if(m_hLibModule){
		FreeLib();
	}
};


BOOL DDIHookClass::LoadLib(char *psAppPath)
{
	OSVERSIONINFO info;
	memset(&info,0,sizeof(OSVERSIONINFO));
	info.dwOSVersionInfoSize=sizeof(OSVERSIONINFO);
	if (!GetVersionEx(&info)){
		log.Print(0,"DDIHOOK: ERROR GetVersionEx() Failed.");
		return FALSE;
	}

	
	char cDLL	[_MAX_PATH]; 
	LPCTSTR strDLL=NULL;
	memset(cDLL,0,_MAX_PATH);

	strcpy(cDLL,psAppPath);

	int ixDir = strlen(cDLL);
	if(info.dwPlatformId==VER_PLATFORM_WIN32_NT){ 
		if(info.dwMajorVersion >= 5){ 
			log.Print(0,"DDIHOOK: Version is over 2000.");		
			
			
			
			strDLL=_T("EP_NSBN.DLL");
			
			
			if(_snprintf(cDLL+ixDir,_MAX_PATH-ixDir,"\\%s",strDLL) < 0)
			{
				
				log.Print(0,"ERROR: DDIHook dll's path exceeds _MAX_PATH = %d",_MAX_PATH);
				return FALSE;
			}

		} else {
			log.Print(0,"DDIHOOK: Version is under 2000.");		
		}
	}else if(info.dwPlatformId==VER_PLATFORM_WIN32_WINDOWS){	
		if(info.dwMinorVersion >= 0){ 
			log.Print(0,"DDIHOOK: Version is over 95.");		
			
			
			
			strDLL=_T("EP_NSB4.DLL");
			

			if(_snprintf(cDLL+ixDir,_MAX_PATH-ixDir,"\\%s",strDLL) < 0)
			{
				
				log.Print(0,"ERROR: DDIHook dll's path exceeds _MAX_PATH = %d",_MAX_PATH);
				return FALSE;
			}

			m_bIsWin9x = TRUE;
		} else {
			log.Print(0,"DDIHOOK: Version is under 95.");		
		}
	} else {
		log.Print(0,"DDIHOOK: Version is unknown.");			
	}

	if(!strDLL){
		log.Print(0,"DDIHOOK: ERROR strDLL NULL.");
		return FALSE;
	}

	
	
	if(!IsWin9x())
	{
		if(!CheckDriverFiles())
			return FALSE;
	}
	

	
	
	

	m_hLibModule = (HMODULE)LoadLibrary(cDLL);

	if(m_hLibModule)
	{
		m_pfnInitialize  = (PROC_Initialize) GetProcAddress(m_hLibModule,_T("Initialize"));
		m_pfnFinalize    = (PROC_Finalize)   GetProcAddress(m_hLibModule,_T("Finalize"));
		m_pfnReadBuffer  = (PROC_ReadBuffer) GetProcAddress(m_hLibModule,_T("ReadBuffer"));
		m_pfnPeekBuffer  = (PROC_PeekBuffer) GetProcAddress(m_hLibModule,_T("PeekBuffer"));
		m_pfnMovePointer = (PROC_MovePointer)GetProcAddress(m_hLibModule,_T("MoveReadPointer"));
		m_pfnReSetOverFlow = (PROC_ReSetOverFlow)GetProcAddress(m_hLibModule,_T("ReSetOverFlow"));


		log.Print(0, "m_pfnInitialize %lu",(unsigned long)m_pfnInitialize);		
		log.Print(0, "m_pfnFinalize %lu",(unsigned long)m_pfnFinalize);			
		log.Print(0, "m_pfnReadBuffer %lu",(unsigned long)m_pfnReadBuffer);		
		log.Print(0, "m_pfnPeekBuffer %lu",(unsigned long)m_pfnPeekBuffer);		
		log.Print(0, "m_pfnMovePointer %lu",(unsigned long)m_pfnMovePointer);		
		log.Print(0, "m_pfnReSetOverFlow %lu",(unsigned long)m_pfnReSetOverFlow);	

		
		if(m_pfnInitialize		 == NULL		
		   || m_pfnFinalize		 == NULL		
		   || m_pfnReadBuffer	 == NULL		
		   || m_pfnPeekBuffer	 == NULL   
		   || m_pfnMovePointer	 == NULL
		   || m_pfnReSetOverFlow == NULL
		   ){
			log.Print(0,"DDIHOOK: ERROR one of the dll functions is NULL.");
			return FALSE;
		}
		else
			return TRUE;
	}
	log.Print(0,"DDIHOOK: ERROR LoadLibrary() failed.");
	return FALSE;
}

BOOL DDIHookClass::FreeLib()
{	
	
	BOOL ret = FALSE;						

	m_pfnInitialize		= NULL;
	m_pfnFinalize		= NULL;
	m_pfnReadBuffer		= NULL;
	m_pfnWriteBuffer	= NULL;

	
	m_pfnMovePointer	= NULL;
	m_pfnReSetOverFlow	= NULL;
	m_pfnPeekBuffer     = NULL;
	

	if(m_hLibModule){
		
		ret = FreeLibrary(m_hLibModule);	
		m_hLibModule =NULL;
	}

	
	return ret;								
}


BOOL DDIHookClass::call_Initialize()
{
	if(!m_hLibModule || !m_pfnInitialize)
	{
		return FALSE;
	}
	return m_pfnInitialize();
}

BOOL DDIHookClass::call_Finalize()
{
	if(!m_hLibModule || !m_pfnFinalize)
	{
		return FALSE;
	}
	return m_pfnFinalize();
}

int DDIHookClass::call_ReadBuffer(char* ptr,int size)
{
	if(!m_hLibModule || !m_pfnReadBuffer)
	{
		return 0;
	}
	return m_pfnReadBuffer(ptr,size);
}

int DDIHookClass::call_PeekBuffer(char* ptr,int size)
{
	if(!m_hLibModule || !m_pfnPeekBuffer)
	{
		return 0;
	}
	return m_pfnPeekBuffer(ptr,size);
}

int DDIHookClass::call_WriteBuffer(char* ptr,int size)
{
	if(!m_hLibModule || !m_pfnWriteBuffer)
	{
		return 0;
	}
	return m_pfnWriteBuffer(ptr,size);
}


int DDIHookClass::call_MovePointer(int size)
{  
	if(!m_hLibModule || !m_pfnMovePointer){
		return 0;
	}
	return m_pfnMovePointer(size);
}

BOOL DDIHookClass::call_ReSetOverFlow()
{
	if(!m_hLibModule || !m_pfnReSetOverFlow)
	{
		return 0;
	}
	return m_pfnReSetOverFlow();
}

BOOL DDIHookClass::call_SetDCHandle(unsigned long dcHandle)
{
	if(!m_hLibModule || !m_pfnSetDCHandle)
	{
		return 0;
	}
	return m_pfnSetDCHandle(dcHandle);
}

int DDIHookClass::ReadRect(RECT *rect)
{

	CONTAINER container;
	container.rect.left = 0;
	container.rect.top = 0;
	container.rect.right = 0;
	container.rect.bottom = 0;

	int size = call_ReadBuffer((char*)&container,sizeof(CONTAINER));

	
	
	

	*rect = container.rect;
	return size;
}


BOOL
DDIHookClass::CheckDriverFiles()
{
	HANDLE file = NULL;
	char driverpath[_MAX_PATH];
	char filepath[_MAX_PATH];
	
	DWORD length = GetSystemDirectory(driverpath,_MAX_PATH);
	log.Print(0,"driver path = %s",driverpath);
	
	if(length > 4)
	{
		if(_snprintf(driverpath,_MAX_PATH,"%s\\",driverpath) < 0)
		{
			
			log.Print(0,"ERROR: DDIHook dll's path exceeds _MAX_PATH = %d",_MAX_PATH);
			return FALSE;
		}
	}
	
	
	if(_snprintf(filepath,_MAX_PATH,"%sEP_NSDG.dll",driverpath) < 0)
	{
		
		log.Print(0,"ERROR: DDIHook dll's path exceeds _MAX_PATH = %d",_MAX_PATH);
		return FALSE;
	}

	file = CreateFile(filepath,0,0,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
		
	if(file == INVALID_HANDLE_VALUE)
	{
		log.Print(0,"ERROR: EP_NSDG.dll missing from %s",driverpath);
		return FALSE;
	}
	else
	{
		if(CloseHandle(file))
			log.Print(0,"CloseHandle  EP_NSDG.dll successful.");
		file = NULL;
	}

	
	if(_snprintf(filepath,_MAX_PATH,"%sdrivers\\EP_NSDG.sys",driverpath) < 0)
	{
		
		log.Print(0,"ERROR: DDIHook dll's path exceeds _MAX_PATH = %d",_MAX_PATH);
		return FALSE;
	}

	file = CreateFile(filepath,0,0,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
		
	if(file == INVALID_HANDLE_VALUE)
	{
		log.Print(0,"ERROR: EP_NSDG.sys missing from %s",driverpath);
		return FALSE;
	}
	else
	{
		if(CloseHandle(file))
			log.Print(0,"CloseHandle EP_NSDG.sys successful.");
		file = NULL;
	}

	
	if(_snprintf(filepath,_MAX_PATH,"%sdrivers\\EP_NSDK.sys",driverpath) < 0)
	{
		
		log.Print(0,"ERROR: DDIHook dll's path exceeds _MAX_PATH = %d",_MAX_PATH);
		return FALSE;
	}

	file = CreateFile(filepath,0,0,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
		
	if(file == INVALID_HANDLE_VALUE)
	{
		log.Print(0,"ERROR: Genport.sys missing from %s",driverpath);
		return FALSE;
	}
	else
	{
		if(CloseHandle(file))
			log.Print(0,"CloseHandle EP_NSDK.sys successful.");
		file = NULL;
	}

	return TRUE;
}