/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
#include "stdhdrs.h"
#include "VNeSThread.h"



#include "VSocket.h"
#include "VNeSConst.h"

#include "vncInstHandler.h"
#include "vncDesktop.h"

#include "VNeS.h"

LRESULT CALLBACK DesktopWndProc (HWND, UINT, WPARAM, LPARAM);




DWORD WINAPI 
MainThreadProc(LPVOID lParam)
{
	log.Print(0,"THREAD: MainThreadProc() called");

	DWORD	dwTimeAdj = 0;
	DWORD	dwTimeInc = 0;
	BOOL	bTimeAdjDisabled = false;

	if(GetSystemTimeAdjustment(&dwTimeAdj,&dwTimeInc,&bTimeAdjDisabled)!=0)
	{
		timelog.Print(0,"SYSTEM TIME settings: TimeAdj = %lu Time Inc = %lu bDisabled = %d"
				,dwTimeAdj,dwTimeInc,bTimeAdjDisabled);
	}
	else
	{
		timelog.Print(0,"GetSystemTimeAdjustment() failed. %lu",GetLastError());
	}


	
	gdwAppStartTime = GetTickCount();

	
	QueryPerformanceFrequency(&lpFreq); 
	

	HWND         hwnd ; 
    WNDCLASS     wndclass ;

    wndclass.style         = CS_HREDRAW | CS_VREDRAW ;
    wndclass.lpfnWndProc   = DesktopWndProc ;
    wndclass.cbClsExtra    = 0 ;
    wndclass.cbWndExtra    = 0 ;
    wndclass.hInstance     = ghAppInstance ;
    wndclass.hIcon         = LoadIcon (NULL, IDI_APPLICATION) ;
    wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW);
    wndclass.hbrBackground = (HBRUSH) GetStockObject (WHITE_BRUSH) ;
    wndclass.lpszMenuName  = (const char *) NULL ;
    wndclass.lpszClassName = gszAppName ;

    if (!RegisterClass(&wndclass))
    {
		int err = GetLastError();
		log.Print(0,"ERROR: Desktop RegisterClassEx() Failed : %d",err);

		char  c[1024];
		FormatMessage(
			FORMAT_MESSAGE_FROM_SYSTEM|
			FORMAT_MESSAGE_IGNORE_INSERTS,
			NULL,
			err,
			MAKELANGID(LANG_NEUTRAL,SUBLANG_DEFAULT),
			(LPTSTR)c,
			1023,NULL
			);

		log.Print(0,"ERROR: Desktop RegisterClassEx() %s",c);
		
		
		
        return -1 ;
    }
     
	hwnd = CreateWindow (gszAppName,                  
                         TEXT ("VNeS RD"),			 
                         WS_OVERLAPPEDWINDOW,        
                         CW_USEDEFAULT,              
                         CW_USEDEFAULT,              
                         CW_USEDEFAULT,              
                         CW_USEDEFAULT,              
                         NULL,                       
                         NULL,                       
                         ghAppInstance,              
                         NULL) ;                     
     

	if(hwnd == NULL)
	{
		int err = GetLastError();
		log.Print(0,"ERROR: Desktop CreateWindow() Failed : %d",err);
		
		
		
		
        return -1 ;
		
	}
  
	
	VSocketSystem socksys;
	if (!socksys.Initialised())
	{
		log.Print(0,"ERROR: Windows Sockets Initialization Failed");
		
		
		
		return -1;
	}

	log.Print(LL_STATE, VNCLOG("INIT: Sockets initialised\n"));

	
	SetProcessShutdownParameters(0x100, 0);

	
	vncInstHandler instancehan;
	if (!instancehan.Init())
	{
		
		log.Print(0,"ERROR: Another Instance of VNeS is already running");
		
		
		
		return -1;
	}

	
	
	
	if(!gZlibLib.LoadLib(gsAppPath))
	{
		log.Print(0,"ERROR: Zlib LoadLib() Failed");
		
		
		
		return -1;
	}
	
	

	
	gpDesktop = new vncDesktop(hwnd,gsAppPath);
	
	

	if(gpDesktop==NULL){
		log.Print(0,"ERROR: Cannot create Desktop");
		
		
		
		return -1;
	}

	
	if(!gpDesktop->SetWindowPointer())
	{
		if(gpDesktop!=NULL){
			delete gpDesktop;
			gpDesktop = NULL;
		}
		
		
		
		return -1;
	}
	

	
	if(!gpDesktop->LoadDDIHookLibrary())
	{
		log.Print(0,"ERROR: DDIHook LoadLib() Failed");

		if(gpDesktop!=NULL){
			delete gpDesktop;
			gpDesktop = NULL;
		}
		
		
		
		return -1;
	}
	else{
		log.Print(0,"INIT: DDIHook LoadLib() successful");
	}
	

	
	
	if(!SetEvent(ghInitWait))
	{
		
		log.Print(0,"ERROR: SetEvent(ghInitWait) failed.");
			 
		if(gpDesktop != NULL)
		{
			delete gpDesktop;
			gpDesktop = NULL;
		}

		
		
		
		return -1;
	}

	  MSG msg;	
	 
	 
	 int bRet;
	 
	 while((WaitForSingleObject(gpMainThread->m_hEventKill,0)== WAIT_TIMEOUT) &&
		   ((bRet = GetMessage(&msg, NULL, 0, 0 )) != 0))
	 {	
		 if (bRet == -1)
		 {
			 
			 log.Print(0,"ERROR: GetMessage() failed.");
			 
			 if(gpDesktop != NULL)
			 {
				 delete gpDesktop;
				 gpDesktop = NULL;
			 }

			 
			 PostMessage(ghEMPWnd,EV_COMMAND,vnesError,VERROR_AppWindow);
			 
			 return -1;
		 }
		 else
		 {
			 TranslateMessage(&msg); 
			 DispatchMessage(&msg); 
		 }
	 }
	 
	 log.Print(0,"MAIN THREAD: GetMessage() exited.");
	 
	 if(gpDesktop != NULL)
	 {
		 log.Print(0,"MAIN THREAD: Calling delete pDesktop.");
		 delete gpDesktop;
		 gpDesktop = NULL;
	 }

	
	
	if(!gZlibLib.FreeLib())
		log.Print(0,"ERROR: Zlib FreeLib() Failed");
	

	
	

	log.Print(0,"THREAD: MainThreadProc() end");

	return msg.wParam;
}
