/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/




#include "stdhdrs.h"
#include "VNeS.h"
#include "VNeSThread.h"
#include "VNeSConst.h"
#include "VNeSUtil.h"


HINSTANCE	ghAppInstance			= NULL;
const char	*gszAppName				= "VNeS Remote Display Server";
char		gsAppPath[_MAX_PATH];
CVNeSThread	*gpMainThread			= NULL;
vncDesktop	*gpDesktop				= NULL;
HWND		ghEMPWnd				= NULL;
HANDLE		ghInitWait				= NULL;


DWORD		gdwAppStartTime			= 0;


CZlibLib		gZlibLib;









int WINAPI DllMain (HINSTANCE hInstance, DWORD fdwReason, PVOID pvReserved)
{	
	if(fdwReason == DLL_PROCESS_ATTACH )
	{
		
		
		
		DWORD dwLogFile = 0;
		GetVNeSAppDir(gsAppPath,_MAX_PATH);

		
		LoadVNeSPathName(gsAppPath,dwLogFile);
		
		
		
	
		
		ghAppInstance = hInstance;

		log.Print(0,"DLL: PROCESS ATTACHED ");
	}
	else if(fdwReason == DLL_PROCESS_DETACH)
	{
		
		
		
		
		
		
		
		
		
		

		if(pvReserved == NULL)
			log.Print(0,"FreeLibrary() exit.");
		else
			log.Print(0,"WARNING: Process exit.");
		
		if(ghInitWait != NULL)
		{
			CloseHandle(ghInitWait);
			ghInitWait = NULL;

			log.Print(0,"DLL: ghInitWait Event Handle closed at FreeLib");
		}

		if(gpMainThread != NULL)
		{
			delete gpMainThread;
			gpMainThread = NULL;
			log.Print(0,"DLL: gpMainThread deleted at Freelib");
		}

		log.Print(0,"DLL: PROCESS DETACHED.");
	}
	return TRUE ;
}



VEXPORT BOOL StartVNeS(HWND hEMPWnd)
{	
	log.Print(0,"DLL: StartVNeS() called.");

	if(hEMPWnd == NULL || gpMainThread != NULL)
		return FALSE;

	
	
	

	
	
	
	
	
	

	
	ghEMPWnd = hEMPWnd;

	
	
	
	ghInitWait = CreateEvent(NULL, TRUE, FALSE, NULL);
	if(ghInitWait == NULL)
		return FALSE;
	

	
	gpMainThread = new CVNeSThread();
	if(gpMainThread == NULL)
		return FALSE;

	
	if(!gpMainThread->CreateVNeSThread(MainThreadProc,NULL,0))
	{	
		delete gpMainThread;
		gpMainThread = NULL;
		log.Print(0,"THREAD ERROR: MainThread CreateVNeSThread() failed");
		return FALSE;
	}

	if(WaitForSingleObject(ghInitWait,VNES_INITWAIT_TIMEOUT) == WAIT_TIMEOUT)
	{
		log.Print(0,"DLL ERROR: InitWait Timed Out.");
		return FALSE;
	}

	return TRUE;
}



VEXPORT BOOL StopVNeS()
{	
	log.Print(0,"DLL: StopVNeS() called.");

	
	if(gpMainThread != NULL)
	{ 
		if(gpDesktop != NULL)
		{	
			gpDesktop->PostCloseMessage();
			log.Print(0,"WM_CLOSE message posted");
		}

		gpMainThread->KillThread();

		
		delete gpMainThread;
		gpMainThread = NULL;
		log.Print(0,"DLL: gpMainThread deleted.");
		
	}

	
	if(ghInitWait != NULL)
	{
		CloseHandle(ghInitWait);
		ghInitWait = NULL;

		log.Print(0,"DLL: ghInitWait Event Handle closed.");
	}
	

	return TRUE;
}


VEXPORT BOOL Connect(CEVStream &ConnInfo)
{
	if(gpMainThread == NULL || gpDesktop == NULL)
		return FALSE;

	
	
	
	BOOL ret = gpDesktop->OnEVConnectRequest(ConnInfo);

	if(!ret)
		Disconnect();

	return ret;
	
}


VEXPORT BOOL Disconnect()
{
	if(gpMainThread == NULL || gpDesktop == NULL)
		return FALSE;

	return gpDesktop->OnEVDisConnectRequest();
}


VEXPORT BOOL IsSending(DWORD &dwBytes)
{
	if(gpMainThread == NULL || gpDesktop == NULL)
		return FALSE;

	return gpDesktop->OnEVSentDataSizeRequest(dwBytes);
}


VEXPORT BOOL SetSendImage(BOOL bPause)
{
	if(gpMainThread == NULL || gpDesktop == NULL)
		return FALSE;
	
	BOOL ret = FALSE;

	if(bPause)
		ret = gpDesktop->OnEVImageSendStop();
	else
		ret = gpDesktop->OnEVImageSendStart();

	return ret;
}


VEXPORT BOOL ReSendFullScrRect(DWORD dwID)
{
	if(gpMainThread == NULL || gpDesktop == NULL)
		return FALSE;

	if(gpDesktop == NULL)
		return FALSE;

	return gpDesktop->OnEVReSendUDPFullScrRect(dwID);
}


VEXPORT BOOL GetScreenFormat(rfbServerInitMsg &format)
{
	log.Print(0,"GETSCREENFORMAT called.");

	ZeroMemory(&format,sizeof(format));

	VNESBMINFO bminfo;

	if(!InitializeBitmap(format, bminfo))
		return FALSE;

	
	

	if(!SetFormat(format, bminfo))
		return FALSE;

	
	

	log.Print(0,"SERVER FORMAT TO EMP: BPP = %d  DEPTH = %d \r\n"
			  "BIG E = %d  TRUECOLOR = %d \r\n"
			  "RGB MAX = 0x%8x  0x%8x  0x%8x \r\n"
			  "RGB SHIFTS = %d  %d  %d"
			  ,format.format.bitsPerPixel,format.format.depth
			  ,format.format.bigEndian,format.format.trueColour
			  ,format.format.redMax
			  ,format.format.greenMax,format.format.blueMax
			  ,format.format.redShift
			  ,format.format.greenShift,format.format.blueShift
			  );

	log.Print(0,"SERVER: width = %lu height %lu"
			   ,format.framebufferWidth,format.framebufferHeight);

	return TRUE;
}



