/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


#include "VNeSConst.h"
#include "VNeSBuff.h"
#include "VNeS.h"






CVNeSBufferList::CVNeSBufferList()
{	
	m_dwListSizeLimit = VNES_DEFAULTMAXBUFFERSIZE;
	
	ResetListSize();
	
	ResetRingPointers();
	
	m_dwScrBufferSize = 0;
}

CVNeSBufferList::~CVNeSBufferList()
{
	Clear();
}


	
void CVNeSBufferList::Clear()
{
	
	vBufList::iterator entry;
	for (entry = m_BufferList.begin(); entry != m_BufferList.end(); entry++)
	{
		
		if((*entry).pScreenBuff != NULL)
		{
			delete [] (*entry).pScreenBuff;
			(*entry).pScreenBuff = NULL;

			bufflog.Print(0,"BUFF LIST: buffer deleted.");
		}

		(*entry).dwScrBuffsize = 0;
	}

	
	m_BufferList.clear();
	ResetListSize();
	ResetRingPointers();

	
}



int
CVNeSBufferList::Add(BYTE *pScrBuff,DWORD dwBufSize)
{
	

	if(m_dwListSize >= m_dwListSizeLimit)
	{
		log.Print(0,"BUFFER LIST: List FULL, Ignoring new entry");
		log.Print(0,"BUFF LIST OVERFLOW!!!");
		
		
		
		
	    
		return VBL_OVERFLOWERROR;
	}

	vBufList::iterator input;
	input = m_RingHead;
	
	
	
	
	
	
	
	if(dwBufSize > m_dwScrBufferSize)
	{
		log.Print(0,"RING BUFF: ERROR! screen buff greater than buff size.");
		
		
		
		
		

		return VBL_BUFFERSIZEERROR;
	}

	
	
	
	memcpy_ex((*input).pScreenBuff,pScrBuff,dwBufSize);
	

	
	(*input).dwScrBuffsize = dwBufSize;

	
	m_dwListSize++;
	IncrementRingHead();

	

	return 1;
}



int	
CVNeSBufferList::Add(VNES_SCRBUFFER &entry)
{
	if(m_dwListSize >= m_dwListSizeLimit)
	{
		log.Print(0,"BUFFER LIST: List FULL, Ignoring new entry2");
		log.Print(0,"BUFF LIST OVERFLOW!!!");
		
		
		
		return VBL_OVERFLOWERROR;
	}

	
	vBufList::iterator input;
	input = m_RingHead;
	
	
	
	
	
	
	

	
	if(entry.dwScrBuffsize > m_dwScrBufferSize)
	{
		log.Print(0,"RING BUFF: ERROR! screen buff greater than buff size.");
		
		
		
		
		return VBL_BUFFERSIZEERROR;
	}

	
	
	
	memcpy_ex((*input).pScreenBuff,entry.pScreenBuff,entry.dwScrBuffsize);
	

	(*input).dwScrBuffsize = entry.dwScrBuffsize;

	m_dwListSize++;
	IncrementRingHead();

	

	return 1;
}



BOOL 
CVNeSBufferList::GetFront(VNES_SCRBUFFER &scrBuffer)
{
	
	BOOL ret = false;

	if(!IsEmpty())
	{
		vBufList::iterator output;
		output = m_RingTail;
		
		
		
		
		
		
		
		
		
		memcpy_ex(scrBuffer.pScreenBuff,(*output).pScreenBuff,(*output).dwScrBuffsize);
		
		scrBuffer.dwScrBuffsize = (*output).dwScrBuffsize;  

		(*output).dwScrBuffsize = 0;

		IncrementRingTail();
		m_dwListSize--;

		ret = true;

	}
	else
	{
		log.Print(0,"WARNING: BUFFER list was empty. GetFront()");
	}

	return ret;
}



BOOL	
CVNeSBufferList::InitRingBuffer(DWORD dwBufferSize,DWORD dwListSizeLimit)
{
	
	
	
	
	m_dwScrBufferSize = dwBufferSize;

	
	m_dwListSizeLimit = dwListSizeLimit;

	
	m_BufferList.resize(m_dwListSizeLimit);

	
	

	vBufList::iterator entry;
	for (entry = m_BufferList.begin(); entry != m_BufferList.end(); entry++)
	{
		
		(*entry).pScreenBuff = NULL;
		(*entry).dwScrBuffsize = 0;
	}
	
	for (entry = m_BufferList.begin(); entry != m_BufferList.end(); entry++)
	{
		
		(*entry).pScreenBuff = new BYTE[m_dwScrBufferSize];

		if((*entry).pScreenBuff == NULL)
		{
			log.Print(0,"RING BUFF: ERROR not enough memory.");
			return FALSE;
		}

		ZeroMemory((*entry).pScreenBuff,m_dwScrBufferSize);
	}
	
	
	ResetRingPointers();

	log.Print(0,"RING BUFF: Initialized successfully. Size: %lu.",m_dwListSizeLimit);
	return TRUE;
}


BOOL	
CVNeSBufferList::IsEmpty()
{ 
	return (m_dwListSize == 0); 
}


BOOL	
CVNeSBufferList::IsFull()  
{ 
	return (m_dwListSize == m_dwListSizeLimit); 
}
