/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
#include "VNeSThread.h"
#include "rfb.h"


#include "VNeSCommon.h"

#include "VNeSConst.h"



#include <assert.h>





CClientThread::CClientThread() 
	: CVNeSThread()
{
	log.Print(0,"THREAD: CClientThread constructor.");
	
	
	m_vrRegionToSend.Clear();
	m_vrFullScreen.Clear();

	
	m_pBuffer		= NULL;
	

	
	SetRectEmpty(&m_fullscreen);

	
	
	
	

	
	
	
	
	

	
	m_bGrabStyle = VNES_DEF_GRABSTYLE;
	m_bSendStyle = VNES_DEF_SENDSTYLE;

	
	m_bSendFullScr = TRUE;
	

	
	SetRectEmpty(&m_rctMouse);
	m_bMouseMoved = FALSE;
	

	
	m_dwHookDetailDelay = VNES_DEF_HOOK_DETAIL_DELAY;
	m_dwDetailArea		= VNES_DEF_DETAIL_AREA; 
	m_dwDetailWidth		= VNES_DEF_DETAIL_WIDTH;
	m_dwDetailHeight	= VNES_DEF_DETAIL_HEIGHT;
	m_dwDetailLimit		= VNES_DEF_DETAIL_LIMIT;

	m_bCheckDetailArea	= VNES_DEF_CHECK_DETAIL_AREA;
	

	
	m_pSendBuffer		= NULL;
	m_dwSendBuffSize	= 0;
	m_dwSendBuffptr		= 0;
	m_bNewUpdate		= false;
	

	
	
	dwTotTime	= 0;
	dwLoopCount = 0;
	
}

CClientThread::~CClientThread()
{
	log.Print(0,"THREAD: CClientThread destructor.");

	
	if(m_pSendBuffer != NULL)
	{
		delete [] m_pSendBuffer;
		m_pSendBuffer = NULL;
		m_dwSendBuffSize	= 0;
		m_dwSendBuffptr		= 0;
	}
	

	
	
	if(dwLoopCount != 0)
		ratelog.Print(-1,"Totaltime = %d \r\n"
			"Loops = %d \r\n"
			"average sending time = %.3f",
			dwTotTime,dwLoopCount,
			(float)(dwTotTime/dwLoopCount)); 
	else
		ratelog.Print(-1,"average sending time = 0"); 
	
}


BOOL
CClientThread::CopyUpdateRegion(vncRegion *captured)
{
	if(!captured->IsEmpty())
	{
		
		m_vrRegionToSend.Combine(*captured);

		
		captured->Clear();
	}
	return TRUE;
}


BOOL
CClientThread::Init(vncBuffer *buffer)
{
	if(buffer == NULL)
		return FALSE;

	m_pBuffer = buffer;

	m_fullscreen = m_pBuffer->GetFullScreenRect();
	
	log.Print(0,"FULLSCREEN: %d %d %d %d"
			  ,m_fullscreen.left,m_fullscreen.top,
			  m_fullscreen.right,m_fullscreen.bottom);

	return TRUE;
}


BOOL		
CClientThread::GrabRect(RECT &rect)
{
	
	return m_pBuffer->GrabRect(rect);
	
	
}


BOOL
CClientThread::GrabRegionBox(vncRegion &region)
{
	RECT grabRect;
	
	
	m_lpStart.QuadPart = 0;
	m_lpStop.QuadPart = 0;
	
	timelog.Print(0,"CAPTURE: ONE RECT GRAB Start = %lu ms"
				,GetTickCount()-gdwAppStartTime);

	QueryPerformanceCounter(&m_lpStart);
	
	
	

	if(!region.GetRegionBox(grabRect))
	{
		
		
		log.Print(0,"WARNING: GetRegionBox() Failed at GrabRegionBox()");
		return TRUE;
		
	}

	
	
	if(!GrabRect(grabRect))
	{
		log.Print(0,"ERROR: GrabRect() Failed at GrabRegionBox()");
		return FALSE;
	}
	

	
	QueryPerformanceCounter(&m_lpStop);

	timelog.Print(0,"CAPTURE: ONE RECT GRAB Stop = %lu ms"
				,GetTickCount()-gdwAppStartTime);

	timelog.Print(0,"CAPTURE: ONE RECT GRAB: time = %.2f us"
		 ,(float)(m_lpStop.QuadPart-m_lpStart.QuadPart)/lpFreq.QuadPart*1000000);
	
	grablog.Print(0,"GRAB BOX: Rect %d %d %d %d  : %d x %d"
			,grabRect.left,grabRect.top,grabRect.right,grabRect.bottom
			,grabRect.right-grabRect.left,grabRect.bottom-grabRect.top);
	

	return TRUE;
}

BOOL
CClientThread::GrabRegion(vncRegion &region)
{
	
	rectlist::iterator i;
	rectlist rects;
	RECT grabRect;
	
	
	m_lpStart.QuadPart = 0;
	m_lpStop.QuadPart = 0;
	
	rects.clear();

	
	if (!region.Rectangles(rects))
	{
		log.Print(0,"ERROR: reigion.Rectangles() failed at GrabRegion().");
		return FALSE;
	}

	
	if(rects.empty())
		return TRUE;
	

	
	SetRectEmpty(&grabRect);

	
	
	
	
	
	
	timelog.Print(0,"CAPTURE: MANY GRAB RECT Start = %lu ms"
			,GetTickCount()-gdwAppStartTime);

	QueryPerformanceCounter(&m_lpStart);
	

	for (i = rects.begin(); i != rects.end(); i++)
	{
		RECT current = *i;
		
		#ifdef _DEBUG
		grablog.Print(0,"SORT: Rect %d %d %d %d  : %d x %d"
				  ,current.left,current.top,current.right,current.bottom
				  ,current.right-current.left,current.bottom-current.top);
		#endif

		
		if (current.top > grabRect.bottom)
		{
			
			if (!IsRectEmpty(&grabRect))
			{
				grablog.Print(0,"GRAB: Rect %d %d %d %d  : %d x %d"
				  ,grabRect.left,grabRect.top,grabRect.right,grabRect.bottom
				  ,grabRect.right-grabRect.left,grabRect.bottom-grabRect.top);

				if(!GrabRect(grabRect))
				{
					log.Print(0,"ERROR: GrabRect() failed at GrabRegion().");
					return FALSE;
				}

				grabRect = current;
			}else{
				grabRect = current;
			}
		} else {
			
			UnionRect(&grabRect, &current, &grabRect);
			
			#ifdef _DEBUG
			grablog.Print(0,"UNION: Rect %d %d %d %d  : %d x %d"
				  ,grabRect.left,grabRect.top,grabRect.right,grabRect.bottom
				  ,grabRect.right-grabRect.left,grabRect.bottom-grabRect.top);
			#endif

		}
	}

	
	if (!IsRectEmpty(&grabRect)){
		
		grablog.Print(0,"GRAB: Last Rect %d %d %d %d  : %d x %d"
				  ,grabRect.left,grabRect.top,grabRect.right,grabRect.bottom
				  ,grabRect.right-grabRect.left,grabRect.bottom-grabRect.top);

		if(!GrabRect(grabRect))
		{
			log.Print(0,"ERROR: GrabRect() failed at GrabRegion().");
			return FALSE;
		}
	}
	
	QueryPerformanceCounter(&m_lpStop);

	timelog.Print(0,"CAPTURE: MANY GRAB RECT Stop = %lu ms"
				,GetTickCount()-gdwAppStartTime);

	timelog.Print(0,"CAPTURE: MANY RECT GRAB: time = %.2f us"
		 ,(float)(m_lpStop.QuadPart-m_lpStart.QuadPart)/lpFreq.QuadPart*1000000);
	

	
	rects.clear();

	return TRUE;
}


BOOL
CClientThread::SendUpdate(RECT &mouseRect, BOOL bJpeg)
{
	
	CheckMouseUpdate(mouseRect,m_vrRegionToSend);
	

	
	
	if(m_vrRegionToSend.IsEmpty() && !m_vrResendRegion.IsEmpty())
	{
		rectlist rects;
		rects.clear();
		
		m_vrResendRegion.Rectangles(rects);
		RECT rect;
		DWORD dwTotalArea = 0;	
		DWORD dwOneArea;		
		while(!rects.empty())
		{
			rect = rects.front();
			
			
			dwOneArea = (rect.right-rect.left) * (rect.bottom-rect.top);
			
			
			if(dwTotalArea + dwOneArea > m_dwResendZlibAreaLimit){
				
				DWORD dwMargin = m_dwResendZlibAreaLimit - dwTotalArea;
				rect.bottom = rect.top + (dwMargin/rect.right)-1;
				
				m_vrResendRegion.SubtractRect(rect);
				
				m_vrRegionToSend.AddRect(rect);
				break;
			} else {
				dwTotalArea += dwOneArea;
				
				m_vrResendRegion.SubtractRect(rect);
				
				m_vrRegionToSend.AddRect(rect);
				rects.pop_front();
			}
		}
		bJpeg = FALSE;
	}
	

	
	
	
	if(!m_vrRegionToSend.IsEmpty())
	
	{
		rectlist toBeSentList;
		toBeSentList.clear();

		
		CheckMouseRegion(m_vrRegionToSend);
		


		
		BOOL bGrabStyle = IsGrabStyle();
		BOOL bSendStyle = IsSendStyle();

		if((!bGrabStyle || !bSendStyle) && IsCheckDetailArea())
		{
			
			
			
			
			
			rectlist rectCheckList;
			rectCheckList.clear();

			
			m_vrRegionToSend.Rectangles(rectCheckList);
		
			int nSmallStuffCount = 0 ; 

			rectlist::iterator i;
			for (i=rectCheckList.begin(); i != rectCheckList.end(); i++)
			{
				int width	= i->right - i->left ;
				int height	= i->bottom - i->top ; 

				if ((width  < GetDetailWidth()) || (height < GetDetailHeight())|| 
					(width * height < GetDetailArea()))  
				{
					nSmallStuffCount++ ; 
				}
			}

			if (nSmallStuffCount > GetDetailLimit()) 
			{
				rectlog.Print(0,"DETAIL CLIENT TREAD: SmallLimit %d < nSmallStuffCount Setting styles to true"
						,GetDetailLimit(),nSmallStuffCount);

				bGrabStyle = TRUE; 
				bSendStyle = TRUE; 
			}

			
			rectCheckList.clear();
		}
		
		
		if(bGrabStyle)
		{
			
			if(!GrabRegionBox(m_vrRegionToSend))
			{
				log.Print(0,"ERROR: GrabRegionBox FAILED.");
				return FALSE;
			}
		}
		else
		{
			
			
			if(!GrabRegion(m_vrRegionToSend))
			{
				log.Print(0,"ERROR: GrabRegion FAILED.");
				return FALSE;
			}
		}

		
		if(m_bMouseMoved)
		{
			
			
			m_rctMouse = m_pBuffer->GrabMouse();
			
			
			m_vrRegionToSend.AddRect(m_rctMouse);
			m_bMouseMoved = FALSE;
		}
		

		
		if(bSendStyle)
		{
			
			RECT rect;

			
			if(!m_vrRegionToSend.GetRegionBox(rect))
			{
				log.Print(0,"WARNING: GetRegionBox() failed at SendUpdate()");
			}
			else
			{
				toBeSentList.push_front(rect);
			}
			
		}
		else
		{
			
			m_vrRegionToSend.Rectangles(toBeSentList);
			
			
			
			
			
			
		}
		

		
		m_vrRegionToSend.Clear();

		if(toBeSentList.empty())
			return TRUE;

		if(!SendUpdateTCP(toBeSentList,bJpeg))
			return FALSE;

		toBeSentList.clear();
	}
	return TRUE;
}

BOOL
CClientThread::SendRectangles(rectlist &rects, BOOL bJpeg, DWORD dwFullScrStartID)
{
	RECT rect;

	
	

	
	DWORD dwStart = GetTickCount(); 
	bool bRectAru = false;
	

	
	
	DWORD dwFullScrID = dwFullScrStartID;
	

	while(!rects.empty())
	{
		rect = rects.front();

		
		
		
		
		if (!SendRectangle(rect, bJpeg,dwFullScrID)) 
		{
			
			rects.clear();
			return FALSE;
		}

		if(dwFullScrID != 0)
		{
			
			
			
			
			
			dwFullScrID++;
		}
	
		

		rects.pop_front();

		socklog.Print(0,"RECT SENT: %d, %d, %d, %d  Fullscreen.ID = %lu",
			rect.left,rect.top,rect.right,rect.bottom,dwFullScrID-1); 

		
		bRectAru = true; 
		
	}
	rects.clear();

	DWORD dwEnd = GetTickCount();

	
	
	
	
	
	
	
	
	
	
	
	
	if(bRectAru){ 
		dwTotTime += dwEnd - dwStart;
		dwLoopCount++;
	}
	

	return TRUE;
}


BOOL
CClientThread::SendRectangle(RECT &rect, BOOL bJpeg, DWORD dwFullScrID)
{
	socklog.Print(0,"SENDING: Rect %d %d %d %d  : %d x %d"
				  ,rect.left,rect.top,rect.right,rect.bottom
				  ,rect.right-rect.left,rect.bottom-rect.top);

	
	
	int left,top,right,bottom = 0; 
	left = rect.left;
	top = rect.top;
	right = rect.right;
	bottom = rect.bottom;

	
	
	m_lpStart.QuadPart = 0;
	m_lpStop.QuadPart = 0;
	m_lpBrk1.QuadPart = 0;

	
	UINT tempDataSize = m_pBuffer->GetVNCEncoder()->GetDataSize(); 
	UINT tempEncodedSize = m_pBuffer->GetVNCEncoder()->GetEncodedSize(); 
	UINT tempTransSize = m_pBuffer->GetVNCEncoder()->GetTransmittedSize(); 
	
	timelog.Print(0,"CAPTURE: COMPRESSION Start = %lu ms"
				,GetTickCount()-gdwAppStartTime);

	
	int a = QueryPerformanceCounter(&m_lpStart); 
		
	
	UINT bytes = m_pBuffer->TranslateRect(rect,bJpeg);  

	
	int b = QueryPerformanceCounter(&m_lpBrk1); 

	timelog.Print(0,"CAPTURE: COMPRESSION Stop = %lu ms"
				,GetTickCount()-gdwAppStartTime);

	
	if(m_pBuffer->IsUsedJpeg())
	{
		rectlist rects;
		m_vrResendRegion.Rectangles(rects);
		if(rects.size() < m_dwResendJpegRects){
			m_vrResendRegion.AddRect(rect);
		}
	}
	

	
	BOOL ret = CopyToSendBuff(m_pBuffer->GetClientBuffer(),bytes);
	
	
	
	
	
	
	int c = QueryPerformanceCounter(&m_lpStop);  

	
	UINT newDataSize = m_pBuffer->GetVNCEncoder()->GetDataSize(); 
	UINT newEncodedSize = m_pBuffer->GetVNCEncoder()->GetEncodedSize(); 
	UINT newTransSize = m_pBuffer->GetVNCEncoder()->GetTransmittedSize(); 
	
	
	ratelog.Print(0,"rawBytes = %d \r\n"
				"compression rate = %.2f \r\n"
				"compression time = %.2f us \r\n"
				"CAPTURE: copy to send buffer time BODY = %.2f us "
				"bytes = %lu \r\n"
				"ecncoded bytes = %d  (check: %d )\r\n"
				"error markers: %d %d %d \r\n"
				"rect area = %dx%d = %d \r\n"
				 ,newDataSize-tempDataSize
				 ,(float)(newEncodedSize-tempEncodedSize)/(newDataSize-tempDataSize)*100
				 ,(float)(m_lpBrk1.QuadPart-m_lpStart.QuadPart)/lpFreq.QuadPart*1000000
				 ,(float)(m_lpStop.QuadPart-m_lpBrk1.QuadPart)/lpFreq.QuadPart*1000000
				 ,bytes
				 ,newEncodedSize-tempEncodedSize,newTransSize-tempTransSize
				 ,a,b,c
				 ,(right-left),(bottom-top),(right-left)*(bottom-top)
				 ); 
		
	timelog.Print(0,"rawBytes = %d  after encoding = %d \r\n "
				"compression rate = %.2f \r\n"
				"compression time = %.2f us \r\n" 
				"CAPTURE: copy to send buffer time (CopyToSendBuff) BODY = %.2f us."
				"bytes = %lu \r\n"
				"rect area = %dx%d = %d \r\n"
				 ,newDataSize-tempDataSize,newEncodedSize-tempEncodedSize
				 ,(float)(newEncodedSize-tempEncodedSize)/(newDataSize-tempDataSize)*100
				 ,(float)(m_lpBrk1.QuadPart-m_lpStart.QuadPart)/lpFreq.QuadPart*1000000
				 ,(float)(m_lpStop.QuadPart-m_lpBrk1.QuadPart)/lpFreq.QuadPart*1000000
				 ,bytes
				 ,(right-left),(bottom-top),(right-left)*(bottom-top)
				 ); 

	socklog.Print(0,"Jpeg = %d bytes = %d  ret = %d",bJpeg,bytes,ret);

	return ret;
}


BOOL		
CClientThread::SendRFBMsg(CARD8 type,BYTE *buffer,int buflen,BOOL bHeader)
{
	
	((rfbServerToClientMsg *)buffer)->type = type;

	
	m_lpStart.QuadPart = 0;
	m_lpStop.QuadPart = 0;

	QueryPerformanceCounter(&m_lpStart);
	
	if(!CopyToSendBuff(buffer,buflen))
		return false;

	QueryPerformanceCounter(&m_lpStop);

	timelog.Print(0,"CAPTURE: copy to send buffer time (CopyToSendBuff) RFB HEADER: time = %.2f us.\r\n"
		 ,(float)(m_lpStop.QuadPart-m_lpStart.QuadPart)/lpFreq.QuadPart*1000000);

	

	
	
	
	
	
		
	
	

	return TRUE;
}
	












	
void	
CClientThread::AddRectToFullScr(RECT &rect)
{
	
	{
		CMyMutexLock lock(m_csFullscreenlock);
		m_vrFullScreen.AddRect(rect);
	}
	
	return;
}


BOOL	
CClientThread::IsFullScrRgnEmpty()
{
	BOOL ret = FALSE;

	
	{
		CMyMutexLock lock(m_csFullscreenlock);
		ret = m_vrFullScreen.IsEmpty();
	}

	return ret;
}

	
void	
CClientThread::AddFullScr()
{
	if(IsRectEmpty(&m_fullscreen))
	{
		log.Print(0,"WARNING: Request for fullscreen refresh too early.");
		return;
	}

	
	{
		CMyMutexLock lock(m_csFullscreenlock);
		m_vrFullScreen.AddRect(m_fullscreen);
	}

	return;
}


BOOL	
CClientThread::SendFullScr(RECT &mouseRect)
{
	
	
	
	
	
	if(m_bSendFullScr)
	{
		rectlist toBeSentList;
		toBeSentList.clear();

		
		{
			CMyMutexLock lock(m_csFullscreenlock);

			
			
			
			vncRegion vrTempRegion;
			vrTempRegion.Combine(m_vrFullScreen);
			CheckMouseUpdate(mouseRect,vrTempRegion);
			
			

			if(m_vrFullScreen.IsEmpty())    
			{
				return TRUE;
			}

			
			
			CheckMouseRegion(vrTempRegion);
			
	
			rectlog.Print(0,"FULLSCREEN: Entered SendFullScr()");

			
			if(IsGrabStyle())
			{
				
				
				
				
				
				if(!GrabRegionBox(vrTempRegion))
				{
					return FALSE;
				}
			}
			else
			{
				
				
				
				
				if(!GrabRegion(vrTempRegion))
				
				{
					return FALSE;
				}
			}

			if(m_bMouseMoved)
			{
				
				
				m_rctMouse = m_pBuffer->GrabMouse();
				
				
				m_vrFullScreen.AddRect(mouseRect);
				
				m_vrFullScreen.AddRect(m_rctMouse);
				
				m_bMouseMoved = FALSE;
			}

		
			
			if(IsSendStyle())
			{
				
				RECT rect;

				
				if(!m_vrFullScreen.GetRegionBox(rect))
				{
					log.Print(0,"WARNING: GetRegionBox() failed at SendFullScr()");
				}
				else
				{
					toBeSentList.push_front(rect);
				}
				
			}
			else
			{
				
				m_vrFullScreen.Rectangles(toBeSentList);
			}

			
			m_vrFullScreen.Clear();
		
			if(toBeSentList.empty())
			{
				log.Print(0,"WARNING: TobeSentList empty at SendFullScr()");
				return TRUE;
			}
		}
		

		if(!SendUpdateTCP(toBeSentList,FALSE))
			return FALSE;
	
		toBeSentList.clear();
	}
	return TRUE;
}

BOOL	
CClientThread::SendUpdateTCP(rectlist &listrect,BOOL bJpeg)
{
	
	
	int numrects = 0;
		
	
	rectlist::iterator i;
	for (i=listrect.begin(); i != listrect.end(); i++)
	{
		numrects += m_pBuffer->GetNumCodedRects(*i);
	}

	if (numrects == 0)
		return TRUE;

	
	rfbFramebufferUpdateMsg header;
	ZeroMemory(&header,sizeof(header));

	header.nRects = Swap16IfLE(numrects);
	if (!SendRFBMsg(rfbFramebufferUpdate,(BYTE *) &header
					, sz_rfbFramebufferUpdateMsg,TRUE))
	{
		return FALSE;
	}
	

	ratelog.Print(0,"FrameBuffUpdateMsg Header Sent"); 
		
	
	if(!SendRectangles(listrect,bJpeg))
		return FALSE;

	
	listrect.clear();

	return TRUE;
}


void	
CClientThread::CheckMouseUpdate(RECT &tmpRect,vncRegion &region)
{
	
	
	if(!EqualRect(&tmpRect,&m_rctMouse))
	{
		
		
		region.AddRect(m_rctMouse);
		
		m_bMouseMoved = TRUE;
		rectlog.Print(0,"Mouse Move TRUE from CheckMouseUpdate()");
	}

	return;
}

void	
CClientThread::CheckMouseRegion(vncRegion &region)
{
	RECT updateRect; 

	
	if(!region.GetRegionBox(updateRect))
		return;
	

	if(IntersectRect(&updateRect,&updateRect,&m_rctMouse))
	{
		
		
		
		m_bMouseMoved = TRUE;
		region.AddRect(m_rctMouse);
	}
	return;
}


BOOL	
CClientThread::SetClientFormat(rfbPixelFormat &format)
{
	BOOL ret = FALSE;

	
	
	
	

	if(format.bitsPerPixel == 32)
	{
		
		format.bitsPerPixel		= 32;
		format.depth			= 32;
		format.bigEndian		= 0;	
		format.trueColour		= 1;
		format.redMax			= 0xff;
		format.greenMax			= 0xff;
		format.blueMax			= 0xff;
		format.redShift			= 16;
		format.greenShift		= 8;
		format.blueShift	   	= 0;
		
	}
	else
	{
		
		format.bitsPerPixel		= 16;
		format.depth			= 16;
		format.bigEndian		= 0;	
		format.trueColour		= 1;
		format.redMax			= 0x1f;
		format.greenMax			= 0x3f;
		format.blueMax			= 0x1f;
		format.redShift			= 11;
		format.greenShift		= 5;
		format.blueShift		= 0;
		
	}

	log.Print(0,"CLIENT FORMAT: BPP = %d  DEPTH = %d \r\n"
			  "BIG E = %d  TRUECOLOR = %d \r\n"
			  "RGB MAX = 0x%8x  0x%8x  0x%8x \r\n"
			  "RGB SHIFTS = %d  %d  %d"
			  ,format.bitsPerPixel,format.depth
			  ,format.bigEndian,format.trueColour
			  ,format.redMax
			  ,format.greenMax,format.blueMax
			  ,format.redShift
			  ,format.greenShift,format.blueShift
			  );

	if(m_pBuffer != NULL)
	{
		ret = m_pBuffer->SetClientFormat(format);
	}
	return ret;
}


BOOL	
CClientThread::CopyToSendBuff(BYTE *buff,DWORD dwSize)
{
	timelog.Print(0,"CAPTURE: CopyToSendBuff Start = %lu ms.  Data size = %lu"
					,GetTickCount()-gdwAppStartTime,dwSize);

	if(buff == NULL || dwSize == 0)
		return false;

	DWORD dwNewSize = m_dwSendBuffptr + dwSize;
	
	if(dwNewSize > m_dwSendBuffSize)
	{
		log.Print(0,"ERROR: dwNewSize %lu > dwSendBuffSize %lu",dwNewSize,m_dwSendBuffSize);
		return false;
	}

	
	
	memcpy_ex(m_pSendBuffer + m_dwSendBuffptr,buff,dwSize);
	

	m_dwSendBuffptr += dwSize;

	SetNewUpdate(true);

	timelog.Print(0,"CAPTURE: CopyToSendBuff Stop = %lu ms. Data size = %lu"
					,GetTickCount()-gdwAppStartTime,dwSize);

	return true;
}


BOOL	
CClientThread::CreateSendBuff(DWORD dwSize)
{
	if(dwSize == 0)
		return false;

	m_dwSendBuffSize = dwSize;

	m_pSendBuffer = new BYTE[m_dwSendBuffSize];

	if(m_pSendBuffer == NULL)
	{
		log.Print(0,"ERROR: memory m_pSendBuff is NULL");
		return false;
	}

	ZeroMemory(m_pSendBuffer,m_dwSendBuffSize);
	return true;
}


void
CClientThread::ResetSendBuffPtr()
{
	m_dwSendBuffptr = 0;
}