/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


#include "stdhdrs.h"
#include "resource.h"
#include "rfb.h"


#include "VNeSThread.h"



#define VNES_REGISTRY_KEY					HKEY_LOCAL_MACHINE	



#define VNES_REGISTRY_SUBKEY				"Software\\SEIKO EPSON\\EMP NS Connection"


const char VNES_REGKEY_RECTGRABSTYLE[]		= "bVRectGrabStyle";	
const char VNES_REGKEY_RECTSENDSTYLE[]		= "bVRectSendStyle";	
const char VNES_REGKEY_MAKE_HOOK_RECT_ONE[]	= "bVHookedRectIntoOne";	
const char VNES_REGKEY_HOOK_LIMIT[]			= "dwVHookCaptureLimit";
const char VNES_REGKEY_LOGFILE[]			= "dwVOutputLogfile";


const char VNES_REGKEY_ENCODINGTYPE[]		= "nVEncodingType";
const char VNES_REGKEY_ENC_COMP_LEVEL[]		= "nVCompressionLevel";
const char VNES_REGKEY_ENC_JPEG_LEVEL[]		= "nVJpegQualityLevel";


const char VNES_REGKEY_CLIENT_WAIT[]		= "dwVClientWaitDelay";
const char VNES_REGKEY_DDIHOOK_WAIT[]		= "dwVDDIHookWaitDelay";	


const char VNES_REGKEY_APP_PATH[]			= "AppPath";


const char	VNES_REGKEY_FULLSCREEN_COUNT[]	= "dwVFullscreenCount";
const char	VNES_REGKEY_FULLSCREEN_DELAY[]	= "dwVFullscreenDelay";
const char	VNES_REGKEY_FULLSCREEN_SEND[]	= "bVFullscreenSend";




const char VNES_REGKEY_BUFFLIST_SIZE[]		= "dwVBufferListSize";
const char VNES_REGKEY_BATCHMAXRECTLIMIT[]	= "dwVBatchMaxRectLimit";


const char VNES_REGKEY_CLIENT_PRIORITY[]	= "dwVPriorityClient";
const char VNES_REGKEY_DDIHOOK_PRIORITY[]	= "dwVPriorityHook";
const char VNES_REGKEY_CHECK_DETAIL_AREA[]	= "bVCheckDetailArea";


const char VNES_REGKEY_HOOK_DETAIL_DELAY[]  = "dwVHookDetailDelay" ; 
const char VNES_REGKEY_DETAIL_AREA[]		= "dwVDetailArea" ; 
const char VNES_REGKEY_DETAIL_WIDTH[]		= "dwVDetailWidth";
const char VNES_REGKEY_DETAIL_HEIGHT[]		= "dwVDetailHeight";
const char VNES_REGKEY_DETAIL_LIMIT[]		= "dwVDetailLimit";



const char VNES_REGKEY_BITBLT_DELAY[]		= "dwVBitBltDelay";
const char VNES_REGKEY_PP_SCR_DIV[]			= "dwVPPScreenDivisions";
const char VNES_REGKEY_PP_PERCENT[]			= "dwVPPScreenPercent";
const char VNES_REGKEY_PP_BOOL[]			= "bVPPDoScreenDivs";


const char VNES_REGKEY_SEND_WAIT_LIMIT[]	= "dwVSendWaitLimit";


const DWORD VNES_DEF_SEND_WAIT_LIMIT		= 15000;	



const char VNES_REGKEY_SENDTRD_PRIORITY[]	= "dwVSendTrdPriority";
const char VNES_REGKEY_MAX_BUFFLIST[]		= "dwVMaxBuffListSize";


const UINT VNES_CAPTURE_ERROR	= WM_USER + 8;
const UINT VNES_DRIVER_ERROR	= WM_USER + 9;



const DWORD	VNES_DEF_CLIENTTHREAD_DELAY	 = 50;		


const DWORD VNES_DEF_DDIHOOKTHREAD_DELAY = 30;		

const DWORD VNES_DEF_HOOKCAPTURE_LIMIT   = 1000;	
const DWORD VNES_DEF_LOG_GENERATE		 = 0;		




const DWORD VNES_DEFAULTMAXBUFFERSIZE	 = 10;



const int VNES_DEF_COMPRESSIONLEVEL		 = 5;		


const int VNES_DEF_JPEQ_QUALITYLEVEL	 = 9;		




const DWORD	VNES_DEF_FULLSCREEN_COUNT	 = 1;					


const DWORD	VNES_DEF_FULLSCREEN_DELAY	 = 100;

const BOOL	VNES_DEF_FULLSCREEN_BSEND	 = TRUE;				
const int VNES_DEF_RFB_ENCODING			 = rfbEncodingTight;	


const DWORD VNES_DEF_BITBLT_DELAY		= 1;					
const BOOL	VNES_DEF_BOOL_POWERPOINT	= true;
const DWORD VNES_DEF_PP_SCR_DIV			= 64;
const DWORD VNES_DEF_PP_SCR_PERCENT		= 90;


const DWORD	VNES_DEF_MAX_RGNBUFFSIZE	  = 15;					 
const char	VNES_REGKEY_REGION_BUFFSIZE[] = "dwVRegionBuffSize"; 



const DWORD VNES_DEF_CLIENT_PRIORITY	= 4;			


const DWORD VNES_DEF_HOOK_PRIORITY		= 5;

const BOOL	VNES_DEF_CHECK_DETAIL_AREA	= false;



const DWORD VNES_DEF_SENDTRD_PRIORITY	= 4;









const DWORD VNES_DEF_HOOK_DETAIL_DELAY	= 0;			
const DWORD VNES_DEF_DETAIL_AREA		= 100; 			
const DWORD VNES_DEF_DETAIL_WIDTH		= 2;			
const DWORD VNES_DEF_DETAIL_HEIGHT		= 2; 			
const DWORD	VNES_DEF_DETAIL_LIMIT		= 50; 			




const BOOL	VNES_DEF_GRABSTYLE			= FALSE;
const BOOL	VNES_DEF_SENDSTYLE			= FALSE;
const BOOL	VNES_DEF_HOOKRECTINTOONE	= FALSE;


const DWORD	VNES_INITWAIT_TIMEOUT		= 3000;			



extern HINSTANCE	ghAppInstance;
extern const char	*gszAppName;
extern char			gsAppPath[_MAX_PATH];
extern CVNeSThread	*gpMainThread;
extern vncDesktop	*gpDesktop;
extern HWND			ghEMPWnd;
extern HANDLE		ghInitWait;


extern DWORD		gdwAppStartTime;



extern DWORD WINAPI ClientThreadProc(LPVOID lParam);
extern DWORD WINAPI DDIHookThreadProc(LPVOID lParam);
extern DWORD WINAPI MainThreadProc(LPVOID lParam);
extern DWORD WINAPI SendThreadProc(LPVOID lParam);



enum VNeSLog
{
	ServerLog		= 0x0001,
	EMPmessagesLog	= 0x0002,
	BuffListLog		= 0x0004,
	CompressionLog	= 0x0008,
	TimeLog			= 0x0010,
	DESlog			= 0x0020,
	SockLog 		= 0x0040,
	RectLog			= 0x0080,
	MouseRectLog	= 0x0100,
	GrabLog			= 0x0200
};



const int	VBL_ERROR				= 1000;
const int	VBL_OVERFLOWERROR		= VBL_ERROR + 1;
const int	VBL_COPYRECTERROR		= VBL_ERROR + 2;
const int   VBL_BUFFERSIZEERROR		= VBL_ERROR + 3;
const int	VBL_MEMORYERROR			= VBL_ERROR + 4;


const DWORD	STRETCH_LIMIT_HEIGHT	= 100000;
const DWORD	STRETCH_LIMIT_WIDTH		= 100000;
const DWORD	XGA_HEIGHT				= 768;
const DWORD	XGA_WIDTH				= 1024;







const char VNES_REGKEY_CLIENT_WAIT_SLOW[]				= "dwVClientWaitDelaySlow";
const char VNES_REGKEY_CLIENT_WAIT_SLOW_LIMIT_RECTS[]	= "dwVClientWaitDelaySlowLimitRects";
const char VNES_REGKEY_CLIENT_WAIT_SLOW_LIMIT_AREA[]	= "dwVClientWaitDelaySlowLimitArea";
const char VNES_REGKEY_RESEND_ZLIB_AREA_LIMIT[]			= "dwVResendZlibAreaLimit";



const char VNES_REGKEY_FULLCAPTURE_LIMITTIME[]			= "dwVFullCaptureLimitTime";
const char VNES_REGKEY_EDGECHECK_COLOR_PERCENT[]		= "dwVEdgeCheckColorPercent";
const char VNES_REGKEY_EDGECHECK_SAMPLING_PIXELS[]		= "dwVEdgeCheckSamplingPixels";
const char VNES_REGKEY_OLDEDGECHECK[]					= "bVOldEdgeCheck";
const char VNES_REGKEY_RESEND_JPEG_RECTS[]				= "dwVResendJpegRects";





const DWORD	VNES_DEF_CLIENTTHREAD_DELAY_SLOW			 = 500;

const DWORD	VNES_DEF_CLIENTTHREAD_DELAY_SLOW_LIMIT_RECTS = 100;










const DWORD VNES_DEF_FULLCAPTURE_LIMITTIME		= 100;

const DWORD VNES_DEF_EDGECHECK_COLOR_PERCENT	= 20;	

const DWORD VNES_DEF_EDGECHECK_SAMPLING_PIXELS	= 8;			
const BOOL	VNES_DEF_OLDEDGECHECK				= false;		
const DWORD VNES_DEF_RESEND_JPEG_RECTS			= 50;			
const DWORD VNES_DEF_RESEND_ZLIB_AREA_LIMIT		= 196608;		

