/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
#include "VNeSThread.h"
#include "VNeSConst.h"




CSendThread::CSendThread()
	: CVNeSThread()
{
	m_pSock				= NULL;
	m_pBuffToSend		= NULL;
	m_dwSendBuffSize	= 0;
	m_dwSendSize		= 0;
	m_dwCopySize		= 0;
	m_dwIPAdd			= 0;
	m_dwSentBytes		= 0;
	
	SetAlive(false);

	
	SetTimeStart(0);

	
	
}

CSendThread::~CSendThread()
{
	log.Print(0,"THREAD: SendThread desstructor. 0x%8x ID = %lu"
		,m_dwIPAdd,m_dwThreadID);

	if(m_pBuffToSend != NULL)
	{
		delete [] m_pBuffToSend;
		m_pBuffToSend = NULL;
	}

	if(m_pSock != NULL)
	{
		delete m_pSock;
		m_pSock = NULL;
	}

	if(m_hSendData != NULL)
	{
		CloseHandle(m_hSendData);
		m_hSendData = NULL;

		log.Print(0,"SEND THREAD: m_SendData Event Handle closed.");
	}
}


BOOL	
CSendThread::Init(VSocket *sock, DWORD dwBuffsize, DWORD dwListSize)
{
	if(sock == NULL || dwBuffsize == 0)
		return false;

	
	m_pSock = sock;

	
	m_dwSendBuffSize = dwBuffsize;

	
	if(!m_SendBuffList.InitRingBuffer(m_dwSendBuffSize,dwListSize))
	{
		log.Print(0,"ERROR: InitRingBuffer() failed 0x%8x",m_dwIPAdd);
		return false;
	}

	m_pBuffToSend = new BYTE[m_dwSendBuffSize];

	if(m_pBuffToSend == NULL)
	{
		log.Print(0,"ERROR: not enough memory Init() failed 0x%8x",m_dwIPAdd);
		return false;
	}

	ZeroMemory(m_pBuffToSend,m_dwSendBuffSize);

	
	
	m_hSendData = CreateEvent(NULL, TRUE, FALSE, NULL);
	
	if(m_hSendData == NULL)
	{
		log.Print(0,"ERROR: m_hSendData CreateEvent() failed. 0x%8x",m_dwIPAdd);
		return FALSE;
	}

	
	SetAlive(true);

	log.Print(0,"Send THREAD: SendThread Init() successful 0x%8x",m_dwIPAdd);

	return true;
}


BOOL		
CSendThread::SignalSendData(BOOL bSet)
{
	if(m_hSendData == NULL)
		return FALSE;

	BOOL ret;

	
	{
		CMyMutexLock lock(m_csSendSignal);
	
		if(bSet)
		{
			ret =  SetEvent(m_hSendData);
			
		}
		else
		{
			ret =  ResetEvent(m_hSendData);
			
		}
	}

	return ret;
};


BOOL		
CSendThread::SendUpdate()
{
	while(!IsListEmpty())
	{
		if(!TransferBuffers())
		{
			log.Print(0,"WARNING: TransferBuffers failed. %lu",m_dwSendSize);
			break;
		}

		

		m_lpTimeStart.QuadPart = 0;
		m_lpTimeStop.QuadPart = 0;
		
		timelog.Print(0,"SEND: SendUpdate Start = %lu ms"
					,GetTickCount()-gdwAppStartTime);

		SetTimeStart(GetTickCount());
		QueryPerformanceCounter(&m_lpTimeStart);

		
		
		if(!m_pSock->SendData((char *)m_pBuffToSend,m_dwSendSize,TRUE,TRUE))
		{
			SetTimeStart(0);
			return false;
		}

		QueryPerformanceCounter(&m_lpTimeStop);
		SetTimeStart(0);

		timelog.Print(0,"SEND: SendUpdate Stop = %lu ms"
					,GetTickCount()-gdwAppStartTime);

		timelog.Print(0,"SEND: (SendUpdate) Send time = %.2f us  datasize(sent data) = %lu \r\n client = 0x%8x"
				   ,(float)(m_lpTimeStop.QuadPart-m_lpTimeStart.QuadPart)/lpFreq.QuadPart*1000000
				   ,m_dwSendSize,m_dwIPAdd);

		bufflog.Print(0,"SendUpdate: %lu data sent 0x%8x", m_dwSendSize,m_dwIPAdd);

		{
			CMyMutexLock lock(m_csBytesSent);
			m_dwSentBytes += m_dwSendSize;
		}

		m_dwSendSize = 0;      
	}

	return true;
}


BOOL		
CSendThread::CopyData(BYTE *buff,DWORD dwSize)
{
	if(buff == NULL || dwSize == 0)
		return false;

	if(dwSize > m_dwSendBuffSize)
	{
		log.Print(0,"ERROR: dwSize %lu > m_dwSendBuffSize %lu"
				   ,dwSize,m_dwSendBuffSize);
		return false;
	}

	{
		m_lpTimeStart.QuadPart = 0;
		m_lpTimeStop.QuadPart = 0;

		CMyMutexLock lock(m_csSendBuffList);

		timelog.Print(0,"CAPTURE: CopyData Start = %lu ms"
					,GetTickCount()-gdwAppStartTime);

		QueryPerformanceCounter(&m_lpTimeStart);

		m_SendBuffList.Add(buff,dwSize);

		QueryPerformanceCounter(&m_lpTimeStop);

		timelog.Print(0,"CAPTURE: CopyData Stop = %lu ms"
					,GetTickCount()-gdwAppStartTime);

		timelog.Print(0,"CAPTURE: data buffer to list of send buff(CopyData) = %.2f us  size = %lu \r\n"
				   ,(float)(m_lpTimeStop.QuadPart-m_lpTimeStart.QuadPart)/lpFreq.QuadPart*1000000
				   ,dwSize);

		bufflog.Print(0,"CopyData: dwSize = %lu (copied) 0x%8x",dwSize,m_dwIPAdd);
	}

	SignalSendData(TRUE);

	return true;
}


BOOL
CSendThread::TransferBuffers()
{
	m_lpTimeStart.QuadPart = 0;
	m_lpTimeStop.QuadPart = 0;

	CMyMutexLock lock(m_csSendBuffList);
	
	timelog.Print(0,"SEND: TransferBuffers Start = %lu ms"
					,GetTickCount()-gdwAppStartTime);

	QueryPerformanceCounter(&m_lpTimeStart);

	VNES_SCRBUFFER	input;
	
	
	input.pScreenBuff	= m_pBuffToSend;

	if(!m_SendBuffList.GetFront(input))
		return false;
	
	
	m_dwSendSize = input.dwScrBuffsize;

	QueryPerformanceCounter(&m_lpTimeStop);

	timelog.Print(0,"SEND: TransferBuffers Stop = %lu ms"
					,GetTickCount()-gdwAppStartTime);

	timelog.Print(0,"SEND: list of send buff to actual send buff (TransferBuffers) = %.2f us size = %lu client = 0x%8x\r\n"
				   ,(float)(m_lpTimeStop.QuadPart-m_lpTimeStart.QuadPart)/lpFreq.QuadPart*1000000
				   ,m_dwSendSize,m_dwIPAdd);

	return true;
}

BOOL	
CSendThread::CloseSocket()
{
	BOOL bRet = true;

	if(m_pSock != NULL)
	{
		bRet = m_pSock->Close();
	}

	return bRet;
}

