/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "VNeSUtil.h"
#include "VNeSConst.h"
#include "Registry.h"



int GetVNeSAppDir (char *pDst, DWORD dwSize)
{
	char szPath[_MAX_PATH];
	if (GetModuleFileName(NULL, szPath, _MAX_PATH)) {
		char szDrive	[_MAX_DRIVE];
		char szDir		[_MAX_DIR];
		char szFname	[_MAX_FNAME];
		char szExt		[_MAX_EXT];
		_splitpath(szPath, szDrive, szDir, szFname, szExt);
		return _snprintf(pDst,dwSize,"%s%s",szDrive, szDir);
	}
	return 0;
}


BOOL 
LoadVNeSPathName(char *sPath, DWORD &dwLog)
{
	BOOL ret = FALSE;
	CRegistry reg;

	if(reg.open(VNES_REGISTRY_KEY, VNES_REGISTRY_SUBKEY, KEY_ALL_ACCESS))
	{
		
		
		reg.getString(VNES_REGKEY_APP_PATH,sPath);

		
		reg.getDWORD(VNES_REGKEY_LOGFILE, &dwLog);
			
		reg.close();

		ret = TRUE;
	}

	if(dwLog & ServerLog)
	{
		
		log.SetMode(Log::ToFile);
		log.SetLevel(LL_ALL);
	}
	if(dwLog & EMPmessagesLog)
	{
		
		evlog.SetMode(Log::ToFile);
		evlog.SetLevel(LL_ALL);
	}
	if(dwLog & RectLog)
	{
		
		rectlog.SetMode(Log::ToFile);
		rectlog.SetLevel(LL_ALL);
	}
	if(dwLog & GrabLog)
	{
		
		grablog.SetMode(Log::ToFile);
		grablog.SetLevel(LL_ALL);
	}
	if(dwLog & CompressionLog)
	{		
		
		ratelog.SetMode(Log::ToFile);
		ratelog.SetLevel(LL_ALL);
	}
	if(dwLog & TimeLog)
	{		
		
		timelog.SetMode(Log::ToFile);
		timelog.SetLevel(LL_ALL);
	}
	if(dwLog & DESlog)
	{
		
		deslog.SetMode(Log::ToFile);
		deslog.SetLevel(LL_ALL);
	}
	if(dwLog & BuffListLog)
	{
		bufflog.SetMode(Log::ToFile);
		bufflog.SetLevel(LL_ALL);
	}
	if(dwLog & MouseRectLog)
	{
		
		
	}
	if(dwLog & SockLog)
	{
		socklog.SetMode(Log::ToFile);
		socklog.SetLevel(LL_ALL);
	}
	

	return ret;
}


BOOL
SaveLogRegKey(DWORD dwLog)
{
	CRegistry reg;
	if(!reg.create(VNES_REGISTRY_KEY,
					VNES_REGISTRY_SUBKEY,
					NULL,
					KEY_ALL_ACCESS)
					){
		log.Print(0,"ERROR: Reg create failed.");
		return FALSE;
	}

	if( !reg.open(VNES_REGISTRY_KEY, VNES_REGISTRY_SUBKEY, KEY_ALL_ACCESS))
		return FALSE;

	
	if(!reg.setDWORD(VNES_REGKEY_LOGFILE, dwLog))
		 return	FALSE;

	return TRUE;
}

BOOL
InitializeBitmap(rfbServerInitMsg &scrinfo, VNESBMINFO &bminfo)
{
	BOOL	ret = FALSE;
	
	HBITMAP	membitmap = NULL;
	HDC		hrootdc,hmemdc = NULL;
	RECT	bmrect;
	SetRectEmpty(&bmrect);

	
	hrootdc = ::GetDC(NULL);
	if (hrootdc == NULL)
		goto exit_sequence;

	bmrect.left = bmrect.top = 0;
	bmrect.right = GetDeviceCaps(hrootdc, HORZRES);
	bmrect.bottom = GetDeviceCaps(hrootdc, VERTRES);

	
	scrinfo.framebufferWidth = (CARD16) (bmrect.right - bmrect.left);		
	scrinfo.framebufferHeight = (CARD16) (bmrect.bottom - bmrect.top);

	
	if(scrinfo.framebufferHeight > STRETCH_LIMIT_HEIGHT || scrinfo.framebufferWidth > STRETCH_LIMIT_WIDTH)
	{
		scrinfo.framebufferHeight	= STRETCH_LIMIT_HEIGHT;
		scrinfo.framebufferWidth	= STRETCH_LIMIT_WIDTH;
	}

	

	
	hmemdc = CreateCompatibleDC(hrootdc);
	if (hmemdc == NULL) {
		log.Print(LL_INTERR, VNCLOG("ERROR: failed to create compatibleDC(%d)\n"), GetLastError());
		goto exit_sequence;
	}

	
	
	
	
	
	
	
	
	
	
	
	
	

	
	membitmap = CreateCompatibleBitmap(hrootdc, bmrect.right, bmrect.bottom);
	if (membitmap == NULL) {
		log.Print(LL_INTERR, VNCLOG("ERROR: failed to create memory bitmap(%d)\n"), GetLastError());
		goto exit_sequence;
	}
	log.Print(LL_INTINFO, VNCLOG("created memory bitmap\n"));

	
	int result;
	bminfo.bmi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	bminfo.bmi.bmiHeader.biBitCount = 0;
	result = ::GetDIBits(hmemdc, membitmap, 0, 1, NULL, &bminfo.bmi, DIB_RGB_COLORS);
	if (result == 0) {
		goto exit_sequence;
	}
	result = ::GetDIBits(hmemdc, membitmap,  0, 1, NULL, &bminfo.bmi, DIB_RGB_COLORS);
	if (result == 0) {
		goto exit_sequence;
	}
	
	
	
	
	
	
	
	

	
	bminfo.bmi.bmiHeader.biHeight = - abs(bminfo.bmi.bmiHeader.biHeight);

	
	if(GetDeviceCaps(hmemdc, RASTERCAPS) & RC_PALETTE) 
        bminfo.truecolour = FALSE;  
	else
		bminfo.truecolour = TRUE;
	
	
	ret = TRUE;

exit_sequence:	

	if (hrootdc != NULL)
	{
		
		if(ReleaseDC(NULL, hrootdc) == 0)
		{
			log.Print(LL_INTERR, VNCLOG("ERROR: hrootdc failed to ReleaseDC\n"));
		}
		hrootdc = NULL;
	}
	
	if (hmemdc != NULL)
	{
		
		if (!DeleteDC(hmemdc))
		{
			log.Print(LL_INTERR, VNCLOG("ERROR: hmemdcfailed to DeleteDC\n"));
		}
		hmemdc = NULL;
	}
	if (membitmap != NULL)
	{
		
		if (!DeleteObject(membitmap))
		{
			log.Print(LL_INTERR, VNCLOG("ERROR: membitmap failed to DeleteObject\n"));
		}
		membitmap = NULL;
	}
	return ret;
}


BOOL
ThunkBitmap(VNESBMINFO &bminfo)
{
	
	
	switch (bminfo.bmi.bmiHeader.biBitCount)
	{
	case 1:
	case 4:
		log.Print(LL_INTINFO, VNCLOG("DBG:used/bits/planes/comp/size = %d/%d/%d/%d/%d\n"),
			(int)bminfo.bmi.bmiHeader.biClrUsed,
			(int)bminfo.bmi.bmiHeader.biBitCount,
			(int)bminfo.bmi.bmiHeader.biPlanes,
			(int)bminfo.bmi.bmiHeader.biCompression,
			(int)bminfo.bmi.bmiHeader.biSizeImage);
		
		
		bminfo.bmi.bmiHeader.biClrUsed = 0;
		bminfo.bmi.bmiHeader.biPlanes = 1;
		bminfo.bmi.bmiHeader.biCompression = BI_RGB;
		bminfo.bmi.bmiHeader.biBitCount = 8;
		bminfo.bmi.bmiHeader.biSizeImage =
			abs((bminfo.bmi.bmiHeader.biWidth *
				bminfo.bmi.bmiHeader.biHeight *
				bminfo.bmi.bmiHeader.biBitCount)/ 8);
		bminfo.bmi.bmiHeader.biClrImportant = 0;
		bminfo.truecolour = FALSE;
		break;	
	case 24:
		
		bminfo.bmi.bmiHeader.biBitCount = 16;
		bminfo.bmi.bmiHeader.biPlanes = 1;
		bminfo.bmi.bmiHeader.biCompression = BI_RGB;
		bminfo.bmi.bmiHeader.biSizeImage =
			abs((bminfo.bmi.bmiHeader.biWidth *
				bminfo.bmi.bmiHeader.biHeight *
				bminfo.bmi.bmiHeader.biBitCount)/ 8);
		break;
	}
	return TRUE;
}

SetFormat(rfbServerInitMsg &scrinfo, VNESBMINFO &bminfo)
{
	
	switch (bminfo.bmi.bmiHeader.biBitCount)
	{
	case 24:
	case 32:
		if(scrinfo.framebufferHeight <= XGA_HEIGHT
		   && scrinfo.framebufferWidth <= XGA_WIDTH)
		{
			scrinfo.format.bitsPerPixel		= 32;
			scrinfo.format.depth			= 32;
			scrinfo.format.bigEndian		= 0;	
			scrinfo.format.trueColour		= 1;
			scrinfo.format.redMax			= 0xff;
			scrinfo.format.greenMax			= 0xff;
			scrinfo.format.blueMax			= 0xff;
			scrinfo.format.redShift			= 16;
			scrinfo.format.greenShift		= 8;
			scrinfo.format.blueShift	   	= 0;
			break;
		}
	default:
		{
			
			scrinfo.format.bitsPerPixel		= 16;
			scrinfo.format.depth			= 16;
			scrinfo.format.bigEndian		= 0;	
			scrinfo.format.trueColour		= 1;
			scrinfo.format.redMax			= 0x1f;
			scrinfo.format.greenMax			= 0x3f;
			scrinfo.format.blueMax			= 0x1f;
			scrinfo.format.redShift			= 11;
			scrinfo.format.greenShift		= 5;
			scrinfo.format.blueShift		= 0;
		}
	}
	

	
	





BOOL
SetShifts(rfbServerInitMsg &scrinfo, VNESBMINFO &bminfo)
{
	
	DWORD redMask=0, blueMask=0, greenMask = 0;

	switch (bminfo.bmi.bmiHeader.biBitCount)
	{
	case 16:
		
		if (bminfo.bmi.bmiHeader.biCompression == BI_RGB)
		{
			
			redMask = 0x7c00; greenMask = 0x03e0; blueMask = 0x001f;
		}
		else
		{
			if (bminfo.bmi.bmiHeader.biCompression == BI_BITFIELDS)
			{
				redMask =   *(DWORD *) &bminfo.bmi.bmiColors[0];
				greenMask = *(DWORD *) &bminfo.bmi.bmiColors[1];
				blueMask =  *(DWORD *) &bminfo.bmi.bmiColors[2];
			}
		}
		break;

	case 32:
		
		if (bminfo.bmi.bmiHeader.biCompression == BI_RGB)
		{
			redMask = 0xff0000; greenMask = 0xff00; blueMask = 0x00ff;
		}
		else
		{
			if (bminfo.bmi.bmiHeader.biCompression == BI_BITFIELDS)
			{
				redMask =   *(DWORD *) &bminfo.bmi.bmiColors[0];
				greenMask = *(DWORD *) &bminfo.bmi.bmiColors[1];
				blueMask =  *(DWORD *) &bminfo.bmi.bmiColors[2];
			}
		}
		break;

	default:
		
		if (bminfo.truecolour)
		{
			log.Print(LL_INTERR, "unsupported truecolour pixel format for setpixshifts\n");
			return FALSE;
		}
		return TRUE;
	}

	
	
	MaskToMaxShift(redMask, scrinfo.format.redMax, scrinfo.format.redShift);
	MaskToMaxShift(greenMask, scrinfo.format.greenMax, scrinfo.format.greenShift);
	MaskToMaxShift(blueMask, scrinfo.format.blueMax, scrinfo.format.blueShift);
	
	return TRUE;
}


void
MaskToMaxShift(DWORD mask, CARD16 &max, CARD8 &shift)
{
	for (shift = 0; (mask & 1) == 0; shift++)
		mask >>= 1;
	max = (CARD16) mask;
}

