/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "VNeSThread.h"




CVNeSThread::CVNeSThread() 
{
	
	m_hThread			= NULL;
	m_hEventKill		= NULL;
	
	m_dwThreadID		= 0;
	m_dwWaitDelay		= 0;

	log.Print(0,"THREAD INIT: Main Thread Kill and Dead events created.");
}


CVNeSThread::~CVNeSThread()
{
	
	if(m_hEventKill!=NULL)
	{
		CloseHandle(m_hEventKill);
		m_hEventKill = NULL;

		log.Print(0,"THREAD: Kill event Handle closed.");
	}
	
	
	
	if(m_hThread!=NULL)
	{
		CloseHandle(m_hThread);
		m_hThread = NULL;

		log.Print(0,"THREAD: Handle closed at Destructor.");
	}
}




DWORD
CVNeSThread::SuspendVNeSThread()
{
	
	log.Print(0,"THREAD: SuspendThread() called");
	

	return SuspendThread(m_hThread);
}


DWORD 
CVNeSThread::ResumeVNeSThread()
{

	log.Print(0,"THREAD: ResumeThread() called");

	
	return ResumeThread(m_hThread);
	
}


BOOL 
CVNeSThread::CreateVNeSThread(LPTHREAD_START_ROUTINE ThreadProc,
							  LPVOID lpParameter, 
						 	  DWORD dwCreationFlags)
{
	
	
	m_hEventKill = CreateEvent(NULL, TRUE, FALSE, NULL);
	
	if(m_hEventKill == NULL)
		return false;

	m_hThread = CreateThread(
				NULL,                        
				0,                           
				ThreadProc,					 
				lpParameter,				 
				dwCreationFlags,             
				&m_dwThreadID);               
 
	if(m_hThread == NULL)
	{
		
		log.Print(0,"THREAD ERROR: CreateThread() failed: %d",GetLastError());
		return false;
	}

	
	log.Print(0,"THREAD: CreateThread() successful. 0x%8x",m_dwThreadID);

	return true;
}


void 
CVNeSThread::KillLoop()
{
	log.Print(0,"THREAD: KillLoop() called. 0x%8x",m_dwThreadID);

	if(m_hEventKill!=NULL)
	{
		if(!SetEvent(m_hEventKill))
			log.Print(0,"THREAD: ERROR couldn't set m_hEventKill.");

		log.Print(0,"THREAD: Kill Event Set.");
	}
}


void 
CVNeSThread::KillThread()
{
	
	

	log.Print(0,"THREAD: KillThread() called. 0x%8x",m_dwThreadID);
	

	
	KillLoop();

	
	if(m_hThread != NULL)
	{
		if(!SetThreadPriority(m_hThread,THREAD_PRIORITY_ABOVE_NORMAL))
			log.Print(0,"THREAD ERROR: SetThreadPriority Failed %d",GetLastError());
	}
	
	

	
	
	
	while((int)ResumeVNeSThread() > 1){ }
	

	
	

	if(m_hThread != NULL)
	{
		log.Print(0,"THREAD: Waiting for thread handle.");
		WaitForSingleObject(m_hThread, INFINITE);
		log.Print(0,"THREAD: Waiting for thread handle finished.");
	}	

	

	log.Print(0,"THREAD: KillThread() Done. 0x%8x",m_dwThreadID);
}


BOOL	
CVNeSThread::SetVNeSThreadPriority(DWORD dwPriority)
{
	if(m_hThread == NULL)
		return FALSE;

	BOOL ret = FALSE;

	switch(dwPriority)
	{
	case 1:	
			ret = SetThreadPriority(m_hThread,THREAD_PRIORITY_LOWEST);
			break;
	case 2:
			ret =  SetThreadPriority(m_hThread,THREAD_PRIORITY_BELOW_NORMAL);
			break;
	case 4:
			ret =  SetThreadPriority(m_hThread,THREAD_PRIORITY_ABOVE_NORMAL);
			break;
	case 5:
			ret =  SetThreadPriority(m_hThread,THREAD_PRIORITY_HIGHEST);
			break;
	default:
			ret =  SetThreadPriority(m_hThread,THREAD_PRIORITY_NORMAL);
			break;
	}

	return ret;
}
