/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/





























class VSocket;




#include "stdhdrs.h"

extern "C" {
#include "vncauth.h"
}

#include "rfb.h"

#include <iostream.h>

#include <stdio.h>
#ifdef __WIN32__
#include <io.h>
#include <winsock.h>
#else
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <netdb.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <signal.h>
#endif
#include <sys/types.h>


#include "VNeSCommon.h"



#include "memcpy_ex.h"





#include "VTypes.h"




#ifdef __WIN32__
#define EWOULDBLOCK WSAEWOULDBLOCK
#endif




#include "VSocket.h"




static WORD winsockVersion = 0;

VSocketSystem::VSocketSystem()
{
  
  

#ifdef __WIN32__
  
  WORD wVersionRequested;
  WSADATA wsaData;
	
  wVersionRequested = MAKEWORD(2, 0);
  if (WSAStartup(wVersionRequested, &wsaData) != 0)
  {
    m_status = VFalse;
	return;
  }
  winsockVersion = wsaData.wVersion;
  log.Print(0,"INIT: WinSock version = %d",winsockVersion);

#else
  
  signal(SIGPIPE, SIG_IGN);
#endif

  
  m_status = VTrue;
}

VSocketSystem::~VSocketSystem()
{
	if (m_status)
	{
		WSACleanup();
	}
}



VSocket::VSocket()
{
	
	sock = -1;
	
	
	m_dwIPAddress		= 0;
	

}



VSocket::~VSocket()
{
  
  Close();
}



VBool
VSocket::Create()
{
	
	if(CreateTCP()!= VTrue)
		return VFalse;

	
	if(BindTCP(0)!= VTrue)
		return VFalse;

	DWORD dwBuffer = 0;
	int size = sizeof(dwBuffer);

	
	if(getsockopt(sock,SOL_SOCKET,SO_SNDBUF,(char *)&dwBuffer,&size) != 0)
		return VFalse;

	socklog.Print(0,"SO_SNDBUF = %lu",dwBuffer);

	dwBuffer = 0x10000;
	
	
	
	if(getsockopt(sock,SOL_SOCKET,SO_SNDBUF,(char *)&dwBuffer,&size) != 0)
		return VFalse;

	socklog.Print(0,"SO_SNDBUF = %lu",dwBuffer);

	





















































VBool 
VSocket::SendData(const char *buff,const VCard bufflen,
				BOOL bBlock, BOOL bHeader)
{
	if(bufflen == 0)
		return VTrue;

	
	int on=1;			
	int off=0;			

	if(bBlock != TRUE)
	{
		ioctlsocket(sock, FIONBIO,(unsigned long*)&on);
	}

	if(bHeader)
	{
		if(SendVNESHeader(bufflen)!= VTrue)
			return VFalse;
	}

	if(SendLoop(buff,bufflen)!=VTrue)
		return VFalse; 

	if(bBlock != TRUE)
	{
		ioctlsocket(sock, FIONBIO,(unsigned long*)&off);
	}
	
	return VTrue;	
}


VBool
VSocket::BindTCP(const VCard port)
{

	struct sockaddr_in addr;

	
	if(sock < 0)
	{
		log.Print(0,"TCP BINDING ERROR: socket not open.");
		return VFalse;
	}
	
	
	addr.sin_family = AF_INET;
	addr.sin_port = htons(port);
	addr.sin_addr.s_addr = m_dwIPAddress;

	
	if (bind(sock, (struct sockaddr *)&addr, sizeof(addr)) == SOCKET_ERROR)
	{
		int err = WSAGetLastError();
		log.Print(0,"TCP SOCKET ERROR: Binding failed: %d ", err);
		return VFalse;
	}

	int size = sizeof(addr);
	if(getsockname(sock,(struct sockaddr *)&addr,&size) != SOCKET_ERROR)
	{
		socklog.Print(0,"TCP SOCKET: bound to port %d",ntohs(addr.sin_port));
	}

	return VTrue;
}


VBool	
VSocket::CreateTCP()
{
	const int one = 1;

	
	if (sock >= 0)
		Close();

	
	if ((sock = socket(AF_INET, SOCK_STREAM, IPPROTO_IP)) < 0)
	{
		int err = WSAGetLastError();
		log.Print(0,"SOCKET ERROR: socket() failed: %d",err);
		return VFalse;
	}

	
	if (setsockopt(sock, IPPROTO_TCP, TCP_NODELAY, (char *)&one, sizeof(one)) !=0)
	{
		log.Print(0,"SOCKET ERROR: setsockopt() tcp_nodelay failed %d",WSAGetLastError());
		return VFalse;
	}

	if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, (char *)&one, sizeof(one)) != 0)
	{
		log.Print(0,"SOCKET ERROR: setsockopt()  so_reuseaddr failed %d",WSAGetLastError());
		return VFalse;
	}

	socklog.Print(0,"TCT socket: CreateTCP() successful.");
	return VTrue;
}


VBool
VSocket::SendVNESHeader(const VCard bufflen)
{
	VNESHEADER vnesHeader;

	vnesHeader.dwMagicNum	= Swap32IfLE(VNES_MAGICNUMBER);
	vnesHeader.dwServerIP	= m_dwIPAddress;
	vnesHeader.dwCommand	= Swap32IfLE(0);
	vnesHeader.dwDataSize	= Swap32IfLE(bufflen);

	
	
	
	memcpy_ex(vnesHeader.MajorVersion,(void*)VNES_MAJORVERSION,2);
	memcpy_ex(vnesHeader.MinorVersion,(void*)VNES_MINORVERSION,2);
	

	if(!SendLoop((char *)&vnesHeader,sizeof(VNESHEADER)))
	{
		log.Print(LL_CONNERR, VNCLOG("ERROR: SendVNESHeader() failed.\n"));
		return VFalse;
	}

	socklog.Print(0,"VNES HEADER: sent");

	return VTrue;
}


void	
VSocket::InsertVnesENCHeader(char *buff,VCard bytes)
{
	VNESENCRYPTIONHEADER header;
	ZeroMemory((char *)header.pad,8);

	
	header.size = Swap32IfLE(bytes);

	
	rdEncryptBytes((unsigned char *)header.pad,8);

	
	
	
	memcpy_ex(buff,(char *)&header,sizeof(VNESENCRYPTIONHEADER));
	

	deslog.Print(0,"ENC: Header %d bytes",bytes);
	socklog.Print(0,"socket: InsertVnesENCHeader() finished %d bytes.",bytes);
}


void	
VSocket::InsertVnesENCData(char *buff,const char *srcbuff,VCard bufflen,
						   int remainder)
{
	VCard currlen = bufflen;

	
	currlen -= remainder;

	if(currlen >= 8)
	{
		
		rdEncryptBytes((unsigned char *)srcbuff,currlen);
			
		
		
		
		memcpy_ex(buff,(void*)srcbuff,currlen);
		

		deslog.Print(0,"ENC: copied %d.",currlen);
	}

	if(remainder!=0){
		char sLastEight[8] = {0,0,0,0,0,0,0,0}; 
			
		
		
		memcpy_ex(sLastEight,(void*)(srcbuff+currlen),remainder);
		
			
		
		rdEncryptBytes((unsigned char *)sLastEight,8);
			
		
		
		
		memcpy_ex(buff+currlen,sLastEight,8);
		

		deslog.Print(0,"ENC: last eight copied.");
	}

	socklog.Print(0,"socket: InsertVnesENCData() finished.");
}