/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "ZlibLib.h"




CZlibLib::CZlibLib()
{
	m_hLibModule		= NULL;
	m_pfnDeflate		= NULL;
	m_pfnDeflateEnd		= NULL;
	m_pfnDeflateParams	= NULL;
	m_pfnDeflateInit2	= NULL;

	
	m_pfnDeflateReset	= NULL;
}


CZlibLib::~CZlibLib()
{
	if(m_hLibModule){
		FreeLib();
	}
}


BOOL
CZlibLib::FreeLib()
{
	BOOL ret = FALSE;

	m_pfnDeflate		= NULL;
	m_pfnDeflateEnd		= NULL;
	m_pfnDeflateParams	= NULL;
	m_pfnDeflateInit2	= NULL;

	
	m_pfnDeflateReset	= NULL;

	if(m_hLibModule){
		ret = FreeLibrary(m_hLibModule);	
		m_hLibModule = NULL;
	}

	return ret;
}


BOOL
CZlibLib::LoadLib(char *psAppPath)
{
	char cDLL	[_MAX_PATH];
	memset(cDLL,0,_MAX_PATH);
	
	if(_snprintf(cDLL,_MAX_PATH,"%s\\zlib.dll",psAppPath) < 0)
	{
		
		log.Print(0,"ERROR: Zlib dll's path exceeds _MAX_PATH = %d",_MAX_PATH);
		return FALSE;
	}

	m_hLibModule = (HMODULE)LoadLibrary(cDLL);
	
	log.Print(0, "m_hLibModule = %lu",(unsigned long)m_hLibModule);

	if(m_hLibModule == NULL)
		return FALSE;

	m_pfnDeflate		= (PROC_Deflate) GetProcAddress(m_hLibModule,TEXT("deflate"));
	m_pfnDeflateEnd		= (PROC_DeflateEnd) GetProcAddress(m_hLibModule,TEXT("deflateEnd"));
	m_pfnDeflateParams  = (PROC_DeflateParams) GetProcAddress(m_hLibModule,TEXT("deflateParams"));
	m_pfnDeflateInit2	= (PROC_DeflateInit2) GetProcAddress(m_hLibModule,TEXT("deflateInit2_"));

	
	m_pfnDeflateReset	= (PROC_DeflateReset) GetProcAddress(m_hLibModule,TEXT("deflateReset"));

	log.Print(0, "m_pfnDeflate %lu",(unsigned long)m_pfnDeflate);		
	log.Print(0, "m_pfnDeflateEnd %lu",(unsigned long)m_pfnDeflateEnd);			
	log.Print(0, "m_pfnDeflateParams %lu",(unsigned long)m_pfnDeflateParams);		
	log.Print(0, "m_pfnDeflateInit2 %lu",(unsigned long)m_pfnDeflateInit2);

	
	log.Print(0, "m_pfnDeflateReset %lu",(unsigned long)m_pfnDeflateReset);

	
	if(m_pfnDeflate				== NULL		
	   || m_pfnDeflateEnd		== NULL		
	   || m_pfnDeflateParams	== NULL		
	   || m_pfnDeflateInit2		== NULL  
	   || m_pfnDeflateReset		== NULL				
	){
			
			log.Print(0,"ZLIB LIB: ERROR one of the dll functions is NULL = 0x%8x.",GetLastError());
			return FALSE;
	}
	else
		return TRUE;
}


int  
CZlibLib::Deflate(z_streamp zpStrm, int nFlush)
{	 
	if(!m_hLibModule || !m_pfnDeflate)
	{
		
		return Z_STREAM_ERROR;
	}
	
	return m_pfnDeflate(zpStrm,nFlush);
}


int  
CZlibLib::DeflateEnd(z_streamp zpStrm)
{ 
	if(!m_hLibModule || !m_pfnDeflateEnd)
	{
		
		return Z_STREAM_ERROR;
	}
	
	return m_pfnDeflateEnd(zpStrm);
}


int  
CZlibLib::DeflateParams(z_streamp zpStrm, int nLevel, int nStrategy)
{
	if(!m_hLibModule || !m_pfnDeflateParams)
	{
		
		return Z_STREAM_ERROR;
	}
	
	return m_pfnDeflateParams(zpStrm,nLevel,nStrategy);
}


int  
CZlibLib::DeflateInit2(z_streamp zpStrm, int nLevel, int nMethod, 
				  int nWindowBits, int nMemLevel,int nStrategy)
{
	if(!m_hLibModule || !m_pfnDeflateInit2)
	{	
		
		return Z_STREAM_ERROR;
	}
	
	return m_pfnDeflateInit2((zpStrm),(nLevel),(nMethod),(nWindowBits),
				(nMemLevel),(nStrategy),ZLIB_VERSION,sizeof(z_stream));
}


int  
CZlibLib::DeflateReset(z_streamp zpStrm)
{	 
	if(!m_hLibModule || !m_pfnDeflateReset)
	{
		
		return Z_STREAM_ERROR;
	}
	
	return m_pfnDeflateReset(zpStrm);
}