/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#define MEMCPYEX

#include "memcpy_ex.h"

#define MEMCPYTYPE_NONE		0
#define MEMCPYTYPE_MEMCPY	1
#define MEMCPYTYPE_MMX		2
#define MEMCPYTYPE_SSE		3

static int gfMemcpyType = MEMCPYTYPE_NONE;

void bench_memcpy();
void memcpy_ex( void* a, void* b, int size );
void memcpy_mmx( void*, void* s, int _size );
void memcpy_sse( void* d, void* s, int _size );
DWORD GetTick();

DWORD GetTick()
{
	Sleep(0);
	LONGLONG time,lnFreq;
	::QueryPerformanceFrequency((LARGE_INTEGER*)&lnFreq);
	::QueryPerformanceCounter((LARGE_INTEGER*)&time);
	return (DWORD)((time * (LONGLONG)1000) / lnFreq);
}

void bench_memcpy();
int CPU();
enum {
	cpu_legacy=0, cpu_mmx=(1<<0),
	cpu_3dnow=(1<<1), cpu_e3dnow=(1<<2),
	cpu_sse=(1<<3), cpu_sse2=(1<<4),
};

int CPU() {
	int flags=0,type=0;
	__asm {
		xor eax,eax;
		cpuid;
		or eax,eax;
		jz quit;
		mov eax,1;
		cpuid;
		mov flags,edx;
	quit:
	}
	if (flags&(1<<31)) {
		type|=cpu_3dnow;
		if (flags&(1<<30)) {
			type|=cpu_e3dnow;
		}
	}

	if (flags&(1<<23)) {
		type|=cpu_mmx;
		if (flags&(1<<25)) {
			type|=cpu_sse;
			if (flags&(1<<26)) {
				type|=cpu_sse2;
			}
		}
	}
	return type;
}

void memcpy_ex( void* a, void* b, int size )
{
#ifdef MEMCPYEX
	if(gfMemcpyType == MEMCPYTYPE_NONE){
		bench_memcpy();
	}
	
	
	
	
	switch(gfMemcpyType){
	case MEMCPYTYPE_MEMCPY:
		memcpy(a,b,size);
		break;
	case MEMCPYTYPE_MMX:
		memcpy_mmx(a,b,size);
		break;
	
	
	
	
	
	}
	
	
	

#else
	memcpy(a,b,size);
#endif
}

void bench_memcpy()
{
#define BENCHBUFLEN	65536
	int cpu=CPU();
	gfMemcpyType = MEMCPYTYPE_MEMCPY;

	LPBYTE pDst = new BYTE[BENCHBUFLEN];
	LPBYTE pSrc = new BYTE[BENCHBUFLEN];

	int i;
	DWORD tm1,tm2;


	DWORD res1,res2;


	tm1 = GetTick();
	for(i = 0;i < 1000;i++)
		memcpy(pDst,pSrc,BENCHBUFLEN);
	tm2 = GetTick();
	res1 = tm2 -tm1;
	
	if (cpu&cpu_mmx) {
		tm1 = GetTick();
		for(i = 0;i < 1000;i++)
			memcpy_mmx(pDst,pSrc,BENCHBUFLEN);
		tm2 = GetTick();
		res2 = tm2 -tm1;
	} else {
		res2 = 0xffffffff;
	}


	if(res1 > res2){
		gfMemcpyType = MEMCPYTYPE_MMX;
	}
	else
	{
		gfMemcpyType = MEMCPYTYPE_MEMCPY;
	}



	

}


void memcpy_mmx( void* d, void* s, int _size ) {
	_asm {
		mov edi,d;
		mov esi,s;
		mov ecx,_size;

		
		mov eax,ecx;		
		shr ecx,3;
		jz odd_copy;		

copy_loop:
		movq mm0,[esi];
		add esi,8;
		movq [edi],mm0;
		add edi,8;
		dec ecx;
		jnz copy_loop;
		emms;

odd_copy:
		mov ecx,eax;		
		and ecx,7;			
		jz memcpy_exit;		
		rep movsb;
memcpy_exit:

	}
}








