/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/









typedef struct {
    CARD16 x;
    CARD16 y;
    CARD16 w;
    CARD16 h;
} rfbRectangle;

#define sz_rfbRectangle 8




typedef struct {

    CARD8 bitsPerPixel;		

    CARD8 depth;		

    CARD8 bigEndian;		

    CARD8 trueColour;		

    

    CARD16 redMax;		

    CARD16 greenMax;		

    CARD16 blueMax;		

    CARD8 redShift;		

    CARD8 greenShift;		

    CARD8 blueShift;		

    CARD8 pad1;
    CARD16 pad2;

} rfbPixelFormat;

#define sz_rfbPixelFormat 16







#define rfbProtocolVersionFormat "RFB %03d.%03d\n"
#define rfbProtocolMajorVersion 3
#define rfbProtocolMinorVersion 3

typedef char rfbProtocolVersionMsg[13];	

#define sz_rfbProtocolVersionMsg 12




#define rfbConnFailed 0
#define rfbNoAuth 1
#define rfbVncAuth 2



#define rfbVncAuthOK 0
#define rfbVncAuthFailed 1
#define rfbVncAuthTooMany 2




typedef struct {
    CARD8 shared;
} rfbClientInitMsg;

#define sz_rfbClientInitMsg 1




typedef struct {
    CARD16 framebufferWidth;
    CARD16 framebufferHeight;
    rfbPixelFormat format;	
    CARD32 nameLength;
    
} rfbServerInitMsg;

#define sz_rfbServerInitMsg (8 + sz_rfbPixelFormat)










#define rfbFramebufferUpdate 0
#define rfbSetColourMapEntries 1
#define rfbBell 2
#define rfbServerCutText 3




#define rfbSetPixelFormat 0
#define rfbFixColourMapEntries 1	
#define rfbSetEncodings 2
#define rfbFramebufferUpdateRequest 3
#define rfbKeyEvent 4
#define rfbPointerEvent 5
#define rfbClientCutText 6






#define rfbEncodingRaw		0
#define rfbEncodingCopyRect	1
#define rfbEncodingRRE		2
#define rfbEncodingCoRRE	4
#define rfbEncodingHextile	5
#define rfbEncodingZlib		6
#define rfbEncodingTight	7

#define rfbEncodingZlibHex	8  





#define rfbEncodingCompressLevel0  0xFFFFFF00
#define rfbEncodingCompressLevel1  0xFFFFFF01
#define rfbEncodingCompressLevel2  0xFFFFFF02
#define rfbEncodingCompressLevel3  0xFFFFFF03
#define rfbEncodingCompressLevel4  0xFFFFFF04
#define rfbEncodingCompressLevel5  0xFFFFFF05
#define rfbEncodingCompressLevel6  0xFFFFFF06
#define rfbEncodingCompressLevel7  0xFFFFFF07
#define rfbEncodingCompressLevel8  0xFFFFFF08
#define rfbEncodingCompressLevel9  0xFFFFFF09

#define rfbEncodingXCursor         0xFFFFFF10
#define rfbEncodingRichCursor      0xFFFFFF11

#define rfbEncodingLastRect        0xFFFFFF20

#define rfbEncodingQualityLevel0   0xFFFFFFE0
#define rfbEncodingQualityLevel1   0xFFFFFFE1
#define rfbEncodingQualityLevel2   0xFFFFFFE2
#define rfbEncodingQualityLevel3   0xFFFFFFE3
#define rfbEncodingQualityLevel4   0xFFFFFFE4
#define rfbEncodingQualityLevel5   0xFFFFFFE5
#define rfbEncodingQualityLevel6   0xFFFFFFE6
#define rfbEncodingQualityLevel7   0xFFFFFFE7
#define rfbEncodingQualityLevel8   0xFFFFFFE8
#define rfbEncodingQualityLevel9   0xFFFFFFE9







typedef struct {
    CARD8 type;			
    CARD8 pad;
    CARD16 nRects;
    
} rfbFramebufferUpdateMsg;

#define sz_rfbFramebufferUpdateMsg 4



typedef struct {
    rfbRectangle r;
    CARD32 encoding;	
} rfbFramebufferUpdateRectHeader;

#define sz_rfbFramebufferUpdateRectHeader (sz_rfbRectangle + 4)







typedef struct {
    CARD16 srcX;
    CARD16 srcY;
} rfbCopyRect;

#define sz_rfbCopyRect 4




typedef struct {
    CARD32 nSubrects;
} rfbRREHeader;

#define sz_rfbRREHeader 4




typedef struct {
    CARD8 x;
    CARD8 y;
    CARD8 w;
    CARD8 h;
} rfbCoRRERectangle;

#define sz_rfbCoRRERectangle 4




#define rfbHextileRaw			(1 << 0)
#define rfbHextileBackgroundSpecified	(1 << 1)
#define rfbHextileForegroundSpecified	(1 << 2)
#define rfbHextileAnySubrects		(1 << 3)
#define rfbHextileSubrectsColoured	(1 << 4)

#define rfbHextilePackXY(x,y) (((x) << 4) | (y))
#define rfbHextilePackWH(w,h) ((((w)-1) << 4) | ((h)-1))
#define rfbHextileExtractX(byte) ((byte) >> 4)
#define rfbHextileExtractY(byte) ((byte) & 0xf)
#define rfbHextileExtractW(byte) (((byte) >> 4) + 1)
#define rfbHextileExtractH(byte) (((byte) & 0xf) + 1)



typedef struct {
    CARD32 nBytes;
} rfbZlibHeader;

#define sz_rfbZlibHeader 4




#define rfbTightExplicitFilter         0x04
#define rfbTightFill                   0x08
#define rfbTightJpeg                   0x09  
#define rfbTightMaxSubencoding         0x09  


#define rfbTightFilterCopy             0x00
#define rfbTightFilterPalette          0x01
#define rfbTightFilterGradient         0x02




#define rfbHextileZlibRaw		(1 << 5)
#define rfbHextileZlibHex		(1 << 6)



typedef struct {
    CARD8 foreRed;
    CARD8 foreGreen;
    CARD8 foreBlue;
    CARD8 backRed;
    CARD8 backGreen;
    CARD8 backBlue;
} rfbXCursorColors;

#define sz_rfbXCursorColors 6






typedef struct {
    CARD8 type;			
    CARD8 pad;
    CARD16 firstColour;
    CARD16 nColours;

    

} rfbSetColourMapEntriesMsg;

#define sz_rfbSetColourMapEntriesMsg 6





typedef struct {
    CARD8 type;			
} rfbBellMsg;

#define sz_rfbBellMsg 1





typedef struct {
    CARD8 type;			
    CARD8 pad1;
    CARD16 pad2;
    CARD32 length;
    
} rfbServerCutTextMsg;

#define sz_rfbServerCutTextMsg 8




typedef union {
    CARD8 type;
    rfbFramebufferUpdateMsg fu;
    rfbSetColourMapEntriesMsg scme;
    rfbBellMsg b;
    rfbServerCutTextMsg sct;
} rfbServerToClientMsg;








typedef struct {
    CARD8 type;			
    CARD8 pad1;
    CARD16 pad2;
    rfbPixelFormat format;
} rfbSetPixelFormatMsg;

#define sz_rfbSetPixelFormatMsg (sz_rfbPixelFormat + 4)




typedef struct {
    CARD8 type;			
    CARD8 pad;
    CARD16 firstColour;
    CARD16 nColours;

    

} rfbFixColourMapEntriesMsg;
 
#define sz_rfbFixColourMapEntriesMsg 6




typedef struct {
    CARD8 type;			
    CARD8 pad;
    CARD16 nEncodings;
    
} rfbSetEncodingsMsg;

#define sz_rfbSetEncodingsMsg 4




typedef struct {
    CARD8 type;			
    CARD8 incremental;
    CARD16 x;
    CARD16 y;
    CARD16 w;
    CARD16 h;
} rfbFramebufferUpdateRequestMsg;

#define sz_rfbFramebufferUpdateRequestMsg 10




typedef struct {
    CARD8 type;			
    CARD8 down;			
    CARD16 pad;
    CARD32 key;			
} rfbKeyEventMsg;

#define sz_rfbKeyEventMsg 8




typedef struct {
    CARD8 type;			
    CARD8 buttonMask;		
    CARD16 x;
    CARD16 y;
} rfbPointerEventMsg;

#define rfbButton1Mask 1
#define rfbButton2Mask 2
#define rfbButton3Mask 4

#define sz_rfbPointerEventMsg 6





typedef struct {
    CARD8 type;			
    CARD8 pad1;
    CARD16 pad2;
    CARD32 length;
    
} rfbClientCutTextMsg;

#define sz_rfbClientCutTextMsg 8





typedef union {
    CARD8 type;
    rfbSetPixelFormatMsg spf;
    rfbFixColourMapEntriesMsg fcme;
    rfbSetEncodingsMsg se;
    rfbFramebufferUpdateRequestMsg fur;
    rfbKeyEventMsg ke;
    rfbPointerEventMsg pe;
    rfbClientCutTextMsg cct;
} rfbClientToServerMsg;
