/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


























#include "stdhdrs.h"


#include "vncRegion.h"
#include "RectList.h"
#include "vncDesktop.h"


#include "DDIHook.h"



#include "Registry.h"


#include "VNeSCommon.h"
#include "VNeSConst.h"
#include "VNeS.h"
#include "EVStream.h"

#include "MyGetDC.h"

#include "vncEncoder.h"
#include "vncEncodeRRE.h"
#include "vncEncodeCoRRE.h"
#include "vncEncodeHexT.h"
#include "vncEncodeZlib.h"
#include "vncEncodeTight.h"
#include "vncEncodeZlibHex.h"  




#include <assert.h>


const TIGHT_CONF_COPY vncDesktop::m_conf_copy[10] = {
	{	512,   32 },
	{  2048,  128 },
	{  6144,  256 },
	{ 10240, 1024 },
	{ 16384, 2048 },
	{ 32768, 2048 },
	{ 65536, 2048 },
	{ 65536, 2048 },
	{ 65536, 2048 },
	{ 65536, 2048 }
};


vncDesktop::vncDesktop(HWND hwnd, char *sPath)
{
	
	
	assert(sPath != NULL);
	strcpy(m_sAppPath,sPath);
	

	
	m_bFirstCapture = TRUE;
	m_bDoSleep = FALSE;
	

	
	m_bPalettechanged = FALSE;

	

	
	m_hcursor = NULL;
	m_cursormoved = TRUE;

	
	
	
	m_hmemdc = NULL;
	m_membitmap = NULL;

	m_bytesPerRow = 0;

	SetRectEmpty(&m_cursorpos);
	SetRectEmpty(&m_bmrect);

	
	m_DIBbits = NULL;
	m_formatmunged = FALSE;

	
	m_hwnd = hwnd;

	
	m_pClientTrd = NULL;
	m_pDDIHookTrd = NULL;

	
	ZeroMemory(&m_scrinfo,sizeof(m_scrinfo));
	ZeroMemory(&m_REALscrinfo,sizeof(m_REALscrinfo));

	
	m_vrUpdateReg.Clear();

	
	m_rlCaptured.clear();

	
	m_bClientReady			= FALSE;
	m_bEncryption			= FALSE;
	m_bDisconnectReq		= FALSE;
	m_bSetsuzokuChuu		= FALSE;

	
	m_bGrabStyle			= VNES_DEF_GRABSTYLE;
	m_bSendStyle			= VNES_DEF_SENDSTYLE;
	m_bHookedRectIntoOne	= VNES_DEF_HOOKRECTINTOONE;

	m_dwHookCaptureLimit	= VNES_DEF_HOOKCAPTURE_LIMIT;
	m_nEncodingType			= VNES_DEF_RFB_ENCODING;
	m_nCompressionLevel		= VNES_DEF_COMPRESSIONLEVEL;
	m_nJpegQualityLevel		= VNES_DEF_JPEQ_QUALITYLEVEL;
	

	
	m_dwClientWaitDelay		= VNES_DEF_CLIENTTHREAD_DELAY;
	m_dwDDIHookWaitDelay	= VNES_DEF_DDIHOOKTHREAD_DELAY;

	
	
	m_bSendFullScreen		= VNES_DEF_FULLSCREEN_BSEND;
	m_dwFullScrRefreshDelay	= VNES_DEF_FULLSCREEN_DELAY;
	m_dwFullScreenCount		= VNES_DEF_FULLSCREEN_COUNT;
	

	
	m_dwBitBltdelay			= VNES_DEF_BITBLT_DELAY;
	m_bPowerBlackOut		= VNES_DEF_BOOL_POWERPOINT;
	m_dwScreenDivision		= VNES_DEF_PP_SCR_DIV;
	m_dwPercentScreen		= VNES_DEF_PP_SCR_PERCENT;

	
	
	

	
	m_dwSendTreadWaitLimit = VNES_DEF_SEND_WAIT_LIMIT;

	
	m_dwSendTrdPriority		= VNES_DEF_SENDTRD_PRIORITY;

	
	
	m_dwRegionBuffSize		= VNES_DEF_MAX_RGNBUFFSIZE;
	

	
	m_dwClientTrdPriority	= VNES_DEF_CLIENT_PRIORITY;
	m_dwHookTrdPriority		= VNES_DEF_HOOK_PRIORITY;
	m_bCheckDetailArea		= VNES_DEF_CHECK_DETAIL_AREA;

	
	m_dwHookDetailDelay		= VNES_DEF_HOOK_DETAIL_DELAY;
	m_dwDetailArea			= VNES_DEF_DETAIL_AREA; 
	m_dwDetailWidth			= VNES_DEF_DETAIL_WIDTH;
	m_dwDetailHeight		= VNES_DEF_DETAIL_HEIGHT;
	m_dwDetailLimit			= VNES_DEF_DETAIL_LIMIT;
	

	
	m_hSendUpdate			= NULL;
	
	
	m_hSendBuffsReady		= NULL;
	m_dwBuffListMaxSize		= VNES_DEFAULTMAXBUFFERSIZE;

	
	m_pDataBuffer			= NULL;
	m_dwDataBuffsize		= 0;
	m_dwDataBuffptr			= 0;
	

	
	m_bUseStretchBlit		= false;
	m_dwBitmapWidth			= 0;
	m_dwBitmapHeight		= 0;
	m_dwServerWidthRatio	= 0;
	m_dwServerHeightRatio	= 0;
	m_dwClientWidthRatio	= 0;
	m_dwClientHeightRatio	= 0;

	
	m_dwSendTrdCount		= 0;

	{
		CMyMutexLock lock(m_csClientArray);

		for(int s = 0; s < VNES_MAXCONNECTIONS; s++)
		{
			m_pSendTrds[s] = NULL;
		}
	}
	
	for(int z = 0; z < VNES_MAXCONNECTIONS; z++)
	{
		m_SendTrdParam[z].pDesktop		= NULL;
		m_SendTrdParam[z].dwClientID	= -1;
	}
	
}

vncDesktop::~vncDesktop()
{
	log.Print(LL_INTINFO, VNCLOG("Desktop deconstructor"));

	
	
	
	
	

	
	ShutdownThreads();
	Shutdown();
	HookFinalize();

	
	if(ddiHook.FreeLib())
		log.Print(0,"DESKTOP: DDIHook FreeLib() successful");
}


BOOL
vncDesktop::Startup()
{
	if (!InitBitmap())
		return FALSE;

	if (!ThunkBitmapInfo())
		return FALSE;

	if (!SetPixFormat())
		return FALSE;

	EnableOptimisedBlits();

	if (!SetPixShifts())
		return FALSE;

	if (!SetPalette())
		return FALSE;

	if (!CreateDataBuff())
		return FALSE;

	
	log.Print(0,"DESKTOP INIT: Startup() successful.");
	return TRUE;
}


BOOL
vncDesktop::Shutdown()
{
	CMyMutexLock lock(m_csShutdown);

	
	



































































































































































































BOOL 
vncDesktop::CaptureScreenDiv(RECT &rect,DWORD &dwHeight)
{
	BOOL ret = false;

	DWORD dwDiv			= m_dwScreenDivision;
	DWORD dwPart		= dwHeight/dwDiv;
	DWORD dwRemainder	= dwHeight % dwDiv;

	
	
	
	
	
	
	CMyGetDC	scrDC;

	if(scrDC.GetMyDC() == NULL)
	{
		log.Print(0,"ERROR: GetMyDC is NULL. at CaptureScreenDiv. %lu",GetLastError());
		return false;
	}
	
	
	
	
	

	for(int i = 0 ; i < dwDiv ; i++)
	{
		RECT newRect;
		newRect.left	= rect.left;
		newRect.right	= rect.right;
		newRect.top		= rect.top + (dwPart * i);
		newRect.bottom	= rect.top + (dwPart * (i+1));

		if(i == (dwDiv-1) && dwRemainder != 0)
		{
			newRect.bottom += dwRemainder;
		}

		
		
		
		if(m_bDoSleep)
			Sleep(m_dwBitBltdelay);
		

		grablog.Print(0,"PP SLEEP: %lu",m_dwBitBltdelay);
		grablog.Print(0,"PP NEWRECT: %lu %lu %lu %lu",newRect.left,newRect.top,newRect.right,newRect.bottom);

		RECT srcRect;
		SetRect(&srcRect,0,0,0,0);
		srcRect.left	= newRect.left;
		srcRect.right	= newRect.right;
		srcRect.top		= newRect.top;
		srcRect.bottom	= newRect.bottom;

		GdiFlush();

		if(!IsStretchBlit())
		{
			
			ret = BitBlt(m_hmemdc, srcRect.left, srcRect.top
						,(srcRect.right-srcRect.left)
						,(srcRect.bottom-srcRect.top)
						,scrDC.GetMyDC(), srcRect.left, srcRect.top, SRCCOPY);
			
			
		}
		else
		{
			RECT destRect;
			SetRect(&destRect,0,0,0,0);
			destRect = CheckStretchBlitRect(newRect,&srcRect);

			
			ret = StretchBlt(m_hmemdc
							,destRect.left,destRect.top
							,(destRect.right - destRect.left)
							,(destRect.bottom - destRect.top)
							,scrDC.GetMyDC(),srcRect.left,srcRect.top
							,(srcRect.right - srcRect.left)
							,(srcRect.bottom - srcRect.top)
							,SRCCOPY);
			
			

			
			
			
		}

		
		
		

		if(ddiHook.IsWin9x())
		{
			
			CMyMutexLock lock(m_csReCaptureList);
			m_rlCaptured.push_back(srcRect);
		}
	}

	return ret;
}

BOOL 
vncDesktop::CaptureScreenNorm(RECT &rect)
{
	BOOL ret = false;

	RECT srcRect;
	SetRect(&srcRect,0,0,0,0);
	srcRect.left	= rect.left;
	srcRect.right	= rect.right;
	srcRect.top		= rect.top;
	srcRect.bottom	= rect.bottom;

	GdiFlush();

	
	if(m_bFirstCapture){
		m_lpTimeStart.QuadPart = 0;
		m_lpTimeStop.QuadPart = 0;
		QueryPerformanceCounter(&m_lpTimeStart);
	}
	

	
	
	if(m_bDoSleep)
		Sleep(m_dwBitBltdelay);
	

	
	
	
	
	
	
	CMyGetDC	scrDC;

	if(scrDC.GetMyDC() == NULL)
	{
		log.Print(0,"ERROR: GetMyDC is NULL. at CaptureScreenNorm. %lu",GetLastError());
		return false;
	}
	
	
	
	
	


	
	if(!IsStretchBlit())
	{
		
		ret = BitBlt(m_hmemdc, srcRect.left, srcRect.top
					,(srcRect.right-srcRect.left)
					,(srcRect.bottom-srcRect.top)
					,scrDC.GetMyDC(), srcRect.left, srcRect.top
					,SRCCOPY);
		             
		
		
	}
	else
	{
		RECT destRect;
		SetRect(&destRect,0,0,0,0);
		destRect = CheckStretchBlitRect(rect,&srcRect);

		
		ret = StretchBlt(m_hmemdc
						,destRect.left,destRect.top
						,(destRect.right - destRect.left)
						,(destRect.bottom - destRect.top)
						,scrDC.GetMyDC(),srcRect.left,srcRect.top
						,(srcRect.right - srcRect.left)
						,(srcRect.bottom - srcRect.top)
						,SRCCOPY);
		
		

		
		
		
	}

	
	
	

	
	if(m_bFirstCapture)
	{
		QueryPerformanceCounter(&m_lpTimeStop);
		m_dwFullScreenCaptureTime = (DWORD)(((m_lpTimeStop.QuadPart - m_lpTimeStart.QuadPart) * (LONGLONG)1000) / lpFreq.QuadPart);
		
		m_bFirstCapture = false;
		
		timelog.Print(0,"FIRST CAPTURE TIME: %lu",m_dwFullScreenCaptureTime);
		
		if(m_dwFullCaptureLimitTime <= m_dwFullScreenCaptureTime)
		{
			m_bDoSleep = TRUE;
			log.Print(0,"Do sleep $B%U%k%9%/%j!<%s$N%-%c%W%A%c$K;~4V$,$+$+$C$?$N$G9u2hLLBP1~$r9T$$$^$9!#(B");
		} else {
			log.Print(0,"No sleep $B%U%k%9%/%j!<%s$N%-%c%W%A%c$O=<J,9bB.$G$9$N$G9u2hLLBP1~$r9T$$$^$;$s!#(B");
		}

		
		
		
		
		
		
		if(ddiHook.IsWin9x()){
			m_bDoSleep = FALSE;
			log.Print(0,"Win9x$B7O$J$N$G9u2hLLBP1~$r9T$$$^$;$s!#(B");
		}
		

	}
	

	if(ddiHook.IsWin9x())
	{
		
		CMyMutexLock lock(m_csReCaptureList);
		m_rlCaptured.push_back(srcRect);
	}

	return ret;
}


void
vncDesktop::SetMouseRect()
{
	POINT CursorPos;
	ICONINFO IconInfo;


	CMyMutexLock lock(m_csMouse);

	CURSORINFO pci;
	pci.cbSize = sizeof(CURSORINFO);
	
	if(GetCursorInfo(&pci))
	{
		SetMyCursor(pci.hCursor);
	}
	else
	{
		log.Print(0,"MOUSE: ERROR Last error = %lu",GetLastError());
		return;
	}

	
	CursorPos.x = pci.ptScreenPos.x;
	CursorPos.y = pci.ptScreenPos.y;

	
	if (GetIconInfo(m_hcursor, &IconInfo))
	{
		CursorPos.x -= ((int) IconInfo.xHotspot);
		CursorPos.y -= ((int) IconInfo.yHotspot);

		
		
		
		
		
		

		if (IconInfo.hbmMask != NULL)
			DeleteObject(IconInfo.hbmMask);
		if (IconInfo.hbmColor != NULL)
			DeleteObject(IconInfo.hbmColor);
	}
	else
	{
		log.Print(0,"MOUSE ERROR: GetIconInfo() failed.");
	}

	
	m_cursorpos.left	= CursorPos.x;
	m_cursorpos.top		= CursorPos.y;
	m_cursorpos.right	= CursorPos.x + GetSystemMetrics(SM_CXCURSOR);
	m_cursorpos.bottom	= CursorPos.y + GetSystemMetrics(SM_CYCURSOR);

	
	
	
	
	

	
	RECT screen;
	screen.left		= 0;
	screen.top		= 0;
	
	
	screen.right	= m_scrinfo.framebufferWidth;
	screen.bottom	= m_scrinfo.framebufferHeight;

	
	IntersectRect(&m_cursorpos, &m_cursorpos, &screen);

	rectlog.Print(0,"MOUSE: SET Rect %d %d %d %d  : %d x %d"
					,m_cursorpos.left,m_cursorpos.top,m_cursorpos.right,m_cursorpos.bottom
					,m_cursorpos.right-m_cursorpos.left,m_cursorpos.bottom-m_cursorpos.top);

	return;
}


void
vncDesktop::CaptureMouse(BYTE *scrBuff, UINT scrBuffSize)
{
	POINT CursorPos;
	ICONINFO IconInfo;

	
	if (m_hcursor == NULL){
		return;
	}

	CMyMutexLock lock(m_csMouse);

	
	CURSORINFO pci;
	pci.cbSize = sizeof(CURSORINFO);

	if(GetCursorInfo(&pci))
	{
		SetMyCursor(pci.hCursor);
	}
	else
	{
		log.Print(0,"MOUSE: ERROR Last error = %lu",GetLastError());
		return;
	}

	
	
	
	CursorPos.x = pci.ptScreenPos.x;
	CursorPos.y = pci.ptScreenPos.y;

	
	if (GetIconInfo(m_hcursor, &IconInfo))
	{
		CursorPos.x -= ((int) IconInfo.xHotspot);
		CursorPos.y -= ((int) IconInfo.yHotspot);

		
		

		
		
			
		

		rectlog.Print(0,"MOUSE: Hotspot x = %lu y= %lu"
			,IconInfo.xHotspot,IconInfo.yHotspot);

		if (IconInfo.hbmMask != NULL)
			DeleteObject(IconInfo.hbmMask);
		if (IconInfo.hbmColor != NULL)
			DeleteObject(IconInfo.hbmColor);
	}
	else
	{
		log.Print(0,"MOUSE WARNING: GetIconInfo() failed.");
	}

	
	HBITMAP oldbitmap;
	if ((oldbitmap = (HBITMAP) SelectObject(m_hmemdc, m_membitmap)) == NULL)
	{
		return;
	}

	
	
	m_cursorpos.left	= CursorPos.x;
	m_cursorpos.top		= CursorPos.y;
	m_cursorpos.right	= CursorPos.x + GetSystemMetrics(SM_CXCURSOR);
	m_cursorpos.bottom	= CursorPos.y + GetSystemMetrics(SM_CYCURSOR);

	
	RECT offsetRect;
	SetRect(&offsetRect,0,0,0,0);

	offsetRect = m_cursorpos;
	
	if(IsStretchBlit())
	{
		offsetRect = CheckMouseStretchRect(m_cursorpos);
	}

	
	if(pci.flags != 0)
	{
		DrawIconEx(
			m_hmemdc,									
			
			offsetRect.left,offsetRect.top,
			m_hcursor,									
			
			
			
			offsetRect.right-offsetRect.left,
			offsetRect.bottom-offsetRect.top,
			0,											
			NULL,										
			
			
			
			DI_NORMAL
			
			);
	}
	

	
	
	RECT screen;
	screen.left		= 0;
	screen.top		= 0;
	screen.right	= m_dwBitmapWidth;
	screen.bottom	= m_dwBitmapHeight;

	
	if(IntersectRect(&offsetRect, &offsetRect, &screen))
		CopyToBuffer(offsetRect, scrBuff, scrBuffSize);

	
	
	
	screen.right	= m_scrinfo.framebufferWidth;
	screen.bottom	= m_scrinfo.framebufferHeight;

	IntersectRect(&m_cursorpos, &m_cursorpos, &screen);

	
	SelectObject(m_hmemdc, oldbitmap);

	rectlog.Print(0,"MOUSE: Capture Rect %d %d %d %d  : %d x %d"
					,offsetRect.left,offsetRect.top,offsetRect.right,offsetRect.bottom
					,offsetRect.right-offsetRect.left,offsetRect.bottom-offsetRect.top);
}


RECT
vncDesktop::CheckMouseStretchRect(RECT &rect)
{
	rectlog.Print(0,"ORIGINAL MOUSE RECT: %lu %lu %lu %lu"
		,rect.left,rect.top,rect.right,rect.bottom);

	RECT newRect;
	SetRect(&newRect,0,0,0,0);

	newRect.left	= rect.left   * m_dwClientWidthRatio  / m_dwServerWidthRatio;
	newRect.right	= rect.right  * m_dwClientWidthRatio  / m_dwServerWidthRatio;
	newRect.top		= rect.top    * m_dwClientHeightRatio / m_dwServerHeightRatio;
	newRect.bottom	= rect.bottom * m_dwClientHeightRatio / m_dwServerHeightRatio ;

	rectlog.Print(0,"NEW MOUSE RECT: %lu %lu %lu %lu"
		,newRect.left,newRect.top,newRect.right,newRect.bottom);

	return newRect;
}


RECT
vncDesktop::MouseRect()
{
	RECT mouseRect;

	
	{
		CMyMutexLock lock(m_csMouse);
		CopyRect(&mouseRect,&m_cursorpos);
	}

	return mouseRect;
}

void
vncDesktop::SetMyCursor(HCURSOR cursor)
{
	if (cursor == NULL)
	{
		m_hcursor = m_hdefcursor;
		
	}
	else
	{
		m_hcursor = cursor;

		
		
		
		

		
	}
}



inline void
vncDesktop::MaskToMaxAndShift(DWORD mask, CARD16 &max, CARD8 &shift)
{
	for (shift = 0; (mask & 1) == 0; shift++)
		mask >>= 1;
	max = (CARD16) mask;
}


BOOL
vncDesktop::CopyToBuffer(RECT &rect, BYTE *destbuff, UINT destbuffsize)
{
	
	if (destbuff == m_DIBbits) {
		
		return true;
	}

	int y_inv;
	BYTE * destbuffpos;

	
	
	
	y_inv = m_dwBitmapHeight-rect.top-(rect.bottom-rect.top);

	
	destbuffpos = destbuff + (m_bytesPerRow * rect.top);

	
	
	m_bminfo.bmi.bmiHeader.biSizeImage = (rect.bottom-rect.top) * m_bytesPerRow;

	
	
	if(m_DIBbits == NULL) {
		
		
		
		
		if (GetDIBits(m_hmemdc, m_membitmap, y_inv,
					(rect.bottom-rect.top), destbuffpos,
					&m_bminfo.bmi, DIB_RGB_COLORS) == 0)
		
		{
			_RPT1(_CRT_WARN, "vncDesktop : [1] GetDIBits failed! %d\n", GetLastError());
		
			_RPT2(_CRT_WARN, "vncDesktop : y = %d, height = %d\n", y_inv, (rect.bottom-rect.top));
		}
	} 
	else
	{
		log.Print(0,"ERROR: m_DIBbits != NULL even when fast blits is being used.");
		return false;
	}

	return true;
}

BOOL vncDesktop::CheckHookValidate(RECT & rect)
{
	BOOL bRet = TRUE;
	rectlist::iterator i;
	int loops = 0;

	CMyMutexLock lock(m_csReCaptureList);
	
	for (i=m_rlCaptured.begin(); i != m_rlCaptured.end(); i++)
	{
		
		if((*i) == rect)
		{	
			if(loops < 10)
				m_rlCaptured.erase(i);
			else
			{
				m_rlCaptured.erase(m_rlCaptured.begin(),i);
				rectlog.Print(0,"IGNORED: Loops = %d greater than 10",loops);
			}

			bRet = FALSE;

			rectlog.Print(0,"IGNORED: Rect %d %d %d %d  loops = %d" 
						,rect.left,rect.top,rect.right,rect.bottom,loops);
			break;
		}

		loops++;
	}

	return bRet;
}


BOOL 
vncDesktop::HookInit()
{
	
	
	
	
	
	if(!ddiHook.call_Initialize()){
		log.Print(0,"DESKTOP ERROR: ddiHook.call_Initialize Failed.");
		return FALSE;
	}
	
	
	m_bSetsuzokuChuu = TRUE;

	return TRUE;
}


BOOL 
vncDesktop::HookFinalize()
{
	CMyMutexLock lock(m_csShutdown);

	
	
	
	
	
	
	if(!ddiHook.call_Finalize() && m_bSetsuzokuChuu)
	{
		log.Print(0,"DESKTOP ERROR: call_Finalize() failed.");
		return FALSE;
	}
	
	
	
	m_bSetsuzokuChuu = FALSE;
	

	log.Print(0,"DESKTOP: HookFinalize() called.");
	return TRUE;
}



BOOL 
vncDesktop::InitThreads()
{
	
	
	if(!m_pClientTrd->CreateVNeSThread(ClientThreadProc,this,CREATE_SUSPENDED))
	{
		log.Print(0,"THREAD ERROR: Client CreateVNeSThread() failed");
		return FALSE;
	}

	if(!m_pDDIHookTrd->CreateVNeSThread(DDIHookThreadProc,this,CREATE_SUSPENDED))
	{	
		log.Print(0,"THREAD ERROR: DDIHook CreateVNeSThread() failed");
		return FALSE;
	}
	
	return TRUE;
}


LRESULT CALLBACK 
DesktopWndProc (HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	
	
	vncDesktop *_this = (vncDesktop*)GetWindowLongPtr(hwnd, GWLP_USERDATA);
	
	
	
	
	
	
	
	
	
	
	
	
	
	

     switch (message)
     {
     case WM_CREATE:
          log.Print(0,"WM_CREATE Message received.");
          return 0 ;

     case WM_PAINT:
          log.Print(0,"WM_PAINT Message received.");
          return 0 ;
          
	 case WM_CLOSE:	
		  log.Print(0,"WM_CLOSE message received.");
		  DestroyWindow(hwnd);
		  return 0;

     case WM_DESTROY:	
		  log.Print(0,"WM_DESTROY Message received.");
          PostQuitMessage(wParam);
          return 0 ;
	
	 case WM_DISPLAYCHANGE:
		 
		 if(!_this->OnDisplayChanged(wParam,lParam))
		 {
		 }
		return 0;

	 case WM_SYSCOLORCHANGE:
		 log.Print(0,"WM_SYSCOLORCHANGE message received. %d",message);
		 return 0;

	 case WM_PALETTECHANGED:
		
		 log.Print(0,"WM_PALETTECHANGED message received. %d",message);
		 
		  if(!_this->OnPaletteChanged())
			  _this->QuitAppMessage(VERROR_Palette);

		 return 0;
	 
	 case WM_NCMOUSEMOVE:
	 case WM_MOUSEMOVE:
		 log.Print(0,"WM_MOUSEMOVE Message received.");
		 
		 return 0;

	 case VNES_DRIVER_ERROR:

		 _this->OnVNeSDriverError();
		return 0;

	 case VNES_CAPTURE_ERROR:

		 _this->OnVNeSClientLost();
		return 0;

	 default:
		return DefWindowProc (hwnd, message, wParam, lParam) ;
     }
}


BOOL 
vncDesktop::OnMouseMove(WPARAM wParam,LPARAM lParam)
{
	BOOL ret = TRUE;

	SetMyCursor((HCURSOR)wParam);
	
	
	return ret;
}


BOOL 
vncDesktop::OnDisplayChanged(WPARAM wParam,LPARAM lParam)
{
	
	
	

	BOOL ret = TRUE;

	 log.Print(0,"Display Changed  %dx%d wxh %d bpp"
			        ,lParam&0x0000FFFF,(lParam&0xFFFF0000)>>16,wParam); 
	
	 
	 rfbServerInitMsg newScreen = m_REALscrinfo;
	 newScreen.framebufferHeight = (unsigned short)((lParam&0xFFFF0000)>>16);
	 newScreen.framebufferWidth = (unsigned short)(lParam&0x0000FFFF);
	 newScreen.format.bitsPerPixel = (unsigned char)wParam;

	
	log.Print(0, VNCLOG("SCR: old screen format %dx%dx %d\n"),
			m_REALscrinfo.framebufferWidth,
			m_REALscrinfo.framebufferHeight,
			m_REALscrinfo.format.bitsPerPixel);
	
	
	if (memcmp(&m_REALscrinfo, &newScreen, sizeof(newScreen)) != 0)
	{
		m_bDisconnectReq = TRUE;
		
		if(m_pClientTrd != NULL || m_pDDIHookTrd != NULL)
		{
			ShutdownThreads();  
			Shutdown();
			HookFinalize();

			
			EVResolutionChange();
		}
	}
	else
	{
		ret = FALSE;
	}

	return ret;
}


BOOL 
vncDesktop::OnPaletteChanged()
{
	BOOL ret = TRUE;

	log.Print(0,"DESKTOP: Palette Changed"); 

	
	if (!SetPalette())
	{
		ret = FALSE;
	}
	else
	{
		SetPaletteChanged(TRUE);
	}

	return ret;
}

BOOL	
vncDesktop::OnEVConnectRequest(CEVStream &ConnInfo)
{
	
	if(m_bSetsuzokuChuu)
	{
		
		log.Print(0,"IGNORE: Connect Request, Hook UnFinalized.");
		return FALSE;
	}

	
	m_bEncryption = ConnInfo.GetEncryption();
	deslog.Print(0,"INIT: Encryption = %d", m_bEncryption);

	
	
	
	unsigned char encryptionKey[9] = {0,0,0,0,0,0,0,0,0};
	
	
	
	
	memcpy_ex(encryptionKey,ConnInfo.GetEncKey(),MAXKEYLENGTH);
	

	
	
	
	rdSetKey(encryptionKey,MAXKEYLENGTH);

	
	if(!Init())
	{
		log.Print(0, "DESKTOP ERROR: Init() failed.");
		return FALSE;
	}

	
	if(!m_pClientTrd->CreateSendBuff(GetDataBuffSize()))
	{
		log.Print(0,"ERROR: CreateSendBuff() failed.");
		return FALSE;
	}
	

	
	int nClientcount = ConnInfo.GetCount();
	evlog.Print(0, "Client count = %d",nClientcount);

	if(nClientcount == 0 || nClientcount > VNES_MAXCONNECTIONS)
	{
		
		log.Print(0, "ERROR: Invalid client count %d.",nClientcount);
		return FALSE;
	}

	
	

	DWORD dwServerIP = ConnInfo.GetSvrAddr();
	socklog.Print(0,"Init: ServerIP = 0x%8x",dwServerIP);

	for(int nClientIndex = 0; nClientIndex < nClientcount; nClientIndex++)
	{
		VSocket* pNewSocket = new VSocket();

		if(pNewSocket == NULL)
		{
			log.Print(0,"ERROR: memory pNewSocket is NULL client count = %d",m_dwSendTrdCount);
			return FALSE;
		}

		
		pNewSocket->SetIPAddress(dwServerIP);

		if(!pNewSocket->Create())
		{
			evlog.Print(0, "SOCKET ERROR: Create() failed client count %d",m_dwSendTrdCount);
			delete pNewSocket;
			pNewSocket = NULL;
			return FALSE;
		}

		
		DWORD	dwIPAdd = ConnInfo.GetIPAddress(nClientIndex);
		
		in_addr address;
		address.S_un.S_addr = dwIPAdd;
		
		
		char clientHostname[EVS_IPADDRESS_LENGTH];
		if(_snprintf(clientHostname,EVS_IPADDRESS_LENGTH,"%s",inet_ntoa(address)) < 0)
		{
			
			log.Print(0,"ERROR: Client's name exceeds EVS_IPADDRESS_LENGTH");
			delete pNewSocket;
			pNewSocket = NULL;
			return FALSE;
		}
		evlog.Print(0,"client name = %s 0x%8x",clientHostname,dwIPAdd);
		
		
		if(!pNewSocket->Connect(dwIPAdd,INCOMING_PORT_OFFSET))
		{
			log.Print(0,"ERROR: Connect() failed.");
			delete pNewSocket;
			pNewSocket = NULL;
			return false;
		}

		
		if(!InitSendThread(pNewSocket,dwIPAdd))
		{
			log.Print(0,"ERROR: InitSendThread() failed.");
			delete pNewSocket;
			pNewSocket = NULL;
			return false;
		}

		
		
	}

	if(!InitThreads())
	{
		log.Print(0, "DESKTOP ERROR: InitThreads() failed");
		return FALSE;
	}

	
	if(!m_pClientTrd->SetVNeSThreadPriority(m_dwClientTrdPriority))
	{
		log.Print(0, "DESKTOP WARNING: SetVNeSThreadPriority() ClientTrd failed.");
	}
	if(!m_pDDIHookTrd->SetVNeSThreadPriority(m_dwHookTrdPriority))
	{
		log.Print(0, "DESKTOP WARNING: SetVNeSThreadPriority() ClientTrd failed.");
	}
	

	
	int nCTcheck = (int) m_pClientTrd->ResumeVNeSThread();
	int nDTcheck = (int) m_pDDIHookTrd->ResumeVNeSThread();

	if(nCTcheck == -1 ||			
	   nDTcheck == -1 ||
	   nCTcheck > 1   ||			
	   nDTcheck > 1)
	{
		log.Print(0,"ERROR: Unable to Resume thread.");
		log.Print(0,"ERROR: nCTcheck = %d  nDTcheck = %d",nCTcheck,nDTcheck);
		return FALSE;
	}

	evlog.Print(0, "OnEVConnectRequest() successful.");
	return TRUE;
}


BOOL 
vncDesktop::OnEVDisConnectRequest()
{
	BOOL ret = TRUE;

	
	evlog.Print(0, "EV_DISCONNECT_REQUEST message received");
	
	
	
	
	m_bDisconnectReq = TRUE;

	
	ShutdownThreads();
	Shutdown();
	HookFinalize();

	evlog.Print(0, "OnEVDisconnectRequest() DONE.");

	return ret;
}

BOOL	
vncDesktop::OnEVSentDataSizeRequest(DWORD &dwBytes)
{
	evlog.Print(0, "OnEVSentDataSizeRequest() called.");

	dwBytes = 0;

	BOOL bAlive = false;

	{
		CMyMutexLock clientlock(m_csClientArray);
		for(int s = 0; s < m_dwSendTrdCount ; s++)
		{
			if(m_pSendTrds[s] == NULL)
				continue;
			
			if(m_pSendTrds[s]->IsAlive())
			{
				dwBytes += m_pSendTrds[s]->GetSentBytes();
				bAlive = true;
			}
		}
	}

	evlog.Print(0,"SENT DATA: Bytes = %lu" ,dwBytes);

	return bAlive;
}

BOOL 
vncDesktop::OnEVImageSendStart()
{
	evlog.Print(0, "EV_IMAGE_SEND_START message received");

	if(m_pDDIHookTrd != NULL && 
	   m_pClientTrd != NULL)
	{
		
		while((int)m_pClientTrd->ResumeVNeSThread() > 1){ }
		while((int)m_pDDIHookTrd->ResumeVNeSThread() > 1){ }
		
		
		
	}
	else
	{
		evlog.Print(0, "IMAGE_SEND_START ERROR: Failed. Threads NULL.");
		return FALSE;
	}

	return TRUE;
}


BOOL 
vncDesktop::OnEVImageSendStop()
{
	evlog.Print(0, "EV_IMAGE_SEND_STOP message received");

	if(m_pDDIHookTrd != NULL && 
	   m_pClientTrd != NULL)
	{
		
		
		
		
		CMyMutexLock lock(m_csRegionLock);
		
		int nDTcheck = m_pDDIHookTrd->SuspendVNeSThread();
		int nCTcheck = m_pClientTrd->SuspendVNeSThread();
		
		if(nDTcheck == -1 || nCTcheck == -1)
		{
			evlog.Print(0,"ERROR: Couldn't suspend threads at OnEVImageSendStop.");
			return FALSE;
		}
		
		
		m_vrUpdateReg.Clear();
		
		
		m_vrUpdateReg.AddRect(m_pClientTrd->m_fullscreen);

	}
	else
	{
		evlog.Print(0, "IMAGE SEND STOP ERROR: Failed. Threads NULL.");
		return FALSE;
	}
	
	return TRUE;
}



BOOL 
vncDesktop::OnVNeSDriverError()
{
	BOOL ret = TRUE;
	
	evlog.Print(0,"VNES_DRIVER_ERROR messaged received.");

	if(m_pDDIHookTrd != NULL || 
	   m_pClientTrd != NULL)
	{	
		ShutdownThreads();
		Shutdown();
		HookFinalize();

		EVDriverError();
	}
	else
	{
		ret = FALSE;
	}

	return ret;
}


BOOL 
vncDesktop::OnVNeSClientLost()
{
	BOOL ret = TRUE;

	evlog.Print(0,"VNES_CAPTURE_ERROR messaged received.");

	
	
	
	if(m_pDDIHookTrd != NULL || 
	   m_pClientTrd != NULL)
	
	{	
		DWORD dwCount = m_dwSendTrdCount;
		DWORD dwIPArray[VNES_MAXCONNECTIONS];
		
		{
			
			
			CMyMutexLock clientlock(m_csClientArray);

			for(int s = 0; s < dwCount ; s++)
			{
				dwIPArray[s] = 0;

				if(m_pSendTrds[s] == NULL)
					continue;
			
				dwIPArray[s] = m_pSendTrds[s]->GetIPAdd();
			}
		}

		ShutdownThreads();
		Shutdown();
		HookFinalize();

		for(int b = 0; b < dwCount; b++)
		{
			EVClientLost(dwIPArray[b]);
		}
	}
	else
	{
		ret = FALSE;
	}

	return ret;
}












BOOL
vncDesktop::EVDriverError()
{
	QuitAppMessage(VERROR_HookDriver);

	evlog.Print(0,"EV_VNES_DRIVER_ERROR message sent");

	return TRUE;
}


BOOL

vncDesktop::EVClientLost()
{	
	
	if(!m_bDisconnectReq)
	{
		
		

		
		
		if(!PostMessage(ghEMPWnd,EV_COMMAND,clientLost,0))
			return FALSE;

		evlog.Print(0,"EV_DISCONNECT_CLIENT message sent");
	}
	
	return TRUE;
}


BOOL	
vncDesktop::EVClientLost(DWORD dwIP)
{
	
	if(!m_bDisconnectReq)
	{
		if(!PostMessage(ghEMPWnd,EV_COMMAND,clientLost,dwIP))
			return FALSE;

		evlog.Print(0,"EV_DISCONNECT_CLIENT message sent 0x%8x",dwIP);
	}
	
	return TRUE;
}


BOOL
vncDesktop::EVResolutionChange()
{    
	evlog.Print(0,"EV_RESOLUTION_CHANGE message sent");
	return PostMessage(ghEMPWnd,EV_COMMAND,resolutionChange,0);
}

void 
vncDesktop::PostDriverErrorMsg()
{
	PostMessage(m_hwnd,VNES_DRIVER_ERROR,0,0);
	log.Print(0,"PostDriverErrorMsg() called. ");
}

void 
vncDesktop::PostCaptureErrorMsg()
{
	PostMessage(m_hwnd,VNES_CAPTURE_ERROR,0,0);
	log.Print(0,"PostCaptureErrorMsg() called. ");
}

BOOL 
vncDesktop::QuitAppMessage(DWORD dwErrorCode)
{
	
	if(!PostMessage(ghEMPWnd,EV_COMMAND,vnesError,dwErrorCode))
	{
		PostCloseMessage();
		return FALSE;
	}

	log.Print(0,"QuitAppMessage() called. ");

	return TRUE;
}


BOOL 
vncDesktop::PostCloseMessage()
{
	return PostMessage(m_hwnd,WM_CLOSE,0,0);
	
}












BOOL	
vncDesktop::LoadRegistry()
{
	CRegistry reg;
	
	
	
	
	if(reg.open(VNES_REGISTRY_KEY, VNES_REGISTRY_SUBKEY, KEY_ALL_ACCESS))
	{
		
		if(!reg.getBOOL(VNES_REGKEY_RECTGRABSTYLE, &m_bGrabStyle))
			m_bGrabStyle = VNES_DEF_GRABSTYLE;

		
		if(!reg.getBOOL(VNES_REGKEY_RECTSENDSTYLE, &m_bSendStyle))
			m_bSendStyle = VNES_DEF_SENDSTYLE;

		
		if(!reg.getBOOL(VNES_REGKEY_MAKE_HOOK_RECT_ONE, &m_bHookedRectIntoOne))
			 m_bHookedRectIntoOne	= VNES_DEF_HOOKRECTINTOONE;
			 
		
		if(!reg.getINT(VNES_REGKEY_ENCODINGTYPE, &m_nEncodingType))
			m_nEncodingType = VNES_DEF_RFB_ENCODING;
	
		
		if(!reg.getINT(VNES_REGKEY_ENC_COMP_LEVEL, &m_nCompressionLevel))
			m_nCompressionLevel = VNES_DEF_COMPRESSIONLEVEL;

		
		if(!reg.getINT(VNES_REGKEY_ENC_JPEG_LEVEL, &m_nJpegQualityLevel))
			m_nJpegQualityLevel = VNES_DEF_JPEQ_QUALITYLEVEL;

		
		if(!reg.getDWORD(VNES_REGKEY_HOOK_LIMIT, &m_dwHookCaptureLimit))
			m_dwHookCaptureLimit= VNES_DEF_HOOKCAPTURE_LIMIT;

		
		if(!reg.getDWORD(VNES_REGKEY_CLIENT_WAIT, &m_dwClientWaitDelay))
			m_dwClientWaitDelay= VNES_DEF_CLIENTTHREAD_DELAY;

		
		
		if(!reg.getDWORD(VNES_REGKEY_CLIENT_WAIT_SLOW, &m_dwClientWaitDelaySlow))
			m_dwClientWaitDelaySlow= VNES_DEF_CLIENTTHREAD_DELAY_SLOW;

		if(!reg.getDWORD(VNES_REGKEY_CLIENT_WAIT_SLOW_LIMIT_RECTS, &m_dwClientWaitDelaySlowLimitRects))
			m_dwClientWaitDelaySlowLimitRects= VNES_DEF_CLIENTTHREAD_DELAY_SLOW_LIMIT_RECTS;

		
		
		
		
		

		
		if(!reg.getDWORD(VNES_REGKEY_DDIHOOK_WAIT, &m_dwDDIHookWaitDelay))
			m_dwDDIHookWaitDelay= VNES_DEF_DDIHOOKTHREAD_DELAY;

		
		
		if(!reg.getDWORD(VNES_REGKEY_FULLSCREEN_COUNT, &m_dwFullScreenCount))
			m_dwFullScreenCount= VNES_DEF_FULLSCREEN_COUNT;

		
		
		if(!reg.getDWORD(VNES_REGKEY_FULLSCREEN_DELAY, &m_dwFullScrRefreshDelay))
			m_dwFullScrRefreshDelay= VNES_DEF_FULLSCREEN_DELAY;

		
		
		if(!reg.getBOOL(VNES_REGKEY_FULLSCREEN_SEND, &m_bSendFullScreen))
			m_bSendFullScreen	= VNES_DEF_FULLSCREEN_BSEND;

		
		if(!reg.getDWORD(VNES_REGKEY_REGION_BUFFSIZE, &m_dwRegionBuffSize))
			m_dwRegionBuffSize	= VNES_DEF_MAX_RGNBUFFSIZE;
		

		
		
		if(!reg.getDWORD(VNES_REGKEY_CLIENT_PRIORITY, &m_dwClientTrdPriority))
			m_dwClientTrdPriority= VNES_DEF_CLIENT_PRIORITY;
		
		
		if(!reg.getDWORD(VNES_REGKEY_DDIHOOK_PRIORITY, &m_dwHookTrdPriority))
			m_dwHookTrdPriority= VNES_DEF_HOOK_PRIORITY;

		
		if(!reg.getBOOL(VNES_REGKEY_CHECK_DETAIL_AREA, &m_bCheckDetailArea))
			m_bCheckDetailArea = VNES_DEF_CHECK_DETAIL_AREA;
		
		
		if (!reg.getDWORD(VNES_REGKEY_HOOK_DETAIL_DELAY, &m_dwHookDetailDelay))
			m_dwHookDetailDelay = VNES_DEF_HOOK_DETAIL_DELAY; 

		if (!reg.getDWORD(VNES_REGKEY_DETAIL_AREA, &m_dwDetailArea))
			m_dwDetailArea = VNES_DEF_DETAIL_AREA; 
		
		if (!reg.getDWORD(VNES_REGKEY_DETAIL_WIDTH, &m_dwDetailWidth))
			m_dwDetailWidth = VNES_DEF_DETAIL_WIDTH; 

		if (!reg.getDWORD(VNES_REGKEY_DETAIL_HEIGHT, &m_dwDetailHeight))
			m_dwDetailHeight= VNES_DEF_DETAIL_HEIGHT; 

		if (!reg.getDWORD(VNES_REGKEY_DETAIL_LIMIT, &m_dwDetailLimit))
			m_dwDetailLimit= VNES_DEF_DETAIL_LIMIT; 
		

		
		if(!reg.getDWORD(VNES_REGKEY_BITBLT_DELAY, &m_dwBitBltdelay))
			m_dwBitBltdelay= VNES_DEF_BITBLT_DELAY;

		if(!reg.getDWORD(VNES_REGKEY_PP_SCR_DIV, &m_dwScreenDivision))
			m_dwScreenDivision = VNES_DEF_PP_SCR_DIV;

		if(!reg.getDWORD(VNES_REGKEY_PP_PERCENT, &m_dwPercentScreen))
			m_dwPercentScreen = VNES_DEF_PP_SCR_PERCENT;

		if(!reg.getBOOL(VNES_REGKEY_PP_BOOL, &m_bPowerBlackOut))
			m_bPowerBlackOut = VNES_DEF_BOOL_POWERPOINT;

		
		
		
		

		
		if(!reg.getDWORD(VNES_REGKEY_FULLCAPTURE_LIMITTIME, &m_dwFullCaptureLimitTime))
			m_dwFullCaptureLimitTime = VNES_DEF_FULLCAPTURE_LIMITTIME;
		
		if(!reg.getDWORD(VNES_REGKEY_EDGECHECK_COLOR_PERCENT, &m_dwEdgeCheckColorPercent))
			m_dwEdgeCheckColorPercent = VNES_DEF_EDGECHECK_COLOR_PERCENT;
		
		if(!reg.getDWORD(VNES_REGKEY_EDGECHECK_SAMPLING_PIXELS, &m_dwEdgeCheckSamplingPixels))
			m_dwEdgeCheckSamplingPixels = VNES_DEF_EDGECHECK_SAMPLING_PIXELS;
		
		if(!reg.getBOOL(VNES_REGKEY_OLDEDGECHECK, &m_bOldEdgeCheck))
			m_bOldEdgeCheck = VNES_DEF_OLDEDGECHECK;

		if(!reg.getDWORD(VNES_REGKEY_RESEND_JPEG_RECTS, &m_dwResendJpegRects))
			m_dwResendJpegRects = VNES_DEF_RESEND_JPEG_RECTS;
		
		if(!reg.getDWORD(VNES_REGKEY_RESEND_ZLIB_AREA_LIMIT, &m_dwResendZlibAreaLimit))
			m_dwResendZlibAreaLimit = VNES_DEF_RESEND_ZLIB_AREA_LIMIT;	
		

		
		if(!reg.getDWORD(VNES_REGKEY_SENDTRD_PRIORITY, &m_dwSendTrdPriority))
			m_dwSendTrdPriority= VNES_DEF_SENDTRD_PRIORITY;

		if(!reg.getDWORD(VNES_REGKEY_MAX_BUFFLIST, &m_dwBuffListMaxSize))
			m_dwBuffListMaxSize= VNES_DEFAULTMAXBUFFERSIZE;

		
		if(!reg.getDWORD(VNES_REGKEY_SEND_WAIT_LIMIT, &m_dwSendTreadWaitLimit))
			m_dwSendTreadWaitLimit = VNES_DEF_SEND_WAIT_LIMIT;
	
		reg.close();

		
		
		
		
		

		
		if(m_dwBuffListMaxSize < 1)
			m_dwBuffListMaxSize = 1;

		
		if(m_dwFullScreenCount == 0)
		{
			
			
			log.Print(0,"WARNING: Can't set fullscreen count to zero. \r\n"
				         "Using default value = %lu",VNES_DEF_FULLSCREEN_COUNT);
			m_dwFullScreenCount = VNES_DEF_FULLSCREEN_COUNT;
		}

		if(m_dwRegionBuffSize == 0)
		{
			log.Print(0,"WARNING: Can't set region buff size to zero. \r\n"
				         "Using default value = %lu",VNES_DEF_MAX_RGNBUFFSIZE);
			m_dwRegionBuffSize	= VNES_DEF_MAX_RGNBUFFSIZE;
		}
		
		if(m_nEncodingType < 0 || 
		   m_nEncodingType == rfbEncodingCopyRect ||
		   m_nEncodingType == 3 ||
		   m_nEncodingType >= 8)
		{
			
			
			log.Print(0,"WARNING: Unknown encoding type. \r\n"
				         "Using = %d (Tight = 7)",VNES_DEF_RFB_ENCODING);
			m_nEncodingType = VNES_DEF_RFB_ENCODING;
		}
	}
	

	
	if(m_bSendStyle)
		timelog.Print(0,"SENDING STYLE: ONE RECT");
	else
		timelog.Print(0,"SENDING STYLE: MANY RECTS");

	if(m_bHookedRectIntoOne)
		timelog.Print(0,"HOOKING STYLE: ONE RECT");
	else
		timelog.Print(0,"HOOKING STYLE: MANY RECTS");
	
	timelog.Print(0,"HookCaptureLimit = %lu ",m_dwHookCaptureLimit);
	timelog.Print(0,"CompressionLevel = %d  ",m_nCompressionLevel);
	timelog.Print(0,"JpegQualityLevel = %d  ",m_nJpegQualityLevel);
	timelog.Print(0,"EncodingType	  = %d  ",m_nEncodingType);
	

	log.Print(0,"bGrabStyle = %d",m_bGrabStyle);
	log.Print(0,"bSendStyle = %d",m_bSendStyle);
	log.Print(0,"bHookedRectIntoOne = %d",m_bHookedRectIntoOne);		
	log.Print(0,"bSendFullScreen = %d",m_bSendFullScreen);	
	log.Print(0,"bCheckDetailArea = %d",m_bCheckDetailArea);		
	log.Print(0,"EncodingType = %d",m_nEncodingType);
	log.Print(0,"CompressionLevel = %d",m_nCompressionLevel);
	log.Print(0,"JpegQualityLevel = %d",m_nJpegQualityLevel);
	log.Print(0,"HookCaptureLimit = %lu",m_dwHookCaptureLimit);
	log.Print(0,"ClientWaitDelay = %lu",m_dwClientWaitDelay);
	log.Print(0,"DDIHookWaitDelay = %lu",m_dwDDIHookWaitDelay);
	log.Print(0,"FullScreenCount = %lu",m_dwFullScreenCount);
	log.Print(0,"FullScrRefreshDelay = %lu",m_dwFullScrRefreshDelay);
	log.Print(0,"RegionBuffSize = %lu",m_dwRegionBuffSize);
	log.Print(0,"ClientTrdPriority = %lu",m_dwClientTrdPriority);
	log.Print(0,"HookTrdPriority = %lu",m_dwHookTrdPriority);
	log.Print(0,"HookDetailDelay = %lu",m_dwHookDetailDelay);
	log.Print(0,"DetailArea = %lu",m_dwDetailArea);
	log.Print(0,"DetailWidth = %lu",m_dwDetailWidth);
	log.Print(0,"DetailHeight = %lu",m_dwDetailHeight);
	log.Print(0,"DetailLimit = %lu",m_dwDetailLimit);
	log.Print(0,"Bitblt delay = %lu",m_dwBitBltdelay);
	log.Print(0,"Send Tread Priority = %lu",m_dwSendTrdPriority);
	log.Print(0,"Max Buff list = %lu",m_dwBuffListMaxSize);
	log.Print(0,"m_dwScreenDivision = %lu",m_dwScreenDivision);
	log.Print(0,"m_dwPercentScreen = %lu",m_dwPercentScreen);
	log.Print(0,"m_bPowerBlackOut = %d",m_bPowerBlackOut);
	
	
	
	log.Print(0,"m_dwSendTreadWaitLimit = %lu",m_dwSendTreadWaitLimit);
			
	return TRUE;
}


BOOL	
vncDesktop::SaveRegistry()
{
	CRegistry reg;
	if(!reg.create(VNES_REGISTRY_KEY,
					VNES_REGISTRY_SUBKEY,
					NULL,
					KEY_ALL_ACCESS)
					){
		log.Print(0,"ERROR: Reg create failed.");
		return FALSE;
	}

	if( !reg.open(VNES_REGISTRY_KEY, VNES_REGISTRY_SUBKEY, KEY_ALL_ACCESS))
		return FALSE;

	
	if(!reg.setBOOL(VNES_REGKEY_RECTGRABSTYLE, m_bGrabStyle))
		 return FALSE;

	
	if(!reg.setBOOL(VNES_REGKEY_RECTSENDSTYLE, m_bSendStyle))
		return FALSE;

	
	
	
	
	

	
	if(!reg.setBOOL(VNES_REGKEY_MAKE_HOOK_RECT_ONE, m_bHookedRectIntoOne))
		 return	FALSE;

	
	if(!reg.setINT(VNES_REGKEY_ENCODINGTYPE, m_nEncodingType))
		return	FALSE;

	
	if(!reg.setINT(VNES_REGKEY_ENC_COMP_LEVEL, m_nCompressionLevel))
		return	FALSE;

	
	if(!reg.setINT(VNES_REGKEY_ENC_JPEG_LEVEL, m_nJpegQualityLevel))
		return	FALSE;

	
	if(!reg.setDWORD(VNES_REGKEY_HOOK_LIMIT, m_dwHookCaptureLimit))
		 return	FALSE;

	
	if(!reg.setDWORD(VNES_REGKEY_CLIENT_WAIT, m_dwClientWaitDelay))
		 return	FALSE;

	
	if(!reg.setDWORD(VNES_REGKEY_DDIHOOK_WAIT, m_dwDDIHookWaitDelay))
		 return	FALSE;

	
	
	if(!reg.setDWORD(VNES_REGKEY_FULLSCREEN_COUNT, m_dwFullScreenCount))
		 return	FALSE;

	
	
	if(!reg.setDWORD(VNES_REGKEY_FULLSCREEN_DELAY, m_dwFullScrRefreshDelay))
		 return	FALSE;

	
	
	if(!reg.setBOOL(VNES_REGKEY_FULLSCREEN_SEND, m_bSendFullScreen))
		 return	FALSE;

	if(!reg.close() )
		return FALSE;

	return TRUE;
}


BOOL	
vncDesktop::SetWindowPointer()
{
	BOOL bRet = TRUE;

	 SetLastError(0);
	
	if(!SetWindowLongPtr(m_hwnd, GWLP_USERDATA, (long)this))
	{	
		if(GetLastError() != 0)
		{
			log.Print(0,"DESKTOP: ERROR - SetWindowLongPtr() Failed.");
			bRet = FALSE;
		}

		
		
	}

	return bRet;
}


BOOL 
vncDesktop::OnEVReSendUDPFullScrRect(DWORD dwRectID)
{
	socklog.Print(0,"UDP ReSend called. RectID = %lu ",dwRectID);
	
	if(m_pClientTrd == NULL)
		return FALSE;

	if(dwRectID == 0)
	{
		m_pClientTrd->AddFullScr();
		socklog.Print(0,"Fullscreen Update added to fullscreen update region.");
		return TRUE;
	}

	
	

	return TRUE;
}


BOOL	
vncDesktop::SignalSendUpdate(BOOL bSet)
{
	if(m_hSendUpdate == NULL)
		return FALSE;

	BOOL ret;

	
	{
		CMyMutexLock lock(m_csSignalEvent);
	
		if(bSet)
		{
			ret =  SetEvent(m_hSendUpdate);
			
		}
		else
		{
			ret =  ResetEvent(m_hSendUpdate);
			
		}
	}

	return ret;
}


BOOL	
vncDesktop::CopyToDataBuffer(BYTE *buff,DWORD dwSize)
{
	if(IsSendTrdEmpty())
		return false;

	if(IsEncrypt())
	{
		m_lpTimeStart.QuadPart = 0;
		m_lpTimeStop.QuadPart = 0;

		timelog.Print(0,"CAPTURE: DES Encryption Start = %lu ms"
					,GetTickCount()-gdwAppStartTime);

		QueryPerformanceCounter(&m_lpTimeStart);

		
		if(!EncryptDataBuffer(buff,dwSize))
		{
			log.Print(0,"ERROR: EncryptDataBuffer() failed.");
			return false;
		}

		QueryPerformanceCounter(&m_lpTimeStop);

		timelog.Print(0,"CAPTURE: DES Encryption Stop = %lu ms"
					,GetTickCount()-gdwAppStartTime);

		timelog.Print(0,"Encryption time = %.2f us  datasize = %lu \r\n"
				   ,(float)(m_lpTimeStop.QuadPart-m_lpTimeStart.QuadPart)/lpFreq.QuadPart*1000000
				   ,dwSize);
	}
	
	

	DWORD newSize = m_dwDataBuffptr + dwSize;

	if(newSize > m_dwDataBuffsize)
	{
		log.Print(0,"ERROR: not enough buffer size m_pDataBuffer");
		return false;
	}

	m_lpTimeStart.QuadPart = 0;
	m_lpTimeStop.QuadPart = 0;
	
	timelog.Print(0,"CAPTURE: CopyToDataBuffer Start = %lu ms"
					,GetTickCount()-gdwAppStartTime);

	QueryPerformanceCounter(&m_lpTimeStart);

	
	memcpy(m_pDataBuffer + m_dwDataBuffptr,buff,dwSize);

	
	
	memcpy_ex(m_pDataBuffer + m_dwDataBuffptr,buff,dwSize);
	

	QueryPerformanceCounter(&m_lpTimeStop);

	timelog.Print(0,"CAPTURE: CopyToDataBuffer Stop = %lu ms"
					,GetTickCount()-gdwAppStartTime);
	
	timelog.Print(0,"CAPTURE: sendbuff to data buffer (CopyToDataBuffer) time = %.2f us datasize = %lu \r\n"
				   ,(float)(m_lpTimeStop.QuadPart-m_lpTimeStart.QuadPart)/lpFreq.QuadPart*1000000
				   ,dwSize);

	
	m_dwDataBuffptr += dwSize;

	bufflog.Print(0,"CopyToDataBuffer: %lu bytes copied %lu buffer size"
				   ,dwSize,m_dwDataBuffptr);

	return true;
}

BOOL	
vncDesktop::CopyFromDataBuffer()
{
	if(IsSendTrdEmpty())
		return false;

	

	{
		CMyMutexLock clientlock(m_csClientArray);

		for(int s = 0; s < m_dwSendTrdCount ; s++)
		{
			if(m_pSendTrds[s] == NULL)
				continue;
			
			if(m_pSendTrds[s]->IsAlive())
			{
				if(!m_pSendTrds[s]->CopyData(m_pDataBuffer,m_dwDataBuffptr))
						return false;
			}
		}
	}

	

	ResetDataBuff();

	return true;
}

void
vncDesktop::ResetDataBuff()
{
	m_dwDataBuffptr = 0;
}


BOOL
vncDesktop::CreateDataBuff()
{
	
	int nBPP = 0;

	
	if(m_scrinfo.format.bitsPerPixel == 32)
		nBPP = 32/8;
	else
		nBPP = 16/8;

	vncEncoder	   *pEncoder;

	switch(m_nEncodingType)
	{
	case rfbEncodingRaw:
			pEncoder = new vncEncoder;
			break;
	case rfbEncodingRRE:
			pEncoder = new vncEncodeRRE;
			break;
	case rfbEncodingCoRRE:
			pEncoder = new vncEncodeCoRRE;
			break;
	case rfbEncodingHextile:
			pEncoder = new vncEncodeHexT;
			break;
	case rfbEncodingZlib:
			pEncoder = new vncEncodeZlib;
			break;
	case rfbEncodingZlibHex:
			pEncoder = new vncEncodeZlibHex;
			break;
	default:
			pEncoder = new vncEncodeTight;
			break;
	}
		
	if(pEncoder == NULL)
	{
		log.Print(0,"ERROR: not enough memory for pEncoder at CreateDataBuff()");
		return false;
	}

	rfbPixelFormat tempformat;
	if(m_scrinfo.format.bitsPerPixel == 32)
		tempformat.bitsPerPixel = 32; 
	else
		tempformat.bitsPerPixel = 16;

	
	pEncoder->SetRemoteFormat(tempformat);
	pEncoder->SetCompressLevel(m_nCompressionLevel);
	pEncoder->SetQualityLevel(m_nJpegQualityLevel);

	
	m_dwDataBuffsize = pEncoder->RequiredBuffSize(m_dwBitmapWidth,m_dwBitmapHeight);

	delete pEncoder;
	pEncoder = NULL;

	
	m_dwDataBuffsize += sizeof(rfbFramebufferUpdateMsg);

	
	
	
	m_dwDataBuffsize += 88;

	if(IsEncrypt())
	{
		
		m_dwDataBuffsize += sizeof(VNESENCRYPTIONHEADER) + 7;
	}

	log.Print(0,"BUFFER: size = %lu",m_dwDataBuffsize);

	m_pDataBuffer = new BYTE[m_dwDataBuffsize];

	if(m_pDataBuffer == NULL)
	{
		log.Print(0,"ERROR: not enough memory CreateDataBuff() failed.");
		return false;
	}

	ZeroMemory(m_pDataBuffer, m_dwDataBuffsize);

	return true;
}





BOOL
vncDesktop::InitSendThread(VSocket *sock,DWORD dwIPAdd)
{
	log.Print(0,"SEND THREAD: InitSendThread() called.");

	CMyMutexLock lock(m_csClientArray);

	
	m_pSendTrds[m_dwSendTrdCount] = new CSendThread();
		
	if(m_pSendTrds[m_dwSendTrdCount] == NULL)
	{
		log.Print(0,"ERROR: memory error CSendThread() failed client= %lu",m_dwSendTrdCount);
		return false;
	}

	
	m_pSendTrds[m_dwSendTrdCount]->SetIPAdd(dwIPAdd);

	
	if(!m_pSendTrds[m_dwSendTrdCount]->Init(sock,GetDataBuffSize(),m_dwBuffListMaxSize))
	{
		log.Print(0,"ERROR: send thread Init() failed ID = %d",m_dwSendTrdCount);
		return false;
	}

	
	m_SendTrdParam[m_dwSendTrdCount].pDesktop	= this;
	m_SendTrdParam[m_dwSendTrdCount].dwClientID	= m_dwSendTrdCount; 

	
	if(!m_pSendTrds[m_dwSendTrdCount]->CreateVNeSThread(SendThreadProc,&m_SendTrdParam[m_dwSendTrdCount],0))
	{	
		log.Print(0,"THREAD ERROR: send thread CreateVNeSThread() failed ID = %d",m_dwSendTrdCount);
		return false;
	}

	if(!m_pSendTrds[m_dwSendTrdCount]->SetVNeSThreadPriority(m_dwSendTrdPriority))
	{
		log.Print(0, "DESKTOP WARNING: SetVNeSThreadPriority() SendTrd failed.");
	}

	m_dwSendTrdCount++;
	log.Print(0,"CLIENT ADD: successful Clients = %lu",m_dwSendTrdCount);
		
	return true;
}


BOOL	
vncDesktop::EncryptDataBuffer(BYTE *buff,DWORD &dwSize)
{
	DWORD dwNewBufflen = dwSize;
	DWORD dwBuffPtr = 0;

	
	int nRemainder = dwSize % 8;

	InsertENCHeader(buff,dwSize);
	dwNewBufflen += sizeof(VNESENCRYPTIONHEADER);
	dwBuffPtr = sizeof(VNESENCRYPTIONHEADER);

	InsertENCData(buff + dwBuffPtr,dwSize,nRemainder);

	if(nRemainder != 0)
		dwNewBufflen += (8 - nRemainder);

	
	dwSize = dwNewBufflen;
	
	return true;
}


void	
vncDesktop::InsertENCHeader(BYTE *buff,DWORD dwBytes)
{
	VNESENCRYPTIONHEADER header;
	ZeroMemory((char *)header.pad,8);

	
	header.size = Swap32IfLE(dwBytes);

	
	rdEncryptBytes((unsigned char *)header.pad,8);

	
	memmove(buff + sizeof(VNESENCRYPTIONHEADER),buff,dwBytes);
	
	
	
	
	memcpy_ex(buff,(char *)&header,sizeof(VNESENCRYPTIONHEADER));
	

	deslog.Print(0,"ENC: Header %d bytes",dwBytes);

	return;
}


void	
vncDesktop::InsertENCData(BYTE *buff,DWORD dwSize,int nRemainder)
{
	DWORD dwCurrlen = dwSize;

	
	dwCurrlen -= nRemainder;

	if(dwCurrlen >= 8)
	{
		
		rdEncryptBytes((unsigned char *)buff,dwCurrlen);

		deslog.Print(0,"ENC: encrypted and copied %d.",dwCurrlen);
	}

	if(nRemainder != 0 )
	{
		char sLastEight[8] = {0,0,0,0,0,0,0,0}; 
			
		
		
		memcpy_ex(sLastEight,buff+dwCurrlen,nRemainder);
		

		
		rdEncryptBytes((unsigned char *)sLastEight,8);

		
		
		
		memcpy_ex(buff+dwCurrlen,sLastEight,8);
		

		deslog.Print(0,"ENC: last eight copied.");
	}

	deslog.Print(0,"ENC: InsertENCData() finished.");

	return;
}


BOOL	
vncDesktop::SignalSendBuffsReady(BOOL bSet)
{
	if(m_hSendBuffsReady == NULL)
		return FALSE;

	BOOL ret;

	
	{
		CMyMutexLock lock(m_csReadySignalEvent);
	
		if(bSet)
		{
			ret =  SetEvent(m_hSendBuffsReady);
			
		}
		else
		{
			ret =  ResetEvent(m_hSendBuffsReady);
			
		}
	}

	return ret;
}


void
vncDesktop::CheckSendBuffers()
{
	

	BOOL  bEmptyExists = false;
	BOOL  bFullExists = false;

	{
		CMyMutexLock clientlock(m_csClientArray);

		for(int s = 0; s < m_dwSendTrdCount ; s++)
		{
			if(m_pSendTrds[s] == NULL)
				continue;
			
			if(m_pSendTrds[s]->IsAlive())
			{
				if(m_pSendTrds[s]->IsListFull())
					bFullExists = true;
				else if(m_pSendTrds[s]->IsListEmpty())
					bEmptyExists = true;
				else
					bufflog.Print(0,"client %lu list size = %lu",s,m_pSendTrds[s]->GetListSize());
			}
		}
	}

	if(bEmptyExists && !bFullExists)
	{
		if(!SignalSendBuffsReady(true))
			bufflog.Print(0,"WARNING: SignalSendBuffReady failed.");
		else
			bufflog.Print(0,"SEND BUFF SIGNALLED.");
	}

	return;
}


BOOL			
vncDesktop::CheckSendTimes()
{
	
	
	DWORD dwTimeNow = GetTickCount();

	if(m_dwSendTreadWaitLimit != 0)
	{
		CMyMutexLock clientlock(m_csClientArray);

		for(int s = 0; s < m_dwSendTrdCount ; s++)
		{
			if(m_pSendTrds[s] == NULL)
				continue;
			
			if(!m_pSendTrds[s]->IsAlive())
				continue;

			DWORD dwClientTime = m_pSendTrds[s]->GetTimeStart();
			
			if(dwClientTime != 0 && dwTimeNow > dwClientTime)
			{
				

				DWORD dwWaitTime = dwTimeNow - dwClientTime;
				if(dwWaitTime > m_dwSendTreadWaitLimit)
				{
					m_pSendTrds[s]->SetAlive(false);
					
					
					
					m_pSendTrds[s]->CloseSocket();
					
					
					m_pSendTrds[s]->KillLoop();
					m_pSendTrds[s]->SignalSendData(TRUE);
					EVClientLost(m_pSendTrds[s]->GetIPAdd());

					log.Print(0,"ERROR: dwWaitTime %lu > limit %lu 0x%8x"
						,dwWaitTime,m_dwSendTreadWaitLimit,m_pSendTrds[s]->GetIPAdd());
				}
			}
		}
	}

	return true;
}



DWORD 
vncDesktop::GetGCF(DWORD dwA, DWORD dwB)
{
	DWORD dwBig = 0, dwSmall = 0, dwTemp = 0;

	dwBig	= max(dwA,dwB);
	dwSmall = min(dwA,dwB);
	dwTemp  = dwBig % dwSmall;

	while(dwTemp > 0)
	{
		dwBig	= dwSmall;
		dwSmall	= dwTemp;
		dwTemp  = dwBig % dwSmall; 
	}
	
	return dwSmall;
}


RECT
vncDesktop::CheckStretchBlitRect(const RECT &rect, RECT *sourceRect)
{
	RECT destRect;
	SetRect(&destRect,0,0,0,0);

	if(!IsStretchBlit())
	{
		return rect;
	}
	else
	{
		DWORD dwLeftOffset		= rect.left % m_dwServerWidthRatio;
		DWORD dwTopOffset		= rect.top % m_dwServerHeightRatio;
		DWORD dwRightOffset		= m_dwServerWidthRatio  - (rect.right % m_dwServerWidthRatio);
		DWORD dwBottomOffset	= m_dwServerHeightRatio - (rect.bottom % m_dwServerHeightRatio); 
			    
		RECT srcRect;
		SetRect(&srcRect,0,0,0,0);

		srcRect.left	= rect.left	  - dwLeftOffset;
		srcRect.top		= rect.top	  - dwTopOffset;
		srcRect.right	= rect.right  + dwRightOffset;
		srcRect.bottom	= rect.bottom + dwBottomOffset;

		destRect.left	= srcRect.left   / m_dwServerWidthRatio * m_dwClientWidthRatio;
		destRect.right  = srcRect.right  / m_dwServerWidthRatio * m_dwClientWidthRatio;
		destRect.top	= srcRect.top    / m_dwServerHeightRatio * m_dwClientHeightRatio;
		destRect.bottom = srcRect.bottom / m_dwServerHeightRatio * m_dwClientHeightRatio;
			
		if(srcRect.right > m_scrinfo.framebufferWidth)
			srcRect.right = m_scrinfo.framebufferWidth;

		if(srcRect.bottom > m_scrinfo.framebufferHeight)
			srcRect.bottom = m_scrinfo.framebufferHeight;

		if(destRect.right > m_dwBitmapWidth)
			destRect.right = m_dwBitmapWidth;

		if(destRect.bottom > m_dwBitmapHeight)
			destRect.bottom = m_dwBitmapHeight;

		if(sourceRect != NULL)
		{
			SetRect(sourceRect,srcRect.left,srcRect.top,srcRect.right,srcRect.bottom);

			rectlog.Print(0,"ORIGINAL TO STRETCH RECT: %lu %lu %lu %lu : %lu x %lu"
				,rect.left,rect.top,rect.right,rect.bottom
				,rect.right-rect.left,rect.bottom-rect.top);
			rectlog.Print(0,"CAPTURE: NEW STRETCH DEST RECT: %lu %lu %lu %lu : %lu x %lu"
				,destRect.left,destRect.top,destRect.right,destRect.bottom
				,destRect.right-destRect.left,destRect.bottom-destRect.top);
			rectlog.Print(0,"CAPTURE: NEW STRETCH SOURCE RECT: %lu %lu %lu %lu : %lu x %lu"
				,sourceRect->left,sourceRect->top,sourceRect->right,sourceRect->bottom
				,sourceRect->right-sourceRect->left,sourceRect->bottom-sourceRect->top);
		}

		return destRect;
	}
}


BOOL 
vncDesktop::GetNewScreenDC()
{
	
	





	return true;
}



BOOL
vncDesktop::CheckWin9x()
{
	OSVERSIONINFO versionInfo;
	versionInfo.dwOSVersionInfoSize = sizeof( OSVERSIONINFO );

	if ( GetVersionEx( &versionInfo ) ){
		switch( versionInfo.dwPlatformId ){
		case VER_PLATFORM_WIN32s:
		case VER_PLATFORM_WIN32_WINDOWS:
			return true;
		case VER_PLATFORM_WIN32_NT:
			return false;
		default:
			return false;
		}
	}else{
		return false;
	}
	return false;
}

