/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/































class vncDesktop;

#if !defined(_WINVNC_VNCDESKTOP)
#define _WINVNC_VNCDESKTOP
#pragma once


#include "stdhdrs.h"

#include "vncRegion.h"
#include "RectList.h"
#include "translate.h"


#include "VNeSThread.h"
#include "DDIHook.h"
#include "MyMutex.h"


#include "EVStream.h"



extern "C" {
#include "vncauth.h"
}

#include "VSocket.h"

class CDDIHookThread;
class CClientThread;
class CSendThread;


typedef struct tagSendTrdParam
{
	vncDesktop	*pDesktop;
	DWORD		dwClientID;

}VNES_SENDTRDPARAM;


struct TIGHT_CONF_COPY {
	int maxRectSize, maxRectWidth;
};


class vncDesktop
{


public:


public:

	
	BOOL	m_bFirstCapture;
	DWORD	m_dwFullScreenCaptureTime;
	BOOL	m_bDoSleep;

	DWORD	m_dwFullCaptureLimitTime;
	DWORD	m_dwEdgeCheckColorPercent;
	DWORD	m_dwEdgeCheckSamplingPixels;
	BOOL	m_bOldEdgeCheck;
	DWORD	m_dwResendJpegRects;
	DWORD	m_dwResendZlibAreaLimit;
	BOOL	CheckWin9x();
	

	
	vncDesktop(HWND hwnd, char *sPath);
	~vncDesktop();

	BOOL Init();

	
	void FillDisplayInfo(rfbServerInitMsg *scrInfo);
	
	BOOL CaptureScreen(RECT &rect, BYTE *scrBuff, UINT scrBuffSize);
	
	BOOL CaptureScreenDiv(RECT &rect, DWORD &dwHeight);
	BOOL CaptureScreenNorm(RECT &rect);

	int ScreenBuffSize();
	HWND Window() {return m_hwnd;};

	
	void CaptureMouse(BYTE *scrBuff, UINT scrBuffSize);
	
	RECT CheckMouseStretchRect(RECT &rect);
	RECT MouseRect();
	void SetMouseRect();
	
	
	void SetMyCursor(HCURSOR cursor);

	
	
	inline VOID *OptimisedBlitBuffer() {return m_DIBbits;};

	
protected:

	
	static const TIGHT_CONF_COPY m_conf_copy[10];

	
	BOOL Startup();
	BOOL Shutdown();
	
	
	BOOL InitBitmap();
	BOOL ThunkBitmapInfo();
	BOOL SetPixFormat();
	BOOL SetPixShifts();
	BOOL SetPalette();

	BOOL CopyToBuffer(RECT &rect, BYTE *scrBuff, UINT scrBuffSize);
	
	
	void EnableOptimisedBlits();

	
	static void MaskToMaxAndShift(DWORD mask, CARD16 &max, CARD8 &shift);
	
	

	
	HWND			m_hwnd;

	
	HDC				m_hmemdc;
	
	
	

	
	HBITMAP			m_membitmap;

	RECT			m_bmrect;
	
	struct _BMInfo {
		BOOL			truecolour;
		BITMAPINFO		bmi;
		
		RGBQUAD			cmap[256];
	} m_bminfo;

	
	rfbServerInitMsg	m_scrinfo;
	rfbServerInitMsg	m_REALscrinfo;

	
	int				m_bytesPerRow;

	
	HCURSOR			m_hcursor;
	
	HCURSOR			m_hdefcursor;
	
	BOOL			m_cursormoved;
	RECT			m_cursorpos;

	
	VOID			*m_DIBbits;
	BOOL			m_formatmunged;
	
	
	rectlist m_rlCaptured;		

	
	BOOL	m_bEncryption;
	BOOL	IsEncrypt(){ return m_bEncryption; };

	
	BOOL	LoadRegistry();
	BOOL	SaveRegistry();

public:
	BOOL	CheckHookValidate(RECT & rect);
	
	
	
	BOOL	HookInit();
	BOOL	HookFinalize();

	
	BOOL	LoadDDIHookLibrary();
	

	
	BOOL	QuitAppMessage(DWORD dwErrorCode);
	
	BOOL	PostCloseMessage();
	

	
	void	PostCaptureErrorMsg ();
	void	PostDriverErrorMsg();

	
	BOOL	InitThreads();
	BOOL	ShutdownThreads();

	
	
	BOOL	OnDisplayChanged(WPARAM wParam,LPARAM lParam);
	BOOL	OnPaletteChanged();
	BOOL	OnEVDisConnectRequest();
	BOOL	OnEVImageSendStart();
	BOOL	OnEVImageSendStop();
	BOOL	OnMouseMove(WPARAM wParam,LPARAM lParam);
	
	BOOL	EVClientLost();
	
	BOOL	EVClientLost(DWORD dwIP);

	BOOL	EVResolutionChange();
	BOOL	OnVNeSClientLost();
	BOOL	OnVNeSDriverError();
	BOOL	EVDriverError();

	
	BOOL	OnEVReSendUDPFullScrRect(DWORD dwRectID);
	BOOL	OnEVConnectRequest(CEVStream &ConnInfo);
	BOOL	OnEVSentDataSizeRequest(DWORD &dwBytes);
	
	
	
	CDDIHookThread		*m_pDDIHookTrd;
	CClientThread		*m_pClientTrd;
	
	
	DDIHookClass		ddiHook;
	
	
	vncRegion			m_vrUpdateReg;
	
	
	

	
	CMyMutex	m_csRegionLock;
	CMyMutex	m_csSignalEvent;
	CMyMutex	m_csReadySignalEvent;
	CMyMutex	m_csReCaptureList;
	CMyMutex	m_csShutdown;
	CMyMutex	m_csMouse;
	

	
	BOOL				m_bClientReady;
	BOOL				m_bGrabStyle;
	BOOL				m_bSendStyle;
	BOOL				m_bDisconnectReq;
	BOOL				m_bHookedRectIntoOne;
	BOOL				m_bSetsuzokuChuu;

	
	DWORD				m_dwClientWaitDelay;
	DWORD				m_dwDDIHookWaitDelay;
	DWORD				m_dwHookCaptureLimit;
	int					m_nEncodingType;
	int					m_nCompressionLevel;
	int					m_nJpegQualityLevel;
	
	DWORD				m_dwClientWaitDelaySlow;
	DWORD				m_dwClientWaitDelaySlowLimitRects;
	
	
	

	

	BOOL				m_bSendFullScreen;
	DWORD				m_dwFullScrRefreshDelay;
	DWORD				m_dwFullScreenCount;
	BOOL				m_bPalettechanged;			
	DWORD				m_dwRegionBuffSize;
	char				m_sAppPath[_MAX_PATH];

	
	DWORD	GetPrefEncoding(){return m_nEncodingType;};
	int		GetPrefCompressLevel(){return m_nCompressionLevel;};
	int		GetPrefJpegQualityLevel(){return m_nJpegQualityLevel;};

	BOOL	SetWindowPointer();
	BOOL	IsPaletteChange(){ return m_bPalettechanged; };
	void	SetPaletteChanged(BOOL bPal){ m_bPalettechanged = bPal;};

	rfbPixelFormat		GetServerFormat(){ return m_scrinfo.format; };

	DWORD	m_dwClientTrdPriority;
	DWORD	m_dwHookTrdPriority;
	BOOL	m_bCheckDetailArea;

	
	DWORD	m_dwDetailLimit;
	DWORD	m_dwHookDetailDelay; 
	DWORD	m_dwDetailArea; 
	DWORD	m_dwDetailWidth; 
	DWORD	m_dwDetailHeight;
	

	
	HANDLE	m_hSendUpdate;
	BOOL	SignalSendUpdate(BOOL bSet);
	HANDLE	GetSendUpdateEventHandle(){ return m_hSendUpdate; };

	
	DWORD	GetEdgeCheckColorPercent(){return m_dwEdgeCheckColorPercent;};
	DWORD	GetEdgeCheckSamplingPixels(){return m_dwEdgeCheckSamplingPixels;};
	BOOL	GetOldEdgeCheck(){return m_bOldEdgeCheck;};
	

	
	HANDLE	m_hSendBuffsReady;
	BOOL	SignalSendBuffsReady(BOOL bSet);
	HANDLE	GetSendBuffsReadyHandle() { return m_hSendBuffsReady; };
	void	CheckSendBuffers();
		
	
	BYTE*		m_pDataBuffer;
	DWORD		m_dwDataBuffsize;
	DWORD		m_dwDataBuffptr;
	DWORD		m_dwBuffListMaxSize;

	BOOL		CopyToDataBuffer(BYTE *buff,DWORD dwSize);
	BOOL		CopyFromDataBuffer();
	void		ResetDataBuff();
	BOOL		CreateDataBuff();
	DWORD		GetDataBuffSize() { return m_dwDataBuffsize; };
	
	BOOL		EncryptDataBuffer(BYTE *buff,DWORD &dwSize);
	void		InsertENCHeader(BYTE *buff,DWORD dwBytes);
	void		InsertENCData(BYTE *buff,DWORD dwSize,int nRemainder);

	BOOL		InitSendThread(VSocket *sock,DWORD dwIPAdd);
	CMyMutex	m_csClientArray;

	
	CSendThread*	m_pSendTrds[VNES_MAXCONNECTIONS];
	DWORD			m_dwSendTrdCount;
	DWORD			m_dwSendTrdPriority;
	BOOL			IsSendTrdEmpty() { return (m_dwSendTrdCount == 0); };
	void			ResetClientCounter() { m_dwSendTrdCount = 0; };

	VNES_SENDTRDPARAM	m_SendTrdParam[VNES_MAXCONNECTIONS];
	

	
	DWORD			m_dwBitBltdelay;
	BOOL			m_bPowerBlackOut;
	DWORD			m_dwPercentScreen;
	DWORD			m_dwScreenDivision;

	
	
	

	
	DWORD			m_dwSendTreadWaitLimit;
	BOOL			CheckSendTimes();

	LARGE_INTEGER	m_lpTimeStart;
	LARGE_INTEGER	m_lpTimeStop;

	
	BOOL	m_bUseStretchBlit;
	DWORD	m_dwBitmapWidth;
	DWORD	m_dwBitmapHeight;
	DWORD	m_dwServerWidthRatio;
	DWORD	m_dwServerHeightRatio;
	DWORD	m_dwClientWidthRatio;
	DWORD	m_dwClientHeightRatio;

	DWORD	GetGCF(DWORD dwA, DWORD dwB);
	DWORD	GetStretchHeight()	{ return m_dwBitmapHeight;	};
	DWORD	GetStretchWidth()	{ return m_dwBitmapWidth;	};
	BOOL	IsStretchBlit()		{ return m_bUseStretchBlit;	};
	RECT	CheckStretchBlitRect(const RECT &rect, RECT *sourceRect = NULL);

	
	BOOL	GetNewScreenDC();
};

#endif 
