/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/




































#include "vncEncodeZlib.h"

vncEncodeZlib::vncEncodeZlib()
{
	m_buffer = NULL;
	m_bufflen = 0;
	compStreamInited = false;
}

vncEncodeZlib::~vncEncodeZlib()
{
	if (m_buffer != NULL)
	{
		delete [] m_buffer;
		m_buffer = NULL;
	}
	if ( compStreamInited == true )
	{
		
		
		gZlibLib.DeflateEnd(&compStream);
		
	}
	compStreamInited = false;
}

void
vncEncodeZlib::Init()
{
	vncEncoder::Init();
}

void
vncEncodeZlib::LogStats()
{
	ratelog.Print(0, VNCLOG("Zlib (pure) encoder stats: dataSize=%d, \r\n"
									"rectangleOverhead=%d, encodedSize=%d, \r\n"
									"transmittedSize=%d, efficiency=%.3f \r\n"
									"compression rate=%.3f \r\n"),
				 dataSize, rectangleOverhead, encodedSize, transmittedSize,
				 ((((float)dataSize-transmittedSize)*100)/dataSize),
				 (float)(transmittedSize)/dataSize*100);
}

UINT
vncEncodeZlib::RequiredBuffSize(UINT width, UINT height)
{
	int result;

	
	
	
	result = vncEncoder::RequiredBuffSize(width, height);
	
	
	
	result += ((result / 100) + 12) + sz_rfbZlibHeader;
	

	return result;
}

UINT
vncEncodeZlib::NumCodedRects(RECT &rect)
{



	const rectW = rect.right - rect.left;
	const rectH = rect.bottom - rect.top;

	
	
	
	return (( rectH - 1 ) / ( ZLIB_MAX_SIZE( rectW ) / rectW ) + 1 );

}




inline UINT
vncEncodeZlib::EncodeRect(BYTE *source, BYTE *dest, const RECT &rect)
{
	ratelog.Print(0,"ZLIB EncodeRect"); 
	

	int  partialSize = 0;
	int  maxLines;
	int  linesRemaining;
	RECT partialRect;

	const rectW = rect.right - rect.left;
	const rectH = rect.bottom - rect.top;

	partialRect.right	= rect.right;
	partialRect.left	= rect.left;
	partialRect.top		= rect.top;
	partialRect.bottom	= rect.bottom;

	
	

	maxLines = ( ZLIB_MAX_SIZE(rectW) / rectW );
	linesRemaining = rectH;

	
	int nBuffSize = 0;
	int nReturnValue = 0;
	

	while ( linesRemaining > 0 ) {

		int linesToComp = 0;

		if ( maxLines < linesRemaining )
			linesToComp = maxLines;
		else
			linesToComp = linesRemaining;

		partialRect.bottom = partialRect.top + linesToComp;

		
		

		
		
		partialSize = EncodeOneRect( source, dest + nBuffSize, partialRect );
		

		linesRemaining -= linesToComp;
		partialRect.top += linesToComp;

		if (( linesRemaining > 0 ) &&
			( partialSize > 0 ))
		{
			
			nBuffSize += partialSize;
			
			

			transmittedSize += partialSize;
		}


	}
	
	transmittedSize += partialSize;

	
	nReturnValue = nBuffSize + partialSize;
	return nReturnValue;
	
	
}


inline UINT
vncEncodeZlib::EncodeOneRect(BYTE *source, BYTE *dest, const RECT &rect)
{
	int totalCompDataLen = 0;
	int previousTotalOut;
	int deflateResult;

	const rectW = rect.right - rect.left;
	const rectH = rect.bottom - rect.top;
	const rawDataSize = (rectW*rectH*m_remoteformat.bitsPerPixel / 8);

	
	
	
	
	const maxCompSize = (rawDataSize + (rawDataSize/200) + 12);
	

	
	if (rawDataSize < VNC_ENCODE_ZLIB_MIN_COMP_SIZE)
		return vncEncoder::EncodeRect(source, dest, rect);

	
	rfbFramebufferUpdateRectHeader *surh=(rfbFramebufferUpdateRectHeader *)dest;
	surh->r.x = (CARD16) rect.left;
	surh->r.y = (CARD16) rect.top;
	surh->r.w = (CARD16) (rectW);
	surh->r.h = (CARD16) (rectH);
	surh->r.x = Swap16IfLE(surh->r.x);
	surh->r.y = Swap16IfLE(surh->r.y);
	surh->r.w = Swap16IfLE(surh->r.w);
	surh->r.h = Swap16IfLE(surh->r.h);
	surh->encoding = Swap32IfLE(rfbEncodingZlib);
	
	dataSize += (rectW * rectH * m_remoteformat.bitsPerPixel) /8;
	rectangleOverhead += sz_rfbFramebufferUpdateRectHeader;

	
	if (m_bufflen < rawDataSize)
	{
		if (m_buffer != NULL)
		{
			delete [] m_buffer;
			m_buffer = NULL;
		}
		m_buffer = new BYTE [rawDataSize+1];
		if (m_buffer == NULL)
			return vncEncoder::EncodeRect(source, dest, rect);
		m_bufflen = rawDataSize;
	}

	
	Translate(source, m_buffer, rect);

	
	compStream.avail_in = rawDataSize;
	compStream.next_in = m_buffer;
	compStream.avail_out = maxCompSize;
	compStream.next_out = (dest+sz_rfbFramebufferUpdateRectHeader+sz_rfbZlibHeader);
	compStream.data_type = Z_BINARY;

	
	if ( compStreamInited == false )
	{

		compStream.total_in = 0;
		compStream.total_out = 0;
		compStream.zalloc = Z_NULL;
		compStream.zfree = Z_NULL;
		compStream.opaque = Z_NULL;

		
		ratelog.Print(0, VNCLOG("Calling deflateInit2 with zlib level:%d \r\n"), m_compresslevel);

		
		
		deflateResult = gZlibLib.DeflateInit2( &compStream,
			                          m_compresslevel,
					                  Z_DEFLATED,
					                  MAX_WBITS,
					                  MAX_MEM_LEVEL,
					                  Z_DEFAULT_STRATEGY );
		

		if ( deflateResult != Z_OK )
		{
			log.Print(LL_INTINFO, VNCLOG("deflateInit2 returned error:%d:%s\n"), deflateResult, compStream.msg);
			return vncEncoder::EncodeRect(source, dest, rect);
		}
		compStreamInited = true;
	}

	
	previousTotalOut = compStream.total_out;

	
	
	
	deflateResult = gZlibLib.Deflate( &compStream, Z_SYNC_FLUSH );

	if(deflateResult != Z_OK)
	{
		log.Print(LL_INTINFO, VNCLOG("WARNING: deflate returned error:%d:%s\n"), deflateResult, compStream.msg);
		return vncEncoder::EncodeRect(source, dest, rect);
	}
		
	if(compStream.avail_out == 0)
	{
		log.Print(0,"WARNING: ZLIB avail out is Zero");
	}
	
	

	
	totalCompDataLen = compStream.total_out - previousTotalOut;

	
	
	
	
	
	

	
	rfbZlibHeader *zlibh=(rfbZlibHeader *)(dest+sz_rfbFramebufferUpdateRectHeader);
	zlibh->nBytes = Swap32IfLE(totalCompDataLen);

	
	ratelog.Print(LL_INTINFO, VNCLOG("rawSize=%d compSize=%d totalRaw=%d totalComp=%d\n"), rawDataSize, totalCompDataLen, compStream.total_in, compStream.total_out);
	encodedSize += sz_rfbZlibHeader + totalCompDataLen;

	
	return sz_rfbFramebufferUpdateRectHeader +
		   sz_rfbZlibHeader +
		   totalCompDataLen;

}
