/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/



























#include "vncEncoder.h"
#include "vncBuffer.h"



static const rfbPixelFormat BGR233Format = {
    8, 8, 0, 1, 7, 7, 3, 0, 3, 6
};



vncEncoder::vncEncoder()
{
	ZeroMemory(&m_remoteformat, sizeof(m_remoteformat));
	ZeroMemory(&m_localformat, sizeof(m_localformat));
	ZeroMemory(&m_transformat, sizeof(m_transformat));
	m_transtable = NULL;
	m_localpalette = NULL;
	m_bytesPerRow = 0;

	
	m_compresslevel = 5;
	m_qualitylevel = -1;

	
	m_dwEdgeCheckColorPercent = 50;
	m_dwEdgeCheckSamplingPixels = 8;
	m_bOldEdgeCheck = FALSE;
	m_bUsedJpeg = FALSE;
	

}

vncEncoder::~vncEncoder()
{
	if (m_transtable != NULL)
	{
		free(m_transtable);
		m_transtable = NULL;
	}
	if (m_localpalette != NULL)
	{
		free(m_localpalette);
		m_localpalette = NULL;
	}
}

void
vncEncoder::Init()
{
	dataSize = 0;
	rectangleOverhead = 0;
	encodedSize = 0;
	transmittedSize = 0;
}

void
vncEncoder::LogStats()
{
	ratelog.Print(0, VNCLOG("Raw Encoder Stats: dataSize = %d, \r\n"
				"rectangle overhead = %d, encoded size = %d, \r\n"
				"transmitted size = %d, efficiency = %.3f \r\n"
				"comp rate = %.3f \r\n"),
				dataSize, rectangleOverhead, encodedSize, transmittedSize,
				((float)dataSize-transmittedSize)/dataSize,
				(float)(transmittedSize)/dataSize* 100);
}

UINT
vncEncoder::RequiredBuffSize(UINT width, UINT height)
{
	return sz_rfbFramebufferUpdateRectHeader +
		(width * height * m_remoteformat.bitsPerPixel)/8;
}

UINT
vncEncoder::NumCodedRects(RECT &rect)
{
	return 1;
}


inline void
vncEncoder::Translate(BYTE *source, BYTE *dest, const RECT &rect)
{
	
	BYTE *sourcepos = (BYTE *)(source + (m_bytesPerRow * rect.top)+(rect.left * (m_localformat.bitsPerPixel / 8)));

	
	(*m_transfunc) (m_transtable,
					&m_localformat,
					&m_transformat,
					(char *)sourcepos,
					(char *)dest,
					m_bytesPerRow,
					rect.right-rect.left,
					rect.bottom-rect.top
					);
}




inline void
vncEncoder::Translate(BYTE *source, BYTE *dest, int w, int h, int bytesPerRow)
{
	
	(*m_transfunc) (m_transtable, &m_localformat, &m_transformat,
					(char *)source, (char *)dest, bytesPerRow, w, h);
}


inline UINT
vncEncoder::EncodeRect(BYTE *source, BYTE *dest, const RECT &rect)
{
	ratelog.Print(0,"RAW EncodeRect"); 
	

	const rectW = rect.right - rect.left;
	const rectH = rect.bottom - rect.top;

	
	rfbFramebufferUpdateRectHeader *surh = (rfbFramebufferUpdateRectHeader *)dest;
	surh->r.x = (CARD16) rect.left;
	surh->r.y = (CARD16) rect.top;
	surh->r.w = (CARD16) (rect.right-rect.left);
	surh->r.h = (CARD16) (rect.bottom-rect.top);
	surh->r.x = Swap16IfLE(surh->r.x);
	surh->r.y = Swap16IfLE(surh->r.y);
	surh->r.w = Swap16IfLE(surh->r.w);
	surh->r.h = Swap16IfLE(surh->r.h);
	surh->encoding = Swap32IfLE(rfbEncodingRaw);

	
	
	rectangleOverhead += sz_rfbFramebufferUpdateRectHeader;
	dataSize += (rectW * rectH * m_remoteformat.bitsPerPixel)/8;
	encodedSize += (rectW * rectH * m_remoteformat.bitsPerPixel)/8;
	transmittedSize += sz_rfbFramebufferUpdateRectHeader + (rectW * rectH * m_remoteformat.bitsPerPixel)/8;


	
	Translate(source, dest + sz_rfbFramebufferUpdateRectHeader, rect);

	
	return sz_rfbFramebufferUpdateRectHeader +
		((rect.right-rect.left)*(rect.bottom-rect.top)*m_remoteformat.bitsPerPixel) / 8;
}
















BOOL
vncEncoder::GetRemotePalette(RGBQUAD *quadlist, UINT ncolours)
{
	log.Print(LL_INTINFO, VNCLOG("remote palette data requested\n"));

	
	
	if (!m_localformat.trueColour)
	{
		if (!SetTranslateFunction())
			return FALSE;
	}

	
	if (m_remoteformat.trueColour)
		return FALSE;

	
	if (m_localformat.trueColour)
	{
		
		log.Print(LL_INTINFO, VNCLOG("generating BGR233 palette data\n"));

		int ncolours = 1 << m_transformat.bitsPerPixel;
		if (m_localpalette != NULL)
			free(m_localpalette);
		m_localpalette = (char *)malloc(ncolours * sizeof(RGBQUAD));
		
		if (m_localpalette != NULL)
		{
			RGBQUAD *colour = (RGBQUAD *)m_localpalette;

			for (int i=0; i<ncolours; i++)
			{
				colour[i].rgbBlue = (((i >> m_transformat.blueShift) & m_transformat.blueMax) * 255) / m_transformat.blueMax;
				colour[i].rgbRed = (((i >> m_transformat.redShift) & m_transformat.redMax) * 255) / m_transformat.redMax;
				colour[i].rgbGreen = (((i >> m_transformat.greenShift) & m_transformat.greenMax) * 255) / m_transformat.greenMax;
			}
		}
	}
	else
	{
		
		log.Print(LL_INTINFO, VNCLOG("generating 8-bit palette data\n"));

		rfbPixelFormat remote;
		remote.trueColour = TRUE;
		remote.bitsPerPixel = 32;
		remote.depth = 24;
		remote.bigEndian = FALSE;
		remote.redMax = remote.greenMax = remote.blueMax = 255;
		remote.redShift = 16;
		remote.greenShift = 8;
		remote.blueShift = 0;

		
		
		(*rfbInitColourMapSingleTableFns[remote.bitsPerPixel / 16])
			(&m_localpalette, &m_localformat, &remote);
	}

	
	if (m_localpalette == NULL)
	{
		log.Print(LL_INTERR, VNCLOG("ERROR: failed to obtain colour map data!\n"));
		return FALSE;
	}

	
	
	
	memcpy_ex(quadlist, m_localpalette, ncolours*sizeof(RGBQUAD));
	

	return TRUE;
}

BOOL
vncEncoder::SetTranslateFunction()
{
	log.Print(LL_INTINFO, VNCLOG("settranslatefunction called\n"));

	
	m_transformat = m_remoteformat;

    

    if ((m_transformat.bitsPerPixel != 8) &&
		(m_transformat.bitsPerPixel != 16) &&
		(m_transformat.bitsPerPixel != 32))
    {
		log.Print(LL_CONNERR,
			VNCLOG("only 8, 16 or 32 bits supported remotely - %d requested\n"),
			m_transformat.bitsPerPixel
			);

		return FALSE;
    }
	
    if ((m_localformat.bitsPerPixel != 8) &&
		(m_localformat.bitsPerPixel != 16) &&
		(m_localformat.bitsPerPixel != 32))
    {
		log.Print(LL_CONNERR,
			VNCLOG("only 8, 16 or 32 bits supported locally - %d in use\n"),
			m_localformat.bitsPerPixel
			);

		return FALSE;
    }

	if (!m_transformat.trueColour && (m_transformat.bitsPerPixel != 8))
	{
		log.Print(LL_CONNERR, VNCLOG("only 8-bit palette format supported remotely\n"));
		return FALSE;
	}
	if (!m_localformat.trueColour && (m_localformat.bitsPerPixel != 8))
	{
		log.Print(LL_CONNERR, VNCLOG("only 8-bit palette format supported locally\n"));
		return FALSE;
	}

	

	
    if (!m_transformat.trueColour)
	{
		
		if (!m_localformat.trueColour &&
			(m_localformat.bitsPerPixel == m_transformat.bitsPerPixel))
		{
			
			log.Print(LL_INTINFO, VNCLOG("no encoding required - both 8-bit palettized\n"));

			m_transfunc = rfbTranslateNone;

			
			
			return TRUE;
		}
		else if (m_localformat.trueColour)
		{
			
			log.Print(LL_INTINFO, VNCLOG("local truecolour, remote palettized.  using BGR233 palette\n"));

			
			m_transformat = BGR233Format;

			
		}
		else
		{
			
			log.Print(LL_CONNERR, VNCLOG("unknown local pixel format in use!\n"));
			return FALSE;
		}
	}

	

	
	if (!m_localformat.trueColour)
	{
		

		
		log.Print(LL_INTINFO, VNCLOG("using 8-bit colourmap to truecolour translation\n"));

		m_transfunc = rfbTranslateWithSingleTableFns
			[m_localformat.bitsPerPixel / 16]
			[m_transformat.bitsPerPixel / 16];

		(*rfbInitColourMapSingleTableFns[m_transformat.bitsPerPixel / 16])
			(&m_transtable, &m_localformat, &m_transformat);
		return m_transtable != NULL;
	}

	

	
    if (PF_EQ(m_transformat,m_localformat))
	{
		
		log.Print(LL_INTINFO, VNCLOG("no translation required\n"));

		m_transfunc = rfbTranslateNone;

		return TRUE;
    }

	
    if (m_localformat.bitsPerPixel == 16)
	{
		
		log.Print(LL_INTINFO, VNCLOG("single LUT used\n"));

		m_transfunc = rfbTranslateWithSingleTableFns
			[m_localformat.bitsPerPixel / 16]
			[m_transformat.bitsPerPixel / 16];

		(*rfbInitTrueColourSingleTableFns[m_transformat.bitsPerPixel / 16])
			(&m_transtable, &m_localformat, &m_transformat);
    }
	else
	{
		
		log.Print(LL_INTINFO, VNCLOG("triple LUT used\n"));

		m_transfunc = rfbTranslateWithRGBTablesFns
			[m_localformat.bitsPerPixel / 16]
			[m_transformat.bitsPerPixel / 16];

		(*rfbInitTrueColourRGBTablesFns[m_transformat.bitsPerPixel / 16])
			(&m_transtable, &m_localformat, &m_transformat);
    }

	return m_transtable != NULL;
}

BOOL
vncEncoder::SetLocalFormat(rfbPixelFormat &pixformat, int width, int height)
{
	
	m_bytesPerRow = width * pixformat.bitsPerPixel/8;

	
	m_localformat = pixformat;
	return SetTranslateFunction();
}

BOOL
vncEncoder::SetRemoteFormat(rfbPixelFormat &pixformat)
{
	
	m_remoteformat = pixformat;

	log.Print(0,"CLIENT PIXEL FORMAT: %d",m_remoteformat.bitsPerPixel);

	return SetTranslateFunction();
}


void
vncEncoder::SetCompressLevel(int level)
{
	m_compresslevel = (level >= 0 && level <= 9) ? level : 6;
}


void
vncEncoder::SetQualityLevel(int level)
{
	m_qualitylevel = (level >= 0 && level <= 9) ? level : -1;

}
