/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/




























#include "stdhdrs.h"



#include "vncRegion.h"



vncRegion::vncRegion()
{
	region = NULL;
}

vncRegion::~vncRegion()
{
	Clear();
}


void vncRegion::AddRect(RECT &rect)
{
	HRGN newregion;

	if (region == NULL)
	{
		
		
		region = CreateRectRgnIndirect(&rect);
	}
	else
	{
		
		
		newregion = CreateRectRgnIndirect(&rect);

		
		if (CombineRgn(region, region, newregion, RGN_OR) == NULLREGION)
			Clear();

		
		DeleteObject(newregion);
	}
}


void vncRegion::SubtractRect(RECT &rect)
{
	HRGN newregion;

	if (region == NULL)
		return;

	
	
	newregion = CreateRectRgnIndirect(&rect);

	
	if (CombineRgn(region, region, newregion, RGN_DIFF) == NULLREGION)
		Clear();

	
	DeleteObject(newregion);
}

void vncRegion::Clear()
{
	
	if (region != NULL)
	{
		DeleteObject(region);
		region = NULL;
	}
}

BOOL vncRegion::IsEmpty()
{
	return region == NULL;
}

void
vncRegion::Combine(vncRegion &rgn)
{
	if (rgn.region == NULL)
		return;
	if (region == NULL)
	{
		region = CreateRectRgn(0, 0, 0, 0);
		if (region == NULL)
			return;

		
		if (CombineRgn(region, rgn.region, 0, RGN_COPY) == NULLREGION)
			Clear();
		return;
	}

	
	if (CombineRgn(region, region, rgn.region, RGN_OR) == NULLREGION)
		Clear();
}

void
vncRegion::Intersect(vncRegion &rgn)
{
	if (rgn.region == NULL)
		return;
	if (region == NULL)
		return;

	
	if (CombineRgn(region, region, rgn.region, RGN_AND) == NULLREGION)
		Clear();
}

void
vncRegion::Subtract(vncRegion &rgn)
{
	if (rgn.region == NULL)
		return;
	if (region == NULL)
		return;

	
	if (CombineRgn(region, region, rgn.region, RGN_DIFF) == NULLREGION)
		Clear();
}




BOOL vncRegion::Rectangles(rectlist &rects)
{
	int buffsize;
	int x;
	RGNDATA *buff;

	
	if (region == NULL)
	{
		log.Print(0,"vncRegion: WARNING region is NULL.");
		
		
		return TRUE;
		
	}

	
	buffsize = GetRegionData(region, NULL, 0);

	
	if(buffsize == 0)
	{
		
		log.Print(0,"WARNING: buffsize = 0  No rect to get..");
		return TRUE;
	}
	

	buff = (RGNDATA *) new BYTE [buffsize];
	if (buff == NULL)
	{
		log.Print(0,"vncRegion: ERROR couldn't create buff.");
		return FALSE;
	}

	
	if (GetRegionData(region, buffsize, buff))
	{
		for (x=0; x<(buff->rdh.nCount); x++)
		{
			
			RECT *rect = (RECT *) (((BYTE *) buff) + sizeof(RGNDATAHEADER) + x * sizeof(RECT));
			
			
			
			rects.push_back(*rect);
		}
	}
	else
	{
		log.Print(0,"WARNING: GetRegionData() Failed. buffsize = %d",buffsize);
	}

	
	
	
	if(buff != NULL){
		delete [] buff;
		buff = NULL;
	}
	

	
	if(rects.empty())
	{
		log.Print(0,"vncRegion: WARNING: rectlist empty after Rectangles().");
	}

	return TRUE;

	
	
	
	
}


BOOL vncRegion::Rectangles(rectlist &rects, RECT &cliprect)
{
	vncRegion cliprgn;

	
	cliprgn.AddRect(cliprect);

	
	cliprgn.Intersect(*this);

	return cliprgn.Rectangles(rects);
}


BOOL 
vncRegion::GetRegionBox(RECT &rect)
{
	if(region == NULL)
		return FALSE;

	return GetRgnBox(region,&rect);
}


BOOL 
vncRegion::IsEqualRgn(vncRegion &rgn)
{
	if(region == NULL || rgn.region == NULL)
		return FALSE;

	return EqualRgn(region,rgn.region);
}

