#include "config.h"
#include "debug.h"
#include "SDL_util.h"
#include "VarNavGuide.h"

VarNavGuide::VarNavGuide(SDL_Surface *screen,
                         const SDL_Rect& rect)
    : NavGuide(screen,rect)
{
    m_BgRsrcName = 0;
}

VarNavGuide::~VarNavGuide()
{    
}

int
VarNavGuide::LoadBgRsrc()
{
    DebugPrint("%s: Entering\n", __PRETTY_FUNCTION__);
    
    SDL_PixelFormat* format = m_screen->format;
    m_BgRsrc = SDL_CreateRGBSurface(SDL_SWSURFACE,
                                    drawable_area.w, drawable_area.h,
                                    format->BitsPerPixel,
                                    format->Rmask,
                                    format->Gmask,
                                    format->Bmask,
                                    format->Amask);
    if ( m_BgRsrc == 0 ) {
        SDL_ERROR();
        return -1;
    }

    /* Load in the left end of the guide bar */
    SDL_Surface* leftImage;
    leftImage = SDL_LoadImage(RESOURCE_DIR "navBarEndLeft.png", 
                              NULL, SDL_ROTATE_0, 
                              SDL_STRETCH_BILINEAR, SDL_FALSE);
    if ( leftImage == 0 ) {
        DebugPrint("%s: Unable to load %s",     
                   __PRETTY_FUNCTION__, "navBarEndLeft.png\n");
        SDL_FreeSurface(m_BgRsrc);
        m_BgRsrc = 0;
        return -1;
    }
    SDL_Rect leftRect = { 0, 0, leftImage->w, leftImage->h };
    SDL_BlitSurface(leftImage, NULL, m_BgRsrc, &leftRect);
    SDL_FreeSurface(leftImage);
    leftImage = 0;

    /* Load in the right end of the guide bar */
    SDL_Surface* rightImage;
    rightImage = SDL_LoadImage(RESOURCE_DIR "navBarEndRight.png", 
                               NULL, SDL_ROTATE_0, 
                               SDL_STRETCH_BILINEAR, SDL_FALSE);
    if ( rightImage == 0 ) {
        DebugPrint("%s: Unable to load %s",     
                   __PRETTY_FUNCTION__, "navBarEndRight.png\n");
        SDL_FreeSurface(m_BgRsrc);
        m_BgRsrc = 0;
        return -1;
    }
    SDL_Rect rightRect = { drawable_area.w - rightImage->w, 0, 
                           rightImage->w, 
                           rightImage->h };
    SDL_BlitSurface(rightImage, NULL, m_BgRsrc, &rightRect);
    SDL_FreeSurface(rightImage);
    rightImage = 0;

    /* Load in and stretch the middle part of the guide bar */
    SDL_Surface* midImage;
    midImage = SDL_LoadImage(RESOURCE_DIR "navBarMiddle.png", 
                             NULL, SDL_ROTATE_0, 
                             SDL_STRETCH_BILINEAR, SDL_FALSE);
    if ( midImage == 0 ) {
        DebugPrint("%s: Unable to load %s",     
                   __PRETTY_FUNCTION__, "navBarMiddle.png\n");
        SDL_FreeSurface(m_BgRsrc);
        m_BgRsrc = 0;
        return -1;
    }
    SDL_Rect midRect = { leftRect.w, 0, 
                         (drawable_area.w - rightRect.w - leftRect.w), 
                         midImage->h };
    SDL_StretchBlit(midImage, NULL, m_BgRsrc, &midRect, SDL_STRETCH_BILINEAR);
    SDL_FreeSurface(midImage);
    midImage = 0;

    DebugPrint("%s: Leaving\n", __PRETTY_FUNCTION__);
    return 0;
}
