/*!
  \file   vnes.h
  \author Aric Cyr
  \date   Thu Jul 18 10:59:58 2002
  
  \brief  VNeS server communication protocol interface

  The VNeS protocol says that we must communicate missing fullscreen
  updates to the server.  This module handles that.
*/

#ifndef VNES_H
#define VNES_H

#include "vnesproto.h"

#ifdef __cplusplus
extern "C" {
#endif

/*!  Connect to the EasyConnect daemon via a unix domain socket.

  \return 0 on success, otherwise error.
*/
int ConnectDomainSocket();

/*!  Connect to the EasyConnect daemon via a unix domain socket.

  \return 0 on success, otherwise error.
*/
void DisconnectDomainSocket();

/*!  Check the socket for any pending messages and process them.

  \return 0 on success, otherwise error.
*/
void PollDomainSocket();


/*!  Notification of the start of a new fullscreen update cycle has
begun.
  
  \param totalID The total number of expected update packets.
*/
void StartFullscreenUpdate(unsigned int totalID);


/*!  Acknowledge receipt of a fullscreen update packet.

  \param itemID ID of the received packet.
*/
void AckFullscreenUpdate(unsigned int ackID);


/*! Process all queued missing fullscreen updates and send a request
  to have them resent.
*/
void EndFullscreenUpdate();

/*! Send a message to the controlling daemon.
*/
int SendDomainMessage(const vnes_msg_t* message);

#ifdef __cplusplus
}
#endif

#endif /* VNES_H */
