/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/




#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <cstdlib>
#include <cassert>
#include <cstring>

#include "debug.h"
#include "libkeyinput.h"
#include "configescvp.h"


#ifdef mips
static const char* DEFAULT_DEVICE = "eth0";
#else
static const char* DEFAULT_DEVICE = "wlan0";
#endif

static const COMID OPTION_TO_COMID[CONFIG_OPTION_LAST] = {
    cmdIMNWKEY,        
    cmdIMNWCH,         
    cmdIMNWESS,        
    cmdIMNWADHOC,      
    cmdIMNWWEP,        
    cmdIMNWWEPMODE,    
    cmdIMNWWEPKEY,     
    cmdIMNWPNAME,      
    cmdIMNWMODE,       
    cmdIMNWDHCP,       
    cmdIMNWIP,         
    cmdIMNWMASK,       
    cmdIMNWGW,         
    cmd_PASSWD,        
    cmdIMNWTRAPIP1,    
    cmdIMNWTRAPIP2,    
    _END_              
};


ConfigESCVP::ConfigESCVP()
{
    
    Read();
}

ConfigESCVP::~ConfigESCVP()
{
#ifdef mips
    if (LKI_CloseSocket() != LKI_SUCCESS) {
        DebugPrint("%s: Failed on call to LKI_CloseSocket\n", __FUNCTION__);
    }
#endif
}


void
ConfigESCVP::Read()
{
    DeleteAllOptions();

    
    
#ifdef mips
    if (LKI_OpenSocket(NULL) != LKI_SUCCESS) {
        DebugPrint("!!!! %s: Failed to open LKI\n", __FUNCTION__);
        return;
    }
#else
    if (LKI_OpenSocket("163.141.162.200") != LKI_SUCCESS) {
        return;
    }
#endif  

    char buf[ESCVP21_MAXLEN];
    int ret;
    for (int i = 0; i < CONFIG_OPTION_LAST; i++ ) {
        if ( OPTION_TO_COMID[i] == _END_ ) {
            switch (i) {
            case CONFIG_DEVICE:
                SetOption(CONFIG_DEVICE, DEFAULT_DEVICE);
                break;

            default:
                DebugPrint("Unknown configure option requested\n");
                break;
            }
        } else {
            ret = GetValue(OPTION_TO_COMID[i], buf, sizeof(buf));
            if ( ret == 0 ) {
                SetOption(static_cast<CONFIG_OPTION>(i), buf);
            }
        }
    }
}


void
ConfigESCVP::Write()
{
    const char* value;
    int ret;

    
    
    
#ifdef mips
    if (LKI_OpenSocket(NULL) != LKI_SUCCESS) {
        DebugPrint("!!!! %s: Failed to open LKI\n", __FUNCTION__);
        return;
    }
#else
    if (LKI_OpenSocket("163.141.162.200") != LKI_SUCCESS) {
        return;
    }
#endif  

    for ( int option = 0; option < CONFIG_OPTION_LAST; option++ ) {
        if (OPTION_TO_COMID[option] == _END_ ) {
            continue;
        }

        value = GetOption(static_cast<CONFIG_OPTION>(option));
        if ( value ) {
            DebugPrint("Writing option %d = '%s'\n", option, value);
            ret = SetValue(OPTION_TO_COMID[option], value);
            if ( ret != 0 ) {
                DebugPrint("Failed to writing option %d\n", option);
            }

        } else {
            DebugPrint("Clearing option %d\n", option);
            if (IsOptionSet(static_cast<CONFIG_OPTION>(option))) {
                ret = SetValue(OPTION_TO_COMID[option], "INIT");
                if ( ret != 0 ) {
                    DebugPrint("Failed to clear option %d\n", option);
                }            
            }
        }
    }
}



int
ConfigESCVP::GetValue(const COMID eID, char* value, int bufsize)
{
    size_t read_size;

    
    
    
    COMID newID = static_cast<COMID>(eID + 1);
	read_size = LKI_Receive(newID, NULL, value, bufsize);
	if (read_size <= 0) {
        if ( read_size < 0 ) {
            DebugPrint("!!!!! %s: Error calling LKI_Receive\n", __FUNCTION__);
        }
		return -1;
	}

    
    while (read_size > 0) {
        if ( value[read_size-1] == ' ' ) {
            value[read_size-1] == '\0';
        } else {
            break;
        }
        read_size--;
    }

	return 0;
}



int
ConfigESCVP::SetValue(const COMID eID, const char* value)
{
    int ret;

    ret = LKI_Send(eID, value);
	if (ret != LKI_SUCCESS) {
        DebugPrint("!!!!! %s: Error calling LKI_Send\n", __FUNCTION__);
		return -1;
    }

	return 0;
}
