/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/




#ifndef CONFIGFILE_H
#define CONFIGFILE_H

#include <map>
#include <string>
#include <cstdio>

enum CONFIG_OPTION {
    CONFIG_KEYWORD,
    CONFIG_CHANNEL,
    CONFIG_ESSID,
    CONFIG_ADHOCMODE,
    CONFIG_WEP,
    CONFIG_WEPMODE,
    CONFIG_WEPKEY,
    CONFIG_PROJECTORNAME,
    CONFIG_NETWORK,
    CONFIG_DHCP,
    CONFIG_IP,
    CONFIG_MASK,
    CONFIG_GATEWAY,
    CONFIG_PASSWORD,
    CONFIG_TRAP1,
    CONFIG_TRAP2,
    CONFIG_DEVICE,
    CONFIG_OPTION_LAST
};


class ConfigFile
{
public:
    ConfigFile(const char* filename = 0);
    ~ConfigFile();

    
    void Read();

    
    void Write();

    
    bool IsOptionSet(const CONFIG_OPTION option) const;

    
    const char* GetOption(const CONFIG_OPTION option) const;

    
    void  SetOption(const CONFIG_OPTION option, const char* value);

    
    void  DeleteOption(const CONFIG_OPTION option);

protected:
    
    void  DeleteAllOptions();
    
    
    static bool IsPrintable(char ch);

    
    static void TrimWhitespace(char* line);

private:
    
    static const int MaxOptionLength = 1024;

    
    typedef std::map<CONFIG_OPTION, std::string> OptionMap;
    OptionMap m_Options;

    FILE* m_FilePtr;
};

#endif 

