/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


#include <new>
#include <cassert>
#include <cerrno>
#include <unistd.h>
#include <sys/socket.h>
#include "tcp.h"

TCP::TCP(int fd) : IP(fd)
{
}

TCP::~TCP()
{
}

bool
TCP::Create()
{
	assert( m_fd == -1 );

	m_fd = socket(PF_INET, SOCK_STREAM, 0);
	if ( m_fd == -1 ) {
		m_errno = errno;
		return false;
	}

	return true;
}

bool
TCP::Listen( int backlog )
{
	assert( m_fd != -1 );

	if ( listen( m_fd, backlog ) == -1 ) {
		m_errno = errno;
		return false;
	}

	return true;
}


TCP* 
TCP::Accept( in_addr_t* addr, unsigned short *port )
{
	assert( m_fd != -1 );

	struct sockaddr_in destaddr;
	socklen_t size = sizeof(destaddr);

	int destfd;

	destfd = accept( m_fd, (struct sockaddr*)&destaddr, &size );
	if ( destfd < 0 ) {
		m_errno = errno;
		return 0;
	}

	TCP* destsock = new(std::nothrow) TCP(destfd);
	if ( destsock == 0 ) {
		close( destfd );
		return 0;
	}

	return destsock;
}
