/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


#include <cassert>
#include <cerrno>
#include <sys/socket.h>
#include "udp.h"

UDP::UDP(int fd) : IP(fd)
{
	
}

UDP::~UDP()
{

}

bool
UDP::Create()
{
	assert( m_fd == -1 );

	m_fd = socket(PF_INET, SOCK_DGRAM, 0);
	if ( m_fd == -1 ) {
		m_errno = errno;
		return false;
	}

	return true;
}

int
UDP::SendTo( const char* buffer, size_t len,
			 in_addr_t addr, unsigned short port )
{
	assert( m_fd >= 0 );

	struct sockaddr_in destaddr;

	destaddr.sin_family = AF_INET;
	destaddr.sin_port = htons(port);
	destaddr.sin_addr.s_addr = htonl(addr);

	int ret = sendto( m_fd, buffer, len, 0, 
					  (struct sockaddr*)&destaddr, 
					  sizeof(destaddr) );

	if ( ret == -1 ) {
		m_errno = errno;
	}

	return ret;
}

int
UDP::RecvFrom( char* buffer, size_t len,
			   in_addr_t* addr, unsigned short* port )
{
	assert( m_fd >= 0 );

	struct sockaddr_in destaddr;
	socklen_t size = sizeof(destaddr);

	int ret = recvfrom( m_fd, buffer, len, 0, 
						(struct sockaddr*)&destaddr, 
						&size );

	if ( ret == -1 ) {
		m_errno = errno;
		return ret;
	}

	if ( size != sizeof(destaddr) ) {
		return -1; 
	}

	*addr = ntohl(destaddr.sin_addr.s_addr);
	*port = ntohs(destaddr.sin_port);

	return ret;
}


bool
UDP::Disconnect()
{
	assert( m_fd >= 0 );

	struct sockaddr_in destaddr;

	destaddr.sin_family = AF_UNSPEC;
	destaddr.sin_port = htons(0);
	destaddr.sin_addr.s_addr = htonl(INADDR_NONE);

	if ( connect(m_fd, (struct sockaddr*)&destaddr, sizeof(destaddr)) ) {
		m_errno = errno;
		return false;
	}

	return true;
}


bool
UDP::EnableBroadcast( bool enable )
{
	int optval = (int)enable;

	if ( setsockopt(m_fd, SOL_SOCKET, SO_BROADCAST, 
					&optval, sizeof(optval)) )
	{
		m_errno = errno;
		return false;
	} else {
		return true;
	}
}
