/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <string.h>
#include <ctype.h>
#include "validate.h"



#define IPADDR_MAXLEN (4*3+3)

#ifndef FALSE
#define FALSE 0
#endif
#ifndef TRUE
#define TRUE 1
#endif

static int
get_addr(unsigned char bytes[4], const char *addr)
{
	 char *tmp, *cur, *end;
	 unsigned long val;
	 int i;
	tmp = strdup(addr);
	if (!tmp)
		return FALSE;

	cur = tmp;
	memset(bytes, 0, sizeof(bytes));
	
	for (i = 0; i <= 3; i++) {
		val = strtoul(cur, &end, 10);
		if (val > UCHAR_MAX) {
			free(tmp);
			return FALSE;
		}
		bytes[i] = val;
		if (end[0] != '.')
			break;
		cur = end + 1;
	}

	free(tmp);

	return TRUE;
}

int
CheckNetmask(unsigned char addr[4])
{
    int i, j;
    int foundZero = FALSE;

    for (i = 0; i < 4; i++) {
        for(j = 0; j < 8; j++) {
            if (addr[i] & 0x80) {
                if (foundZero) {
                    return FALSE;
                }            
            } else {
                foundZero = TRUE;
            }
            addr[i] <<= 1;
        }
    }

    return TRUE;
}

int
IsValidAddress(const char *addr, ValidateType type)
{
	 int len = strlen(addr);
	 int i, dot = 0;
	 unsigned char bytes[4];

	if (!addr)
		return FALSE;
	if (len > IPADDR_MAXLEN)
		return FALSE;

	for (i = 0; i < len; i++) {
		if (!isdigit(addr[i])) {
			if (addr[i] != '.')
				return FALSE;
			else
				dot++;
		}
	}
	if (dot != 3)
		return FALSE;

	if (get_addr(bytes, addr) != TRUE)
		return FALSE;

	if (type == typeIPAddr) {
		if (bytes[0] == 127 || bytes[0] >= 224)
			return FALSE;
		if (bytes[0] == 0)
			return FALSE;

	} else if (type == typeNetMask) {
		if ((bytes[0] & bytes[1] & bytes[2] & bytes[3]) == UCHAR_MAX)
 			return FALSE;
		if (bytes[0] != 0 && bytes[0] != 255)
			return FALSE;

        return CheckNetmask(bytes);

	} else {
		return FALSE;
	}

	return TRUE;
}

#ifdef TEST
int main()
{
	const char addr[] = "192.168.1.255";
	printf("%s is %s\n", addr, IsValidAddress(addr, typeIPAddr) == TRUE ?
		"valid" : "invalid" );
	
}
#endif
