/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#ifndef UIRSRCMGR_H
#define UIRSRCMGR_H

#include <SDL.h>

enum Language {
	LangEnglish,	
	LangJapanese,	
	LangFrench,		
	LangGermany,	
	LangItaly,		
	LangSpanish,	
	LangPortuguese,	
	LangChinese,	
	LangKorean		
};

struct rsrc_file_t;
class UIRsrcMgr {
public:
	enum Rsrc {
		
		iconSelector,		
		iconSelectorOn,		
		iconFolder,			
		iconParFolder,		
		iconFileJPEG,		
		iconFileMPEG,		
		iconFileSIT1,		
		iconFileSIT2,		
		iconFileDPOF,		
		iconFileWav,		

		
		btbBtnSelector,		
		btbBtnSelectorOn,	
		btbButton,			

		
		btbNextPage,		
		btbPrevPage,		
		btbThumbnail,		
		btbIcon,			
		btbOK,				
		btbCancel,			
		btbReset,			
        btbBtnAdvanced,     

		
		propUnselectedL,	
		propUnselectedB,	
		propUnselectedR,	
		propSelectedL,		
		propSelectedB,		
		propSelectedR,		
		propShadowL,		
		propShadowB,		
		propShadowR,		
		propOnL,			
		propOnB,			
		propOnR,			

		
		propOpenFolder,		
		propPlaySlideShow,	
		propSetOptions,		
		propPlayScenario,	
		propEditScenario,	
		propPlayPict,		
		propPlayMovie,		
		propMove,			
		propShowHide,		
		propCancel,			
		propToUp,			

		
		optRepeat,			
		optInterval,		
		optSort,			
		optEffect,			
		optUIMode,
		optHLine,			

		
		rbtnBtnOn,
		rbtnBtnOff,
		rbtnBtnSelector,
		rbtnBtnSelectorOn,
		rbtnTag10sec,
		rbtnTag20sec,
		rbtnTag30sec,
		rbtnTag1min,
		rbtnTagDisolve,
		rbtnTagNone,
		rbtnTagOn,
		rbtnTagOff,
		rbtnTagSortName,
		rbtnTagSortTime,
		rbtnTagWipe1,
		rbtnTagWipe2,
		rbtnTagGuideMode,
		rbtnTagQuickMode,
		rbtnTagNetEasy,
		rbtnTagNetAP,

		
		navIconEnter,
		navIconEscape,
		navIconCursors,
		navIconPageUp,
		navIconPageDown,
		navIconArwVert,
		navIconArwHorz,

		
		navTagDecide,
		navTagSelect,
		navTagNextPage,	
		navTagPrevPage,
		navTagBack,	
		navTagCancel,	
		navTagOpFolder,
		navTagProperty,	
		navTagPlMovie,	
		navTagPlPict,	
		navTagPlScenario,	
		navTagShowPict,	
		navTagHidePict,	
		navTagToUp,	
		navTagMove,
		navTagToTop,
		navTagMoveCursor,

		
		dlgBackground,

		
		dlgExit,
		dlgContinue,
		
		
		sedSelector,
		sedSelectorOn,
		sedMarker,
		sedMarkerOn,
		sedDisable,

		
		pvtFolder,
		pvtPreview,
		pvtOption,
		pvtShow,
		pvtHide,
		pvtMoveFile,

		
		cfgFrameL,
		cfgFrameM,
		cfgFrameR,
		cfgNetDetail,
		cfgNetwork,
		cfgAccessPoint,
		cfgEasy, 
		cfgWired,
        cfgAntennaOn,
        cfgAntennaOff,
        cfgAntennaGray,
        cfgAntenna,

		
		msgBg,
		msgQuitScenario,
		msgQuitSlideshow,

        
        erdTagChSet,
        erdTagDHCP,
        erdTagESSID,
        erdTagGateway,
        erdTagIPAddress,
        erdTagKeyword,
        erdTagMACAddress,
        erdTagPassword,
        erdTagProjectorName,
        erdTagSubnet,
        erdTagTrapIP1,
        erdTagTrapIP2,
        erdTagWEPKey,
        erdTagSeparator,
        erdTagAdhocMode,
        erdTagAdhocEssID,
        erdTagAdhocChannel,
        erdInputShort,
        erdInputLong,
        erdInputLong26,

        
        btnClr,
        btnClrSelectorOff,
        btnClrSelectorOn,
        btnClrLabel,

        
        navBarBase,
        navBtn,
        navBtnGray,
        navBtnPress,
        navBtnSelect,
        navBtnLabel,
        labGuide1,
        labGuide2_1,
        labGuide2_2,
        labGuide2_3,
        labGuide2_4,
        labGuide3,
        labGuide4,
        msgBarBack1,
        msgBarBack2,

	
	stuTitle,			
	stuNsTitle,			
	stuCpTitle,			
	stuNsDsc,			
	stuCpDsc,			

		
		nofFileNotFound,
		nofBadFile
	};

	enum Module {
		ModNone		= 0x0,
		ModEMP		= 0x1,	
		ModVNC		= 0x2,	
		ModCONF		= 0x4,	
		ModAll 		= ModEMP|ModVNC|ModCONF
	};

public:
	static UIRsrcMgr*	Instance(const Language& lang)
	{
		instance = Instance();
		if (instance && instance->SetLang(lang)) {
			delete instance;
			instance = NULL;
		}
		return instance;
	}	

	static UIRsrcMgr*	Instance(Uint8 lang)
	{
		return Instance(E21Lang2UILang(lang));
	}

	static UIRsrcMgr*	Instance()
	{
		if (instance == NULL) {
			instance = new UIRsrcMgr();
		}
		return instance;
	}

protected: 
	UIRsrcMgr();
		~UIRsrcMgr();

	static Language	E21Lang2UILang(Uint8 lang);

public:
	int				Init		(Module using_module = ModNone);

	SDL_Surface*	GetResource	(const Rsrc	what);

	void			Dispose		();

	int				SetLang		(const Uint8 lang);

protected:
	void			DisposeLocalizeRsrc();

	int				Load		(rsrc_file_t& rsrc_file);

	int				SetLang		(const Language& lang);

	const Language&	GetLang		() {return m_lang;}

protected:
	static UIRsrcMgr*	instance;

	Language		m_lang;

	Module			m_using_module;
};

#endif 
