/*
    Copyright (C) SEIKO EPSON CORPORATION 2002-2003. All Rights Reserved.

    3-3-5, Owa, Suwa-shi Nagano-ken, 392-8502 Japan
    Phone: +81-266-52-3131

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include <string.h>
#include "config.h"


#include "SDL_util.h"

#include "UIRsrcMgr.h"


UIRsrcMgr*	UIRsrcMgr::instance = NULL;

enum LocalRsrc {
	lIconSelector,
	lIconSelectorOn,
	lFolder,
	lFolderParent,
	lFileSIT1,
	lFileJPEG,
	lFileMPEG,

	lBtbBtnSelector,
	lBtbBtnSelectorOn,
	lBtbButton,
	lBtbNextPage,
	lBtbPrevPage,
	lBtbThumbnail,
	lBtbIcon,
	lBtbOK,
	lBtbCancel,
	lBtbReset,
    lBtbBtnAdvanced,

	lPropUnselectedL,
	lPropUnselectedB,
	lPropUnselectedR,
	lPropSelectedL,
	lPropSelectedB,
	lPropSelectedR,
	lPropShadowL,
	lPropShadowB,
	lPropShadowR,
	lPropOnL,
	lPropOnB,
	lPropOnR,

	lPropOpenFolder,
	lPropPlaySlideShow,
	lPropSetOptions,
	lPropPlayScenario,
	lPropEditScenario,
	lPropPlayPict,
	lPropPlayMovie,
	lPropMove,
	lPropShowHide,
	lPropCancel,
	lPropToUp,

	lOptRepeat,
	lOptInterval,
	lOptSort,
	lOptEffect,
	lOptUIMode,
	lOptHLine,

	lRbtnBtnOn,
	lRbtnBtnOff,
	lRbtnBtnSelector,
	lRbtnBtnSelectorOn,
	lRbtnTag10sec,
	lRbtnTag20sec,
	lRbtnTag30sec,
	lRbtnTag1min,
	lRbtnTagDisolve,
	lRbtnTagNone,
	lRbtnTagOn,
	lRbtnTagOff,
	lRbtnTagSortName,
	lRbtnTagSortTime,
	lRbtnTagWipe1,
	lRbtnTagWipe2,
	lRbtnTagGuideMode,
	lRbtnTagQuickMode,
    lRbtnTagNetEasy,
    lRbtnTagNetAP,
    
	lNavIconEnter,
	lNavIconEscape,
	lNavIconCursors,
	lNavIconPageUp,
	lNavIconPageDown,
	lNavIconArwVert,
	lNavIconArwHorz,

	lNavTagDecide,
	lNavTagSelect,
	lNavTagNextPage,
	lNavTagPrevPage,
	lNavTagBack,
	lNavTagCancel,
	lNavTagOpFolder,
	lNavTagProperty,
	lNavTagPlMovie,
	lNavTagPlPict,
	lNavTagPlScenario,
	lNavTagShowPict,
	lNavTagHidePict,
	lNavTagToUp,
	lNavTagMove,
	lNavTagToTop,
	lNavTagMoveCursor,

	lDlgBackground,
	lDlgExit,
	lDlgContinue,

	lSedSelector,
	lSedSelectorOn,
	lSedMarker,
	lSedMarkerOn,
	lSedDisable,

	lPvtFolder,
	lPvtPreview,
	lPvtOption,
	lPvtShow,
	lPvtHide,
	lPvtMoveFile,

	lCfgFrameL,
	lCfgFrameM,
	lCfgFrameR,
	lCfgNetDetail,
	lCfgNetwork,
	lCfgAccessPoint,
	lCfgEasy,
	lCfgWired,
    lCfgAntennaOn,
    lCfgAntennaOff,
    lCfgAntennaGray,
    lCfgAntenna,

	lMsgBg,
	lMsgQuitScenario,
	lMsgQuitSlideshow,

    lErdTagChSet,
    lErdTagDHCP,
    lErdTagESSID,
    lErdTagGateway,
    lErdTagIPAddress,
    lErdTagKeyword,
    lErdTagMACAddress,
    lErdTagPassword,
    lErdTagProjectorName,
    lErdTagSubnet,
    lErdTagTrapIP1,
    lErdTagTrapIP2,
    lErdTagWEPKey,
    lErdTagSeparator,    
    lErdTagAdhocMode,
    lErdTagAdhocEssID,
    lErdTagAdhocChannel,
    lErdInputShort,
    lErdInputLong,
    lErdInputLong26,

    lBtnClr,
    lBtnClrSelectorOff,
    lBtnClrSelectorOn,
    lBtnClrLabel,

    lNavBarBase,
    lNavBtn,
    lNavBtnGray,
    lNavBtnPress,
    lNavBtnSelect,
    lNavBtnLabel,
    lLabGuide1,
    lLabGuide2_1,
    lLabGuide2_2,
    lLabGuide2_3,
    lLabGuide2_4,
    lLabGuide3,
    lLabGuide4,
    lMsgBarBack1,
    lMsgBarBack2,

    lStuTitle,
    lStuNsTitle,
    lStuCpTitle,
    lStuNsDsc,
    lStuCpDsc,

	lNofFileNotFound,
	lNofBadFile
};

#include "UIRsrc.cpp"


const char *LANG_IDS[] = { "_e", "_j", "_f", "_g", "_i", "_s", "_p", "_c", "_k"};


UIRsrcMgr::
UIRsrcMgr	()
{
	m_lang = LangEnglish;
	m_using_module = ModNone;
}


UIRsrcMgr::
~UIRsrcMgr	()
{
	Dispose();
}

int UIRsrcMgr::
Load		(rsrc_file_t& rsrc_file)
{
	if (rsrc_file.surface)
		return 0;

	 char tmp[FILENAME_MAX + 1];
	assert(sizeof(tmp) > (sizeof(RESOURCE_DIR) + strlen(rsrc_file.file)));
	memset(tmp, 0, sizeof(tmp));
	strcpy(tmp, RESOURCE_DIR);

	if (rsrc_file.isLocalizable) {
		 char *p = strchr(rsrc_file.file, '.');
		if (!p) {
			return -1;
		}
		strcat(tmp, LANG_IDS[m_lang]);
		strcat(tmp, "/");		
		strncat(tmp, rsrc_file.file, p - rsrc_file.file);
		strcat(tmp, LANG_IDS[m_lang]);
		strcat(tmp, p);
	} else {
		strcat(tmp, rsrc_file.file);
	}

	DebugF(("Load %s\n", tmp));
	rsrc_file.surface = ::SDL_LoadImage(tmp, NULL, SDL_ROTATE_0, SDL_STRETCH_BILINEAR, SDL_TRUE);
	if (!rsrc_file.surface) {
		DebugF(("cannot load %s.\n", tmp));
		return -1;
	}
	return 0;
}


int UIRsrcMgr::
Init		(Module using_module)
{
	m_using_module = using_module;
	 int result;
	for (int i = 0; i < (int)(sizeof(rsrc_files)/sizeof(rsrc_file_t)); i++) {
		if (!(rsrc_files[i].align & using_module) || rsrc_files[i].surface ||
			rsrc_files[i].isLocalizable == true) {
			continue;
		}

		result = Load(rsrc_files[i]);
		if (result != 0) {
			return result;
		}
	}
	for (int i = 0; i < (int)(sizeof(rsrc_files)/sizeof(rsrc_file_t)); i++) {
		if (!(rsrc_files[i].align & using_module) || rsrc_files[i].surface ||
			rsrc_files[i].isLocalizable == false) {
			continue;
		}

		result = Load(rsrc_files[i]);
		if (result != 0) {
			return result;
		}
	}
	return 0;
}


void UIRsrcMgr::
DisposeLocalizeRsrc		()
{
	for (int i = 0; i < (int)(sizeof(rsrc_files)/sizeof(rsrc_file_t)); i++) {
		if (rsrc_files[i].surface && rsrc_files[i].isLocalizable) {
			SDL_FreeSurface(rsrc_files[i].surface);
			rsrc_files[i].surface = NULL;
		}
	}
}

void UIRsrcMgr::
Dispose		()
{
	for (int i = 0; i < (int)(sizeof(rsrc_files)/sizeof(rsrc_file_t)); i++) {
		if (rsrc_files[i].surface) {
			SDL_FreeSurface(rsrc_files[i].surface);
			rsrc_files[i].surface = NULL;
		}
	}
}

::SDL_Surface* UIRsrcMgr::
GetResource	(Rsrc id)
{
	const static struct rsrc_conv_t {
		UIRsrcMgr::Rsrc	pub;
		LocalRsrc		local;
	} rsrc_conv[] = {
		{iconSelector,		lIconSelector},
		{iconSelectorOn,	lIconSelectorOn},
		{iconFolder,		lFolder},
		{iconParFolder,		lFolderParent},
		{iconFileJPEG,		lFileJPEG},
		{iconFileMPEG,		lFileMPEG},
		{iconFileSIT1,		lFileSIT1},
		{iconFileSIT2,		lFileSIT1},
		{iconFileDPOF,		lFileSIT1},
		{iconFileWav,		lFileMPEG},
		{btbBtnSelector,	lBtbBtnSelector},
		{btbBtnSelectorOn,	lBtbBtnSelectorOn},
		{btbButton,			lBtbButton},
		{btbNextPage,		lBtbNextPage},
		{btbPrevPage,		lBtbPrevPage},
		{btbThumbnail,		lBtbThumbnail},
		{btbIcon,			lBtbIcon},
		{btbOK,				lBtbOK},
		{btbCancel,			lBtbCancel},
		{btbReset,			lBtbReset},
		{btbBtnAdvanced,	lBtbBtnAdvanced},
		{propUnselectedL,	lPropUnselectedL},
		{propUnselectedB,	lPropUnselectedB},
		{propUnselectedR,	lPropUnselectedR},
		{propSelectedL,		lPropSelectedL},
		{propSelectedB,		lPropSelectedB},
		{propSelectedR,		lPropSelectedR},
		{propShadowL,		lPropShadowL},
		{propShadowB,		lPropShadowB},
		{propShadowR,		lPropShadowR},
		{propOnL,			lPropOnL},
		{propOnB,			lPropOnB},
		{propOnR,			lPropOnR},


		{propOpenFolder,	lPropOpenFolder},
		{propPlaySlideShow,	lPropPlaySlideShow},
		{propSetOptions,	lPropSetOptions},
		{propPlayScenario,	lPropPlayScenario},
		{propEditScenario,	lPropEditScenario},
		{propPlayPict,		lPropPlayPict},
		{propPlayMovie,		lPropPlayMovie},
		{propMove,			lPropMove},
		{propShowHide,		lPropShowHide},
		{propCancel,		lPropCancel},
		{propToUp,			lPropToUp},
		{optRepeat,			lOptRepeat},
		{optInterval,		lOptInterval},
		{optSort,			lOptSort},
		{optEffect,			lOptEffect},
		{optUIMode,			lOptUIMode},
		{optHLine,			lOptHLine},

		{rbtnBtnOn,			lRbtnBtnOn},
		{rbtnBtnOff,		lRbtnBtnOff},
		{rbtnBtnSelector,	lRbtnBtnSelector},
		{rbtnBtnSelectorOn,	lRbtnBtnSelectorOn},
		{rbtnTag10sec,		lRbtnTag10sec},
		{rbtnTag20sec,		lRbtnTag20sec},
		{rbtnTag30sec,		lRbtnTag30sec},
		{rbtnTag1min,		lRbtnTag1min},
		{rbtnTagDisolve,	lRbtnTagDisolve},
		{rbtnTagNone,		lRbtnTagNone},
		{rbtnTagOn,			lRbtnTagOn},
		{rbtnTagOff,		lRbtnTagOff},
		{rbtnTagSortName,	lRbtnTagSortName},
		{rbtnTagSortTime,	lRbtnTagSortTime},
		{rbtnTagWipe1,		lRbtnTagWipe1},
		{rbtnTagWipe2,		lRbtnTagWipe2},
		{rbtnTagGuideMode,	lRbtnTagGuideMode},
		{rbtnTagQuickMode,	lRbtnTagQuickMode},
		{rbtnTagNetEasy,	lRbtnTagNetEasy},
		{rbtnTagNetAP,	    lRbtnTagNetAP},

		{navIconEnter,		lNavIconEnter},
		{navIconEscape,		lNavIconEscape},
		{navIconCursors,	lNavIconCursors},
		{navIconPageUp,		lNavIconPageUp},
		{navIconPageDown,	lNavIconPageDown},
		{navIconArwVert,	lNavIconArwVert},
		{navIconArwHorz,	lNavIconArwHorz},

		{navTagDecide,		lNavTagDecide},
		{navTagSelect,		lNavTagSelect},
		{navTagNextPage,	lNavTagNextPage},
		{navTagPrevPage,	lNavTagPrevPage},
		{navTagBack,		lNavTagBack},
		{navTagCancel,		lNavTagCancel},
		{navTagOpFolder,	lNavTagOpFolder},
		{navTagProperty,	lNavTagProperty},
		{navTagPlMovie,		lNavTagPlMovie},
		{navTagPlPict,		lNavTagPlPict},
		{navTagPlScenario,	lNavTagPlScenario},
		{navTagShowPict,	lNavTagShowPict},
		{navTagHidePict,	lNavTagHidePict},
		{navTagToUp,		lNavTagToUp},
		{navTagMove,		lNavTagMove},
		{navTagToTop,		lNavTagToTop},
		{navTagMoveCursor,	lNavTagMoveCursor},

		{dlgBackground,		lDlgBackground},
		{dlgExit,			lDlgExit},
		{dlgContinue,		lDlgContinue},
		{sedSelector,		lSedSelector},
		{sedSelectorOn,		lSedSelectorOn},
		{sedMarker,			lSedMarker},
		{sedMarkerOn,		lSedMarkerOn},
		{sedDisable,		lSedDisable},
		{pvtFolder,			lPvtFolder},
		{pvtPreview,		lPvtPreview},
		{pvtOption,			lPvtOption},
		{pvtShow,			lPvtShow},
		{pvtHide,			lPvtHide},
		{pvtMoveFile,		lPvtMoveFile},
		{cfgFrameL,			lCfgFrameL},
		{cfgFrameM,			lCfgFrameM},
		{cfgFrameR,			lCfgFrameR},
		{cfgNetDetail,		lCfgNetDetail},
		{cfgNetwork,		lCfgNetwork},
		{cfgAccessPoint,	lCfgAccessPoint},
		{cfgEasy,			lCfgEasy},
		{cfgWired,			lCfgWired},
		{cfgAntennaOn,		lCfgAntennaOn},
		{cfgAntennaOff,		lCfgAntennaOff},
		{cfgAntennaGray,	lCfgAntennaGray},
		{cfgAntenna,	    lCfgAntenna},

		{msgBg,				lMsgBg},			
		{msgQuitScenario,	lMsgQuitScenario},
		{msgQuitSlideshow,	lMsgQuitSlideshow},

        {erdTagChSet,       lErdTagChSet},
        {erdTagDHCP,        lErdTagDHCP},
        {erdTagESSID,       lErdTagESSID},
        {erdTagGateway,     lErdTagGateway},
        {erdTagIPAddress,   lErdTagIPAddress},
        {erdTagKeyword,     lErdTagKeyword},
        {erdTagMACAddress,  lErdTagMACAddress},
        {erdTagPassword,    lErdTagPassword},
        {erdTagProjectorName, lErdTagProjectorName},
        {erdTagSubnet,      lErdTagSubnet},
        {erdTagTrapIP1,     lErdTagTrapIP1},
        {erdTagTrapIP2,     lErdTagTrapIP2},
        {erdTagWEPKey,      lErdTagWEPKey},
        {erdTagSeparator,   lErdTagSeparator},
        {erdTagAdhocMode,   lErdTagAdhocMode},
        {erdTagAdhocEssID,  lErdTagAdhocEssID},
        {erdTagAdhocChannel,lErdTagAdhocChannel},
        {erdInputShort,     lErdInputShort},
        {erdInputLong,      lErdInputLong},
        {erdInputLong26,    lErdInputLong26},

        {btnClr,            lBtnClr},
        {btnClrSelectorOff, lBtnClrSelectorOff},
        {btnClrSelectorOn,  lBtnClrSelectorOn},
        {btnClrLabel,       lBtnClrLabel},

        {navBarBase,        lNavBarBase},
        {navBtn,            lNavBtn},
        {navBtnGray,        lNavBtnGray},
        {navBtnPress,       lNavBtnPress},
        {navBtnSelect,      lNavBtnSelect},
        {navBtnLabel,       lNavBtnLabel},
        {labGuide1,         lLabGuide1},
        {labGuide2_1,       lLabGuide2_1},
        {labGuide2_2,       lLabGuide2_2},
        {labGuide2_3,       lLabGuide2_3},
        {labGuide2_4,       lLabGuide2_4},
        {labGuide3,         lLabGuide3},
        {labGuide4,         lLabGuide4},
        {msgBarBack1,       lMsgBarBack1},
        {msgBarBack2,       lMsgBarBack2},

	{stuTitle,          lStuTitle},
	{stuNsTitle,	    lStuNsTitle},
	{stuCpTitle,	    lStuCpTitle},
	{stuNsDsc,          lStuNsDsc},
	{stuCpDsc,          lStuCpDsc},

		{nofFileNotFound,	lNofFileNotFound},
		{nofBadFile,		lNofBadFile}
	};

	for (int i = 0; i < (int)(sizeof(rsrc_conv)/sizeof(rsrc_conv_t)); i++) {
		if (rsrc_conv[i].pub == id) {
			if (rsrc_files[rsrc_conv[i].local].surface == NULL) {
				Load(rsrc_files[rsrc_conv[i].local]);
			}

			return rsrc_files[rsrc_conv[i].local].surface;
		}
	}

	return NULL;
}

int UIRsrcMgr::
SetLang	(const Language& lang)
{
	if (m_lang == lang) {
		return 0;
	}
	
	m_lang = lang;

	
	DisposeLocalizeRsrc();

	
	 int result = Init(m_using_module);
	if (result != 0) {
		return result;
	}

	return 0;
}

Language UIRsrcMgr::
E21Lang2UILang(Uint8 lang)
{
	const static struct lang_conv {
		Uint8 lang;
		Language ui_lang;
	} langs[] = {
		{0x01, LangEnglish},
		{0x02, LangJapanese},
		{0x03, LangFrench},
		{0x04, LangGermany},
		{0x05, LangItaly},
		{0x06, LangSpanish},
		{0x07, LangPortuguese},
		{0x08, LangChinese},
		{0x09, LangKorean}
	};
	for (int i = 0; i < (sizeof(langs)/sizeof(lang_conv)); i++) {
		if (langs[i].lang == lang) {
			return langs[i].ui_lang;
		}
	}
	return LangEnglish;
}

int UIRsrcMgr::
SetLang	(const Uint8 lang)
{
	return SetLang(E21Lang2UILang(lang));
}
